import { trigger, state, style, transition, animate } from '@angular/animations';
import { DomSanitizer } from '@angular/platform-browser';
import { of } from 'rxjs';
import { map, tap } from 'rxjs/operators';
import { CommonModule, registerLocaleData } from '@angular/common';
import localePt from '@angular/common/locales/pt';
import localePtExtra from '@angular/common/locales/extra/pt';
import { Router, ActivatedRoute, RouterModule } from '@angular/router';
import { FlexLayoutModule } from '@angular/flex-layout';
import { MatButtonModule } from '@angular/material/button';
import { MatCardModule } from '@angular/material/card';
import { MatExpansionModule } from '@angular/material/expansion';
import { MatIconModule } from '@angular/material/icon';
import { MatMenuModule } from '@angular/material/menu';
import { MatListModule } from '@angular/material/list';
import { MatProgressBarModule } from '@angular/material/progress-bar';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatTableModule } from '@angular/material/table';
import { MatTooltipModule } from '@angular/material/tooltip';
import { NgxMdService, NgxMdModule } from 'ngx-md';
import { LayoutService, LayoutModule } from '@datasus/layout';
import { MatPaginatorIntl, MatPaginatorModule } from '@angular/material/paginator';
import { MatStepperIntl } from '@angular/material/stepper';
import { MatSortHeaderIntl, MatSortModule } from '@angular/material/sort';
import { MatDatepickerIntl } from '@angular/material/datepicker';
import { Component, Input, Output, EventEmitter, Pipe, Inject, NgModule, LOCALE_ID } from '@angular/core';
import { ConfigService, DATASUS_ROUTE_HOME, AuthService, SecurityService, AuthGuard, SecurityGuard, DATASUS_ROUTE_CHOOSE_PROFILE, DATASUS_ROUTE_ERROR_BACKEND, DATASUS_ROUTE_FORBIDDEN, DATASUS_ROUTE_RELEASE_NOTES, PathGuard, CoreModule } from '@datasus/core';
import { NativeDateAdapter, MAT_DATE_LOCALE, MAT_DATE_FORMATS, DateAdapter } from '@angular/material/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NotFoundComponent {
    /**
     * @param {?} layoutService
     */
    constructor(layoutService) {
        this.layoutService = layoutService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.layoutService.off();
        console.log('%c404', 'font-size: 1.2em; color: #f66;');
    }
}
NotFoundComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-not-found',
                template: "<mat-card>\n  <mat-card-header>\n    <mat-card-title>\n      <code>:(</code>\n      Ops!\n    </mat-card-title>\n    <mat-card-subtitle>P\u00E1gina n\u00E3o encontrada</mat-card-subtitle>\n    <button mat-fab color=\"accent\" routerLink=\"/\">\n      <mat-icon>home</mat-icon>\n    </button>\n  </mat-card-header>\n  <img mat-card-image alt=\"Imagem de uma tartaruga virada, sem conseguir voltar para a posi\u00E7\u00E3o normal. Tadinha!\" src=\"/assets/datasus/commons/not-found/tartaruga-virada.gif\" />\n  <mat-card-content>\n    <p>\n      Tente digitar outra URL na barra de endere\u00E7o, ou acesse a <a routerLink=\"/\">p\u00E1gina inicial</a> do sistema.\n    </p>\n  </mat-card-content>\n</mat-card>",
                styles: ["mat-card{width:328px;margin:auto}mat-card mat-card-header{position:relative}mat-card mat-card-header button[mat-fab]{position:absolute;top:42px;right:30px}mat-card img{width:360px;min-width:360px;max-width:360px;height:225px;min-height:225px;max-height:225px}mat-card mat-card-content{position:relative;top:0;left:0}mat-card mat-card-content::after{content:\"404\";color:#fff;position:absolute;top:-52px;right:-18px;font-size:48px}"]
            }] }
];
/** @nocollapse */
NotFoundComponent.ctorParameters = () => [
    { type: LayoutService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FabExtendedComponent {
    constructor() {
        this.color = 'primary';
        this.showTextOnlyOnHover = false;
        this.showTextOnlyOnFocus = false;
    }
}
FabExtendedComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-fab-extended',
                template: "<button\n    type=\"button\"\n    class=\"fab-extended\"\n    mat-fab\n    [color]=\"color\"\n    [attr.ariaLabel]=\"description\"\n    [matTooltip]=\"description\"\n    [class.fixed-text]=\"!showTextOnlyOnFocus && !showTextOnlyOnHover\"\n    [class.apply-hover-state]=\"showTextOnlyOnHover\"\n    [class.apply-focus-state]=\"showTextOnlyOnFocus\"\n>\n    <mat-icon class=\"icon\">{{ icon }}</mat-icon>\n    <span class=\"text\">{{ text }}</span>\n</button>\n",
                styles: ["button.fab-extended{position:relative;top:0;left:0;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;border-radius:50px;padding:0 16px;max-width:56px;transition:width 350ms,max-width .5s ease-out}button.fab-extended span.text{position:absolute;top:16px;left:100px}button.fab-extended.apply-focus-state:focus,button.fab-extended.apply-hover-state:hover,button.fab-extended.fixed-text{width:auto;max-width:315px;transition:width 350ms,max-width .5s ease-in}button.fab-extended.apply-focus-state:focus span.text,button.fab-extended.apply-hover-state:hover span.text,button.fab-extended.fixed-text span.text{position:relative;top:0;left:0;margin-left:16px}"]
            }] }
];
FabExtendedComponent.propDecorators = {
    color: [{ type: Input }],
    description: [{ type: Input }],
    icon: [{ type: Input }],
    text: [{ type: Input }],
    showTextOnlyOnHover: [{ type: Input }],
    showTextOnlyOnFocus: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DynamicActionsComponent {
    constructor() {
        this.actions = [];
        this.showActions = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (!this.type) {
            throw new Error('É necessário adicionar o tipo.');
        }
    }
    /**
     * @param {?} menu
     * @return {?}
     */
    emitClick(menu) {
        if (!this.actionCallbackParam) {
            return menu.callback();
        }
        return menu.callback(this.actionCallbackParam);
    }
}
DynamicActionsComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-dynamic-actions',
                template: "<div *ngIf=\"type === 'menu'\" #menuTrigger>\n    <button mat-icon-button [matMenuTriggerFor]=\"menuTrigger\">\n        <mat-icon>more_vert</mat-icon>\n    </button>\n    <mat-menu #menuTrigger=\"matMenu\">\n        <button\n            mat-menu-item\n            [ngClass]=\"'color-' + action.color\"\n            [disabled]=\"action.disabled\"\n            *ngFor=\"let action of actions\"\n            (click)=\"emitClick(action)\"\n        >\n            <mat-icon [color]=\"action.color\">{{ action.icon }}</mat-icon>\n            <span>{{ action.label }}</span>\n        </button>\n    </mat-menu>\n</div>\n\n<div\n    *ngIf=\"(type === 'actions' || type === 'staticActions')\"\n    [class.actions-wrapper]=\"type === 'actions'\"\n    (mousemove)=\"showActions = true\"\n    (mouseleave)=\"showActions = false\"\n    (pointermove)=\"showActions = true\"\n    (pointerleave)=\"showActions = false\"\n>\n    <button\n        *ngFor=\"let action of actions\"\n        class=\"action-button\"\n        type=\"button\"\n        mat-icon-button\n        [class.focus]=\"showActions\"\n        [disabled]=\"action.disabled\"\n        [color]=\"action.color\"\n        (blur)=\"showActions = false\"\n        (focus)=\"showActions = true\"\n        (click)=\"emitClick(action)\"\n        matTooltip=\"{{action.label}}\"\n    >\n        <mat-icon>{{ action.icon }}</mat-icon>\n    </button>\n</div>\n\n\n\n",
                styles: [".actions-wrapper{overflow:hidden}.actions-wrapper .action-button:not(:focus){transition:.4s ease-in-out;transform:translateX(200px);opacity:0}.actions-wrapper .action-button.focus,tr:hover .actions-wrapper .action-button{transform:translateX(0);opacity:1}"]
            }] }
];
DynamicActionsComponent.propDecorators = {
    type: [{ type: Input }],
    actions: [{ type: Input }],
    showActions: [{ type: Input }],
    actionCallbackParam: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DynamicTableComponent {
    constructor() {
        this.columns = [];
        this.data = [];
        this.total = 0;
        this.pageSize = 0;
        this.actions = [];
        this.loading = false;
        this.request = new EventEmitter(true);
        this.error = new EventEmitter(true);
        this.actionButtonCtrl = new Map();
        this.showActions = false;
        this.resquestDataCache = {};
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this.actions.length > 0) {
            this.displayedColumns = this.columns.map((/**
             * @param {?} column
             * @return {?}
             */
            column => column.cell)).concat(['actions']);
        }
        else {
            this.displayedColumns = this.columns.map((/**
             * @param {?} column
             * @return {?}
             */
            column => column.cell));
        }
        this.resquestDataCache = {};
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.resquestDataCache = {};
    }
    /**
     * @param {?} data
     * @return {?}
     */
    onRequest(data) {
        /** @type {?} */
        const newData = Object.assign({}, this.resquestDataCache, data);
        this.request.emit(newData);
        this.resquestDataCache = newData;
    }
    /**
     * @param {?} path
     * @return {?}
     */
    urlWrap(path) {
        return `url('${path}')`;
    }
}
DynamicTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-dynamic-table',
                template: "<mat-progress-bar *ngIf=\"loading\" aria-label=\"Carregando...\" mode=\"indeterminate\"></mat-progress-bar>\n<table\n    class=\"mat-elevation-z2\"\n    mat-table\n    matSort\n    [dataSource]=\"data\"\n    (matSortChange)=\"onRequest({sort: $event})\">\n\n\n    <ng-container\n        *ngFor=\"let column of columns\"\n        [matColumnDef]=\"column.cell\"\n    >\n        <th\n            *matHeaderCellDef\n            mat-header-cell\n            [mat-sort-header]=\"column.cell\"\n            [disabled]=\"!column.sort\"\n        >\n            {{column.header}}\n        </th>\n        <td mat-cell *matCellDef=\"let row\">\n            <div\n                *ngIf='column.type?.image'\n                mat-card-avatar\n                class=\"avatar mat-elevation-z2\"\n                [style.backgroundImage]=\"urlWrap(row[column.cell] | datasusUrlDownload$ | async) | datasusSafeStyle\"\n            ></div>\n            <div *ngIf='column.type?.date'>\n                {{ row[column.cell] | date: column.type.date && column.type.format? column.type.format : 'mediumDate' }}\n            </div>\n            <div *ngIf='!column.type'>\n                {{row[column.cell]}}\n            </div>\n        </td>\n    </ng-container>\n    <ng-container matColumnDef=\"actions\">\n        <th mat-header-cell *matHeaderCellDef></th>\n        <td mat-cell\n            *matCellDef=\"let row\"\n            class=\"align-right\"\n        >\n            <datasus-dynamic-actions\n                [showActions]=\"row.showActions\"\n                [type]=\"typeActions\"\n                [actions]=\"actions\"\n                [actionCallbackParam]=\"row\"\n            >\n            </datasus-dynamic-actions>\n        </td>\n    </ng-container>\n    <tr mat-header-row *matHeaderRowDef=\"displayedColumns\"></tr>\n    <tr\n        *matRowDef=\"let row; columns: displayedColumns;\"\n        mat-row\n        (mousemove)=\"row.showActions = true\"\n        (mouseleave)=\"row.showActions = false\"\n        (pointermove)=\"row.showActions = true\"\n        (pointerleave)=\"row.showActions = false\"\n    ></tr>\n\n</table>\n<mat-progress-bar *ngIf=\"loading\" aria-label=\"Carregando...\" mode=\"indeterminate\"></mat-progress-bar>\n<mat-paginator\n    class=\"mat-elevation-z3-upside-down\"\n    (page)=\"onRequest({pageEvent: $event})\"\n    [length]=\"total\"\n    [pageSize]=\"pageSize\"\n    [pageSizeOptions]=\"[5, 10, 25, 50, 100]\">\n</mat-paginator>\n",
                styles: [".text-limit{max-width:40ch;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.tabs{display:none}.actions-wrapper{overflow:hidden}.actions-wrapper .action-button:not(:focus){transition:.4s ease-in-out;transform:translateX(200px);opacity:0}.actions-wrapper .action-button.focus,tr:hover .actions-wrapper .action-button{transform:translateX(0);opacity:1}.avatar{background-size:cover;background-color:transparent;width:50px;height:50px}"]
            }] }
];
DynamicTableComponent.propDecorators = {
    columns: [{ type: Input }],
    data: [{ type: Input }],
    total: [{ type: Input }],
    pageSize: [{ type: Input }],
    actions: [{ type: Input }],
    loading: [{ type: Input }],
    typeActions: [{ type: Input }],
    request: [{ type: Output }],
    error: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ExpandableTableComponent {
    constructor() {
        this.columns = [];
        this.data = [];
        this.total = 0;
        this.pageSize = 0;
        this.actions = [];
        this.loading = false;
        this.request = new EventEmitter(true);
        this.error = new EventEmitter(true);
        this.actionButtonCtrl = new Map();
        this.resquestDataCache = {};
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.displayedColumns = this.columns.map((/**
         * @param {?} column
         * @return {?}
         */
        column => column.cell));
        this.resquestDataCache = {};
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.resquestDataCache = {};
    }
    /**
     * @param {?} data
     * @return {?}
     */
    onRequest(data) {
        /** @type {?} */
        const newData = Object.assign({}, this.resquestDataCache, data);
        this.request.emit(newData);
        this.resquestDataCache = newData;
    }
    /**
     * @param {?} menu
     * @param {?} item
     * @return {?}
     */
    emitClick(menu, item) {
        menu.callback(item);
    }
    /**
     * @param {?} path
     * @return {?}
     */
    urlWrap(path) {
        return `url('${path}')`;
    }
}
ExpandableTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-expandable-table',
                template: "<mat-progress-bar *ngIf=\"loading\" aria-label=\"Carregando...\" mode=\"indeterminate\"></mat-progress-bar>\n\n<table\n    class=\"mat-elevation-z2\"\n    mat-table\n    matSort\n    [dataSource]=\"data\"\n    multiTemplateDataRows\n    (matSortChange)=\"onRequest({sort: $event})\">\n\n    <ng-container\n        *ngFor=\"let column of columns\"\n        [matColumnDef]=\"column.cell\" >\n        <th mat-header-cell\n            [mat-sort-header]=\"column.cell\"\n            [disabled]=\"!column.sort\"\n            *matHeaderCellDef>\n            {{column.header}}\n        </th>\n        <td class=\"eee\" mat-cell *matCellDef=\"let element\">\n            <div mat-card-avatar\n                *ngIf='column.type?.image'\n                class=\"avatar mat-elevation-z2\"\n                [style.backgroundImage]=\"urlWrap(element[column.cell] | datasusUrlDownload$ | async) | datasusSafeStyle\">\n            </div>\n            <div *ngIf='column.type?.date'>\n                {{ element[column.cell] | date: column.type.date && column.type.format? column.type.format : 'mediumDate' }}\n            </div>\n            <div *ngIf='!column.type'>\n                {{element[column.cell]}}\n            </div>\n        </td>\n    </ng-container>\n    <ng-container matColumnDef=\"expandedDetail\">\n        <td mat-cell *matCellDef=\"let element\" [attr.colspan]=\"displayedColumns.length\">\n            <div\n                class=\"element-detail\"\n                [@detailExpand]=\"element == expandedElement ? 'expanded' : 'collapsed'\"\n            >\n                <ng-container  *ngTemplateOutlet=\"expandedTemplateRef; context: {$implicit: element}\"></ng-container>\n            </div>\n        </td>\n    </ng-container>\n\n    <tr mat-header-row *matHeaderRowDef=\"displayedColumns\"></tr>\n    <tr\n        class=\"element-row\"\n        mat-row\n        *matRowDef=\"let element; columns: displayedColumns;\"\n        [class.example-expanded-row]=\"expandedElement === element\"\n        (click)=\"expandedElement = expandedElement === element ? null : element\"\n    >\n    </tr>\n    <tr mat-row *matRowDef=\"let row; columns: ['expandedDetail']\" class=\"example-detail-row\"></tr>\n</table>\n\n<mat-progress-bar *ngIf=\"loading\" aria-label=\"Carregando...\" mode=\"indeterminate\"></mat-progress-bar>\n<mat-paginator\n    class=\"mat-elevation-z3-upside-down\"\n    (page)=\"onRequest({pageEvent: $event})\"\n    [length]=\"total\"\n    [pageSize]=\"pageSize\"\n    [pageSizeOptions]=\"[5, 10, 25, 50, 100]\"\n>\n</mat-paginator>\n",
                animations: [
                    trigger('detailExpand', [
                        state('collapsed', style({ height: '0px', minHeight: '0' })),
                        state('expanded', style({ height: '*' })),
                        transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
                    ]),
                ],
                styles: [".eee{cursor:pointer}.text-limit{max-width:40ch;overflow:hidden;text-overflow:ellipsis;white-space:nowrap}.tabs{display:none}.actions-wrapper{overflow:hidden}.actions-wrapper .action-button:not(:focus){transition:.4s ease-in-out;transform:translateX(200px);opacity:0}.actions-wrapper .action-button.focus,tr:hover .actions-wrapper .action-button{transform:translateX(0);opacity:1}.avatar{background-size:cover;background-color:transparent;width:50px;height:50px}tr.example-detail-row{height:0}tr.element-row:not(.example-expanded-row):hover{background:#f5f5f5}tr.element-row:not(.example-expanded-row):active{background:#efefef}.element-row td{border-bottom-width:0}.element-detail{overflow:hidden;display:flex}.example-element-diagram{min-width:80px;border:2px solid #000;padding:8px;font-weight:lighter;margin:8px 0;height:104px}.example-element-symbol{font-weight:700;font-size:40px;line-height:normal}.example-element-description{padding:16px}.example-element-description-attribution{opacity:.5}"]
            }] }
];
ExpandableTableComponent.propDecorators = {
    expandedTemplateRef: [{ type: Input }],
    columns: [{ type: Input }],
    data: [{ type: Input }],
    total: [{ type: Input }],
    pageSize: [{ type: Input }],
    actions: [{ type: Input }],
    loading: [{ type: Input }],
    request: [{ type: Output }],
    error: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UrlDownloadPipe {
    /**
     * @param {?} configService
     */
    constructor(configService) {
        this.configService = configService;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    transform(id) {
        if (!id) {
            return of('');
        }
        return this.configService.getList(['backendUrl', 'clientId']).pipe(map((/**
         * @param {?} configs
         * @return {?}
         */
        (configs) => `${configs[0]}/arquivo/sistema/${configs[1]}/id/${id}/download`)));
    }
}
UrlDownloadPipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusUrlDownload$' },] }
];
/** @nocollapse */
UrlDownloadPipe.ctorParameters = () => [
    { type: ConfigService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UrlDownloadPublicPipe {
    /**
     * @param {?} configService
     */
    constructor(configService) {
        this.configService = configService;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    transform(id) {
        console.log('%c[!] UrlDownloadPublicPipe.transform(): Vc está usando um método temporário para atender um singelo felino', 'color: #ff6; background-color: #424242; font-size: 1.2em');
        if (!id) {
            return of('');
        }
        return this.configService.getList(['backendUrl', 'clientId']).pipe(map((/**
         * @param {?} configs
         * @return {?}
         */
        (configs) => `${configs[0]}/public/arquivo/sistema/${configs[1]}/id/${id}/download`)));
    }
}
UrlDownloadPublicPipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusUrlDownloadPublic$' },] }
];
/** @nocollapse */
UrlDownloadPublicPipe.ctorParameters = () => [
    { type: ConfigService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ConfigPipe {
    /**
     * @param {?} configService
     */
    constructor(configService) {
        this.configService = configService;
    }
    /**
     * @param {?} property
     * @return {?}
     */
    transform(property) {
        if (!property) {
            return of('');
        }
        return this.configService.get(property).pipe(map((/**
         * @param {?} value
         * @return {?}
         */
        (value) => value)));
    }
}
ConfigPipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusConfig$' },] }
];
/** @nocollapse */
ConfigPipe.ctorParameters = () => [
    { type: ConfigService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SafeHtmlPipe {
    /**
     * @param {?} sanitizer
     */
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
    }
    /**
     * @param {?} url
     * @return {?}
     */
    transform(url) {
        return this.sanitizer.bypassSecurityTrustHtml(url);
    }
}
SafeHtmlPipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusSafeHtml' },] }
];
/** @nocollapse */
SafeHtmlPipe.ctorParameters = () => [
    { type: DomSanitizer }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SafeResourceUrlPipe {
    /**
     * @param {?} sanitizer
     */
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
    }
    /**
     * @param {?} url
     * @return {?}
     */
    transform(url) {
        return this.sanitizer.bypassSecurityTrustResourceUrl(url);
    }
}
SafeResourceUrlPipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusSafeResourceUrl' },] }
];
/** @nocollapse */
SafeResourceUrlPipe.ctorParameters = () => [
    { type: DomSanitizer }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SafeUrlPipe {
    /**
     * @param {?} sanitizer
     */
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
    }
    /**
     * @param {?} url
     * @return {?}
     */
    transform(url) {
        return this.sanitizer.bypassSecurityTrustUrl(url);
    }
}
SafeUrlPipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusSafeUrl' },] }
];
/** @nocollapse */
SafeUrlPipe.ctorParameters = () => [
    { type: DomSanitizer }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SafeStylePipe {
    /**
     * @param {?} sanitizer
     */
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
    }
    /**
     * @param {?} style
     * @return {?}
     */
    transform(style$$1) {
        return this.sanitizer.bypassSecurityTrustStyle(style$$1);
    }
}
SafeStylePipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusSafeStyle' },] }
];
/** @nocollapse */
SafeStylePipe.ctorParameters = () => [
    { type: DomSanitizer }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NormalizePipe {
    /**
     * @param {?} text
     * @return {?}
     */
    transform(text) {
        if (!text) {
            return '';
        }
        /** @type {?} */
        let i;
        /** @type {?} */
        let normalized = text.toLowerCase();
        /** @type {?} */
        const map$$1 = {
            a: '[àáâãäå]',
            e: '[èéêë]',
            i: '[ìíîï]',
            o: '[òóôõö]',
            u: '[ùúûűü]',
            y: '[ýÿ]',
            c: 'ç',
            n: 'ñ',
            '-': '–',
        };
        for (i in map$$1) {
            if (map$$1.hasOwnProperty(i)) {
                normalized = normalized.replace(new RegExp(map$$1[i], 'img'), i);
            }
        }
        normalized = normalized.replace(/\s+/g, ' ');
        return normalized;
    }
}
NormalizePipe.decorators = [
    { type: Pipe, args: [{
                name: 'datasusNormalize'
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FilterPipe {
    /**
     * @param {?} list
     * @param {?=} filter
     * @param {?=} key
     * @return {?}
     */
    transform(list, filter, key) {
        if (!list) {
            return [];
        }
        if (!filter) {
            return [...list];
        }
        if (!key) {
            return list.filter((/**
             * @param {?} item
             * @return {?}
             */
            (item) => this.test(item, filter)));
        }
        return list.filter((/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            try {
                return this.test(item[key], filter);
            }
            catch (e) {
                console.error(e);
                return false;
            }
        }));
    }
    /**
     * @private
     * @param {?} item
     * @param {?} filter
     * @return {?}
     */
    test(item, filter) {
        return this.normalize(item).includes(this.normalize(filter));
    }
    /**
     * @private
     * @param {?} text
     * @return {?}
     */
    normalize(text) {
        /** @type {?} */
        const normalizePipe = new NormalizePipe();
        return normalizePipe.transform('' + text);
    }
}
FilterPipe.decorators = [
    { type: Pipe, args: [{
                name: 'datasusFilter'
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UrlDownloadAssinaturaPipe {
    /**
     * @param {?} configService
     */
    constructor(configService) {
        this.configService = configService;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    transform(id) {
        if (!id) {
            return of('');
        }
        return this.configService.getList(['backendUrl']).pipe(map((/**
         * @param {?} configs
         * @return {?}
         */
        (configs) => `${configs[0]}/assinatura-digital/arquivo-assinado-modificado/${id}`)));
    }
}
UrlDownloadAssinaturaPipe.decorators = [
    { type: Pipe, args: [{
                name: 'urlDownloadAssinatura$'
            },] }
];
/** @nocollapse */
UrlDownloadAssinaturaPipe.ctorParameters = () => [
    { type: ConfigService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class WelcomeComponent {
    /**
     * @param {?} commonsConfig
     * @param {?} layoutService
     */
    constructor(commonsConfig, layoutService) {
        this.commonsConfig = commonsConfig;
        this.layoutService = layoutService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.layoutService.setTitle('Bem-vindo!');
        this.welcomeText = this.commonsConfig.welcomeText || 'Bem-vindo!';
        this.welcomeHtml = this.commonsConfig.welcomeHtml || '';
        console.log('%cOi WelcomeComponent', 'font-size: 1.2em; color: cyan;');
    }
    /**
     * @return {?}
     */
    openMenu() {
        this.layoutService.menuSidenavExpanded();
    }
}
WelcomeComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-welcome',
                template: `
    <datasus-container>
      <p datasusOnMobile="hide">
          <mat-icon>arrow_back</mat-icon>
          Selecione no menu ao lado um item.
      </p>
      <h1 *ngIf="!!welcomeText">{{ welcomeText }}</h1>
      <div *ngIf="!!welcomeHtml" [innerHTML]="welcomeHtml | datasusSafeHtml"></div>
      <p datasusOnMobile="show">
          <button type="button" mat-icon-button (click)="openMenu()"><mat-icon>menu</mat-icon></button>
          Abra o menu e selecione um item.
      </p>
    </datasus-container>
  `,
                styles: [`
    p {
      margin-bottom: 100px;
    }
    h1 {
      text-align: center;
    }
  `]
            }] }
];
/** @nocollapse */
WelcomeComponent.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: ['CommonsConfig',] }] },
    { type: LayoutService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ErrorBackendComponent {
    /**
     * @param {?} router
     * @param {?} layoutService
     */
    constructor(router, layoutService) {
        this.router = router;
        this.layoutService = layoutService;
        this.progress = 0;
        this.flipProgress = false;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.layoutService.off();
        /** @type {?} */
        const reloadAndFlipIf = (/**
         * @param {?} test
         * @return {?}
         */
        (test) => {
            if (test) {
                this.router.navigate([`/${DATASUS_ROUTE_HOME}`]);
                this.flipProgress = !this.flipProgress;
            }
        });
        this.interval = setInterval((/**
         * @return {?}
         */
        () => {
            reloadAndFlipIf(this.flipProgress && --this.progress <= -1);
            reloadAndFlipIf(!this.flipProgress && ++this.progress >= 101);
        }), 100);
        console.log('%cError back-end!', 'font-size: 1.2em; color: #f66;');
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        clearInterval(this.interval);
    }
    /**
     * @return {?}
     */
    reload() {
        this.progress = this.flipProgress ? 100 : 0;
    }
}
ErrorBackendComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-error-backend',
                template: "<mat-card>\n  <mat-card-header>\n    <mat-card-title>\n      <code>:(</code>\n      Xiii!\n    </mat-card-title>\n    <mat-card-subtitle>Problema ao conectar com o servidor</mat-card-subtitle>\n    <mat-progress-spinner color=\"primary\" diameter=\"60\" stroke=\"4\" [value]=\"progress\" [class.flipped]=\"flipProgress\"></mat-progress-spinner>\n    <button mat-fab color=\"accent\" routerLink=\"/\" (click)=\"reload()\">\n      <mat-icon>replay</mat-icon>\n    </button>\n  </mat-card-header>\n  <img mat-card-image alt=\"Imagem de uma tela de uma TV sem sinal\" src=\"/assets/datasus/commons/error-backend/no-signal.gif\" />\n  <mat-card-content>\n    <p>\n      Tente digitar outra URL na barra de endere\u00E7o, ou acesse a <a routerLink=\"/\">p\u00E1gina inicial</a> do sistema.\n    </p>\n  </mat-card-content>\n</mat-card>\n\n",
                styles: ["mat-card{width:328px;margin:auto}mat-card mat-card-header{position:relative}mat-card mat-card-header button[mat-fab],mat-card mat-card-header mat-progress-spinner{position:absolute}mat-card mat-card-header mat-progress-spinner{top:40px;right:28px}mat-card mat-card-header mat-progress-spinner.flipped{transform:rotateY(180deg)}mat-card mat-card-header button[mat-fab]{top:42px;right:30px}mat-card img{width:360px;min-width:360px;max-width:360px;height:225px;min-height:225px;max-height:225px}mat-card mat-card-content{position:relative;top:0;left:0}mat-card mat-card-content::after{content:\"SEM SINAL!\";color:#fff;opacity:.6;position:absolute;top:-150px;left:50%;font-size:32px;transform:translateX(-50%)}"]
            }] }
];
/** @nocollapse */
ErrorBackendComponent.ctorParameters = () => [
    { type: Router },
    { type: LayoutService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ForbiddenComponent {
    /**
     * @param {?} router
     * @param {?} layoutService
     * @param {?} authService
     */
    constructor(router, layoutService, authService) {
        this.router = router;
        this.layoutService = layoutService;
        this.authService = authService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.layoutService.off();
        this.authService.isAuthenticated().toPromise()
            .then((/**
         * @return {?}
         */
        () => this.goToHome()))
            .catch((/**
         * @param {?} error
         * @return {?}
         */
        (error) => {
            if (error.code === 401) {
                this.goToHome();
            }
        }));
    }
    /**
     * @return {?}
     */
    goToHome() {
        this.router.navigate([`/${DATASUS_ROUTE_HOME}`]);
    }
    /**
     * @return {?}
     */
    getOut() {
        this.authService.logout();
    }
}
ForbiddenComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-forbidden',
                template: "<mat-card>\n    <mat-card-header>\n        <mat-card-title>\n            <code>:O</code>\n            Pera ai!\n        </mat-card-title>\n        <mat-card-subtitle>Voc\u00EA n\u00E3o tem acesso</mat-card-subtitle>\n    </mat-card-header>\n    <img mat-card-image alt=\"Imagem de um sinal de proib\u00EDdo\" src=\"/assets/datasus/commons/forbidden/proibido.gif\" />\n    <mat-card-content>\n        <p>\n            Fa\u00E7a o cadastro de um perfil para acessar o sistema.\n        </p>\n    </mat-card-content>\n    <mat-card-actions class=\"align-right\">\n        <button type=\"button\" mat-stroked-button color=\"warn\" (click)=\"getOut()\">\n            SAIR\n            <mat-icon>exit_to_app</mat-icon>\n        </button>\n    </mat-card-actions>\n</mat-card>\n",
                styles: ["mat-card{width:328px;margin:auto}mat-card img{height:83px;min-height:83px;max-height:83px;width:81px;min-width:81px;max-width:81px;transform:translatex(126px);margin:86px 0}"]
            }] }
];
/** @nocollapse */
ForbiddenComponent.ctorParameters = () => [
    { type: Router },
    { type: LayoutService },
    { type: AuthService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ChooseProfileComponent {
    /**
     * @param {?} router
     * @param {?} activatedRoute
     * @param {?} layoutService
     * @param {?} authService
     * @param {?} securityService
     */
    constructor(router, activatedRoute, layoutService, authService, securityService) {
        this.router = router;
        this.activatedRoute = activatedRoute;
        this.layoutService = layoutService;
        this.authService = authService;
        this.securityService = securityService;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.layoutService.off();
        this.perfilAtual$ = this.securityService.getPerfilSelecionado();
        this.esferaAtual$ = this.securityService.getEsferaSelecionada();
        this.perfis$ = this.securityService.getPerfis().pipe(tap((/**
         * @param {?} perfis
         * @return {?}
         */
        (perfis) => {
            if (!perfis.length) {
                console.log('[ChooseProfileComponent] Ops, sem perfil! Volta para a tela inicial');
                this.router.navigate([`/${DATASUS_ROUTE_HOME}`]);
            }
            else {
                this.nomeCompleto$ = this.authService.getUser().pipe(map((/**
                 * @param {?} usuario
                 * @return {?}
                 */
                (usuario) => usuario.nome)));
            }
        })));
    }
    /**
     * @param {?} matSelectionListChange
     * @return {?}
     */
    select(matSelectionListChange) {
        matSelectionListChange.source.deselectAll();
        matSelectionListChange.option.selected = true;
        setTimeout((/**
         * @return {?}
         */
        () => {
            this.activatedRoute.params.subscribe((/**
             * @param {?} params
             * @return {?}
             */
            (params) => {
                this.securityService.chooseProfileAndNavigate({
                    perfil: matSelectionListChange.option.value.perfil,
                    esfera: matSelectionListChange.option.value.esfera,
                    returnPath: params.returnPath,
                });
            }));
        }), 500);
    }
}
ChooseProfileComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-choose-profile',
                template: "<ng-template #loading>\n  <mat-progress-bar mode=\"indeterminate\"></mat-progress-bar>\n</ng-template>\n\n<section *ngIf=\"(perfis$ | async) as perfis; else loading\" fxLayout=\"column\">\n\n  <mat-card class=\"mat-elevation-z4\" fxFlex=\"20\">\n    <mat-card-header>\n      <mat-card-title>{{ nomeCompleto$ | async }}</mat-card-title>\n      <mat-card-subtitle [ngPlural]=\"perfis.length\">\n        Voc\u00EA \n        <ng-template ngPluralCase=\"=0\">n\u00E3o tem nenhum perfil</ng-template>\n        <ng-template ngPluralCase=\"=1\">tem somente um perfil</ng-template>\n        <ng-template ngPluralCase=\"=2\">tem dois perfis</ng-template>\n        <ng-template ngPluralCase=\"=3\">tem tr\u00EAs perfis</ng-template>\n        <ng-template ngPluralCase=\"=4\">tem quatro perfis</ng-template>\n        <ng-template ngPluralCase=\"other\">tem {{ perfis.length }} perfis</ng-template>\n        para selecionar\n      </mat-card-subtitle>\n    </mat-card-header>\n    <mat-card-content>\n      <p *ngIf=\"(perfilAtual$ | async) as perfilAtual\">\n        Perfil atual: <strong>{{ perfilAtual.descricao }}</strong>\n      </p>\n      <ng-container *ngIf=\"(esferaAtual$ | async) as esferaAtual\">\n        <p>Esfera: <strong>{{ esferaAtual.descricao }}</strong></p>\n        <p *ngIf=\"!!esferaAtual?.descricaoEsferaAtuacao\">Atua\u00E7\u00E3o: <strong>{{ esferaAtual.descricaoEsferaAtuacao | datasusSwitchSlashToDashPipe }}</strong></p>\n      </ng-container>\n    </mat-card-content>\n  </mat-card>\n\n  <mat-accordion>\n    <mat-expansion-panel *ngFor=\"let perfil of perfis\">\n      <mat-expansion-panel-header>\n        <strong>{{ perfil.descricao }}</strong>\n      </mat-expansion-panel-header>\n      <mat-selection-list (selectionChange)=\"select($event)\">\n        <mat-list-option *ngFor=\"let esfera of perfil.esferas;\" [value]=\"{ perfil: perfil, esfera: esfera }\">\n          <ng-template #withoutDescricaoEsferaAtuacao>\n            <p>{{ esfera.descricao }}</p>\n          </ng-template>\n          <p\n            *ngIf=\"!!esfera?.descricaoEsferaAtuacao else withoutDescricaoEsferaAtuacao\"\n            [matTooltip]=\"esfera.descricao\"\n          >\n            {{ esfera?.descricaoEsferaAtuacao | datasusSwitchSlashToDashPipe }}\n          </p>\n        </mat-list-option>\n      </mat-selection-list>\n    </mat-expansion-panel>\n  </mat-accordion>\n</section>",
                styles: ["section{width:90%;margin:auto}section /deep/ .mat-pseudo-checkbox{border-radius:50%!important}"]
            }] }
];
/** @nocollapse */
ChooseProfileComponent.ctorParameters = () => [
    { type: Router },
    { type: ActivatedRoute },
    { type: LayoutService },
    { type: AuthService },
    { type: SecurityService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ReleaseNotesComponent {
    /**
     * @param {?} layoutService
     * @param {?} markdown
     */
    constructor(layoutService, markdown) {
        this.layoutService = layoutService;
        this.markdown = markdown;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.layoutService.off();
        this.markdown.renderer.paragraph = (/**
         * @param {?} text
         * @return {?}
         */
        (text) => {
            /** @type {?} */
            const regex = /banco\s+de\s+dados.*c[oó]digo[\s+-]fonte/gmi;
            /** @type {?} */
            const regexInvertido = /c[oó]digo[\s+-]fonte.*banco\s+de\s+dados/gmi;
            if (regex.test(text) || regexInvertido.test(text)) {
                /** @type {?} */
                const regexBandoDados = /banco\s+de\s+dados\s*\((\s*x\s*)\)/gmi;
                /** @type {?} */
                const bancoDados = regexBandoDados.test(text) ? '✓' : '';
                /** @type {?} */
                const regexCodigoFonte = /c[oó]digo[\s+-]fonte\s*\((\s*x\s*)\)/gmi;
                /** @type {?} */
                const codigoFonte = regexCodigoFonte.test(text) ? '✓' : '';
                return `
          <div class="container flex">
            <div class="flex">
              Banco de dados:
              <span class="banco-dados">${bancoDados}</span>
            </div>
            <div class="flex">
              Codigo fonte:
              <span class="codigo-fonte">${codigoFonte}</span>
            </div>
          </div>`;
            }
            return `<p>${text}</p>`;
        });
    }
}
ReleaseNotesComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-release-notes',
                template: "<div fxLayout=\"column\" fxLayoutAlign=\"start center\">\n    <button class=\"fab-stick\" mat-mini-fab color=\"accent\" routerLink=\"/\">\n        <mat-icon>home</mat-icon>\n    </button>\n    <mat-card>\n        <div class=\"release-notes\" NgxMd [path]=\"'./RELEASENOTES.md'\"></div>\n    </mat-card>\n</div>\n\n\n",
                styles: ["@charset \"UTF-8\";.fab-stick{position:-webkit-sticky;position:sticky;top:27px;z-index:102;margin-top:10px}.mat-card{margin-top:-10px;min-width:60vw;max-width:80vw;word-break:break-word}.release-notes /deep/ .container{max-width:100%;margin:0 auto 15px}.release-notes /deep/ .wrap{flex-wrap:wrap}.release-notes /deep/ .flex{display:flex}.release-notes /deep/ .banco-dados,.release-notes /deep/ .codigo-fonte{color:#2e7d32;margin-right:15px;margin-left:5px}.release-notes /deep/ .banco-dados:empty::after,.release-notes /deep/ .codigo-fonte:empty::after{color:#9b0000;content:\"\u00D7\"}.release-notes /deep/ h1{color:#003c8f}.release-notes /deep/ h2{margin-top:16px;box-shadow:0 3px 1px -2px rgba(0,0,0,.2),0 2px 2px 0 rgba(0,0,0,.14),0 1px 5px 0 rgba(0,0,0,.12);padding:10px;background:#fff;position:-webkit-sticky;position:sticky;top:3px}.release-notes /deep/ h2 code{color:#9b0000}.release-notes /deep/ h2 strong{float:right;font-size:.7em;font-weight:300;color:#6d6d6d}@media all and (max-width:959px){.fab-stick{margin-top:-55px;bottom:0;order:1;transform:translate(35vw,-19px)}.mat-card{margin-bottom:50px}.release-notes /deep/ h2{padding-top:20px}}.release-notes /deep/ h3,.release-notes /deep/ h4{font-weight:600}.release-notes /deep/ h3{border-bottom:1px solid #c7c7c7}.release-notes /deep/ p strong{color:#424242}.release-notes /deep/ li{list-style-type:circle}.release-notes /deep/ li strong{color:#424242}.release-notes /deep/ hr{display:block;margin:0;border-top:.2px solid rgba(0,0,0,.12)}"]
            }] }
];
/** @nocollapse */
ReleaseNotesComponent.ctorParameters = () => [
    { type: LayoutService },
    { type: NgxMdService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
const ɵ0 = {
    layoutColor: 'primary',
    layoutAnimation: true,
};
/** @type {?} */
const welcomeRoute = {
    component: WelcomeComponent,
    path: DATASUS_ROUTE_HOME,
    data: ɵ0,
    canActivate: [
        AuthGuard,
        SecurityGuard,
    ],
};
const ɵ1 = {
    layoutColor: 'accent',
    layoutAnimation: true,
};
/** @type {?} */
const selectProfileRoute = {
    component: ChooseProfileComponent,
    path: DATASUS_ROUTE_CHOOSE_PROFILE,
    data: ɵ1,
    canActivate: [
        AuthGuard,
    ],
};
const ɵ2 = {
    layoutColor: 'error',
    layoutAnimation: false,
};
/** @type {?} */
const errorBackendRoute = {
    component: ErrorBackendComponent,
    path: DATASUS_ROUTE_ERROR_BACKEND,
    data: ɵ2
};
const ɵ3 = {
    layoutColor: 'error',
    layoutAnimation: true,
};
/** @type {?} */
const forbiddenRoute = {
    component: ForbiddenComponent,
    path: DATASUS_ROUTE_FORBIDDEN,
    data: ɵ3,
};
const ɵ4 = {
    layoutColor: 'error',
    layoutAnimation: true,
};
/** @type {?} */
const notFoundRoute = {
    component: NotFoundComponent,
    path: '**',
    data: ɵ4
};
const ɵ5 = {
    layoutColor: 'success',
    layoutAnimation: true,
};
/** @type {?} */
const releaseNotesRoute = {
    component: ReleaseNotesComponent,
    path: DATASUS_ROUTE_RELEASE_NOTES,
    data: ɵ5
};
/** @type {?} */
const routesDefault = [
    welcomeRoute,
    selectProfileRoute,
    errorBackendRoute,
    forbiddenRoute,
    releaseNotesRoute,
    notFoundRoute,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const guardsDefault = [
    AuthGuard,
    SecurityGuard,
    PathGuard,
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class PaginatorIntl extends MatPaginatorIntl {
    constructor() {
        super(...arguments);
        this.itemsPerPageLabel = 'Itens por página';
        this.nextPageLabel = 'Próxima página';
        this.previousPageLabel = 'Página anterior';
        this.firstPageLabel = 'Primeira página';
        this.lastPageLabel = 'Última página';
        /**
         * ✓ 0: Nenhum item
         * ✓ 1 até 1 de 1: Único item
         * ✓ 1 até X de X: X itens
         * ✓ X até X de X: Último item dos X itens
         * ✓ X até Y de Y itens
         * ✓ X até Y de Z itens
         */
        this.getRangeLabel = (/**
         * @param {?} page
         * @param {?} pageSize
         * @param {?} length
         * @return {?}
         */
        (page, pageSize, length) => {
            if (length === 0 || pageSize === 0) {
                return 'Nenhum item';
            }
            length = Math.max(length, 0);
            /** @type {?} */
            const index = page * pageSize;
            /** @type {?} */
            const end = index < length ? Math.min(index + pageSize, length) : index + pageSize;
            /** @type {?} */
            const start = index + 1;
            if (start === 1) {
                if (start === length) {
                    return 'Único item';
                }
                if (end === length) {
                    return `${length} itens`;
                }
            }
            if (start === end && end === length) {
                return `Último item dos ${length} itens`;
            }
            return `${start} até ${end} de ${length} itens`;
        });
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class StepperIntl extends MatStepperIntl {
    constructor() {
        super(...arguments);
        this.optionalLabel = 'Opcional';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SortHeaderIntl extends MatSortHeaderIntl {
    constructor() {
        super(...arguments);
        this.sortButtonLabel = (/**
         * @param {?} id
         * @return {?}
         */
        (id) => `Muda a ordenação de ${id}`);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class DatepickerIntl extends MatDatepickerIntl {
    constructor() {
        super(...arguments);
        /**
         * A label for the calendar popup (used by screen readers).
         */
        this.calendarLabel = 'Calendário';
        /**
         * A label for the button used to open the calendar popup (used by screen readers).
         */
        this.openCalendarLabel = 'Abrir o calendário';
        /**
         * A label for the previous month button (used by screen readers).
         */
        this.prevMonthLabel = 'Mês anterior';
        /**
         * A label for the next month button (used by screen readers).
         */
        this.nextMonthLabel = 'Próximo mês';
        /**
         * A label for the previous year button (used by screen readers).
         */
        this.prevYearLabel = 'Ano anterior';
        /**
         * A label for the next year button (used by screen readers).
         */
        this.nextYearLabel = 'Próximo ano';
        /**
         * A label for the previous multi-year button (used by screen readers).
         */
        this.prevMultiYearLabel = '20 anos anteriores';
        /**
         * A label for the next multi-year button (used by screen readers).
         */
        this.nextMultiYearLabel = 'Próximos 20 anos';
        /**
         * A label for the 'switch to month view' button (used by screen readers).
         */
        this.switchToMonthViewLabel = 'Escolher uma data';
        /**
         * A label for the 'switch to year view' button (used by screen readers).
         */
        this.switchToMultiYearViewLabel = 'Escolher um ano';
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class UrlDownloadDeprecatedPipe {
    /**
     * @param {?} configService
     */
    constructor(configService) {
        this.configService = configService;
    }
    /**
     * @param {?} id
     * @return {?}
     */
    transform(id) {
        console.log('%cPipe "datasusUrlDownload" deprecated. Use "datasusUrlDownload$" yet!', 'font-size: 2em; color: #ff6; background-color: #424242');
        /** @type {?} */
        const pipe = new UrlDownloadPipe(this.configService);
        return pipe.transform(id);
    }
}
UrlDownloadDeprecatedPipe.decorators = [
    { type: Pipe, args: [{ name: 'datasusUrlDownload' },] }
];
/** @nocollapse */
UrlDownloadDeprecatedPipe.ctorParameters = () => [
    { type: ConfigService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable-next-line: max-line-length
/** @type {?} */
const regex = /^(?:(?:31(\/|-|\.)(?:0?[13578]|1[02]))\1|(?:(?:29|30)(\/|-|\.)(?:0?[13-9]|1[0-2])\2))(?:(?:1[6-9]|[2-9]\d)?\d{2})$|^(?:29(\/|-|\.)0?2\3(?:(?:(?:1[6-9]|[2-9]\d)?(?:0[48]|[2468][048]|[13579][26])|(?:(?:16|[2468][048]|[3579][26])00))))$|^(?:0?[1-9]|1\d|2[0-8])(\/|-|\.)(?:(?:0?[1-9])|(?:1[0-2]))\4(?:(?:1[6-9]|[2-9]\d)?\d{2})$/;
class DatePtAdapter extends NativeDateAdapter {
    constructor() {
        super(...arguments);
        this.locale = 'pt';
    }
    /**
     * @param {?} value
     * @return {?}
     */
    parse(value) {
        try {
            // console.log('[Date "pt" Adapter] parse:', value);
            if (typeof value !== 'string') {
                return null;
            }
            if (!regex.test(value)) {
                return null;
            }
            if (value.length !== 10) {
                return null;
            }
            /** @type {?} */
            const regExExecution = regex.exec(value);
            /** @type {?} */
            const separator = regExExecution[4] || regExExecution[3] || regExExecution[2] || regExExecution[1];
            // lançar! 🚀
            /** @type {?} */
            const values = value.split(separator).map((/**
             * @param {?} val
             * @return {?}
             */
            (val) => +val));
            return new Date(values[2], values[1] - 1, values[0]);
        }
        catch (error) {
            console.error('[Date "pt" Adapter] error:', error);
        }
        return null;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const PORTUGUESE = 'pt';
/** @type {?} */
const DATE_FORMATS = {
    parse: {
        dateInput: 'LL',
    },
    display: {
        dateInput: 'LL',
        monthYearLabel: 'MMM YYYY',
        dateA11yLabel: 'LL',
        monthYearA11yLabel: 'MMMM YYYY',
    },
};
registerLocaleData(localePt, PORTUGUESE, localePtExtra);
const ɵ0$1 = DATE_FORMATS, ɵ1$1 = {};
class CommonsModule {
    /**
     * Criando o Módulo com suas configurações
     * @param {?} commonsConfig Configuração do módulo comum: Texto da home inicial, etc...
     * @return {?}
     */
    static forRoot(commonsConfig) {
        return {
            ngModule: CommonsModule,
            providers: [{
                    provide: 'CommonsConfig',
                    useValue: commonsConfig,
                }],
        };
    }
}
CommonsModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    // Angular
                    CommonModule,
                    RouterModule,
                    // Angular Flex-Layout
                    FlexLayoutModule,
                    // Angular Material
                    MatButtonModule,
                    MatCardModule,
                    MatExpansionModule,
                    MatIconModule,
                    MatListModule,
                    MatMenuModule,
                    MatProgressBarModule,
                    MatProgressSpinnerModule,
                    MatPaginatorModule,
                    MatSortModule,
                    MatTableModule,
                    MatTooltipModule,
                    // Angular Markdown (NgxMd)
                    NgxMdModule,
                    // Datasus
                    CoreModule,
                    LayoutModule,
                ],
                declarations: [
                    // Pipes
                    UrlDownloadPipe,
                    UrlDownloadDeprecatedPipe,
                    UrlDownloadPublicPipe,
                    ConfigPipe,
                    SafeHtmlPipe,
                    SafeResourceUrlPipe,
                    SafeUrlPipe,
                    SafeStylePipe,
                    FilterPipe,
                    NormalizePipe,
                    UrlDownloadAssinaturaPipe,
                    // Components
                    ChooseProfileComponent,
                    ErrorBackendComponent,
                    ForbiddenComponent,
                    NotFoundComponent,
                    WelcomeComponent,
                    FabExtendedComponent,
                    DynamicActionsComponent,
                    DynamicTableComponent,
                    ExpandableTableComponent,
                    ReleaseNotesComponent,
                ],
                exports: [
                    // Pipes
                    UrlDownloadPipe,
                    UrlDownloadDeprecatedPipe,
                    UrlDownloadPublicPipe,
                    ConfigPipe,
                    SafeHtmlPipe,
                    SafeResourceUrlPipe,
                    SafeUrlPipe,
                    SafeStylePipe,
                    FilterPipe,
                    NormalizePipe,
                    UrlDownloadAssinaturaPipe,
                    // Components
                    NotFoundComponent,
                    FabExtendedComponent,
                    DynamicActionsComponent,
                    DynamicTableComponent,
                    ExpandableTableComponent,
                    ReleaseNotesComponent
                ],
                providers: [
                    {
                        provide: LOCALE_ID,
                        useValue: PORTUGUESE,
                    },
                    {
                        provide: MAT_DATE_LOCALE,
                        useValue: PORTUGUESE,
                    },
                    {
                        provide: DateAdapter,
                        useClass: DatePtAdapter,
                    },
                    {
                        provide: MAT_DATE_FORMATS,
                        useValue: ɵ0$1,
                    },
                    {
                        provide: MatPaginatorIntl,
                        useClass: PaginatorIntl,
                    },
                    {
                        provide: MatStepperIntl,
                        useClass: StepperIntl,
                    },
                    {
                        provide: MatSortHeaderIntl,
                        useClass: SortHeaderIntl,
                    },
                    {
                        provide: MatDatepickerIntl,
                        useClass: DatepickerIntl,
                    },
                    {
                        provide: 'CommonsConfig',
                        useValue: ɵ1$1,
                    },
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NotFoundComponent, FabExtendedComponent, DynamicActionsComponent, DynamicTableComponent, ExpandableTableComponent, UrlDownloadPipe, UrlDownloadPublicPipe, ConfigPipe, SafeHtmlPipe, SafeResourceUrlPipe, SafeUrlPipe, SafeStylePipe, FilterPipe, NormalizePipe, UrlDownloadAssinaturaPipe, routesDefault, guardsDefault, CommonsModule, DatePtAdapter as ɵh, ChooseProfileComponent as ɵc, ErrorBackendComponent as ɵd, ForbiddenComponent as ɵe, ReleaseNotesComponent as ɵf, WelcomeComponent as ɵa, DatepickerIntl as ɵl, PaginatorIntl as ɵi, SortHeaderIntl as ɵk, StepperIntl as ɵj, UrlDownloadDeprecatedPipe as ɵg };

//# sourceMappingURL=datasus-commons.js.map