import { HttpClient } from '@angular/common/http';
import { Observable } from 'rxjs';
/**
 * Serviço responsável por trazer as configurações
 * de `/config/env.json`
 * (local das configurações por ambiente.)
 */
export declare class ConfigService {
    private httpClient;
    /**
     * Chave para ser usada como
     * identificaçõe da app rodando
     */
    appKey: string;
    /**
     * Cache das configurações
     */
    private cache;
    /**
     * Inicializador da classe
     * @param httpClient Cliente para as requisiçõe HTTP
     */
    constructor(httpClient: HttpClient);
    /**
     * Registra nas configurações para fins de cache
     * qual aplicação em questão
     * @param name Nome da aplicação
     * @param version Versão da aplicação
     */
    registerApp(name: string, version: string): void;
    /**
     * Retorna um `Observable`. Se assinado vai
     * trazer todas as configurações em
     * um objeto só
     */
    getAll(): Observable<any>;
    /**
     * Retorna um `Observable`. Se assinado vai
     * trazer a configuração selecionada
     * @param property Propriedade da configuração a ser retornada no `Observable`
     */
    get(property: string): Observable<any>;
    /**
     * Retorna um `Observable`. Se assinado vai
     * trazer as configurações selecionadas
     * em uma lista
     * @param properties Lista de propriedades da configuração a serem retornadas no `Observable`
     */
    getList(properties: string[]): Observable<any[]>;
}
