import { Location } from '@angular/common';
import { Router } from '@angular/router';
import { Observable } from 'rxjs';
import { ConfigService } from './config.service';
import { PerfilModel } from '../models/perfil.model';
import { EsferaModel } from '../models/esfera.model';
import { HttpClient } from '@angular/common/http';
/**
 * Serviço que tem os atalhos para acessar
 * informações como perfil e esfera
 */
export declare class SecurityService {
    private location;
    private router;
    private httpClient;
    private configService;
    private perfis$;
    private perfilSelecionado$;
    private esferaSelecionada$;
    constructor(location: Location, router: Router, httpClient: HttpClient, configService: ConfigService);
    requestProfiles(): Observable<PerfilModel[]>;
    navigateToChooseProfile(forceRedirectToHome?: boolean): void;
    chooseProfileAndNavigate({ returnPath, perfil, esfera }: {
        returnPath?: string;
        perfil: PerfilModel;
        esfera?: EsferaModel;
    }): void;
    /**
     * Limpa os dados de perfil e esfera
     */
    clear(): void;
    /**
     * Limpa a lista de perfis
     */
    clearPerfis(): void;
    /**
     * Verifica se tem lista de perfis
     */
    hasPerfis(): boolean;
    /**
     * Recupera a lista de perfis
     */
    getPerfis(): Observable<PerfilModel[]>;
    /**
     * Salva a lista de perfis
     * @param perfis Lista de perfis
     */
    setPerfis(perfis: PerfilModel[]): void;
    /**
     * Limpa o perfil selecionado
     */
    clearPerfilSelecionado(): void;
    /**
     * Verifica se tem perfil selecionado
     */
    hasPerfilSelecionado(): boolean;
    /**
     * Recupera o observable do perfil selecionado
     */
    getPerfilSelecionado(): Observable<PerfilModel>;
    /**
     * Recupera o perfil selecionado armazenada
     */
    getStoredPerfilSelecionado(): PerfilModel;
    /**
     * Salva o perfil selecionado
     * @param perfil Perfil selecionado
     */
    setPerfilSelecionado(perfil: PerfilModel): void;
    /**
     * Limpa a esfera selecionada
     */
    clearEsferaSelecionada(): void;
    /**
     * Verifica se tem esfera selecionada
     */
    hasEsferaSelecionada(): boolean;
    /**
     * Recupera o observable da esfera selecionada
     */
    getEsferaSelecionada(): Observable<EsferaModel>;
    /**
     * Recupera a esfera selecionada armazenada
     */
    getStoredEsferaSelecionada(): EsferaModel;
    /**
     * Salva a esfera selecionada
     * @param esfera Esfera selecionada
     */
    setEsferaSelecionada(esfera: EsferaModel): void;
    private decoratePerfis;
    private decoratePerfil;
    private decorateEsferas;
    private decorateEsfera;
    private extractList;
    private hasDescricao;
    private key;
}
