import { CommonModule } from '@angular/common';
import { MatIconModule } from '@angular/material/icon';
import { MatButtonModule } from '@angular/material/button';
import { MatTooltipModule } from '@angular/material/tooltip';
import { CdkVirtualScrollViewport, ScrollingModule } from '@angular/cdk/scrolling';
import { Subject } from 'rxjs';
import { debounceTime, map } from 'rxjs/operators';
import { FilterPipe, CommonsModule } from '@datasus/commons';
import { compareObjects } from '@datasus/core';
import { Directive, ElementRef, HostListener, Input, ChangeDetectionStrategy, ChangeDetectorRef, Component, EventEmitter, forwardRef, Optional, Output, Self, ViewChild, NgModule } from '@angular/core';
import { MatFormFieldControl } from '@angular/material/form-field';
import { MatSelectModule } from '@angular/material/select';
import { MatProgressSpinnerModule } from '@angular/material';
import { FormControlName, NgControl, FormsModule, ReactiveFormsModule } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CpfValidator {
    /**
     * @param {?=} value
     * @return {?}
     */
    static isValid(value = '') {
        value = String(value);
        /** @type {?} */
        const cpf = value ? value.replace(/[^\d]+/g, '') : '';
        if (cpf === '') {
            return false;
        }
        // Elimina CPFs invalidos conhecidos
        if (cpf.length !== 11 ||
            cpf === '0'.repeat(11) ||
            cpf === '1'.repeat(11) ||
            cpf === '2'.repeat(11) ||
            cpf === '3'.repeat(11) ||
            cpf === '4'.repeat(11) ||
            cpf === '5'.repeat(11) ||
            cpf === '6'.repeat(11) ||
            cpf === '7'.repeat(11) ||
            cpf === '8'.repeat(11) ||
            cpf === '9'.repeat(11)) {
            return false;
        }
        // Valida 1o digito
        /** @type {?} */
        let add = 0;
        for (let i = 0; i < 9; i++) {
            add += parseInt(cpf.charAt(i), 10) * (10 - i);
        }
        /** @type {?} */
        let rev = 11 - (add % 11);
        if (rev === 10 || rev === 11) {
            rev = 0;
        }
        if (rev !== parseInt(cpf.charAt(9), 10)) {
            return false;
        }
        // Valida 2o digito
        add = 0;
        for (let i = 0; i < 10; i++) {
            add += parseInt(cpf.charAt(i), 10) * (11 - i);
        }
        rev = 11 - (add % 11);
        if (rev === 10 || rev === 11) {
            rev = 0;
        }
        if (rev !== parseInt(cpf.charAt(10), 10)) {
            return false;
        }
        return true;
    }
}
CpfValidator.CPF_REG_EXP = new RegExp('^\\d{0,3}\\.{0,1}\\d{0,3}\\.{0,1}\\d{0,3}\\-{0,1}\\d{0,2}$');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CnpjValidator {
    /**
     * @param {?=} value
     * @return {?}
     */
    static isValid(value = '') {
        value = String(value);
        /** @type {?} */
        const cnpj = value ? value.replace(/[^\d]+/g, '') : '';
        if (cnpj === '') {
            return false;
        }
        /** @type {?} */
        const MAX_LENGTH = 14;
        if (cnpj.length !== MAX_LENGTH) {
            return false;
        }
        // Elimina CNPJs invalidos conhecidos
        if (cnpj.length !== MAX_LENGTH ||
            cnpj === '0'.repeat(MAX_LENGTH) ||
            cnpj === '1'.repeat(MAX_LENGTH) ||
            cnpj === '2'.repeat(MAX_LENGTH) ||
            cnpj === '3'.repeat(MAX_LENGTH) ||
            cnpj === '4'.repeat(MAX_LENGTH) ||
            cnpj === '5'.repeat(MAX_LENGTH) ||
            cnpj === '6'.repeat(MAX_LENGTH) ||
            cnpj === '7'.repeat(MAX_LENGTH) ||
            cnpj === '8'.repeat(MAX_LENGTH) ||
            cnpj === '9'.repeat(MAX_LENGTH)) {
            return false;
        }
        // Valida DVs
        /** @type {?} */
        let tamanho = cnpj.length - 2;
        /** @type {?} */
        let numeros = cnpj.substring(0, tamanho);
        /** @type {?} */
        const digitos = cnpj.substring(tamanho);
        /** @type {?} */
        let soma = 0;
        /** @type {?} */
        let pos = tamanho - 7;
        for (let i = tamanho; i >= 1; i--) {
            /** @type {?} */
            const num = Number(numeros.charAt(tamanho - i));
            soma += num * pos--;
            if (pos < 2) {
                pos = 9;
            }
        }
        /** @type {?} */
        let resultado = soma % 11 < 2 ? 0 : 11 - (soma % 11);
        if (resultado !== Number(digitos.charAt(0))) {
            return false;
        }
        tamanho = tamanho + 1;
        numeros = cnpj.substring(0, tamanho);
        soma = 0;
        pos = tamanho - 7;
        for (let i = tamanho; i >= 1; i--) {
            /** @type {?} */
            const num = Number(numeros.charAt(tamanho - i));
            soma += num * pos--;
            if (pos < 2) {
                pos = 9;
            }
        }
        resultado = soma % 11 < 2 ? 0 : 11 - (soma % 11);
        if (resultado !== Number(digitos.charAt(1))) {
            return false;
        }
        return true;
    }
}
CnpjValidator.CNPJ_REG_EXP = new RegExp('^\\d{0,2}\\.{0,1}\\d{0,3}\\.{0,1}\\d{0,3}/{0,1}\\d{0,4}\\-{0,1}\\d{0,2}$');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CnsValidator {
    /**
     * @param {?=} value
     * @return {?}
     */
    static isValid(value = '') {
        value = String(value);
        /** @type {?} */
        const cns = value ? value.replace(/[^\d]+/g, '') : '';
        if (cns === '') {
            return false;
        }
        /** @type {?} */
        const PIS_MAX_TAM = 11;
        /** @type {?} */
        const CNS_MAX_TAM = 15;
        /** @type {?} */
        let i = 0;
        /** @type {?} */
        let j = 0;
        /** @type {?} */
        let pis = '';
        /** @type {?} */
        let sum = 0;
        /** @type {?} */
        let mod = 0;
        /** @type {?} */
        let dv = 0;
        /** @type {?} */
        let res = 0;
        /** @type {?} */
        let tipo = '';
        if (cns.length !== CNS_MAX_TAM) {
            return false;
        }
        if (parseInt(cns, 10).toString() !== cns) {
            return false;
        }
        tipo = cns.charAt(0);
        if ((tipo === '1') || (tipo === '2')) {
            /* Validacao de numeros definitivos (Baseados no PIS do usuario) */
            pis = cns.substring(0, PIS_MAX_TAM);
            for (i = 0, j = 15; i < PIS_MAX_TAM; i++, j--) {
                sum += (parseInt(pis.charAt(i), 10) * j);
            }
            mod = sum % 11;
            dv = 11 - mod;
            if (dv === 11) {
                dv = 0;
            }
            if (dv === 10) {
                sum += 2;
                mod = sum % 11;
                dv = 11 - mod;
                res = (parseInt(pis, 10) * 10000) + 10 + dv;
            }
            else {
                res = (parseInt(pis, 10) * 10000) + dv;
            }
            if (res !== parseInt(cns, 10)) {
                return false;
            }
        }
        else if ((tipo === '7') || (tipo === '8') || (tipo === '9')) {
            /* Validacao de numeros provisorios */
            for (i = 0, j = 15; i < CNS_MAX_TAM; i++, j--) {
                sum += (parseInt(cns.charAt(i), 10) * j);
            }
            mod = sum % 11;
            if (mod) {
                return false;
            }
        }
        else {
            return false;
        }
        return true;
    }
}
CnsValidator.CNS_REG_EXP = new RegExp('\\d{0,15}$');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FormValidators {
    /**
     * @param {?} ctrl
     * @return {?}
     */
    static cpfCnpj(ctrl) {
        if (FormValidators.isNull(ctrl)) {
            return null;
        }
        /** @type {?} */
        const value = ctrl.value ? ctrl.value.replace(/[^\d]+/g, '') : '';
        if (value.length === 11) {
            return FormValidators.cpf(ctrl);
        }
        else if (value.length === 14) {
            return FormValidators.cnpj(ctrl);
        }
        else {
            return null;
        }
    }
    /**
     * @param {?} ctrl
     * @return {?}
     */
    static cpf(ctrl) {
        if (FormValidators.isNull(ctrl)) {
            return null;
        }
        if (CpfValidator.isValid(ctrl.value)) {
            return null;
        }
        return { cpfInvalid: true };
    }
    /**
     * @param {?} ctrl
     * @return {?}
     */
    static cnpj(ctrl) {
        if (FormValidators.isNull(ctrl)) {
            return null;
        }
        if (CnpjValidator.isValid(ctrl.value)) {
            return null;
        }
        return { cnpjInvalid: true };
    }
    /**
     * @param {?} ctrl
     * @return {?}
     */
    static cns(ctrl) {
        if (FormValidators.isNull(ctrl)) {
            return null;
        }
        if (CnsValidator.isValid(ctrl.value)) {
            return null;
        }
        return { cnsInvalid: true };
    }
    /**
     * @param {?} checkFn
     * @return {?}
     */
    static requiredIf(checkFn) {
        /**
         * @param {?} ctrl
         * @return {?}
         */
        function validator(ctrl) {
            if (checkFn(ctrl)) {
                if (!!ctrl.value) {
                    return null;
                }
                return { requiredIf: true };
            }
            return null;
        }
        return validator;
    }
    /**
     * @param {?} max
     * @return {?}
     */
    static maxLength(max) {
        /**
         * @param {?} ctrl
         * @return {?}
         */
        function validator(ctrl) {
            if (FormValidators.isNull(ctrl)) {
                return null;
            }
            if (ctrl.value.length <= max) {
                return null;
            }
            return { maxLength: true };
        }
        return validator;
    }
    /**
     * @param {?} min
     * @return {?}
     */
    static minLength(min) {
        /**
         * @param {?} ctrl
         * @return {?}
         */
        function validator(ctrl) {
            if (FormValidators.isNull(ctrl)) {
                return null;
            }
            if (ctrl.value.length >= min) {
                return null;
            }
            return { minLength: true };
        }
        return validator;
    }
    /**
     * @private
     * @param {?} ctrl
     * @return {?}
     */
    static isNull(ctrl) {
        // if (ctrl.touched) {
        //   return false;
        // }
        return !(ctrl.value !== undefined && ctrl.value !== '' && ctrl.value !== null);
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class CustomErrorStateMatcher {
    /**
     * @param {?} control
     * @param {?} form
     * @return {?}
     */
    isErrorState(control, form) {
        /** @type {?} */
        const isSubmitted = form && form.submitted;
        return !!(control && control.invalid && (control.dirty || control.touched || isSubmitted));
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable-next-line: max-line-length
/** @type {?} */
const DATE_REG_EXP = /^(?:(?:31(\/|-|\.)(?:0?[13578]|1[02]))\1|(?:(?:29|30)(\/|-|\.)(?:0?[1,3-9]|1[0-2])\2))(?:(?:1[6-9]|[2-9]\d)?\d{2})$|^(?:29(\/|-|\.)0?2\3(?:(?:(?:1[6-9]|[2-9]\d)?(?:0[48]|[2468][048]|[13579][26])|(?:(?:16|[2468][048]|[3579][26])00))))$|^(?:0?[1-9]|1\d|2[0-8])(\/|-|\.)(?:(?:0?[1-9])|(?:1[0-2]))\4(?:(?:1[6-9]|[2-9]\d)?\d{2})$/gm;
class DateMaskDirective {
    /**
     * @param {?} elementRef
     */
    constructor(elementRef) {
        this.elementRef = elementRef;
        this.dateMaskSeparator = '/';
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        /** @type {?} */
        const allowed = ['-', '.', '/'];
        if (!allowed.includes(this.dateMaskSeparator)) {
            this.dateMaskSeparator = '/';
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onInput(value) {
        this.elementRef.nativeElement.value = this.formatedValue(value);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    onBlur(value) {
        /** @type {?} */
        const valid = DATE_REG_EXP.test(value);
        if (!valid) {
            this.elementRef.nativeElement.value = '';
        }
    }
    /**
     * @private
     * @param {?=} value
     * @return {?}
     */
    formatedValue(value = '') {
        if (!value || typeof value !== 'string') {
            return '';
        }
        return this.limit(this.addSeparator(this.clear(value)));
    }
    /**
     * @private
     * @param {?=} value
     * @return {?}
     */
    clear(value = '') {
        return value.replace(/[^(-./)\d]/g, '');
    }
    /**
     * @private
     * @param {?=} value
     * @return {?}
     */
    addSeparator(value = '') {
        if (value.length === 2 // dd + / = dd/
            ||
                value.length === 5 // dd/mm + / = dd/mm/
        ) {
            return value.concat(this.dateMaskSeparator);
        }
        return value;
    }
    /**
     * @private
     * @param {?} value
     * @return {?}
     */
    limit(value) {
        if (value.length > 10) { // dd/mm/aaaa + ?
            return value.substring(0, 10);
        }
        return value;
    }
}
DateMaskDirective.decorators = [
    { type: Directive, args: [{
                selector: 'input[datasusDateMask], input[matInput][matDatepicker]'
            },] }
];
/** @nocollapse */
DateMaskDirective.ctorParameters = () => [
    { type: ElementRef }
];
DateMaskDirective.propDecorators = {
    dateMaskSeparator: [{ type: Input }],
    onInput: [{ type: HostListener, args: ['input', ['$event.target.value'],] }],
    onBlur: [{ type: HostListener, args: ['blur', ['$event.target.value'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SelectAutocompleteComponent {
    /**
     * @param {?} ngControl
     * @param {?} controlName
     * @param {?} changeDetectorRef
     */
    constructor(ngControl, controlName, changeDetectorRef) {
        this.controlName = controlName;
        this.changeDetectorRef = changeDetectorRef;
        this.filteredOptions = [];
        this.filteredOptionsChange = new Subject();
        this.selectedValue = null;
        this.floatLabel = true;
        this.options = [];
        this.disabled = false;
        this.display = 'display';
        this.inputSearchPlaceholder = 'Pesquisar';
        this.appearance = 'standard';
        this.loading = false;
        this.loadingDiameter = '18';
        this.selectionChange = new EventEmitter();
        this.openedChangeSelect = new EventEmitter();
        this.onChange = (/**
         * @return {?}
         */
        () => { });
        this.onTouched = (/**
         * @return {?}
         */
        () => { });
        if (ngControl) {
            ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get errorStateMatcher() {
        return this.selectElem.errorStateMatcher;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set errorStateMatcher(val) {
        this.selectElem.errorStateMatcher = val;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.options = this.options || [];
        this.filteredOptions = [...this.options];
        /** @type {?} */
        const filterPipe = new FilterPipe();
        this.filteredOptionsChangeSubscription = this.filteredOptionsChange.pipe(debounceTime(400), map((/**
         * @param {?} value
         * @return {?}
         */
        (value) => filterPipe.transform(this.options, value, this.filterSearch)))).subscribe((/**
         * @param {?} filtro
         * @return {?}
         */
        (filtro) => {
            this.filteredOptions = [...filtro];
            this.rerender();
        }), (/**
         * @param {?} error
         * @return {?}
         */
        (error) => console.error('Erro SelectAutocompleteComponent', error.message)));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.filteredOptionsChangeSubscription.unsubscribe();
    }
    /**
     * @return {?}
     */
    ngOnChanges() {
        this.options = this.options || [];
        this.filteredOptions = [...this.options];
        this.setValue();
    }
    /**
     * @return {?}
     */
    ngDoCheck() {
        this.selectElem.updateErrorState();
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    writeValue(obj) {
        this.selectedValue = obj;
        this.setValue();
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.selectElem.registerOnChange(fn);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.selectElem.registerOnTouched(fn);
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.selectElem.setDisabledState(isDisabled);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    filterItem(value) {
        this.filteredOptionsChange.next(value);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    stopEventAndAppendSpace(event) {
        event.stopPropagation();
        event.preventDefault();
        this.searchInput.nativeElement.value += ' ';
    }
    /**
     * @return {?}
     */
    clearFilterItem() {
        this.searchInput.nativeElement.value = '';
        this.filterItem(this.searchInput.nativeElement.value);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onSelectionChange(event) {
        this.selectedValue = event.value;
        this.selectedOptionDisplay = event.source.triggerValue;
        this.setValue();
        this.onChange(this.selectedValue);
        this.onTouched();
        this.selectionChange.emit(this.selectedValue);
        this.rerender();
    }
    /**
     * @param {?} isOpen
     * @return {?}
     */
    openedChange(isOpen) {
        if (isOpen) {
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToMiddle()), 0);
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToBottom()), 1);
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToMiddle()), 2);
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToTop()), 3);
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToMiddle()), 4);
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToBottom()), 5);
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToMiddle()), 6);
            setTimeout((/**
             * @return {?}
             */
            () => this.scrollToTop()), 7);
        }
        this.openedChangeSelect.emit(isOpen);
        this.rerender();
    }
    /**
     * @param {?} o1
     * @param {?} o2
     * @return {?}
     */
    compare(o1, o2) {
        if (!!this.optionField && o1 != null) {
            return o1 === o2[this.optionField];
        }
        return compareObjects(o1, o2);
    }
    /**
     * @return {?}
     */
    setValue() {
        this.selectedOptionDisplay = null;
        if (this.selectedValue !== null && this.optionField) {
            /** @type {?} */
            const selectedOptionValue = this.options.find((/**
             * @param {?} item
             * @return {?}
             */
            item => item[this.optionField] === this.selectedValue));
            this.selectedOptionDisplay = this.findFieldInObj(this.display, selectedOptionValue);
            this.selectedOptionTooltip = this.findFieldInObj(this.tooltip, selectedOptionValue);
        }
        this.rerender();
    }
    /**
     * @param {?} field
     * @param {?} obj
     * @return {?}
     */
    findFieldInObj(field, obj) {
        if (obj instanceof Object && obj.hasOwnProperty(field)) {
            return obj[field];
        }
        return null;
    }
    /**
     * @private
     * @return {?}
     */
    scrollToTop() {
        /** @type {?} */
        const x = 0;
        /** @type {?} */
        const y = 0;
        this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollTo(x, y);
        this.rerender();
    }
    /**
     * @private
     * @return {?}
     */
    scrollToMiddle() {
        /** @type {?} */
        const x = 0;
        /** @type {?} */
        const y = Math.ceil(this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollHeight / 2);
        this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollTo(x, y);
        this.rerender();
    }
    /**
     * @private
     * @return {?}
     */
    scrollToBottom() {
        /** @type {?} */
        const x = 0;
        /** @type {?} */
        const y = this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollHeight;
        this.cdkVirtualScrollViewport.elementRef.nativeElement.scrollTo(x, y);
        this.rerender();
    }
    /**
     * @private
     * @return {?}
     */
    rerender() {
        this.changeDetectorRef.detectChanges();
        this.cdkVirtualScrollViewport.checkViewportSize();
    }
}
SelectAutocompleteComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-select-autocomplete',
                template: "<mat-form-field [appearance]=\"appearance\" class=\"adjust-mat-field\">\n    <mat-label *ngIf=\"floatLabel\">{{ placeholder }}</mat-label>\n    <mat-select\n        #selectElem\n        class=\"select-custom virtual-scroll\"\n        [disabled]=\"disabled\"\n        [placeholder]=\"placeholder\"\n        [required]=\"required\"\n        (openedChange)=\"openedChange($event)\"\n        (selectionChange)=\"onSelectionChange($event)\"\n        [(ngModel)]=\"selectedValue\"\n    >\n        <div class=\"box-search styck-input\">\n            <input\n                #searchInput\n                type=\"text\"\n                (keydown.space)=\"stopEventAndAppendSpace($event)\"\n                (input)=\"filterItem(searchInput.value)\"\n                [placeholder]=\"inputSearchPlaceholder\"\n            />\n            <div class=\"box-search-icon\" (click)=\"clearFilterItem()\">\n                <button mat-icon-button class=\"search-button\">\n                    <mat-icon class=\"mat-24\" aria-label=\"Search icon\">clear</mat-icon>\n                </button>\n            </div>\n        </div>\n\n        <mat-option\n            *ngIf=\"!!selectedOptionDisplay || !!selectedValue\"\n            class=\"option-item\"\n            [value]=\"selectedValue || (!!optionField ? selectedValue[display] : selectedValue)\"\n            [matTooltip]=\"selectedOptionTooltip || findFieldInObj(tooltip, selectedValue)\"\n        >\n            {{ selectedOptionDisplay || findFieldInObj(display, selectedValue) }}\n        </mat-option>\n\n        <cdk-virtual-scroll-viewport\n            [class.has-mat-option-selected]=\"!!selectedOptionDisplay || !!selectedValue\"\n            itemSize=\"50\"\n            minBufferPx=\"200\"\n            maxBufferPx=\"400\"\n        >\n            <ng-container *cdkVirtualFor=\"let option of filteredOptions\">\n                <mat-option\n                    class=\"option-item\"\n                    [class.hidden]=\"compare(selectedValue, option)\"\n                    [disabled]=\"!!option?.disabled\"\n                    [value]=\"!!optionField ? option[optionField] : option\"\n                    [matTooltip]=\"findFieldInObj(tooltip, option)\"\n                >\n                    {{ findFieldInObj(display, option) || '\u2026' }}\n                </mat-option>\n            </ng-container>\n        </cdk-virtual-scroll-viewport>\n    </mat-select>\n    <mat-spinner *ngIf=\"loading\" matSuffix [diameter]=\"loadingDiameter\"></mat-spinner>\n    <mat-error *ngIf=\"customErroMessage\">{{customErroMessage}}</mat-error>\n</mat-form-field>\n\n",
                providers: [{
                        provide: MatFormFieldControl,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => SelectAutocompleteComponent)),
                        multi: true
                    },
                ],
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".box-search{margin:8px;border-radius:2px;box-shadow:0 2px 4px -1px rgba(0,0,0,.2),0 4px 5px 0 rgba(0,0,0,.14),0 1px 10px 0 rgba(0,0,0,.12);transition:box-shadow .2s cubic-bezier(.4,0,.2,1);display:flex;z-index:1002}.adjust-mat-field{margin-top:-14px}.box-search input{flex:1;border:none;outline:0;padding:5px 9px}.styck-input{position:-webkit-sticky;position:sticky;top:10px}.search-button{width:36px;height:36px;line-height:33px;color:#424242}.pl-1{padding-left:1rem}mat-form-field{width:100%}cdk-virtual-scroll-viewport{height:200px!important}cdk-virtual-scroll-viewport.has-mat-option-selected{height:150px!important}.option-item{height:50px!important}"]
            }] }
];
/** @nocollapse */
SelectAutocompleteComponent.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: FormControlName, decorators: [{ type: Optional }] },
    { type: ChangeDetectorRef }
];
SelectAutocompleteComponent.propDecorators = {
    placeholder: [{ type: Input }],
    floatLabel: [{ type: Input }],
    options: [{ type: Input }],
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    display: [{ type: Input }],
    optionField: [{ type: Input }],
    inputSearchPlaceholder: [{ type: Input }],
    filterSearch: [{ type: Input }],
    appearance: [{ type: Input }],
    tooltip: [{ type: Input }],
    customErroMessage: [{ type: Input }],
    loading: [{ type: Input }],
    loadingDiameter: [{ type: Input }],
    errorStateMatcher: [{ type: Input }],
    selectionChange: [{ type: Output }],
    openedChangeSelect: [{ type: Output }],
    selectElem: [{ type: ViewChild, args: ['selectElem',] }],
    cdkVirtualScrollViewport: [{ type: ViewChild, args: [CdkVirtualScrollViewport,] }],
    searchInput: [{ type: ViewChild, args: ['searchInput',] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SelectAutocompleteMultipleComponent {
    /**
     * @param {?} ngControl
     * @param {?} changeDetectorRef
     * @param {?} controlName
     */
    constructor(ngControl, changeDetectorRef, controlName) {
        this.changeDetectorRef = changeDetectorRef;
        this.controlName = controlName;
        this.searchText = '';
        this.selectedValue = null;
        this.floatLabel = true;
        this.options = [];
        this.disabled = false;
        this.display = 'display';
        this.inputSearchPlaceholder = 'Pesquisar';
        this.appearance = 'standard';
        this.loading = false;
        this.loadingDiameter = '18';
        this.selectionChange = new EventEmitter();
        this.openedChangeSelect = new EventEmitter();
        this.onChange = (/**
         * @return {?}
         */
        () => { });
        this.onTouched = (/**
         * @return {?}
         */
        () => { });
        if (ngControl) {
            ngControl.valueAccessor = this;
        }
    }
    /**
     * @return {?}
     */
    get errorStateMatcher() {
        return this.selectElem.errorStateMatcher;
    }
    /**
     * @param {?} val
     * @return {?}
     */
    set errorStateMatcher(val) {
        console.log('vall --> ', val);
        this.selectElem.errorStateMatcher = val;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.options = this.options || [];
    }
    /**
     * @return {?}
     */
    ngDoCheck() {
        this.selectElem.updateErrorState();
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    writeValue(obj) {
        this.selectedValue = obj;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.selectElem.registerOnChange(fn);
        this.onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this.selectElem.registerOnTouched(fn);
        this.onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this.selectElem.setDisabledState(isDisabled);
    }
    /**
     * @return {?}
     */
    clearSearchText() {
        this.searchText = '';
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onSelectionChange(event) {
        this.selectedValue = event.value;
        this.onChange(this.selectedValue);
        this.selectElem.writeValue(this.selectedValue);
        this.onTouched();
        this.selectionChange.emit(this.selectedValue);
        this.changeDetectorRef.detectChanges();
    }
    /**
     * @param {?} isOpen
     * @return {?}
     */
    openedChange(isOpen) {
        this.openedChangeSelect.emit(isOpen);
        this.changeDetectorRef.detectChanges();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    stopEventAndAppendSpace(event) {
        event.stopPropagation();
        event.preventDefault();
        this.searchText += ' ';
    }
    /**
     * @param {?} field
     * @param {?} obj
     * @return {?}
     */
    findFieldInObj(field, obj) {
        if (obj instanceof Object && obj.hasOwnProperty(field)) {
            return obj[field];
        }
        return null;
    }
}
SelectAutocompleteMultipleComponent.decorators = [
    { type: Component, args: [{
                selector: 'datasus-select-autocomplete-multiple',
                template: "<mat-form-field [appearance]=\"appearance\" class=\"adjust-mat-field\">\n    <mat-label *ngIf=\"floatLabel\">{{ placeholder }}</mat-label>\n    <mat-select\n        #selectElem\n        class=\"select-custom\"\n        [disabled]=\"disabled\"\n        [placeholder]=\"placeholder\"\n        multiple\n        [required]=\"required\"\n        (openedChange)=\"openedChange($event)\"\n        (selectionChange)=\"onSelectionChange($event)\"\n        [(ngModel)]=\"selectedValue\"\n    >\n        <div class=\"box-search styck-input\">\n            <input\n                class=\"pl-1\"\n                type=\"text\"\n                [(ngModel)]=\"searchText\"\n                (keydown.space)=\"stopEventAndAppendSpace($event)\"\n                [placeholder]=\"inputSearchPlaceholder\"\n            />\n            <div class=\"box-search-icon\" (click)=\"clearSearchText()\">\n                <button mat-icon-button class=\"search-button\">\n                    <mat-icon class=\"mat-24\" aria-label=\"Search icon\">clear</mat-icon>\n                </button>\n            </div>\n        </div>\n        <mat-option\n            *ngFor=\"let option of (options | datasusFilter: searchText: filterSearch)\"\n            [disabled]=\"!!option?.disabled\"\n            [value]=\"!!optionField ? option[optionField] : option\"\n            [matTooltip]=\"findFieldInObj(tooltip, option)\"\n        >\n            {{ findFieldInObj(display, option) }}\n        </mat-option>\n        <mat-select-trigger [ngPlural]=\"selectedValue?.length\">\n            Voc\u00EA\n            <ng-template ngPluralCase=\"=0\">n\u00E3o selecionou</ng-template>\n            <ng-template ngPluralCase=\"=1\">selecionou somente um registro</ng-template>\n            <ng-template ngPluralCase=\"=2\">selecionou dois registros</ng-template>\n            <ng-template ngPluralCase=\"other\">selecionou {{ selectedValue?.length }} registros</ng-template>\n        </mat-select-trigger>\n    </mat-select>\n    <mat-spinner *ngIf=\"loading\" matSuffix [diameter]=\"loadingDiameter\"></mat-spinner>\n    <mat-error *ngIf=\"customErroMessage\">{{customErroMessage}}</mat-error>\n</mat-form-field>\n\n",
                providers: [{
                        provide: MatFormFieldControl,
                        useExisting: forwardRef((/**
                         * @return {?}
                         */
                        () => SelectAutocompleteMultipleComponent)),
                        multi: true
                    }],
                changeDetection: ChangeDetectionStrategy.OnPush,
                styles: [".box-search{margin:8px;border-radius:2px;box-shadow:0 2px 4px -1px rgba(0,0,0,.2),0 4px 5px 0 rgba(0,0,0,.14),0 1px 10px 0 rgba(0,0,0,.12);transition:box-shadow .2s cubic-bezier(.4,0,.2,1);display:flex;z-index:1002}.adjust-mat-field{margin-top:-14px}.box-search input{flex:1;border:none;outline:0;padding:5px 9px}.styck-input{position:-webkit-sticky;position:sticky;top:10px}.search-button{width:36px;height:36px;line-height:33px;color:#424242}.pl-1{padding-left:1rem}mat-form-field{width:100%}"]
            }] }
];
/** @nocollapse */
SelectAutocompleteMultipleComponent.ctorParameters = () => [
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] },
    { type: ChangeDetectorRef },
    { type: FormControlName, decorators: [{ type: Optional }] }
];
SelectAutocompleteMultipleComponent.propDecorators = {
    selectElem: [{ type: ViewChild, args: ['selectElem',] }],
    placeholder: [{ type: Input }],
    floatLabel: [{ type: Input }],
    options: [{ type: Input }],
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    display: [{ type: Input }],
    optionField: [{ type: Input }],
    inputSearchPlaceholder: [{ type: Input }],
    filterSearch: [{ type: Input }],
    appearance: [{ type: Input }],
    tooltip: [{ type: Input }],
    customErroMessage: [{ type: Input }],
    loading: [{ type: Input }],
    loadingDiameter: [{ type: Input }],
    errorStateMatcher: [{ type: Input }],
    selectionChange: [{ type: Output }],
    openedChangeSelect: [{ type: Output }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FormsModule$1 {
}
FormsModule$1.decorators = [
    { type: NgModule, args: [{
                imports: [
                    // Angular
                    CommonModule,
                    FormsModule,
                    ReactiveFormsModule,
                    // Angular Material
                    MatSelectModule,
                    MatIconModule,
                    ScrollingModule,
                    MatButtonModule,
                    MatTooltipModule,
                    MatProgressSpinnerModule,
                    // Datasus
                    CommonsModule,
                ],
                declarations: [
                    // Directives
                    DateMaskDirective,
                    // Components
                    SelectAutocompleteComponent,
                    SelectAutocompleteMultipleComponent,
                ],
                exports: [
                    // Directives
                    DateMaskDirective,
                    // Components
                    SelectAutocompleteComponent,
                    SelectAutocompleteMultipleComponent,
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FormValidators, CnpjValidator, CpfValidator, CnsValidator, CustomErrorStateMatcher, FormsModule$1 as FormsModule, SelectAutocompleteMultipleComponent as ɵc, SelectAutocompleteComponent as ɵb, DateMaskDirective as ɵa };

//# sourceMappingURL=datasus-forms.js.map