# DATASUS Angular

|Pacotes Disponíveis|
|:--|
|[`@datasus/core`](./projects/datasus/core/README.md)|
|[`@datasus/layout`](./projects/datasus/layout/README.md)|
|[`@datasus/commons`](./projects/datasus/commons/README.md)|
|[`@datasus/forms`](./projects/datasus/forms/README.md)|
|[`@datasus/externals`](./projects/datasus/externals/README.md)|
|~~[`@datasus/welcome`](./projects/datasus/welcome/README.md)~~|
|~~[`@datasus/not-found`](./projects/datasus/not-found/README.md)~~|

## Não esquecer

Sempre que criar uma _library_ com `ng g lib ...`, usar o `@datasus/` como escopo e passar o parâmetro `--prefix datasus`.

> Sendo que `[nome-do-modulo]` é um dos pacotes disponíveis.

```bash
# Generate
ng generate library --prefix datasus @datasus/[nome-do-modulo]

# Build
ng build @datasus/[nome-do-modulo]

# Copying files from especific folder
node bin/copy-files.js ./projects/datasus/[nome-do-modulo]/src/[pasta] ./dist/datasus/[nome-do-modulo]/[pasta]

# Replacing term in especific file
node bin/replace-in.js ./projects/datasus/[nome-do-modulo]/[caminho-do-arquivo] [termo-anterior] [termo-novo]

# Packing
npm pack dist/datasus/[nome-do-modulo]

# Publishing
npm publish dist/datasus/[nome-do-modulo]
```

Adicionar no `package.json` os _scripts_ de _build_/_publish_ do módulo:

> Observar se tem que copiar os arquivos de uma `[pasta]` de um módulo, se sim, colocar o `node bin/copy-files.js ./projects/datasus/[nome-do-modulo]/src/[pasta] ./dist/datasus/[nome-do-modulo]/[pasta]` no `postbuild:@datasus/[nome-do-modulo]`.

```json
{
  ...
  "scripts": {
    ...
    "build:@datasus/[nome-do-modulo]": "ng build @datasus/[nome-do-modulo]",
    "postbuild:@datasus/[nome-do-modulo]": "node bin/copy-files.js ./projects/datasus/[nome-do-modulo]/src/[pasta] ./dist/datasus/[nome-do-modulo]/[pasta]",
    "publish:@datasus/[nome-do-modulo]": "npm publish dist/datasus/[nome-do-modulo]",
    ...
    "build:@datasus": "... && npm run build:@datasus/[nome-do-modulo]",
    "publish:@datasus": "... && npm run publish:@datasus/[nome-do-modulo]",
  },
  ...
}
```
```bash
# Build all modules
npm run build:@datasus

# Build module [nome-do-modulo]
npm run build:@datasus/[nome-do-modulo]
```

Colocar na lib criada, no `package.json`:

```json
...
  "publishConfig": {
    "registry": "http://nexus-cicd.apps.saude.gov/repository/npm-hosted/"
  },
...
```

### Para usar o Nexus do DATASUS

Primeiramente logar no registry:

```bash
npm login --registry http://nexus-cicd.apps.saude.gov/repository/npm-hosted/
```

Após isso, executar os _builds_ e chamar o _publish_:

```bash
# Publish all modules
npm run publish:@datasus

# Publish module [nome-do-modulo]
npm run publish:@datasus/[nome-do-modulo]
```

## Instruções para os sistemas

### Iniciando uma nova aplicação

- Estrutura inicial:

```bash
# New app
ng new sigla --prefix sigla --style scss --routing --skip-git
cd sigla

# NPM Instaling dependencies
npm install
npm install @angular/material @angular/cdk @angular/animations @angular/flex-layout

# Polyfills
npm install --save classlist.js web-animations-js
```

- Descomente os _polyfills_ em `src/polyfills.ts`;
- Modifique o `src/index.html`:

```html
<!DOCTYPE html>
<html lang="pt-br">
  <head>
    <meta charset="utf-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1" />
    <base href="/" />

    <title>Nome do Sistema</title>
    <link rel="icon" type="image/x-icon" href="favicon.ico?datasus-logo" />
  </head>
  <body>
    <sigla-root></sigla-root>
    <noscript>Favor habilitar o JavaScript no seu navegador para poder iniciar a aplicação.</noscript>
  </body>
</html>
```

- Importe o `BrowserAnimationsModule` vindo de `@angular/platform-browser/animations`:

```typescript
import { NgModule } from '@angular/core';
import { BrowserModule } from '@angular/platform-browser';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';

@NgModule({
  imports: [
    BrowserModule,
    BrowserAnimationsModule,
  ],
})
export class AppModule { }
```

- Modifique o `tsconfig.json` e adicione no `"compilerOptions"`:

```json
  ...
  "compilerOptions": {
    "resolveJsonModule": true,
    ...
```

> Com isso, podemos usar no import um JSON e utilizá-lo.

#### Build de PRD

Lembrar de colocar os parametros:

- `--prod`;
- `--build-optimizer`;
- `--optimization`;
- `--i18nLocale=pt`.

Exemplo:

```bash
ng build --prod --build-optimizer --optimization --i18nLocale=pt --no-progress
```

### Para usar o módulo `@datasus/core`

- Instalar via NPM:

```bash
npm install @datasus/core
```
- Importar o módulo `CoreModule`, com as configurações, vindo do `@datasus/core`.

#### Para usar o serviço `ConfigService`

```typescript
import { ConfigService } from '@datasus/core';
```
```typescript
constructor(private configService: ConfigService) { }
```
```typescript
this.configService.getAll().subscribe(
  (config: any) => console.log('todas as configurações', config)
);

this.configService.getList(['config1', 'config2']).subscribe(
  (configs: any[]) => console.log('lista com: config1 e config2', configs[0], configs[1])
);

this.configService.get('config3').subscribe(
  (config3: any) => console.log('valor de config3', config3)
);
```

#### Para usar o serviço `DominioService`

```typescript
import { DominioService } from '@datasus/core';
```
- Se for usar `DominioModel, DominioItemModel` então tem que importar também:
`import { DominioService, DominioModel, DominioItemModel } from '@datasus/core';`

```typescript
constructor(private dominioService: DominioService) { }
```
```typescript
this.dominioService.getItemNames().subscribe(
  (domainNames: string[]) => console.log('lista com os nome dos dominios disponíveis', domainNames)
  // [ "nome-do-dominio", "outro-nome-do-dominio", ... ]
);

this.dominioService.get('nome-do-dominio').subscribe(
  (domain: DominioModel) => console.log('dominio do nome informado', domain)
);

this.dominioService.getList(['nome-do-dominio', 'outro-nome-do-dominio']).subscribe(
  (domains: DominioModel[]) => console.log('lista com os dominios dos nomes informados', domains)
);

this.dominioService.getAll().subscribe(
  (domains: DominioModel[]) => console.log('todos os dominios', domains)
);
```

#### Tratamento de erro nas requisições HTTP

```typescript
import { ErrorModel } from '@datasus/core';
```
```typescript
this.errorMessage = '';
this.errorCode = 0;
this.errorData: any = {};

this.loading = true;

this.someService.request().pipe(
  finalize(() => this.loading = false)
).subscribe(
  (result: any[]) => console.log('resultado do request', result),
  (error: ErrorModel) => {
    this.errorMessage = error.message; // mensagem de erro
    this.errorCode = error.code; // código do erro
    this.errorData = error.data; // origem do erro
  },
);
```

#### Para usar o serviço `LocalStorage` ou `SessionStorage`

```typescript
import { LocalStorage } from '@datasus/core';
```
```typescript
LocalStorage.setItem('CHAVE1', { abc: 123 });
LocalStorage.setItem('CHAVE2', 'abc');
LocalStorage.setItem('CHAVE3', 123);
LocalStorage.setItem('CHAVE4', [ 'a','b','c', 1, 2, 3 ]);
LocalStorage.setItem('CHAVE5', true);

LocalStorage.getItem('CHAVE1'); // {abc: 123}
LocalStorage.getItem('CHAVE2'); // 'abc'
LocalStorage.getItem('CHAVE3'); // 123
LocalStorage.getItem('CHAVE4'); // ['a','b','c', 1, 2, 3]
LocalStorage.getItem('CHAVE5'); // true
```

> `SessionStorage` tem os mesmas métodos

#### Serviços / _back-end_

Todos os serviços tem que herdar de `BaseService`, com isso teremos todos os métodos que vão no _back-end_ prontos.

```typescript
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService } from '@datasus/core';

import { AppModel } from './caminho/dos/models/app.model';

@Injectable({
  providedIn: 'root'
})
export class AppService extends BaseService {

  metodoGet(id: number): Observable<AppModel> {
    return this.baseGet<AppModel>(`caminho/do/get/${id}`);
  }

  metodoPut(id: number, data: AppModel): Observable<void> {
    return this.basePut<void>(`caminho/do/put/${id}`, data);
  }

  metodoPost(data: AppModel): Observable<void> {
    return this.basePost<void>('caminho/do/post', data);
  }

  metodoDelete(id: number): Observable<boolean> {
    return this.baseDelete<boolean>(`caminho/do/delete/${id}`);
  }

}

```

### Para usar o módulo `@datasus/commons`

- Instalar via NPM:

```bash
npm install @datasus/commons
```
- Importar o módulo `CommonsModule`, com as configurações, vindo do `@datasus/commons`.
- Colocar os _assets_ para serem incluidos no _build_ da aplicação, no `angular.json`:

```json
...
"assets": [
  ...
  {
    "input": "./node_modules/@datasus/commons/assets",
    "glob": "*/**",
    "output": "assets/datasus/commons"
  },
  ...
],
```

#### Declarando as rotas da aplicação

A recomendação é de importar o `routesDefault` e concatenar na configuração de rota da aplicação.

```typescript
import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

import { routesDefault, guardsDefault } from '@datasus/commons';

import { RotaProtegidaComponent } from './rota-protegida.component';
import { RotaNaoProtegidaComponent } from './rota-nao-protegida.component';

const appRoutes: Routes = [
  {
    path: 'rota-protegida',
    component: RotaProtegidaComponent,
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: 'rota-desprotegida',
    component: RotaNaoProtegidaComponent,
  },
  {
    path: 'rota-que-carrega-quando-acessa-protegida',
    loadChildren: './modulo-protegido.module#ModuloProtegidoModule',
    canActivate: [
      ...guardsDefault,
    ],
    canLoad: [
      ...guardsDefault,
    ]
  },
  {
    path: 'rota-que-carrega-quando-acessa-nao-protegida',
    loadChildren: './modulo-nao-protegido.module#ModuloNaoProtegidoModule',
  },
];

@NgModule({
  imports: [
//  RouterModule.forRoot(appRoutes.concat(routesDefault), {
    RouterModule.forRoot([...appRoutes, ...routesDefault], {
      useHash: false,
      onSameUrlNavigation: 'reload',
      scrollPositionRestoration: 'top',
    }),
  ],
  exports: [
    RouterModule,
  ],
})
export class AppRoutingModule { }
```

#### Components Comuns

##### FAB Extended

Uma forma de FAB (_float action button_) com um texto extendido.

Podendo mostrar somente o texto quando interagir com os estados de `hover` ou `focus`.

|Atributo|Obrigatório|Tipo|Padrão|Observação|
|:--|:-:|:--|:--|:--|
|`text`|Sim|`string`|``||
|`icon`|Sim|`string`|``||
|`color`|Não|`ThemePalette` (`@angular/material`)|`'primary'`||
|`description`|Não|`string`|`''`||
|`showTextOnlyOnHover`|Não|`boolean`|`false`|Caso não passado mostra o texto sempre|
|`showTextOnlyOnFocus`|Não|`boolean`|`false`|Caso não passado mostra o texto sempre|

Exemplo:

```html
<datasus-fab-extended
    text="TEXTO"
    icon="info"
    color="primary"
    description="Clique para blah blah blah"
    showTextOnlyOnHover="true"
    showTextOnlyOnFocus="true"
    (click)="doSomething()"
></datasus-fab-extended>
```

### Para usar o módulo `@datasus/forms`

- Instalar via NPM:

```bash
npm install @datasus/forms
```
- Importar o módulo `FormsModule`, com as configurações, vindo do `@datasus/forms`.

#### _Validators_

Para usar em validações gerais e/ou nos `FormControl` do Angular.

Validadores disponíveis:
- CPF
- CNPJ
- CNS

```typescript
import { FormValidators, CpfValidator } from '@datasus/forms';
```
```typescript
this.formGroup = this.formBuilder.group({
  // ...
  cpf: [null, FormValidators.cpf],
  cnpj: [null, Validators.compose([FormValidators.cnpj, Validators.required])],
});
```
```typescript
const cpf = '12345678900';

if (CpfValidator.isValid(cpf)) {
  console.log('CPF válido');
} else {
  console.log('CPF inválido');
}
```

### Para usar o módulo `@datasus/layout`

- Instalar via NPM:

```bash
npm install @datasus/layout
```

- Altere todas as configurações do _assets_ do _favicon_, no `angular.json`:

```json
...
"assets": [
  ...
  {
    "input": "./node_modules/@datasus/layout/resources",
    "glob": "favicon.ico",
    "output": "."
  },
  {
    "input": "./node_modules/@datasus/layout/assets",
    "glob": "*/**",
    "output": "assets/datasus/layout"
  },
  ...
],
...
```

- Importar no `src/styles.scss` na primeira linha:

```scss
@import "~@datasus/layout/resources/theme";

//... continuação do SCSS global do sistema
```
- Importar o módulo `LayoutModule`, com as configurações, vindo do `@datasus/layout`:

```typescript
import { NgModule } from '@angular/core';

import { LayoutModule } from '@datasus/layout';

import * as packageJson from '../../package.json';

@NgModule({
  imports: [
    LayoutModule.forRoot({
      withGovBrBar: true,
      appName: 'Nome do Sistema',
      appVersion: packageJson.version
    }),
  ],
  exports: [
    LayoutModule,
  ],
})
export class DatasusModule { }
```
- No seu componente principal, utilize o `<datasus-root />` como `template`:

```typescript
import { Component } from '@angular/core';

@Component({
  selector: 'app-root',
  template: '<datasus-root></datasus-root>',
})
export class AppComponent { }
```

#### _Helpers_ de CSS

Alguns _helpers_ para ajudar no CSS:

- `.transparent`: Esconde da tela preservando a área do elemento na tela.
- `.hidden`: Remove da tela preservando a árvore do DOM.
- `.align-center`: Alinha o que tem dentro no centro.
- `.align-left`: Alinha o que tem dentro na esquerda.
- `.align-right`: Alinha o que tem dentro na direita.

#### Alertas e confirmações

O módulo _layout_ disponibiliza dois tipos de diálogos: de alerta e confirmação.

Para utilizar utilize o `DialogService`, usando os métodos: `showAlert(...)` e o `showConfirmation(...)`.

```typescript
import { DialogService } from '@datasus/layout';
```
```typescript
constructor(private dialogService: DialogService) { }
```
```typescript
alert(): void {
  this.dialogService.showAlert({
    message: 'Teste um dois, um dois...',
    type: 'warn', // 'primary', 'accent' ou 'warn'
    callback: () => console.log('fechou o alerta'),
  });
  // ou
  this.dialogService.showAlert('Teste um dois, um dois...'); // vai com o tipo 'warn' e sem callback
}

confirmation(): void {
  const buttons: Map<string, () => void> = new Map();
  buttons.set('sim', () => console.log('Que bom!'));
  buttons.set('não', () => console.log('Que pena!'));
  buttons.set('talvez', () => console.log('Que coisa, não?!'));

  this.dialogService.showConfirmation({
    title: 'Ei...',
    question: 'Você está vivo?',
    anwsers: buttons
  });
}

prompt(): void {
  this.dialogService.showPrompt({
    label: 'Escreva um texto',
    button: 'agora, clique aqui',
    required: true,
    callback: (result: string) => console.log('fechou o prompt', result),
  });
}

notification(): void {
  this.dialogService.showNotification('Somente a mensagem');
  // ou
  this.dialogService.showNotification({
    message: 'Mensagem de notificação. Vai fechar sozinha ou clica no botão do lado.',
    secondsToClose: 10,
    action: { 'Ação extra': () => console.log('ação extra') },
    dismiss: () => console.log('dismiss notification'),
  });
}
```

Para mais opções verifique as configurações `DialogAlertConfig` e `DialogConfirmationConfig`.

#### Responsividade

Caso precise de especificar o estilo só para _viewport_ menor, utilize o mixin `datasus-media-mobile` assim:

```scss
@import "~@datasus/layout/resources/media";

.title {
  font-size: 2em;
}

@include datasus-media-mobile {
  .title {
    font-size: 1.5em;
  }
}
```

Também tem disponível todos os _medias_ de acordo com o Angular Flex Layout:

|SCSS Mixin|Media Query|
|:--|:--|
|`@include datasus-media-xs { ... }`|`@media all and (max-width: 599px) { ... }`|
|`@include datasus-media-sm { ... }`|`@media all and (min-width: 600px) and (max-width: 959px) { ... }`|
|`@include datasus-media-md { ... }`|`@media all and (min-width: 960px) and (max-width: 1279px) { ... }`|
|`@include datasus-media-lg { ... }`|`@media all and (min-width: 1280px) and (max-width: 1919px) { ... }`|
|`@include datasus-media-xl { ... }`|`@media all and (min-width: 1920px) and (max-width: 5000px) { ... }`|
|`@include datasus-media-lt-sm { ... }`|`@media all and (max-width: 599px) { ... }`|
|`@include datasus-media-lt-md { ... }`|`@media all and (max-width: 959px) { ... }`|
|`@include datasus-media-lt-lg { ... }`|`@media all and (max-width: 1279px) { ... }`|
|`@include datasus-media-lt-xl { ... }`|`@media all and (max-width: 1919px) { ... }`|
|`@include datasus-media-gt-xs { ... }`|`@media all and (min-width: 600px) { ... }`|
|`@include datasus-media-gt-sm { ... }`|`@media all and (min-width: 960px) { ... }`|
|`@include datasus-media-gt-md { ... }`|`@media all and (min-width: 1280px) { ... }`|
|`@include datasus-media-gt-lg { ... }`|`@media all and (min-width: 1920px) { ... }`|

##### Diretiva `datasusOnMobile="hide|show"`

Com ela você mostra ou esconde elementos quando estiver numa _viewport_ menor.

```html
<button type="button" datasusOnMobile="show">
  MOSTRA NO MOBILE
</button>

<button type="button" datasusOnMobile="hide">
  ESCONDE NO MOBILE
</button>
```

#### Containers

Para colocar os elementos de cada tela. Delimita o espaço dentro do conteúdo principal.

Com um _background_ claro:

```html
<datasus-container>
  ...
</datasus-container>
```

Com um _background_ transparente:

```html
<datasus-container-transparent>
  ...
</datasus-container-transparent>
```

#### Ícones

Caso precise de especifica o tamanho de um icone:

```scss
@import "~@datasus/layout/resources/icon";

.icon {
  @include datasus-icon-sizing(16px);
}
```

#### Cores

> Embasado em: https://material.io/design/color/

Paleta disponível:

- `dusky`: Escura; Texto padrão; Fundo com contraste;
- `bright`: Clara; Fundo padrão; Texto com contraste;
- `primary`: Principal; Destaques não destoantes; **Da paleta oficial do Material Design**;
- `accent`: Acentuada; Destaque destoante; **Da paleta oficial do Material Design**;
- `warn`: Alerta; Atenção; **Da paleta oficial do Material Design**;
- `success`: Sucesso; Indicativo de ok, sem problema;
- `erro`: Erro; Indicativo de erro, problema;

Com as variantes: `light` e `dark` para cada cor.

##### Clases no CSS:

- `color-dusky`
- `color-duskyLight`
- `color-duskyDark`
- `color-bright`
- `color-brightLight`
- `color-brightDark`
- `color-primary`
- `color-primaryLight`
- `color-primaryDark`
- `color-accent`
- `color-accentLight`
- `color-accentDark`
- `color-warn`
- `color-warnLight`
- `color-warnDark`
- `color-success`
- `color-successLight`
- `color-successDark`
- `color-error`
- `color-errorLight`
- `color-errorDark`

##### Variáveis no SCSS:

###### Escura (Grey 800)

- <span style="color: #424242;">▉</span> `$datasus-dusky-color`
- <span style="color: #6d6d6d;">▉</span> `$datasus-duskyLight-color`
- <span style="color: #1b1b1b;">▉</span> `$datasus-duskyDark-color`

###### Clara (Grey 50)

- <span style="color: #fafafa;">▉</span> `$datasus-bright-color`
- <span style="color: #ffffff;">▉</span> `$datasus-brightLight-color`
- <span style="color: #c7c7c7;">▉</span> `$datasus-brightDark-color`

###### Principal (Blue 800)

- <span style="color: #1565c0;">▉</span> `$datasus-primary-color`
- <span style="color: #5e92f3;">▉</span> `$datasus-primaryLight-color`
- <span style="color: #003c8f;">▉</span> `$datasus-primaryDark-color`

###### Acentuada (Amber 800)

- <span style="color: #ff8f00;">▉</span> `$datasus-accent-color`
- <span style="color: #ffc046;">▉</span> `$datasus-accentLight-color`
- <span style="color: #c56000;">▉</span> `$datasus-accentDark-color`

###### Alerta (Pink 700)

- <span style="color: #c2185b;">▉</span> `$datasus-warn-color`
- <span style="color: #fa5788;">▉</span> `$datasus-warnLight-color`
- <span style="color: #8c0032;">▉</span> `$datasus-warnDark-color`

###### Sucesso (Green 800)

- <span style="color: #2e7d32;">▉</span> `$datasus-success-color`
- <span style="color: #60ad5e;">▉</span> `$datasus-successLight-color`
- <span style="color: #005005;">▉</span> `$datasus-successDark-color`

###### Erro (Red A700)

- <span style="color: #d50000;">▉</span> `$datasus-error-color`
- <span style="color: #ff5131;">▉</span> `$datasus-errorLight-color`
- <span style="color: #9b0000;">▉</span> `$datasus-errorDark-color`

###### Links

- [<span style="color: #6d6d6d;">▉</span><span style="color: #1b1b1b;">▉</span><span style="color: #424242">▉</span><span style="color: #1565c0">▉</span><span style="color: #003c8f;">▉</span><span style="color: #5e92f3;">▉</span> Dusky | Primary](https://material.io/tools/color/#!/?primary.color=424242&secondary.color=1565C0)
- [<span style="color: #6d6d6d;">▉</span><span style="color: #1b1b1b;">▉</span><span style="color: #424242">▉</span><span style="color: #ff8f00">▉</span><span style="color: #c56000;">▉</span><span style="color: #ffc046;">▉</span> Dusky | Accent](https://material.io/tools/color/#!/?primary.color=424242&secondary.color=FF8F00)
- [<span style="color: #6d6d6d;">▉</span><span style="color: #1b1b1b;">▉</span><span style="color: #424242">▉</span><span style="color: #c2185b">▉</span><span style="color: #8c0032;">▉</span><span style="color: #fa5788;">▉</span> Dusky | Warn](https://material.io/tools/color/#!/?primary.color=424242&secondary.color=C2185B)
- [<span style="color: #6d6d6d;">▉</span><span style="color: #1b1b1b;">▉</span><span style="color: #424242">▉</span><span style="color: #2e7d32">▉</span><span style="color: #005005;">▉</span><span style="color: #60ad5e;">▉</span> Dusky | Success](https://material.io/tools/color/#!/?primary.color=424242&secondary.color=2E7D32)
- [<span style="color: #6d6d6d;">▉</span><span style="color: #1b1b1b;">▉</span><span style="color: #424242">▉</span><span style="color: #d50000">▉</span><span style="color: #9b0000;">▉</span><span style="color: #ff5131;">▉</span> Dusky | Error](https://material.io/tools/color/#!/?primary.color=424242&secondary.color=D50000)
- [<span style="color: #6d6d6d;">▉</span><span style="color: #1b1b1b;">▉</span><span style="color: #424242">▉</span><span style="color: #fafafa">▉</span><span style="color: #c7c7c7;">▉</span><span style="color: #ffffff;">▉</span> Dusky | Bright](https://material.io/tools/color/#!/?primary.color=424242&secondary.color=FAFAFA)
- [<span style="color: #5e92f3;">▉</span><span style="color: #003c8f;">▉</span><span style="color: #1565c0">▉</span><span style="color: #fafafa">▉</span><span style="color: #c7c7c7;">▉</span><span style="color: #ffffff;">▉</span> Primary | Bright](https://material.io/tools/color/#!/?primary.color=1565C0&secondary.color=FAFAFA)
- [<span style="color: #ffc046;">▉</span><span style="color: #c56000;">▉</span><span style="color: #ff8f00">▉</span><span style="color: #fafafa">▉</span><span style="color: #c7c7c7;">▉</span><span style="color: #ffffff;">▉</span> Accent | Bright](https://material.io/tools/color/#!/?primary.color=FF8F00&secondary.color=FAFAFA)
- [<span style="color: #fa5788;">▉</span><span style="color: #8c0032;">▉</span><span style="color: #c2185b">▉</span><span style="color: #fafafa">▉</span><span style="color: #c7c7c7;">▉</span><span style="color: #ffffff;">▉</span> Warn | Bright](https://material.io/tools/color/#!/?primary.color=C2185B&secondary.color=FAFAFA)
- [<span style="color: #60ad5e;">▉</span><span style="color: #005005;">▉</span><span style="color: #2e7d32">▉</span><span style="color: #fafafa">▉</span><span style="color: #c7c7c7;">▉</span><span style="color: #ffffff;">▉</span> Success | Bright](https://material.io/tools/color/#!/?primary.color=2E7D32&secondary.color=FAFAFA)
- [<span style="color: #ff5131;">▉</span><span style="color: #9b0000;">▉</span><span style="color: #d50000">▉</span><span style="color: #fafafa">▉</span><span style="color: #c7c7c7;">▉</span><span style="color: #ffffff;">▉</span> Error | Bright](https://material.io/tools/color/#!/?primary.color=D50000&secondary.color=FAFAFA)


#### Animação/Cor do conteúdo principal

Para as rotas que teram animação inicial, é só passar como parametro `data: { layoutColor: '...', layoutAnimation: '...' }` na rota.

Padrão é sem cor e sem animação.

```typescript
const appRoutes: Routes = [
  {
    path: 'tela-sem-animacao-com-cor',
    component: TelaSemAnimacaoComCorComponent,
    data: {
      layoutColor: 'warn',
      layoutAnimation: false,
    },
    ...guardsDefault,
  },
  {
    path: 'tela-sem-animacao-sem-cor',
    component: TelaSemAnimacaoSemCorComponent,
    data: {
      layoutColor: 'none',
      layoutAnimation: false,
    },
    ...guardsDefault,
  },
  {
    path: 'tela-com-animacao',
    component: TelaComAnimacaoComponent,
    data: {
      layoutColor: 'primary',
      layoutAnimation: true,
    },
    ...guardsDefault,
  },
```

##### Opções de cores

As opções disponíveis segue o padrão do nome das cores.

Sempre exibindo a mesma animação, com diferença na cor a ser usada:

- Cor com a animação desligada:
  - `none`
  - `dusky`
  - `bright`
  - `primary`
  - `accent`
  - `warn`
  - `success`
  - `erro`
- Cor com a a Animação ligada:
  - Todas de cima, menos o `none`.

## Dicas

Tentar colocar sempre os atributos no template seguindo essa ordem:

| Atributo | Exemplo |
|:--|:--|
| de _template reference variable_ | `#nameOfElement` |
| de _template input variable_ | `*ngIf="..."` |
| do HTML padrão  | `class="..."` |
| de _directives_ | `matTooltip="..."`<br />`mat-mini-fab` |
| do tipo _property (on-way data) bind_ "[]" | `[nameOfTarget]="..."`<br />`bind-nameOfTarget="..."` |
| do tipo _event (on-way data) bind_ "()" | `(nameOfEvent)="..."`<br />`on-nameOfEvent="..."` |
| do tipo _two-way data bind_ "[()]" | `[(ngModel)]="..."`<br />`bindon-ngModel="..."` |

## Pendências

- Push notification: https://onesignal.com/
- Manter a licença ISC?
