import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { MatSelectionListChange } from '@angular/material/list';
import { Observable } from 'rxjs';
import { tap, map } from 'rxjs/operators';

import {
  AuthService,
  SecurityService,
  PerfilModel,
  EsferaModel,
  UsuarioModel,
  DATASUS_ROUTE_HOME,
} from '@datasus/core';

import { LayoutService } from '@datasus/layout';

@Component({
  selector: 'datasus-choose-profile',
  templateUrl: './choose-profile.component.html',
  styleUrls: ['./choose-profile.component.scss']
})
export class ChooseProfileComponent implements OnInit {

  perfis$: Observable<PerfilModel[]>;
  nomeCompleto$: Observable<string>;
  perfilAtual$: Observable<PerfilModel>;
  esferaAtual$: Observable<EsferaModel>;

  constructor(
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private layoutService: LayoutService,
    private authService: AuthService,
    private securityService: SecurityService,
  ) { }

  ngOnInit() {
    this.layoutService.off();
    this.perfilAtual$ = this.securityService.getPerfilSelecionado();
    this.esferaAtual$ = this.securityService.getEsferaSelecionada();
    this.perfis$ = this.securityService.getPerfis().pipe(
      tap((perfis: PerfilModel[]) => {
        if (!perfis.length) {
          console.log('[ChooseProfileComponent] Ops, sem perfil! Volta para a tela inicial');
          this.router.navigate([`/${DATASUS_ROUTE_HOME}`]);
        } else {
          this.nomeCompleto$ = this.authService.getUser().pipe(map((usuario: UsuarioModel) => usuario.nome));
        }
      })
    );
  }

  select(matSelectionListChange: MatSelectionListChange): void {
    matSelectionListChange.source.deselectAll();
    matSelectionListChange.option.selected = true;
    setTimeout(() => {
      this.activatedRoute.params.subscribe(
        (params: {[returnPath: string]: string}) => {
          this.securityService.chooseProfileAndNavigate({
            perfil: matSelectionListChange.option.value.perfil,
            esfera: matSelectionListChange.option.value.esfera,
            returnPath: params.returnPath,
          });
        }
      );
    }, 500);
  }

}
