import { Component, OnInit, Input, EventEmitter, Output, OnDestroy, TemplateRef } from '@angular/core';
import { trigger, state, style, transition, animate } from '@angular/animations';

import { ErrorModel } from '@datasus/core';

import { DynamicTableRequestDataConfig } from '../dynamic-table/dynamic-table-request-data.config';
import { ActionConfig } from '../../configs/actions.config';

@Component({
  selector: 'datasus-expandable-table',
  templateUrl: './expandable-table.component.html',
  styleUrls: ['./expandable-table.component.scss'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({ height: '0px', minHeight: '0' })),
      state('expanded', style({ height: '*' })),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)')),
    ]),
  ],

})


export class ExpandableTableComponent implements OnInit, OnDestroy {


  @Input() expandedTemplateRef: TemplateRef<any>;
  @Input() columns: any[] = [];
  @Input() data: any[] = [];
  @Input() total = 0;
  @Input() pageSize = 0;
  @Input() actions: ActionConfig[] = [];
  @Input() loading = false;
  @Output() request: EventEmitter<DynamicTableRequestDataConfig> = new EventEmitter<DynamicTableRequestDataConfig>(true);
  @Output() error: EventEmitter<ErrorModel> = new EventEmitter<ErrorModel>(true);

  displayedColumns: string[];
  actionButtonCtrl: Map<any, boolean> = new Map();
  expandedElement: any | null;

  private resquestDataCache: DynamicTableRequestDataConfig = {};

  ngOnInit(): void {
    this.displayedColumns = this.columns.map(column => column.cell);
    this.resquestDataCache = {};
  }

  ngOnDestroy(): void {
    this.resquestDataCache = {};
  }

  onRequest(data: DynamicTableRequestDataConfig): void {
    const newData = { ...this.resquestDataCache, ...data };
    this.request.emit(newData);
    this.resquestDataCache = newData;
  }

  emitClick(menu: any, item: any): void {
    menu.callback(item);
  }

  urlWrap(path: string): string {
    return `url('${path}')`;
  }

}


