import { Component, OnInit, Inject } from '@angular/core';

import { LayoutService } from '@datasus/layout';
import { CommonsConfig } from '../../configs/commons.config';

@Component({
  selector: 'datasus-welcome',
  template: `
    <datasus-container>
      <p datasusOnMobile="hide">
          <mat-icon>arrow_back</mat-icon>
          Selecione no menu ao lado um item.
      </p>
      <h1 *ngIf="!!welcomeText">{{ welcomeText }}</h1>
      <div *ngIf="!!welcomeHtml" [innerHTML]="welcomeHtml | datasusSafeHtml"></div>
      <p datasusOnMobile="show">
          <button type="button" mat-icon-button (click)="openMenu()"><mat-icon>menu</mat-icon></button>
          Abra o menu e selecione um item.
      </p>
    </datasus-container>
  `,
  styles: [`
    p {
      margin-bottom: 100px;
    }
    h1 {
      text-align: center;
    }
  `],
})
export class WelcomeComponent implements OnInit {

  welcomeText: string;
  welcomeHtml: string;

  constructor(
    @Inject('CommonsConfig')
    private commonsConfig: CommonsConfig,
    private layoutService: LayoutService,
  ) { }

  ngOnInit(): void {
    this.layoutService.setTitle('Bem-vindo!');
    this.welcomeText = this.commonsConfig.welcomeText || 'Bem-vindo!';
    this.welcomeHtml = this.commonsConfig.welcomeHtml || '';
    console.log('%cOi WelcomeComponent', 'font-size: 1.2em; color: cyan;');
  }

  openMenu(): void {
    this.layoutService.menuSidenavExpanded();
  }

}
