import { Injectable } from '@angular/core';
import { HttpEvent, HttpHandler, HttpInterceptor, HttpRequest } from '@angular/common/http';
import { Observable } from 'rxjs';

import { SecurityService } from '../services/security.service';
import { PerfilModel } from '../models/perfil.model';
import { EsferaModel } from '../models/esfera.model';
import { HTTP_HEADER_PERFIL, HTTP_HEADER_ESFERA, HTTP_HEADER_ESFERA_ATUACAO } from '../constants/security.constants';

@Injectable({
  providedIn: 'root'
})
export class SecurityInterceptor implements HttpInterceptor {

  constructor(
    private securityService: SecurityService
  ) { }

  intercept(
    request: HttpRequest<any>,
    next: HttpHandler
  ): Observable<HttpEvent<any>> {
    const perfil: PerfilModel = this.securityService.getStoredPerfilSelecionado() || new PerfilModel();
    const esfera: EsferaModel = this.securityService.getStoredEsferaSelecionada() || new EsferaModel();

    let headers = request.headers;

    if (perfil.sigla) {
      headers = headers.set(HTTP_HEADER_PERFIL, perfil.sigla.toString());
    }
    if (esfera.sigla) {
      headers = headers.set(HTTP_HEADER_ESFERA, esfera.sigla.toString());
    }
    if (esfera.esferaAtuacao) {
      headers = headers.set(HTTP_HEADER_ESFERA_ATUACAO, esfera.esferaAtuacao);
    }

    // console.log('[SecurityInterceptor::intercept] Headers enviado', headers);
    return next.handle(request.clone({ headers }));
  }
}
