/**
 * Atalho para `window.location.href`
 * @param url URL que redirecionado
 */
export function redirectTo(url: string): void {
  (window as Window).location.href = url;
}

/**
 * Atalho para comparar Objetos.
 * @param obj1 Objeto 1 de comparação
 * @param obj2 Objeto 2 de comparação
 */
export function compareObjects(obj1: any = {}, obj2: any = {}): boolean {
  if (typeof obj1 !== typeof obj2) {
    return false;
  }

  if (obj1 === obj2) {
    return true;
  }

  const o1 = obj1 || {};
  const o2 = obj2 || {};

  if (typeof o1 !== 'object' || typeof o2 !== 'object') {
    return false;
  }

  function fnLength(o: any): number {
    return Object.keys(o).length;
  }

  if (fnLength(o1) !== fnLength(o2)) {
    return false;
  }

  return Object.keys(o1).every((key: string) => compareObjects(o1[key], o2[key]));
}

/**
 * Atalho para comparar objeto em lista
 * @param list Lista de objetos
 * @param obj Objeto
 */
export function checkObjectInList({ list, obj }: { list: any[]; obj: any; }): boolean {
  return list.some(
    (item: any) => compareObjects(item, obj)
  );
}

/**
 * Metodo de verificação de intervalo dia, mes e ano
 * @param startDate Data inicio
 * @param stopDate Data fim
 * @param incrementMode Tipo `('days' | 'months' | 'years')`
 */
export function intervalDates(startDate: Date, stopDate: Date, incrementMode: ('days' | 'months' | 'years')): Date[] {


  if (startDate > stopDate) {
    return [];
  }

  function countYears(): number {
    return Math.ceil((+stopDate - +startDate + 31622400000) / 31622400000);
  }

  function countMounths(): number {
    return Math.ceil(((+stopDate + 2678400000) - +startDate) / 2678400000);
  }

  function countDays(): number {
    return Math.ceil(((+stopDate + 86400000) - +startDate) / 86400000);
  }

  const incrementMap = {
    /** @todo precisar melhoarar o algoritimo  */
    days: (): Date[] => {
      let day: number = startDate.getDate();
      return new Array(countDays()).fill(undefined).map(
        () => new Date(startDate.getFullYear(), startDate.getMonth(), day++)
      );
    },
    months: (): Date[] => {
      let month: number = startDate.getMonth();
      return new Array(countMounths()).fill(undefined).map(
        () => new Date(startDate.getFullYear(), month++)
      );
    },

    years: (): Date[] => {
      let year: number = startDate.getFullYear();
      return new Array(countYears()).fill(undefined).map(
        () => new Date(year++)
      );
    },
  };
  const dateArray = incrementMap[incrementMode]();
  return dateArray;
}


