import { PageEvent } from '@angular/material/paginator';
import { Sort } from '@angular/material/sort';

interface Config {
  to: {pageSize: number};
  withPageEvent?: PageEvent;
  withSort?: Sort;
  changingPageSizeDefaultTo?: number;
}

interface RequestParams {
  pageNumber: number;
  pageSize: number;
  sortField?: string;
  sortType?: ('ASC'|'DESC');
}

/**
 * Monta para o envelope padrão da paginação os
 * parâmetros de número da página
 * e tamanho da página
 * @see PageableRequestModel
 * @param config
 *  withPageEvent: Evento do Paginador
 *  to: component Componente que contenha o atributo `pageSize`
 *  changingPageSizeDefaultTo: Valor padrão para o atributo `pageSize`: **10** itens por página.
 */
export function requestItemsParams(config: Config): RequestParams {
  let pageNumber = 1;
  config.to.pageSize = config.changingPageSizeDefaultTo || 10;

  if (config.withPageEvent) {
    pageNumber = (config.withPageEvent.pageIndex || 0) + 1;
    config.to.pageSize = config.withPageEvent.pageSize || config.to.pageSize;
  }
  const sort: {sortField?: string, sortType?: ('ASC'|'DESC')} = {};
  if (config.withSort && !!config.withSort.direction) {
    sort.sortField = config.withSort.active;
    sort.sortType = config.withSort.direction.toUpperCase() as ('ASC'|'DESC');
  }

  return {
    pageSize: config.to.pageSize,
    pageNumber,
    ...sort
  };
}
