import { FormControl } from '@angular/forms';

import { CnpjValidator } from './cnpj.validator';
import { FormValidators } from './form.validators';

describe('CNPJ Validator', () => {

  const CNPJ_VALID_MASKED = '10.565.000/0001-92';
  const CNPJ_INVALID_MASKED = '10.565.000/0001-99';
  const CNPJ_VALID_UNMASKED = '10565000000192';
  const CNPJ_INVALID_UNMASKED = '10565000000199';
  const CNPJ_EMPTY_LIST = [null, undefined, ''];
  const CNPJ_INVALID_LIST = ['9898', '', 'asd', 123];

  it(`Teste da expressão regular o CNPJ: ${CNPJ_VALID_MASKED} (válido)`, () => {
    expect(CnpjValidator.CNPJ_REG_EXP.test(CNPJ_VALID_MASKED)).toBeTruthy();
  });

  it(`Valida o CNPJ: ${CNPJ_VALID_MASKED} (válido)`, () => {
    expect(CnpjValidator.isValid(CNPJ_VALID_MASKED)).toBeTruthy();
  });

  it(`Valida o CNPJ: ${CNPJ_INVALID_MASKED} (inválido)`, () => {
    expect(CnpjValidator.isValid(CNPJ_INVALID_MASKED)).toBeFalsy();
  });

  it(`Valida o CNPJ: ${CNPJ_VALID_UNMASKED} (válido)`, () => {
    expect(CnpjValidator.isValid(CNPJ_VALID_UNMASKED)).toBeTruthy();
  });

  it(`Valida o CNPJ: ${CNPJ_INVALID_UNMASKED} (inválido)`, () => {
    expect(CnpjValidator.isValid(CNPJ_INVALID_UNMASKED)).toBeFalsy();
  });

  it(`Valida CNPJ inválidos`, () => {
    CNPJ_INVALID_LIST.map((CNPJ: string) => expect(CnpjValidator.isValid(CNPJ)).toBeFalsy());
  });

  it(`Valida o FormControl com o CNPJ: ${CNPJ_VALID_MASKED} (válido)`, () => {
    const formControlValid = new FormControl(CNPJ_VALID_MASKED, FormValidators.cnpj);
    expect(formControlValid.valid).toBeTruthy();
  });

  it(`Valida o FormControl com o CNPJ: ${CNPJ_INVALID_MASKED} (inválido)`, () => {
    const formControlInvalid = new FormControl(CNPJ_INVALID_MASKED, FormValidators.cnpj);
    expect(formControlInvalid.valid).toBeFalsy();
  });

  it(`Valida o FormControl com o CNPJ: ${CNPJ_VALID_UNMASKED} (válido)`, () => {
    const formControlValid = new FormControl(CNPJ_VALID_UNMASKED, FormValidators.cnpj);
    expect(formControlValid.valid).toBeTruthy();
  });

  it(`Valida o FormControl com o CNPJ: ${CNPJ_INVALID_UNMASKED} (inválido)`, () => {
    const formControlInvalid = new FormControl(CNPJ_INVALID_UNMASKED, FormValidators.cnpj);
    expect(formControlInvalid.valid).toBeFalsy();
  });

  it(`Valida FormControls com CNPJ vazios`, () => {
    CNPJ_EMPTY_LIST.map((CNPJ: string) => {
      const formControlValid = new FormControl(CNPJ, FormValidators.cnpj);
      expect(formControlValid.valid).toBeTruthy();
    });
  });

});
