import { Component, OnInit, Inject } from '@angular/core';

import { ConfigService } from '@datasus/core';

import { LayoutConfig } from '../../configs/layout.config';

@Component({
    selector: 'datasus-root',
    styleUrls: ['./root.component.scss'],
    templateUrl: './root.component.html',
})
export class RootComponent implements OnInit {

  withGovBrBar = false;

  constructor(
    @Inject('LayoutConfig')
    private layoutConfig: LayoutConfig,
    private configService: ConfigService,
  ) { }

  ngOnInit(): void {
    this.configService.registerApp(this.layoutConfig.appName, this.layoutConfig.appVersion);

    this.appendMatTypographyClass();

    this.withGovBrBar = this.layoutConfig.withGovBrBar;
    if (this.withGovBrBar) {
      this.appendGovBrBarInDom();
    }
  }

  private appendMatTypographyClass(): void {
    const doc: Document = (window as Window).document;
    const body = doc.body;
    body.classList.add('mat-typography');
  }

  private appendGovBrBarInDom(): void {
    try {
      const doc: Document = (window as Window).document;
      const head = doc.head;
      const body = doc.body;
      const meta = doc.createElement('meta');
      const script = doc.createElement('script');

      const URL_ESTRUTURA_ORGANIZACIONAL = 'http://estruturaorganizacional.dados.gov.br/doc/unidade-organizacional/2393';
      const URL_JS_BARRA_GOV_BR = 'https://barra.brasil.gov.br/barra_2.0.js';

      meta.setAttribute('content', URL_ESTRUTURA_ORGANIZACIONAL);
      meta.setAttribute('property', 'creator.productor');

      script.setAttribute('src', URL_JS_BARRA_GOV_BR);
      script.setAttribute('defer', 'defer');
      script.setAttribute('type', 'text/javascript');

      head.appendChild(meta);
      body.appendChild(script);
    } catch (e) {
      console.error('Não foi possível injetar no <head /> e no <body /> as coisas da barra do GOV.BR', e);
    }
  }
}
