import { RelativeDatePipe } from './relative-date.pipe';

describe('RelativeDatePipe', () => {

  let pipe: RelativeDatePipe;

  beforeEach(() => {
    pipe = new RelativeDatePipe();
  });

  it(
    'Instancia o pipe',
    () => expect(pipe).toBeTruthy()
  );

  it(
    'Erro na comparação',
    () => {
      expect(pipe.transform(new Date(2001), new Date(2000))).toEqual('Data futura');
      expect(pipe.transform(+new Date(2000) + 1, new Date(2000))).toEqual('Data futura');
      expect(pipe.transform(new Date(), new Date(2000))).toEqual('Data futura');
    }
  );

  it(
    'Retornos com a precisão baixa: Hoje',
    () => {
      expect(pipe.transform(new Date(), new Date(), true)).toEqual('Hoje');
      expect(pipe.transform(new Date(2000, 0, 1, 0, 0, 0), new Date(2000, 0, 1, 0, 0, 42), true)).toEqual('Hoje');
      expect(pipe.transform(new Date(2000, 0, 1, 0, 0, 0), new Date(2000, 0, 1, 23, 59, 59), true)).toEqual('Hoje');
      expect(pipe.transform(new Date(2000, 0, 1, 0, 0, 0), new Date(2000, 0, 1, 23, 59, 60), true) === 'Hoje').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão baixa: Ontem',
    () => {
      expect(pipe.transform(new Date(2000, 0, 1), new Date(2000, 0, 2), true)).toEqual('Ontem');
      expect(pipe.transform(new Date(2000, 0, 1), new Date(2000, 0, 2, 23, 59, 59), true)).toEqual('Ontem');
      expect(pipe.transform(new Date(2000, 0, 1, 0, 0, 0), new Date(2000, 0, 2, 23, 59, 60), true) === 'Ontem').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão baixa: Essa semana',
    () => {
      expect(pipe.transform(new Date(2019, 9, 31), new Date(2019, 10, 2), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 30), new Date(2019, 10, 2), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 29), new Date(2019, 10, 2), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 28), new Date(2019, 10, 2), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 27), new Date(2019, 10, 2), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 26), new Date(2019, 10, 2), true) === 'Essa semana').toBeFalsy();
      expect(pipe.transform(new Date(2019, 9, 8), new Date(2019, 9, 10), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 10), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 6), new Date(2019, 9, 10), true)).toEqual('Essa semana');
      expect(pipe.transform(new Date(2019, 9, 5), new Date(2019, 9, 10), true) === 'Essa semana').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão baixa: Semana passada',
    () => {
      expect(pipe.transform(new Date(2019, 9, 5), new Date(2019, 9, 10), true)).toEqual('Semana passada');
      expect(pipe.transform(new Date(2019, 9, 4), new Date(2019, 9, 10), true)).toEqual('Semana passada');
      expect(pipe.transform(new Date(2019, 9, 3), new Date(2019, 9, 10), true)).toEqual('Semana passada');
      expect(pipe.transform(new Date(2019, 9, 2), new Date(2019, 9, 10), true)).toEqual('Semana passada');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2019, 9, 10), true)).toEqual('Semana passada');
      expect(pipe.transform(new Date(2019, 8, 30), new Date(2019, 9, 10), true)).toEqual('Semana passada');
      expect(pipe.transform(new Date(2019, 8, 29), new Date(2019, 9, 10), true)).toEqual('Semana passada');
      expect(pipe.transform(new Date(2019, 8, 28), new Date(2019, 9, 10), true) === 'Semana passada').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão baixa: Mês passado',
    () => {
      expect(pipe.transform(new Date(2019, 8, 28), new Date(2019, 9, 10), true)).toEqual('Mês passado');
      expect(pipe.transform(new Date(2019, 8, 27), new Date(2019, 9, 10), true)).toEqual('Mês passado');
      expect(pipe.transform(new Date(2019, 8, 1), new Date(2019, 9, 10), true)).toEqual('Mês passado');
      expect(pipe.transform(new Date(2019, 7, 31), new Date(2019, 9, 10), true) === 'Mês passado').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão baixa: Mais antiga',
    () => {
      expect(pipe.transform(new Date(2019, 7, 31), new Date(2019, 9, 10), true)).toEqual('Mais antiga');
      expect(pipe.transform(new Date(2018, 7, 31), new Date(2019, 9, 10), true)).toEqual('Mais antiga');
      expect(pipe.transform(new Date(2017), new Date(2019, 9, 10), true)).toEqual('Mais antiga');
      expect(pipe.transform(new Date(2000), new Date(2019, 9, 10), true)).toEqual('Mais antiga');
      expect(pipe.transform(new Date(1971), new Date(2019, 9, 10), true)).toEqual('Mais antiga');
    }
  );

  it(
    'Retornos com a precisão alta: Agora pouco',
    () => {
      expect(pipe.transform(new Date(), new Date())).toEqual('Agora pouco');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 0, 30))).toEqual('Agora pouco');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 0, 60))).toEqual('Agora pouco');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 1, 0))).toEqual('Agora pouco');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 2, 0)) === 'Agora pouco').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão alta: Minutos atrás',
    () => {
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 2, 0))).toEqual('Minutos atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 2, 30))).toEqual('Minutos atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 9, 0))).toEqual('Minutos atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 30, 0))).toEqual('Minutos atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 58, 0))).toEqual('Minutos atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 0, 59, 0))).toEqual('Minutos atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 1, 0, 0))).toEqual('Minutos atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 1, 5, 0)) === 'Minutos atrás').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão alta: Horas atrás',
    () => {
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 1, 5, 0))).toEqual('Horas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 5, 5, 0))).toEqual('Horas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 15, 5, 0))).toEqual('Horas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 23, 5, 0))).toEqual('Horas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 7, 23, 59, 59))).toEqual('Horas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 8))).toEqual('Horas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 8, 1)) === 'Horas atrás').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão alta: Dias atrás',
    () => {
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 8, 1))).toEqual('Dias atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 9))).toEqual('Dias atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 10))).toEqual('Dias atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 14))).toEqual('Dias atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 15)) === 'Dias atrás').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão alta: Semanas atrás',
    () => {
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 15))).toEqual('Semanas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 9, 24))).toEqual('Semanas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 10, 1))).toEqual('Semanas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 10, 30))).toEqual('Semanas atrás');
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 11, 3)) === 'Semanas atrás').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão alta: Meses atrás',
    () => {
      expect(pipe.transform(new Date(2019, 9, 7), new Date(2019, 11, 3))).toEqual('Meses atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2020, 3, 1))).toEqual('Meses atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2020, 8, 1))).toEqual('Meses atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2020, 8, 25))).toEqual('Meses atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2020, 9, 1))).toEqual('Meses atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2020, 10, 1)) === 'Meses atrás').toBeFalsy();
    }
  );

  it(
    'Retornos com a precisão alta: Anos atrás',
    () => {
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2020, 10, 1))).toEqual('Anos atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2021, 1))).toEqual('Anos atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2021, 0))).toEqual('Anos atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2025, 0))).toEqual('Anos atrás');
      expect(pipe.transform(new Date(2019, 9, 1), new Date(2100, 0))).toEqual('Anos atrás');
    }
  );

});
