import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { Observable, of as observableOf } from 'rxjs';
import { switchMap, tap, startWith, map, delay, mapTo } from 'rxjs/operators';

import { ConfigService } from '@datasus/core';

import { SistemaModel } from '../models/sistema.model';

@Injectable({
  providedIn: 'root'
})
export class SistemaService {

  cache: SistemaModel[] = [];

  constructor(
    private httpClient: HttpClient,
    private configService: ConfigService,
  ) { }

  get(): Observable<SistemaModel[]> {
    if (!!this.cache && !!this.cache.length) {
      console.log('%cSistemas do cache', 'color: cyan; background-color: black;');
      return observableOf(this.cache).pipe(delay(0));
    }

    return this.configService.getList([
      'backendUrl',
      'clientId'
    ]).pipe(
      switchMap((configs: string[]) => {
        const backendUrl = configs[0];
        const clientId = configs[1];
        console.log('%cSistemas do back-end', 'color: cyan; background-color: black;');
        return this.httpClient.get<SistemaModel[]>(`${backendUrl}/acesso/sistema`).pipe(
          map(
            (sistemas: SistemaModel[]) => sistemas.filter(
              (sistema: SistemaModel) => sistema.sigla !== clientId
            )
          ),
        );
      }),
      tap((sistemas: SistemaModel[]) => this.cache = sistemas),
      startWith([]),
    );

    }
}
