import { Component, OnInit } from '@angular/core';
import { Observable, of as observableOf } from 'rxjs';
import { delay } from 'rxjs/operators';

import { LayoutService, DialogService, DialogAlertConfig } from '@datasus/layout';
import { ErrorModel } from '@datasus/core';

import { SomeService } from './some.service';

@Component({
  selector: 'datasus-app-lazy',
  template: `
  <datasus-container>
    <h1>Componente APP Lazy</h1>
    <ng-template #loading>
      <mat-spinner class="loading"></mat-spinner>
    </ng-template>
    <p *ngIf="(request$ | async) as request; else loading">
      Retorno do request <code>{{ request }}</code>.
    </p>
    <button mat-raised-button color="warn" (click)="alert('warn')">ALERTA (warn)</button>
    <button mat-raised-button color="primary" (click)="alert('primary')">ALERTA (primary)</button>
    <button mat-raised-button color="accent" (click)="alert('accent')">ALERTA (accent)</button>
    |
    <button mat-stroked-button color="warn" (click)="confirmation()">CONFIRMAÇÃO</button>
    |
    <button mat-stroked-button color="primary" (click)="prompt()">PERGUNTA</button>
    |
    <button mat-flat-button (click)="notification()">NOTIFICAÇÃO</button>
    |
    <button mat-button routerLink="crud">CRUD</button>

    <mat-select value="DF" [compareWith]="compareUf">
      <mat-option *ngFor="let uf of ufs" [value]="uf">
        {{ uf.sigla }} - {{ uf.nome }}
      </mat-option>
    </mat-select>

    <div>
        <code>{{ select1 | json }}</code>
        <p-multiSelect
          #pMultiSelect1
          itemSize="50"
          optionLabel="nome"
          selectionLimit="1"
          selectedItemsLabel=""
          defaultLabel="Select #1"
          [resetFilterOnHide]="true"
          [showToggleAll]="false"
          [options]="options"
          [virtualScroll]="true"
          [(ngModel)]="select1"
        >
        <ng-template let-option pTemplate="item">
          <span
            #pMultiSelect1Item
            class="p-multi-select-item"
            [datasusMultiSelect]="[pMultiSelect1, pMultiSelect1Item]"
            [matTooltip]="option.value['descricao']"
          >
            {{ option.value['nome'] }}
          </span>
        </ng-template>
      </p-multiSelect>
    </div>

    <div>
      <code>{{ select2 | json }}</code>
      <p-multiSelect
        #pMultiSelect2
        itemSize="50"
        optionLabel="nome"
        selectionLimit="1"
        selectedItemsLabel=""
        defaultLabel="Select #2"
        [resetFilterOnHide]="true"
        [showToggleAll]="false"
        [options]="options"
        [virtualScroll]="true"
        [(ngModel)]="select2"
      >
        <ng-template let-option pTemplate="item">
            <span
              #pMultiSelect2Item
              class="p-multi-select-item"
              [datasusMultiSelect]="[pMultiSelect2, pMultiSelect2Item]"
              [matTooltip]="option?.value?.descricao"
            >
              {{ option?.value?.nome }}
            </span>
        </ng-template>
      </p-multiSelect>
    </div>
  </datasus-container>`,
})
export class LazyComponent implements OnInit {

  ufs: {id: number, sigla: string, nome: string}[] = [
    {id: 3, sigla: 'RJ', nome: 'Rio de Janeiro'},
    {id: 1, sigla: 'DF', nome: 'Distrito Federal'},
    {id: 2, sigla: 'MG', nome: 'Minas Gerais'},
  ];

  request$: Observable<number>;

  select1: any;
  select2: any;
  options = [
    { id: 261, nome: 'A PONTO DE RESPOSTA'},
    { id: 181, nome: 'ALTERAR EVIDENCIA'},
    { id: 45, nome: 'B Ponto de resposta'},
    { id: 182, nome: 'BBBBBBB'},
    { id: 46, nome: 'C Ponto de restposta'},
    { id: 184, nome: 'CCCCCCC'},
    { id: 221, nome: 'DCSGDFGSD'},
    { id: 183, nome: 'DDDDDDDD'},
    { id: 101, nome: 'DENASUS'},
    { id: 1, nome: 'MINISTÉRIO DA SAÚDE'},
    { id: 141, nome: 'MS ¿ Ministério da Saúde'},
    { id: 186, nome: 'OUVIDORIA EVIDENCIA 1'},
    { id: 262, nome: 'OUVIDORIA NÍVEL 1'},
    { id: 44, nome: 'OUVIDORIA NÍVEL 2'},
    { id: 203, nome: 'OUVIDORIA NÍVEL 3'},
    { id: 110, nome: 'OUVIDORIA-GERAL DO SUS'},
    { id: 2, nome: 'Ouvidoria Gabriel 1'},
    { id: 18, nome: 'Ouvidoria Gabriel 2'},
    { id: 42, nome: 'PONTO DE RESPOSTA GABRIEL'},
    { id: 103, nome: 'PONTO DE RESPOSTA SECRETARIA DE SAUDE DO DISTRITO FEDERAL'},
    { id: 7, nome: 'SECRETARIA DE SAUDE DO DISTRITO FEDERAL'},
    { id: 63, nome: 'TESTE CAIXA ALTA ALTERADO'},
    { id: 62, nome: 'TESTE CAIXA ALTA NO CAMPO ALTERADOA'},
    { id: 64, nome: 'TESTE CAIXA ALTA ORGAO ALTERADO'},
    { id: 109, nome: 'TESTE ITEM 670'},
    { id: 121, nome: 'TESTE OUVIDORIA OD'},
    { id: 205, nome: 'TESTE PONTO DE REPOSTA REGISTRO RL_TEMA'},
    { id: 187, nome: 'TESTE RELACAO CO_TEMA'},
    { id: 3, nome: 'teste pesquisa de ouvidorias'},
  ];

  constructor(
    private someService: SomeService,
    private layoutService: LayoutService,
    private dialogService: DialogService,
  ) { }

  compareUf(option: {id: number, sigla: string, nome: string}, selectValue: string): boolean {
    return selectValue === option.sigla;
  }

  ngOnInit(): void {
    this.layoutService.setTitle('Módulo Preguiçoso');
    this.request$ = this.someService.requestInteger1().pipe(delay(5000));
    this.someService.requestError400().subscribe(() => console.log('ok'), (error400: ErrorModel) => {
      this.dialogService.showNotification({
        message: '400: ' + error400.message,
        dismiss: () => {
          this.someService.requestError500().subscribe(() => console.log('ok'), (error500: ErrorModel) => {
            this.dialogService.showNotification('500: ' + error500.message);
          });
        }
      });
    });
    console.log('Oi componente preguiçoso');
  }

  alert(type: string): void {
    const config: DialogAlertConfig = {
      title: `Alerta do tipo: ${type}`,
      type,
      message: 'Teste um dois, um dois...',
      callback: () => {

        this.request$ = observableOf(999);

        console.log('fechou o alerta');
      },
    };
    if (type === 'primary') {
      config.messages = Array(10).fill(config.message);
      delete config.message;
    }
    this.dialogService.showAlert(config);
  }

  confirmation(): void {
    const buttons: Map<string, () => void> = new Map();
    buttons.set('sim', () => console.log('Que bom!'));
    buttons.set('não', () => console.log('Que pena!'));
    buttons.set('talvez', () => console.log('Que coisa, não?!'));

    this.dialogService.showConfirmation({
      title: 'Ei...',
      question: 'Você está vivo?',
      anwsers: buttons,
    });
  }

  prompt(): void {
    this.dialogService.showPrompt({
      label: 'Escreva um texto',
      button: 'agora, clique aqui',
      required: true,
      callback: (result: string) => console.log('fechou o prompt', result),
    });
  }

  notification(): void {
    this.dialogService.showNotification({
      message: 'Mensagem de notificação. Vai fechar sozinha ou clica no botão do lado.',
      secondsToClose: 10,
      action: { 'Ação extra': () => console.log('ação extra') },
      dismiss: () => console.log('dismiss notification'),
    });
  }
}
