--------------------------------------------------------
--  DDL for Table tb_tipo_documento
--------------------------------------------------------
CREATE SEQUENCE dbscodes.sq_tipodoc_coseqtipodoc;

CREATE TABLE dbscodes.tb_tipo_documento
(
    co_seq_tipo_documento bigint NOT NULL DEFAULT nextval('dbscodes.sq_tipodoc_coseqtipodoc'::regclass),
    ds_tipo_documento character varying(50) NOT NULL,
    CONSTRAINT pk_tipodocumento PRIMARY KEY (co_seq_tipo_documento)
);

COMMENT ON TABLE dbscodes.tb_tipo_documento IS 'Armazena dados dos tipo de documentos.';
COMMENT ON COLUMN dbscodes.tb_tipo_documento.co_seq_tipo_documento IS 'Codigo de identificacão do tipo documento, incrementado pela sequence sq_tipodoc_coseqtipodoc.';
COMMENT ON COLUMN dbscodes.tb_tipo_documento.ds_tipo_documento IS 'Descriçao do tipo de documento.';

--------------------------------------------------------
--  DDL for Table tb_unidade_transferencia
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_unidtransf_cosequnidtransf;
CREATE TABLE dbscodes.tb_unidade_transferencia
(
    co_seq_unidade_transferencia bigint NOT NULL DEFAULT nextval('dbscodes.sq_unidtransf_cosequnidtransf'::regclass),
    ds_unidade_transferencia character varying(100) NOT NULL,
    CONSTRAINT pk_unidadetransferencia PRIMARY KEY (co_seq_unidade_transferencia)
);

COMMENT ON TABLE dbscodes.tb_unidade_transferencia IS 'Armazena dados de unidades de transferência.';
COMMENT ON COLUMN dbscodes.tb_unidade_transferencia.co_seq_unidade_transferencia IS 'Codigo de identificacão da unidade de transferência, incrementado pela sequence sq_unidtransf_cosequnidtransf.';
COMMENT ON COLUMN dbscodes.tb_unidade_transferencia.ds_unidade_transferencia IS 'Descriçao do unidade de transferência.';

--------------------------------------------------------
--  DDL for Table tb_transferencia
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_transf_coseqtransferencia;
CREATE TABLE dbscodes.tb_transferencia
(
    co_seq_transferencia bigint NOT NULL DEFAULT nextval('dbscodes.sq_transf_coseqtransferencia'::regclass),
    ds_transferencia character varying(30) NOT NULL,
    tp_transferencia bigint NOT NULL,
    co_unidade_transferencia bigint NOT NULL,
    CONSTRAINT pk_transferencia PRIMARY KEY (co_seq_transferencia)
);

COMMENT ON TABLE dbscodes.tb_transferencia IS 'Armazena dados de transferência.';
COMMENT ON COLUMN dbscodes.tb_transferencia.co_seq_transferencia IS 'Codigo de identificacão da transferência, incrementado pela sequence sq_transf_coseqtransferencia.';
COMMENT ON COLUMN dbscodes.tb_transferencia.ds_tranferencia IS 'Descriçao da transferência.';
COMMENT ON COLUMN dbscodes.tb_transferencia.tp_transferencia IS 'Tipo de transferência.';
COMMENT ON COLUMN dbscodes.tb_transferencia.co_unidade_transferencia IS 'Chave estrangeira para a tabela dbscodes.tb_unidade_transferencia.';

--------------------------------------------------------
--  Constraints for Table tb_transferencia
--------------------------------------------------------

---ALTERADO CAMPO DA REFERENCIA DE co_usuario PARA co_unidade_transferencia
ALTER TABLE dbscodes.tb_transferencia ADD CONSTRAINT fk_unidtransf_transferencia FOREIGN KEY(co_unidade_transferencia)
    REFERENCES dbscodes.tb_unidade_transferencia(co_seq_unidade_transferencia);

--------------------------------------------------------
--  DDL for Table tb_doacao
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_doacao_coseqdoacao;
CREATE TABLE dbscodes.tb_doacao
(
    co_seq_doacao bigint NOT NULL DEFAULT nextval('dbscodes.sq_doacao_coseqdoacao'::regclass),
    no_doacao character varying(100) NOT NULL,
    ds_cpf_cnpj_doacao character varying(20) NOT NULL,
    CONSTRAINT pk_doacao PRIMARY KEY (co_seq_doacao)
);

COMMENT ON TABLE dbscodes.tb_doacao IS 'Armazena dados de doação.';
COMMENT ON COLUMN dbscodes.tb_doacao.co_seq_doacao IS 'Codigo de identificacão da doação, incrementado pela sequence sq_doacao_coseqdoacao.';
COMMENT ON COLUMN dbscodes.tb_doacao.no_doacao IS 'Nome do responsável pela doação.';
COMMENT ON COLUMN dbscodes.tb_doacao.ds_cpf_cnpj_doacao IS 'CPF ou CNPJ do responsável pela doação.';

--------------------------------------------------------
--  DDL for Table tb_tipo_movimentacao
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_tipomov_coseqtipomov;
CREATE TABLE dbscodes.tb_tipo_movimentacao
(
    co_seq_tipo_movimentacao bigint NOT NULL DEFAULT nextval('dbscodes.sq_tipomov_coseqtipomov'::regclass),
    ds_tipo_movimentacao character varying(80) NOT NULL,
    tp_tipo_movimentacao character varying(1) NOT NULL,
    CONSTRAINT pk_tipomovimentacao PRIMARY KEY (co_seq_tipo_movimentacao)
);

COMMENT ON TABLE dbscodes.tb_tipo_movimentacao IS 'Armazena dados de tipo movimentação.';
COMMENT ON COLUMN dbscodes.tb_tipo_movimentacao.co_seq_tipo_movimentacao IS 'Codigo de identificacão do tipo movimentação, incrementado pela sequence sq_tipomov_coseqtipomov.';
COMMENT ON COLUMN dbscodes.tb_tipo_movimentacao.ds_tipo_movimentacao IS 'Nome do responsável pelo tipo movimentação.';
COMMENT ON COLUMN dbscodes.tb_tipo_movimentacao.tp_tipo_movimentacao IS 'Tipo do tipo movimentação.';

--------------------------------------------------------
--  DDL for Table tb_compra_modalidade
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_comprmod_coseqcomprmod;
CREATE TABLE dbscodes.tb_compra_modalidade
(
    co_seq_compra_modalidade bigint NOT NULL DEFAULT nextval('dbscodes.sq_comprmod_coseqcomprmod'::regclass),
    ds_compra_modalidade character varying(30) NOT NULL,
    CONSTRAINT pk_compramodalidade PRIMARY KEY (co_seq_compra_modalidade)
);

COMMENT ON TABLE dbscodes.tb_compra_modalidade IS 'Armazena dados de modalidade da compra.';
COMMENT ON COLUMN dbscodes.tb_compra_modalidade.co_seq_compra_modalidade IS 'Codigo de identificacão da tb_compra_modalidade, incrementado pela sequence sq_comprmod_coseqcomprmod.';
COMMENT ON COLUMN dbscodes.tb_compra_modalidade.ds_compra_modalidade IS 'Descrição das modalidades de compra.';

--------------------------------------------------------
--  DDL for Table tb_fornecedor
--------------------------------------------------------
CREATE SEQUENCE dbscodes.sq_fornecedor_coseqfornecedor;

CREATE TABLE dbscodes.tb_fornecedor
(
    co_seq_fornecedor bigint NOT NULL DEFAULT nextval('dbscodes.sq_fornecedor_coseqfornecedor'::regclass),
    ds_descricao character varying(100) NOT NULL,
    nu_cnpj character varying(20) NOT NULL,
    CONSTRAINT pk_fornecedor PRIMARY KEY (co_seq_fornecedor)
);

COMMENT ON TABLE dbscodes.tb_fornecedor IS 'Armazena dados do fornecedor.';
COMMENT ON COLUMN dbscodes.tb_fornecedor.co_seq_fornecedor IS 'Codigo de identificacão do fornecedor, incrementado pela sequence sq_fornecedor_coseqfornecedor.';
COMMENT ON COLUMN dbscodes.tb_fornecedor.ds_descricao IS 'Descricao do fornecedor.';
COMMENT ON COLUMN dbscodes.tb_fornecedor.nu_cnpj IS 'CNPJ do fornecedor.';

--------------------------------------------------------
--  DDL for Table tb_compra_troca
--------------------------------------------------------
CREATE SEQUENCE dbscodes.sq_comprtroca_coseqcompratroca;

CREATE TABLE dbscodes.tb_compra_troca
(
    co_seq_compra_troca bigint NOT NULL DEFAULT nextval('dbscodes.sq_comprtroca_coseqcompratroca'::regclass),
    co_compra_modalidade bigint NOT NULL,
    ds_nota_empenho character varying(30) NOT NULL,
    ds_nota_fiscal character varying(30) NOT NULL,
    co_fornecedor bigint NOT NULL,
    CONSTRAINT pk_compratroca PRIMARY KEY (co_seq_compra_troca)
);

COMMENT ON TABLE dbscodes.tb_compra_troca IS 'Armazena dados de compra e troca.';
COMMENT ON COLUMN dbscodes.tb_compra_troca.co_seq_compra_troca IS 'Codigo de identificacão da compra e troca, incrementado pela sequence sq_comprtroca_coseqcompratroca.';
COMMENT ON COLUMN dbscodes.tb_compra_troca.co_compra_modalidade IS 'Chave estrangeira para a tabela dbscodes.tb_compra_modalidade.';
COMMENT ON COLUMN dbscodes.tb_compra_troca.ds_nota_empenho IS 'Descricao da nota de empenho';
COMMENT ON COLUMN dbscodes.tb_compra_troca.ds_nota_fiscal IS 'Descricao da nota fiscal';
COMMENT ON COLUMN dbscodes.tb_compra_troca.co_fornecedor IS 'Chave estrangeira para a tabela dbscodes.tb_fornecedor.';

--------------------------------------------------------
--  Constraints for Table tb_transferencia
--------------------------------------------------------

--AJUSTADO NOMES DAS FK
ALTER TABLE dbscodes.tb_compra_troca ADD CONSTRAINT fk_compramod_compratroca FOREIGN KEY(co_compra_modalidade)
    REFERENCES dbscodes.tb_compra_modalidade(co_seq_compra_modalidade);

ALTER TABLE dbscodes.tb_compra_troca ADD CONSTRAINT fk_fornecedor_compratroca FOREIGN KEY(co_fornecedor)
    REFERENCES dbscodes.tb_fornecedor(co_seq_fornecedor);


--------------------------------------------------------
--  DDL for Table tb_entrada
--------------------------------------------------------

--ALTERADO CAMPO DE dh_entrada PARA dt_entrada
CREATE SEQUENCE dbscodes.sq_entrada_coseqentrada;

CREATE TABLE dbscodes.tb_entrada
(
    co_seq_entrada bigint NOT NULL DEFAULT nextval('dbscodes.sq_entrada_coseqentrada'::regclass),
    dt_entrada timestamp NOT NULL,
    co_usuario bigint NOT NULL,
    co_doacao bigint NOT NULL,
    co_transferencia bigint NOT NULL,
    co_compra_troca bigint NOT NULL,
    co_tipo_movimentacao bigint NOT NULL,
    ds_motivo character varying(200) NOT NULL,
    CONSTRAINT pk_entrada PRIMARY KEY (co_seq_entrada)
);

COMMENT ON TABLE dbscodes.tb_entrada IS 'Armazena dados de entrada .';
COMMENT ON COLUMN dbscodes.tb_entrada.co_seq_entrada IS 'Codigo de identificacão da entrada, incrementado pela sequence sq_entrada_coseqentrada.';
COMMENT ON COLUMN dbscodes.tb_entrada.dt_entrada IS 'Data e Hora da entrada.';
COMMENT ON COLUMN dbscodes.tb_entrada.co_usuario IS 'Chave estrangeira para a tabela dbscodes.tb_usuario.';
COMMENT ON COLUMN dbscodes.tb_entrada.co_doacao IS 'Chave estrangeira para a tabela dbscodes.tb_doacao.';
COMMENT ON COLUMN dbscodes.tb_entrada.co_transferencia IS 'Chave estrangeira para a tabela dbscodes.tb_transferencia.';
COMMENT ON COLUMN dbscodes.tb_entrada.co_compra_troca IS 'Chave estrangeira para a tabela dbscodes.tb_compra_troca.';
COMMENT ON COLUMN dbscodes.tb_entrada.co_tipo_movimentacao IS 'Chave estrangeira para a tabela dbscodes.tb_tipo_movimentacao.';
COMMENT ON COLUMN dbscodes.tb_entrada.ds_motivo IS 'Motivo da entrada.';

--------------------------------------------------------
--  Constraints for Table tb_entrada
--------------------------------------------------------

ALTER TABLE dbscodes.tb_entrada ADD CONSTRAINT fk_usuario_entrada FOREIGN KEY(co_usuario)
    REFERENCES dbscodes.tb_usuario(co_seq_usuario);

ALTER TABLE dbscodes.tb_entrada ADD CONSTRAINT fk_doacao_entrada FOREIGN KEY(co_doacao)
    REFERENCES dbscodes.tb_doacao(co_seq_doaca);

ALTER TABLE dbscodes.tb_entrada ADD CONSTRAINT fk_transferencia_entrada FOREIGN KEY(co_transferencia)
    REFERENCES dbscodes.tb_transferencia(co_seq_transferencia);

ALTER TABLE dbscodes.tb_entrada ADD CONSTRAINT fk_compratroca_entrada FOREIGN KEY(co_compra_troca)
    REFERENCES dbscodes.tb_compra_troca(co_seq_compratroca);

ALTER TABLE dbscodes.tb_entrada ADD CONSTRAINT fk_tipomov_entrada FOREIGN KEY(co_tipo_movimentacao)
    REFERENCES dbscodes.tb_tipo_movimentacao(co_seq_tipo_movimentacao);

--------------------------------------------------------
--  DDL for Table tb_entrada
--------------------------------------------------------

---ALTERADO CAMPO dh_saida PARA dt_saida
CREATE SEQUENCE dbscodes.sq_saida_coseqsaida;
CREATE TABLE dbscodes.tb_saida
(
    co_seq_saida bigint NOT NULL DEFAULT nextval('dbscodes.sq_saida_coseqsaida'::regclass),
    co_compra_troca bigint NOT NULL,
    co_transferencia bigint NOT NULL,
    ds_motivo character varying(500) NOT NULL,
    dt_saida timestamp NOT NULL,
    co_usuario bigint NOT NULL,
    co_tipo_movimentacao bigint NOT NULL,
    CONSTRAINT pk_saida PRIMARY KEY (co_seq_saida)
);


COMMENT ON TABLE dbscodes.tb_saida IS 'Armazena dados de saída.';
COMMENT ON COLUMN dbscodes.tb_saida.co_seq_saida IS 'Codigo de identificacão da entrada, incrementado pela sequence sq_saida_coseqsaida.';
COMMENT ON COLUMN dbscodes.tb_saida.co_compra_troca IS 'Chave estrangeira para a tabela dbscodes.tb_compra_troca.';
COMMENT ON COLUMN dbscodes.tb_saida.co_transferencia IS 'Chave estrangeira para a tabela dbscodes.tb_transferencia.';
COMMENT ON COLUMN dbscodes.tb_saida.ds_motivo IS 'Motivo da saida.';
COMMENT ON COLUMN dbscodes.tb_saida.dt_saida IS 'Data e hora da saída.';
COMMENT ON COLUMN dbscodes.tb_saida.co_usuario IS 'Chave estrangeira para a tabela dbscodes.tb_usuario.';
COMMENT ON COLUMN dbscodes.tb_saida.co_tipo_movimentacao IS 'Chave estrangeira para a tabela dbscodes.tb_tipo_movimentacao.';

--------------------------------------------------------
--  Constraints for Table tb_saida
--------------------------------------------------------
ALTER TABLE dbscodes.tb_saida ADD CONSTRAINT fk_compratroca_saida FOREIGN KEY(co_compra_troca)
    REFERENCES dbscodes.tb_compra_troca(co_seq_compratroca);

ALTER TABLE dbscodes.tb_saida ADD CONSTRAINT fk_transferencia_saida FOREIGN KEY(co_transferencia)
    REFERENCES dbscodes.tb_transferencia(co_seq_transferencia);

ALTER TABLE dbscodes.tb_saida ADD CONSTRAINT fk_usuario_saida FOREIGN KEY(co_usuario)
    REFERENCES dbscodes.tb_usuario(co_seq_usuario);

ALTER TABLE dbscodes.tb_saida ADD CONSTRAINT fk_tipomov_saida FOREIGN KEY(co_tipo_movimentacao)
    REFERENCES dbscodes.tb_tipo_movimentacao(co_seq_tipo_movimentacao);


--------------------------------------------------------
--  DDL for Table tb_natureza_despesa
--------------------------------------------------------

---AJUSTADO NOME DA PK DE pk_natureza_despesa PARA pk_naturezadespesa

CREATE SEQUENCE dbscodes.sq_natdespesa_coseqnatudespesa;
CREATE TABLE dbscodes.tb_natureza_despesa
(
    co_seq_natureza_despesa bigint NOT NULL DEFAULT nextval('dbscodes.sq_natdespesa_coseqnatudespesa'::regclass),
    nu_natureza_despesa bigint NOT NULL,
    ds_natureza_despesa character varying(800) NOT NULL,
    CONSTRAINT pk_naturezadespesa PRIMARY KEY (co_seq_natureza_despesa)
);

COMMENT ON TABLE dbscodes.tb_natureza_despesa IS 'Armazena dados de natureza de espesa.';
COMMENT ON COLUMN dbscodes.tb_natureza_despesa.co_seq_natureza_despesa IS 'Codigo de identificacão da natureza de espesa, incrementado pela sequence sq_natdespesa_coseqnatudespesa.';
COMMENT ON COLUMN dbscodes.tb_natureza_despesa.nu_natureza_despesa IS 'Número da natureza de espesa.';
COMMENT ON COLUMN dbscodes.tb_natureza_despesa.ds_natureza_despesa IS 'Descrição da natureza de despesa.';

--------------------------------------------------------
--  DDL for Table tb_tipo_cobranca
--------------------------------------------------------

---AJUSTADO NOME DA PK DE pk_natureza_despesa PARA pk_tipocobranca
--alterado campo na PK de co_seq_natureza_despesa para co_seq_tipo_cobranca
--Alterado o data type do campo st_tipo_cobraca estava com bigint

CREATE SEQUENCE dbscodes.sq_tpcobranca_coseqtpcobranca
CREATE TABLE dbscodes.tb_tipo_cobranca
(
    co_seq_tipo_cobranca bigint NOT NULL DEFAULT nextval('dbscodes.sq_tpcobranca_coseqtpcobranca'::regclass),
    st_tipo_cobranca boolean NOT NULL DEFAULT FALSE,
    ds_tipo_cobranca character varying(30) NOT NULL,
    CONSTRAINT pk_tipocobranca PRIMARY KEY (co_seq_tipo_cobranca)
);

COMMENT ON TABLE dbscodes.tb_tipo_cobranca IS 'Armazena dados dos tipos de cobranca.';
COMMENT ON COLUMN dbscodes.tb_tipo_cobranca.co_seq_tipo_cobranca IS 'Codigo de identificacão dos tipos de cobranca, incrementado pela sequence sq_tpcobranca_coseqtpcobranca.';
COMMENT ON COLUMN dbscodes.tb_tipo_cobranca.st_tipo_cobranca IS 'Status do tipo de cobrança.';
COMMENT ON COLUMN dbscodes.tb_tipo_cobranca.ds_tipo_cobranca IS 'Descrição tipo de cobrança.';
