CREATE SEQUENCE dbscodes.sq_tipodemanda_coseqtipodem;
CREATE TABLE dbscodes.tb_tipo_demanda (
    co_seq_tipo_demanda
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_tipodemanda_coseqtipodem'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_tipodemanda
            PRIMARY KEY,
    ds_tipo_demanda
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_tipo_demanda
    IS 'Registros de tipo demanda';
COMMENT ON COLUMN dbscodes.tb_tipo_demanda.co_seq_tipo_demanda
    IS 'Codigo de identificacão de tipo demanda, gerador pela sequence: dbscodes.sq_tipodemanda_coseqtipodem';
COMMENT ON COLUMN dbscodes.tb_tipo_demanda.ds_tipo_demanda
    IS 'Descrição de tipo demanda';

CREATE SEQUENCE dbscodes.sq_tipomodelo_coseqtipomod;
CREATE TABLE dbscodes.tb_tipo_modelo (
    co_seq_tipo_modelo
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_tipomodelo_coseqtipomod'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_tipomodelo
            PRIMARY KEY,
    ds_tipo_modelo
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_tipo_modelo
    IS 'Registros de tipo modelo';
COMMENT ON COLUMN dbscodes.tb_tipo_modelo.co_seq_tipo_modelo
    IS 'Codigo de identificacão de tipo modelo, gerador pela sequence: dbscodes.sq_tipomodelo_coseqtipomod';
COMMENT ON COLUMN dbscodes.tb_tipo_modelo.ds_tipo_modelo
    IS 'Descrição de tipo modelo';

CREATE TABLE dbscodes.rl_tipo_demanda_tipo_modelo (
    co_tipo_demanda
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipodemanda_tipodemtipomod
            REFERENCES dbscodes.tb_tipo_demanda(co_seq_tipo_demanda),
    co_tipo_modelo
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipomodelo_tipodemtipomod
            REFERENCES dbscodes.tb_tipo_modelo(co_seq_tipo_modelo),
    CONSTRAINT pk_tipodemandatipomodelo
        PRIMARY KEY(co_tipo_demanda, co_tipo_modelo)
);
COMMENT ON TABLE dbscodes.rl_tipo_demanda_tipo_modelo
    IS 'Tabela de relacionamento entre as tabelas de tipo de demanda e tipo de modelo.';
COMMENT ON COLUMN dbscodes.rl_tipo_demanda_tipo_modelo.co_tipo_demanda
    IS 'Codigo de identificacão do tipo de demanda, chave estrangeira para dbscodes.tb_tipo_demanda';
COMMENT ON COLUMN dbscodes.rl_tipo_demanda_tipo_modelo.co_tipo_modelo
    IS 'Codigo de identificacão do tipo de modelo, chave estrangeira para dbscodes.tb_tipo_modelo';

CREATE SEQUENCE dbscodes.sq_modelo_coseqmodelo;
CREATE TABLE dbscodes.tb_modelo (
    co_seq_modelo
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_modelo_coseqmodelo'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_modelo
            PRIMARY KEY,
    co_tipo_demanda
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipodemanda_modelo
            REFERENCES dbscodes.tb_tipo_demanda(co_seq_tipo_demanda),
    co_tipo_modelo
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipomodelo_modelo
            REFERENCES dbscodes.tb_tipo_modelo(co_seq_tipo_modelo),
    ds_descritor
        CHARACTER VARYING(80)
        NOT NULL,
    ds_modelo
        CLOB
        NOT NULL,
    st_inativo
        BOOLEAN
        DEFAULT false
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_modelo
    IS 'Registros de modelo ofício';
COMMENT ON COLUMN dbscodes.tb_modelo.co_seq_modelo
    IS 'Codigo de identificacão de modelo de ofício ou relatório técnico, gerador pela sequence: dbscodes.sq_modelo_coseqmodelo';
COMMENT ON COLUMN dbscodes.tb_modelo.co_tipo_demanda
    IS 'Codigo de identificacão do tipo de demanda, chave estrangeira para dbscodes.tb_tipo_demanda';
COMMENT ON COLUMN dbscodes.tb_modelo.co_tipo_modelo
    IS 'Codigo de identificacão do tipo de modelo, chave estrangeira para dbscodes.tb_tipo_modelo';
COMMENT ON COLUMN dbscodes.tb_modelo.ds_descritor
    IS 'Descritor do modelo.';
COMMENT ON COLUMN dbscodes.tb_modelo.ds_modelo
    IS 'Descrição do modelo.';
COMMENT ON COLUMN dbscodes.tb_modelo.st_inativo
    IS 'Indicativo se o modelo está inativo ou não.';


-------------------------------------------------------
--  DML for tb_tipo_demanda
-------------------------------------------------------

INSERT INTO dbscodes.tb_tipo_demanda (co_seq_tipo_demanda, ds_tipo_demanda)
VALUES
(1,'Administrativa'),
(2,'Judicial'),
(3,'Jefaz'),
(4,'Comissão de Farmacologia');
