CREATE SEQUENCE dbscodes.sq_categoria_coseqcategoria;
CREATE TABLE dbscodes.tb_categoria (
    co_seq_categoria
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_categoria_coseqcategoria'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_categoria
            PRIMARY KEY,
    no_categoria
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_categoria
    IS 'Registros de categoria';
COMMENT ON COLUMN dbscodes.tb_categoria.co_seq_categoria
    IS 'Codigo de identificacão de categoria, gerado pela sequence: dbscodes.sq_categoria_coseqcategoria';
COMMENT ON COLUMN dbscodes.tb_categoria.no_categoria
    IS 'Nome da categoria.';

CREATE SEQUENCE dbscodes.sq_tipo_controlado_coseqtpcont;
CREATE TABLE dbscodes.tb_tipo_controlado (
    co_seq_tipo_controlado
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_tipo_controlado_coseqtpcont'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_tipocontrolado
            PRIMARY KEY,
    no_tipo_controlado
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_tipo_controlado
    IS 'Registros de tipo controlado';
COMMENT ON COLUMN dbscodes.tb_tipo_controlado.co_seq_tipo_controlado
    IS 'Codigo de identificacão de tipo controlado, gerado pela sequence: dbscodes.sq_tipo_controlado_coseqtpcont';
COMMENT ON COLUMN dbscodes.tb_tipo_controlado.no_tipo_controlado
    IS 'Nome do tipo controlado.';

CREATE SEQUENCE dbscodes.sq_produto_coseqproduto;
CREATE TABLE dbscodes.tb_produto (
    co_seq_produto
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_produto_coseqproduto'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_produto
            PRIMARY KEY,
    co_categoria
        BIGINT
        NOT NULL
        CONSTRAINT fk_categoria_produto
            REFERENCES dbscodes.tb_categoria(co_seq_categoria),
    co_tipo_controlado
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipocontrolado_produto
            REFERENCES dbscodes.tb_tipo_controlado(co_seq_tipo_controlado),
    no_produto
        CHARACTER VARYING(200)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_produto
    IS 'Registros de produto';
COMMENT ON COLUMN dbscodes.tb_produto.co_seq_produto
    IS 'Codigo de identificacão de produto, gerado pela sequence: dbscodes.sq_produto_coseqproduto';
COMMENT ON COLUMN dbscodes.tb_produto.co_categoria
    IS 'Codigo de identificacão da categoria, chave estrangeira para dbscodes.tb_categoria';
COMMENT ON COLUMN dbscodes.tb_produto.co_tipo_controlado
    IS 'Codigo de identificacão do tipo controlado, chave estrangeira para dbscodes.tb_tipo_controlado';
COMMENT ON COLUMN dbscodes.tb_produto.no_produto
    IS 'Nome do produto.';

CREATE SEQUENCE dbscodes.sq_apresentacao_coseqapresent;
CREATE TABLE dbscodes.tb_apresentacao (
    co_seq_apresentacao
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_apresentacao_coseqapresent'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_apresentacao
            PRIMARY KEY,
    no_apresentacao
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_apresentacao
    IS 'Registros de apresentação';
COMMENT ON COLUMN dbscodes.tb_apresentacao.co_seq_apresentacao
    IS 'Codigo de identificacão de apresentação, gerado pela sequence: dbscodes.sq_apresentacao_coseqapresent';
COMMENT ON COLUMN dbscodes.tb_apresentacao.no_apresentacao
    IS 'Nome da apresentação.';

CREATE SEQUENCE dbscodes.sq_grupo_especificacao_coseqge;
CREATE TABLE dbscodes.tb_grupo_especificacao (
    co_seq_grupo_especificacao
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_grupo_especificacao_coseqge'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_grupoespecificacao
            PRIMARY KEY,
    co_categoria
        BIGINT
        NOT NULL
        CONSTRAINT fk_categoria_grupoespecif
            REFERENCES dbscodes.tb_categoria(co_seq_categoria),
    no_grupo_especificacao
        CHARACTER VARYING(50)
        NOT NULL,
    co_grupo_anatomico
        CHAR
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_grupo_especificacao
    IS 'Registros de grupo da especificacao';
COMMENT ON COLUMN dbscodes.tb_grupo_especificacao.co_seq_grupo_especificacao
    IS 'Codigo de identificacão de grupo da especificacao, gerado pela sequence: dbscodes.sq_grupo_especificacao_coseqge';
COMMENT ON COLUMN dbscodes.tb_grupo_especificacao.co_categoria
    IS 'Codigo de identificacão da categoria, chave estrangeira para dbscodes.tb_categoria';
COMMENT ON COLUMN dbscodes.tb_grupo_especificacao.no_grupo_especificacao
    IS 'Nome do grupo da especificacao.';
COMMENT ON COLUMN dbscodes.tb_grupo_especificacao.co_grupo_anatomico
    IS 'Codigo alfanumerico (letras) que obedecem ao codigo de grupos anatomicos (ATC).';

CREATE SEQUENCE dbscodes.sq_especificacao_coseqespecif;
CREATE TABLE dbscodes.tb_especificacao (
    co_seq_especificacao
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_especificacao_coseqespecif'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_especificacao
            PRIMARY KEY,
    co_produto
        BIGINT
        NOT NULL
        CONSTRAINT fk_produto_especificacao
            REFERENCES dbscodes.tb_produto(co_seq_produto),
    co_apresentacao
        BIGINT
        NOT NULL
        CONSTRAINT fk_apresentacao_especificacao
            REFERENCES dbscodes.tb_apresentacao(co_seq_apresentacao),
    co_grupo_especificacao
        BIGINT
        NOT NULL
        CONSTRAINT fk_grupoespecif_especificacao
            REFERENCES dbscodes.tb_grupo_especificacao(co_seq_grupo_especificacao),
    no_especificacao
        CHARACTER VARYING(500)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_especificacao
    IS 'Registros de especificação';
COMMENT ON COLUMN dbscodes.tb_especificacao.co_seq_especificacao
    IS 'Codigo de identificacão de especificação, gerado pela sequence: dbscodes.sq_especificacao_coseqespecif';
COMMENT ON COLUMN dbscodes.tb_especificacao.co_produto
    IS 'Codigo de identificacão do produto, chave estrangeira para dbscodes.tb_produto';
COMMENT ON COLUMN dbscodes.tb_especificacao.co_apresentacao
    IS 'Codigo de identificacão da apresentação, chave estrangeira para dbscodes.tb_apresentacao';
COMMENT ON COLUMN dbscodes.tb_especificacao.co_grupo_especificacao
    IS 'Codigo de identificacão do grupo, chave estrangeira para dbscodes.tb_grupo_especificacao';
COMMENT ON COLUMN dbscodes.tb_especificacao.no_especificacao
    IS 'Nome da especificação.';

CREATE SEQUENCE dbscodes.sq_marca_coseqmarca;
CREATE TABLE dbscodes.tb_marca (
    co_seq_marca
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_marca_coseqmarca'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_marca
            PRIMARY KEY,
    no_marca
        CHARACTER VARYING(100)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_marca
    IS 'Registros de marca';
COMMENT ON COLUMN dbscodes.tb_marca.co_seq_marca
    IS 'Codigo de identificacão de marca, gerado pela sequence: dbscodes.sq_marca_coseqmarca';
COMMENT ON COLUMN dbscodes.tb_marca.no_marca
    IS 'Nome da marca.';

CREATE SEQUENCE dbscodes.sq_programa_coseqprograma;
CREATE TABLE dbscodes.tb_programa (
    co_seq_programa
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_programa_coseqprograma'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_programa
            PRIMARY KEY,
    no_programa
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_programa
    IS 'Registros de programa';
COMMENT ON COLUMN dbscodes.tb_programa.co_seq_programa
    IS 'Codigo de identificacão de programa, gerado pela sequence: dbscodes.sq_programa_coseqprograma';
COMMENT ON COLUMN dbscodes.tb_programa.no_programa
    IS 'Nome do programa.';

CREATE SEQUENCE dbscodes.sq_tipo_item_coseqtipoitem;
CREATE TABLE dbscodes.tb_tipo_item (
    co_seq_tipo_item
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_tipo_item_coseqtipoitem'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_tipoitem
            PRIMARY KEY,
    no_tipo_item
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_tipo_item
    IS 'Registros de tipo do item';
COMMENT ON COLUMN dbscodes.tb_tipo_item.co_seq_tipo_item
    IS 'Codigo de identificacão de tipo do item, gerado pela sequence: dbscodes.sq_tipo_item_coseqtipoitem';
COMMENT ON COLUMN dbscodes.tb_tipo_item.no_tipo_item
    IS 'Nome do tipo do item.';

CREATE SEQUENCE dbscodes.sq_tipo_dispensacao_coseqtpdis;
CREATE TABLE dbscodes.tb_tipo_dispensacao (
    co_seq_tipo_dispensacao
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_tipo_dispensacao_coseqtpdis'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_tipodispensacao
            PRIMARY KEY,
    no_tipo_dispensacao
        CHARACTER VARYING(50)
        NOT NULL
);
COMMENT ON TABLE dbscodes.tb_tipo_dispensacao
    IS 'Registros de tipo de dispensação';
COMMENT ON COLUMN dbscodes.tb_tipo_dispensacao.co_seq_tipo_dispensacao
    IS 'Codigo de identificacão de tipo de dispensação, gerado pela sequence: dbscodes.sq_tipo_dispensacao_coseqtpdis';
COMMENT ON COLUMN dbscodes.tb_tipo_dispensacao.no_tipo_dispensacao
    IS 'Nome do tipo de dispensação.';

CREATE SEQUENCE dbscodes.sq_item_coseqitem;
CREATE TABLE dbscodes.tb_item (
    co_seq_item
        BIGINT
        DEFAULT NEXTVAL('dbscodes.sq_item_coseqitem'::REGCLASS)
        NOT NULL
        CONSTRAINT pk_item
            PRIMARY KEY,
    co_especificacao
        BIGINT
        NOT NULL
        CONSTRAINT fk_especificacao_item
            REFERENCES dbscodes.tb_especificacao(co_seq_especificacao),
    co_marca
        BIGINT
        NOT NULL
        CONSTRAINT fk_marca_item
            REFERENCES dbscodes.tb_marca(co_seq_marca),
    co_programa
        BIGINT
        NOT NULL
        CONSTRAINT fk_programa_item
            REFERENCES dbscodes.tb_programa(co_seq_programa),
    co_natureza_despesa
        BIGINT
        NULL
        CONSTRAINT fk_natdesp_item
            REFERENCES dbscodes.tb_natureza_despesa(co_seq_natureza_despesa),
    co_tipo_item
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipoitem_item
            REFERENCES dbscodes.tb_tipo_item(co_seq_tipo_item),
    co_tipo_dispensacao
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipodispensacao_dispensacao
            REFERENCES dbscodes.tb_tipo_dispensacao(co_seq_tipo_dispensacao),
    co_item
        CHARACTER VARYING(30)
        NULL,
    st_importado
        BOOLEAN
        DEFAULT false
        NOT NULL,
    dt_importado
        DATE
        NULL,
    co_siafisico
        CHARACTER VARYING(20)
        NULL,
    dt_revisado_siafisico
        DATE
        NULL,
    ds_siafisico
        CHARACTER VARYING(400)
        NULL,
    nu_minimo_dispensacao
        INT
        NOT NULL,
    nu_minimo_alerta
        INT
        NOT NULL,
    ds_observacao
        CHARACTER VARYING(500)
        NULL,
    nu_custo
        NUMERIC(10, 3)
        NULL
);
COMMENT ON TABLE dbscodes.tb_item
    IS 'Registros de item';
COMMENT ON COLUMN dbscodes.tb_item.co_seq_item
    IS 'Codigo de identificacão de item, gerado pela sequence: dbscodes.sq_item_coseqitem';
COMMENT ON COLUMN dbscodes.tb_item.co_especificacao
    IS 'Codigo de identificacão da especificação, chave estrangeira para dbscodes.tb_especificacao';
COMMENT ON COLUMN dbscodes.tb_item.co_marca
    IS 'Codigo de identificacão da marca, chave estrangeira para dbscodes.tb_marca';
COMMENT ON COLUMN dbscodes.tb_item.co_programa
    IS 'Codigo de identificacão da programa, chave estrangeira para dbscodes.tb_programa';
COMMENT ON COLUMN dbscodes.tb_item.co_natureza_despesa
    IS 'Codigo de identificacão da natureza despesa, chave estrangeira para dbscodes.tb_natureza_despesa';
COMMENT ON COLUMN dbscodes.tb_item.co_tipo_item
    IS 'Codigo de identificacão do tipo do item, chave estrangeira para dbscodes.tb_tipo_item';
COMMENT ON COLUMN dbscodes.tb_item.co_tipo_dispensacao
    IS 'Codigo de identificacão do tipo da dispensação, chave estrangeira para dbscodes.tb_tipo_dispensacao';
COMMENT ON COLUMN dbscodes.tb_item.co_item
    IS 'Código do item.';
COMMENT ON COLUMN dbscodes.tb_item.st_importado
    IS 'Indicativo se o item é importado ou não.';
COMMENT ON COLUMN dbscodes.tb_item.dt_importado
    IS 'Data de quando o item foi importado.';
COMMENT ON COLUMN dbscodes.tb_item.co_siafisico
    IS 'Código Siafisico do item.';
COMMENT ON COLUMN dbscodes.tb_item.dt_revisado_siafisico
    IS 'Data de quando o código Siafisico do item foi revisado.';
COMMENT ON COLUMN dbscodes.tb_item.ds_siafisico
    IS 'Descrição do código Siafisico do item.';
COMMENT ON COLUMN dbscodes.tb_item.nu_minimo_dispensacao
    IS 'Define quantidade minima de dias que o item pode ser dispensado.';
COMMENT ON COLUMN dbscodes.tb_item.nu_minimo_alerta
    IS 'Define quantidade minima de meses que o item pode ser alertado.';
COMMENT ON COLUMN dbscodes.tb_item.ds_observacao
    IS 'Descrição da observação do item.';
COMMENT ON COLUMN dbscodes.tb_item.nu_custo
    IS 'Custo do item. Formato 9999999.999 -> 9.999.999,999';

CREATE TABLE dbscodes.rl_tipo_demanda_item (
    co_tipo_demanda
        BIGINT
        NOT NULL
        CONSTRAINT fk_tipodemanda_tipodemtipoitem
            REFERENCES dbscodes.tb_tipo_demanda(co_seq_tipo_demanda),
    co_item
        BIGINT
        NOT NULL
        CONSTRAINT fk_item_tipodemtipoitem
            REFERENCES dbscodes.tb_item(co_seq_item),
    CONSTRAINT pk_tipodemandaitem
        PRIMARY KEY(co_tipo_demanda, co_item)
);
COMMENT ON TABLE dbscodes.rl_tipo_demanda_item
    IS 'Tabela de relacionamento entre as tabelas de tipo de demanda e item.';
COMMENT ON COLUMN dbscodes.rl_tipo_demanda_item.co_tipo_demanda
    IS 'Codigo de identificacão do tipo de demanda, chave estrangeira para dbscodes.tb_tipo_demanda';
COMMENT ON COLUMN dbscodes.rl_tipo_demanda_item.co_item
    IS 'Codigo de identificacão do item, chave estrangeira para dbscodes.tb_item';
