-------------------------------
-- DBSCODES.TB_FORNECEDOR
-------------------------------
ALTER TABLE dbscodes.tb_fornecedor RENAME COLUMN ds_descricao TO no_fornecedor;
ALTER TABLE dbscodes.tb_fornecedor ALTER COLUMN nu_cnpj TYPE CHARACTER VARYING(14);

COMMENT ON COLUMN dbscodes.tb_fornecedor.no_fornecedor IS 'Nome do fornecedor.';

-------------------------------
-- DBSCODES.TB_TIPO_MOVIMENTACAO
-------------------------------
COMMENT ON COLUMN dbscodes.tb_tipo_movimentacao.tp_tipo_movimentacao IS 'Tipo do tipo movimentação, onde: E = Entrada, S = Saida.';

-------------------------------
-- DBSCODES.TB_FABRICANTE
-------------------------------
CREATE SEQUENCE dbscodes.sq_fabricante_coseqfabricante;

CREATE TABLE dbscodes.tb_fabricante
(
    co_seq_fabricante BIGINT NOT NULL DEFAULT NEXTVAL('dbscodes.sq_fabricante_coseqfabricante'::regclass),
    no_fabricante CHARACTER VARYING(100) NOT NULL,
    nu_cnpj CHARACTER VARYING(14) NOT NULL,
    constraint PK_FABRICANTE PRIMARY KEY(co_seq_fabricante)
);

COMMENT ON TABLE  dbscodes.tb_fabricante IS 'Armazena dados do fabricante.';
COMMENT ON COLUMN dbscodes.tb_fabricante.co_seq_fabricante IS 'Codigo de identificacão do fabricante, incrementado pela sequence dbscodes.sq_fabricante_coseqfabricante.';
COMMENT ON COLUMN dbscodes.tb_fabricante.no_fabricante IS 'Nome do fabricante.';
COMMENT ON COLUMN dbscodes.tb_fabricante.nu_cnpj IS 'CNPJ do fabricante.';

-------------------------------
-- DBSCODES.TB_DOACAO
-------------------------------
ALTER TABLE dbscodes.tb_doacao RENAME COLUMN no_doacao TO ds_doacao;

ALTER TABLE dbscodes.tb_doacao RENAME COLUMN ds_cpf_cnpj_doacao TO nu_cpf_cnpj_doacao;
ALTER TABLE dbscodes.tb_doacao ALTER COLUMN nu_cpf_cnpj_doacao TYPE CHARACTER VARYING(14);

-------------------------------
-- DBSCODES.TB_COMPRA_TROCA
-------------------------------
ALTER TABLE dbscodes.tb_compra_troca ALTER COLUMN co_fornecedor DROP NOT NULL;
ALTER TABLE dbscodes.tb_compra_troca ALTER COLUMN co_compra_modalidade DROP NOT NULL;
ALTER TABLE dbscodes.tb_compra_troca ALTER COLUMN ds_nota_fiscal DROP NOT NULL;

-------------------------------
-- DBSCODES.TB_TRANSFERENCIA
-------------------------------
ALTER TABLE dbscodes.tb_transferencia ADD CONSTRAINT ck_transferencia_tipotransf CHECK (tp_transferencia = 1 OR tp_transferencia = 2)
COMMENT ON COLUMN dbscodes.tb_transferencia.tp_transferencia IS 'Tipo de transferência, onde 1 = Entrada e 2 = Saída.';

--------------------------------------------------------
-- DBSCODES.TB_ENTRADA_ESTOQUE
--------------------------------------------------------

DROP SEQUENCE dbscodes.sq_entrada_coseqentrada;
DROP TABLE dbscodes.tb_entrada;

CREATE SEQUENCE dbscodes.sq_entradaestoque_coseqentrada;

CREATE TABLE dbscodes.tb_entrada_estoque
(
    co_seq_entrada_estoque bigint NOT NULL DEFAULT nextval('dbscodes.sq_entradaestoque_coseqentrada'::regclass),
    dt_entrada timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    co_usuario bigint NOT NULL,
    co_tipo_movimentacao bigint NOT NULL,
    co_compra_troca bigint NULL,
    co_doacao bigint NULL,
    co_transferencia bigint NULL,
    ds_motivo character varying(200) NULL,
    CONSTRAINT pk_entradaestoque PRIMARY KEY (co_seq_entrada_estoque),
	CONSTRAINT fk_usuario_entradaestoque FOREIGN KEY(co_usuario) REFERENCES dbscodes.tb_usuario(co_seq_usuario),
	CONSTRAINT fk_tipomoviment_entradaestoque FOREIGN KEY(co_tipo_movimentacao) REFERENCES dbscodes.tb_tipo_movimentacao(co_seq_tipo_movimentacao),
	CONSTRAINT fk_compratroca_entradaestoque FOREIGN KEY(co_compra_troca) REFERENCES dbscodes.tb_compra_troca(co_seq_compra_troca),
	CONSTRAINT fk_doacao_entradaestoque FOREIGN KEY(co_doacao) REFERENCES dbscodes.tb_doacao(co_seq_doacao),
	CONSTRAINT fk_transferencia_entradaestoque FOREIGN KEY(co_transferencia) REFERENCES dbscodes.tb_transferencia(co_seq_transferencia)
);

COMMENT ON TABLE  dbscodes.tb_entrada_estoque IS 'Armazena dados de entrada no estoque.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.co_seq_entrada_estoque IS 'Codigo de identificacão da entrada, incrementado pela sequence dbscodes..sq_entradaestoque_coseqentrada.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.dt_entrada IS 'Data e hora da entrada.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.co_usuario IS 'Chave estrangeira para a tabela dbscodes.tb_usuario.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.co_tipo_movimentacao IS 'Chave estrangeira para a tabela dbscodes.tb_tipo_movimentacao.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.co_compra_troca IS 'Chave estrangeira para a tabela dbscodes.tb_compra_troca.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.co_doacao IS 'Chave estrangeira para a tabela dbscodes.tb_doacao.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.co_transferencia IS 'Chave estrangeira para a tabela dbscodes.tb_transferencia.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque.ds_motivo IS 'Motivo da entrada.';

--------------------------------------------------------
-- DBSCODES.TB_ENTRADA_ESTOQUE_ITEM
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_entradaestoitem_coseqentraaestoitem;

CREATE TABLE dbscodes.tb_entrada_estoque_item (
	co_seq_entrada_estoque_item bigint not null DEFAULT nextval('dbscodes.sq_entradaestoitem_coseqentraaestoitem'::regclass),
	co_entrada_estoque bigint not null,
	co_item bigint not null,
	co_unidade_dispensadora bigint not null,
	dt_entrada_item timestamp not null default current_timestamp,
	qt_item bigint not null,
	st_termolabel boolean not null,
	co_fabricante bigint not null,
	nu_valor_item numeric(10,2) null,
	qt_acerto bigint null,
	CONSTRAINT pk_entradaestoitem PRIMARY KEY (co_seq_entrada_estoque_item),
    CONSTRAINT fk_entradaesto_entradaestoitem FOREIGN KEY(co_entrada_estoque) REFERENCES dbscodes.tb_entrada_estoque(co_seq_entrada_estoque),
    CONSTRAINT fk_item_entradaestoitem FOREIGN KEY(co_item) REFERENCES dbscodes.tb_item(co_seq_item),
    CONSTRAINT fk_unidadedisp_entradaestoitem FOREIGN KEY(co_unidade_dispensadora) REFERENCES dbscodes.tb_unidade_dispensadora(co_seq_unidade_dispensadora),
	CONSTRAINT fk_fabricante_entradaestoitem FOREIGN KEY(co_fabricante) REFERENCES dbscodes.tb_fabricante(co_seq_fabricante),
	CONSTRAINT ck_entradaestoitem_termolabel CHECK (st_termolabel::text = 'true'::text OR st_termolabel::text = 'false'::text)
);

COMMENT ON TABLE  dbscodes.tb_entrada_estoque_item IS 'Armazena dados da entrada de itens no estoque.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.co_seq_entrada_estoque_item IS 'Codigo de identificacão da entrada do item, incrementado pela sequence dbscodes.sq_entradaestoitem_coseqentraaestoitem.';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.co_entrada_estoque IS  'Chave estrageira da tabela dbscodes.tb_entrada_estoque';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.co_item IS  'Chave estrageira da tabela dbscodes.tb_item';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.co_unidade_dispensadora IS  'Chave estrageira da tabela dbscodes.tb_unidade_dispensadora';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.dt_entrada_item IS  'Data e hora da entrada do item';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.qt_item IS  'Quantidade de itens na entrada';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.st_termolabel IS 'Se é o item é termolábel, onde true = sim e false = não';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.co_fabricante IS 'Chave estrageira da tabela dbscodes.tb_fabricante';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.nu_valor_item IS 'Valor do item';
COMMENT ON COLUMN dbscodes.tb_entrada_estoque_item.qt_acerto IS 'Quantidade de acerto da embalagem';

--------------------------------------------------------
-- DBSCODES.TB_SAIDA_ESTOQUE
--------------------------------------------------------

DROP SEQUENCE dbscodes.sq_saida_coseqsaida;
DROP TABLE dbscodes.tb_saida;

CREATE SEQUENCE dbscodes.sq_saidaestoque_coseqsaidaestoque;

CREATE TABLE dbscodes.tb_saida_estoque
(
    co_seq_saida_estoque bigint NOT NULL DEFAULT nextval('dbscodes.sq_saidaestoque_coseqsaidaestoque'::regclass),
    dt_saida timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    co_usuario bigint NOT NULL,
    co_tipo_movimentacao bigint NOT NULL,
    co_compra_troca bigint NULL,
    co_transferencia bigint NULL,
    ds_motivo character varying(200) NULL,
    CONSTRAINT pk_saidaestoque PRIMARY KEY (co_seq_saida_estoque),
	CONSTRAINT fk_usuario_saidaestoque FOREIGN KEY(co_usuario) REFERENCES dbscodes.tb_usuario(co_seq_usuario),
	CONSTRAINT fk_tipomoviment_saidaestoque FOREIGN KEY(co_tipo_movimentacao) REFERENCES dbscodes.tb_tipo_movimentacao(co_seq_tipo_movimentacao),
	CONSTRAINT fk_compratroca_saidaestoque FOREIGN KEY(co_compra_troca) REFERENCES dbscodes.tb_compra_troca(co_seq_compra_troca),
	CONSTRAINT fk_transferencia_saidaestoque FOREIGN KEY(co_transferencia) REFERENCES dbscodes.tb_transferencia(co_seq_transferencia)
);

COMMENT ON TABLE  dbscodes.tb_saida_estoque IS 'Armazena dados de saida no estoque.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque.co_seq_saida_estoque IS 'Codigo de identificacão da saida, incrementado pela sequence dbscodes.sq_saidaestoque_coseqsaidaestoque.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque.dt_saida IS 'Data e hora da saida de estoque.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque.co_usuario IS 'Chave estrangeira para a tabela dbscodes.tb_usuario.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque.co_tipo_movimentacao IS 'Chave estrangeira para a tabela dbscodes.tb_tipo_movimentacao.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque.co_compra_troca IS 'Chave estrangeira para a tabela dbscodes.tb_compra_troca.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque.co_transferencia IS 'Chave estrangeira para a tabela dbscodes.tb_transferencia.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque.ds_motivo IS 'Motivo da saida.';

--------------------------------------------------------
-- DBSCODES.TB_SAIDA_ESTOQUE_ITEM
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_saidaestoitem_coseqsaidaestoqueitem;

CREATE TABLE dbscodes.tb_saida_estoque_item (
	co_seq_saida_estoque_item bigint not null DEFAULT nextval('dbscodes.sq_saidaestoitem_coseqsaidaestoqueitem'::regclass),
	co_saida_estoque bigint not null,
	co_item bigint not null,
	co_unidade_dispensadora bigint not null,
	dt_saida_item timestamp not null default current_timestamp,
	qt_item bigint not null,
	CONSTRAINT pk_saidaestoitem PRIMARY KEY (co_seq_saida_estoque_item),
    CONSTRAINT fk_saidaesto_saidaestoitem FOREIGN KEY(co_saida_estoque) REFERENCES dbscodes.tb_saida_estoque(co_seq_saida_estoque),
    CONSTRAINT fk_item_saidaestoitem FOREIGN KEY(co_item) REFERENCES dbscodes.tb_item(co_seq_item),
    CONSTRAINT fk_unidadedisp_saidaestoitem FOREIGN KEY(co_unidade_dispensadora) REFERENCES dbscodes.tb_unidade_dispensadora(co_seq_unidade_dispensadora)
);

COMMENT ON TABLE  dbscodes.tb_saida_estoque_item IS 'Armazena dados da saida de itens do estoque.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque_item.co_seq_saida_estoque_item IS 'Codigo de identificacão da saida do item, incrementado pela sequence dbscodes.sq_saidaestoitem_coseqsaidaestoqueitem.';
COMMENT ON COLUMN dbscodes.tb_saida_estoque_item.co_saida_estoque IS 'Chave estrageira da tabela dbscodes.tb_saida_estoque';
COMMENT ON COLUMN dbscodes.tb_saida_estoque_item.co_item IS 'Chave estrageira da tabela dbscodes.tb_item';
COMMENT ON COLUMN dbscodes.tb_saida_estoque_item.co_unidade_dispensadora IS 'Chave estrageira da tabela dbscodes.tb_unidade_dispensadora';
COMMENT ON COLUMN dbscodes.tb_saida_estoque_item.dt_saida_item IS 'Data e hora da saida do item';
COMMENT ON COLUMN dbscodes.tb_saida_estoque_item.qt_item IS 'Quantidade de itens na saida';

--------------------------------------------------------
-- DBSCODES.TB_ESTOQUE_AUXILIAR
--------------------------------------------------------

CREATE TABLE dbscodes.tb_estoque_auxiliar (
    co_item bigint not null,
    co_unidade_dispensadora bigint not null,
    qt_item bigint not null,
    dt_atualizacao timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
    constraint pk_estoqueaux primary key(co_unidade_dispensadora, co_item),
    constraint fk_item_estoqueaux foreign key(co_item) references dbscodes.tb_item(co_seq_item),
    constraint fk_unidadedisp_estoqueaux foreign key(co_unidade_dispensadora) references dbscodes.tb_unidade_dispensadora(co_seq_unidade_dispensadora)
);

COMMENT ON TABLE  dbscodes.tb_estoque_auxiliar IS 'Armazena dados auxiliares do estoque e item';
COMMENT ON COLUMN dbscodes.tb_estoque_auxiliar.co_item IS 'codigo do item e chave estrageira da tabela dbscodes.tb_item';
COMMENT ON COLUMN dbscodes.tb_estoque_auxiliar.co_unidade_dispensadora IS 'codigo da unidade dispensadora e chave estrageira da tabela dbscodes.tb_unidade_dispensadora';
COMMENT ON COLUMN dbscodes.tb_estoque_auxiliar.qt_item IS 'Quantidade do item em estoque';
COMMENT ON COLUMN dbscodes.tb_estoque_auxiliar.dt_atualizacao IS 'Data e hora da ultima atualizacao';

--------------------------------------------------------
-- DBSCODES.TB_MOVIMENTACAO
--------------------------------------------------------
CREATE SEQUENCE dbscodes.sq_movimentacao_coseqmovimentacao;

CREATE TABLE dbscodes.tb_movimentacao (
	co_seq_movimentacao bigint NOT NULL DEFAULT nextval('dbscodes.sq_movimentacao_coseqmovimenta'::regclass),
	dt_movimentacao timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	co_item bigint not null,
	co_unidade_dispensadora bigint not null,
	co_tipo_movimentacao bigint not null,
	qt_movimentacao bigint not null,
	qt_movimentacao_total bigint not null,
	co_entrada_estoque bigint null,
	co_saida_estoque bigint null,
    CONSTRAINT pk_movimentacao PRIMARY KEY (co_seq_movimentacao),
	CONSTRAINT fk_item_movimentacao FOREIGN KEY(co_item) REFERENCES dbscodes.tb_item(co_seq_item),
	CONSTRAINT fk_unidadedispens_movimentacao FOREIGN KEY(co_unidade_dispensadora) REFERENCES dbscodes.tb_unidade_dispensadora(co_seq_unidade_dispensadora),
	CONSTRAINT fk_tipomovimenta_movimentacao FOREIGN KEY(co_tipo_movimentacao) REFERENCES dbscodes.tb_tipo_movimentacao(co_seq_tipo_movimentacao),
	CONSTRAINT fk_entradaestoque_movimentacao FOREIGN KEY(co_entrada_estoque) REFERENCES dbscodes.tb_entrada_estoque(co_seq_entrada_estoque),
	CONSTRAINT fk_saidaestoque_movimentacao FOREIGN KEY(co_saida_estoque) REFERENCES dbscodes.tb_saida_estoque(co_seq_saida_estoque)
);

COMMENT ON TABLE  dbscodes.tb_movimentacao IS 'Armazena dados das movimentações no estoque.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.co_seq_movimentacao IS 'Codigo de identificacão da movimentação, incrementado pela sequence dbscodes.sq_movimentacao_coseqmovimenta.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.dt_movimentacao IS 'Data e hora da movimentação.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.co_item IS 'Chave estrangeira para a tabela dbscodes.tb_item.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.co_unidade_dispensadora IS 'Chave estrangeira para a tabela dbscodes.tb_unidade_dispensadora.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.co_tipo_movimentacao IS 'Chave estrangeira para a tabela dbscodes.tb_tipo_movimentacao.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.qt_movimentacao IS 'Quantidade de itens movimentados.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.qt_movimentacao_total IS 'Quantidade total de itens em estoque.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.co_entrada_estoque IS 'Chave estrangeira para a tabela dbscodes.tb_entrada_estoque.';
COMMENT ON COLUMN dbscodes.tb_movimentacao.co_saida_estoque IS 'Chave estrangeira para a tabela dbscodes.tb_saida_estoque.';

