CREATE SEQUENCE dbscodes.sq_inventario_coseqinventario;

CREATE TABLE dbscodes.tb_inventario (
    co_seq_inventario bigint NOT NULL DEFAULT nextval('dbscodes.sq_inventario_coseqinventario'::regclass),
    ds_inventario character varying(60) NOT NULL,
	dt_cadastro timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
	co_usuario bigint NOT NULL,
	co_unidade_dispensadora bigint NOT NULL,
    st_ativo boolean NOT NULL DEFAULT false,
    CONSTRAINT pk_inventario PRIMARY KEY (co_seq_inventario),
    CONSTRAINT fk_uniddispensa_inventario FOREIGN KEY(co_unidade_dispensadora) REFERENCES dbscodes.tb_unidade_dispensadora(co_seq_unidade_dispensadora),
    CONSTRAINT fk_usuario_inventario FOREIGN KEY(co_usuario) REFERENCES dbscodes.tb_usuario(co_seq_usuario),
	CONSTRAINT ck_inventario_stativo CHECK (st_ativo::text = 'true'::text OR st_ativo::text = 'false'::text)
);

COMMENT ON TABLE  dbscodes.tb_inventario IS 'Armazena dados do Inventário.';
COMMENT ON COLUMN dbscodes.tb_inventario.co_seq_inventario IS 'Codigo de Identifição do inventário, incrementado pela sequence dbscodes.sq_inventario_coseqinventario';
COMMENT ON COLUMN dbscodes.tb_inventario.ds_inventario IS 'Descrição do inventário.';
COMMENT ON COLUMN dbscodes.tb_inventario.dt_cadastro IS 'Data de cadastro do inventário.';
COMMENT ON COLUMN dbscodes.tb_inventario.co_usuario IS 'Chave estrangeira para a tabela dbscodes.tb_usuario';
COMMENT ON COLUMN dbscodes.tb_inventario.co_unidade_dispensadora IS 'Chave estrangeira para a tabela dbscodes.tb_unidade_dispensadora';
COMMENT ON COLUMN dbscodes.tb_inventario.st_ativo IS 'Indica inventário ativo, onde: true = ativo e false = inativo';

--------------------------------------------------------
-- DBSCODES.TB_INVENTARIO_ITEM
--------------------------------------------------------

CREATE SEQUENCE dbscodes.sq_inventitem_coseqinventitem;

CREATE TABLE dbscodes.tb_inventario_item (
    co_seq_inventario_item bigint NOT NULL DEFAULT nextval('dbscodes.sq_inventitem_coseqinventitem'::regclass),
	co_inventario bigint NOT NULL,
	co_item bigint NOT NULL,
	qt_inventario_item bigint NOT NULL,
	dt_contagem timestamp NOT NULL,
	st_alterado boolean NOT NULL DEFAULT false,
    qt_compra_entrada bigint null,
    qt_outra_entrada bigint null,
    qt_perda_saida bigint null,
    qt_transferencia_saida bigint null,
    qt_dispensacao_saida bigint null,
    qt_estorno_entrada bigint null,
    qt_recibo_invalidado_entrada bigint null,
    qt_outra_saida bigint null,
    qt_perda_vencimento_saida bigint null,
    qt_quebra_saida bigint null,
    qt_ajuste_saida bigint null,
    qt_estoque_data_contagem bigint null,
    qt_item_pos_calculos bigint null,
	qt_item_diferenca bigint null,
	vl_item numeric(10,2) null,
    CONSTRAINT pk_inventarioitem PRIMARY KEY (co_seq_inventario_item),
    CONSTRAINT fk_inventario_inventarioitem FOREIGN KEY(co_inventario) REFERENCES dbscodes.tb_inventario(co_seq_inventario),
    CONSTRAINT fk_item_inventarioitem FOREIGN KEY(co_item) REFERENCES dbscodes.tb_item(co_seq_item),
    CONSTRAINT uk_inventarioitem_inventarioitem UNIQUE(co_inventario, co_item),
    CONSTRAINT ck_inventarioitem_stalterado CHECK (st_alterado::text = 'true'::text OR st_alterado::text = 'false'::text)
);

COMMENT ON TABLE  dbscodes.tb_inventario_item IS 'Armazena dados do itens do vinculado ao inventário.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.co_seq_inventario_item IS 'Codigo de Identifição do item do inventário, incrementado pela sequence dbscodes.sq_inventitem_coseqinventitem';
COMMENT ON COLUMN dbscodes.tb_inventario_item.co_inventario IS 'Chave estrangeira para a tabela dbscodes.tb_inventario';
COMMENT ON COLUMN dbscodes.tb_inventario_item.co_item IS 'Chave estrangeira para a tabela dbscodes.tb_item';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_inventario_item IS 'Quantidade de itens no inventario';
COMMENT ON COLUMN dbscodes.tb_inventario_item.dt_contagem IS 'Data e hora da contagem';
COMMENT ON COLUMN dbscodes.tb_inventario_item.st_alterado IS 'Se o item foi alterado, true para sim e false para não.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_compra_entrada IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo compra entrada.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_outra_entrada IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo outra entrada.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_perda_saida IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo perda saida.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_transferencia_saida IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo transferência saida.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_dispensacao_saida IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo dispensação saida.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_estorno_entrada IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo estorno entrada.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_recibo_invalidado_entrada IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo recibo invalidado entrada.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_outra_saida IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo outra saida.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_perda_vencimento_saida IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo perda de vencimento saida.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_quebra_saida IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo quebra saida.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_ajuste_saida IS 'Após inventário consolidado armazena a quantidade de movimentações do tipo ajustes saida.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_estoque_data_contagem IS 'Após inventário consolidado armazena a quantidade de itens no estoque na data da contagem.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_item_pos_calculos IS 'Após inventário consolidado armazena a quantidade de itens após entradas e saídas totalizadas.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.qt_item_diferenca IS 'Após inventário consolidado armazena a diferença da quantidade de itens após entradas e saídas totalizadas.';
COMMENT ON COLUMN dbscodes.tb_inventario_item.vl_item IS 'Após inventário consolidado armazena o valor do item da última entrada realizada.';
