package br.gov.datasus.scodes.acl;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.web.servlet.support.SpringBootServletInitializer;
import org.springframework.cloud.openfeign.EnableFeignClients;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.core.env.Environment;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;

import java.net.InetAddress;
import java.net.UnknownHostException;

@EnableFeignClients
@SpringBootApplication
@EntityScan(basePackages = {"br.gov.datasus.scodes.acl.model.entity" })
@EnableJpaRepositories(basePackages = {"br.gov.datasus.scodes.acl.repository", "br.gov.datasus.api.core.dynamicquery.repository" })
@ComponentScan(value = { "br.gov.datasus" })
public class ApplicationBoot extends SpringBootServletInitializer {

    private static final Logger LOG = LoggerFactory.getLogger(ApplicationBoot.class);

    public static void main(String[] args) {
        ConfigurableApplicationContext ctx = SpringApplication.run(ApplicationBoot.class, args);
        try {
            // init
            Environment env = ctx.getEnvironment();

            LOG.info("\n\n *** \n\n" + "\tAplicacao {} iniciada com sucesso!\n" + "\tDisponivel nos enderecos:\n"
                    + "\tLocal: http://localhost:{}\n" + "\tExterno: http://{}:{}\n" + "\tSwagger Url: http://{}:{}\n"
                    + "\tLocal Swagger Url: http://localhost:{}\n" + "\n *** \n\n",
                    env.getProperty("spring.application.name"),
                    env.getProperty("server.port") + env.getProperty("server.servlet.contextPath"),
                    InetAddress.getLocalHost().getHostAddress(),
                    env.getProperty("server.port") + env.getProperty("server.servlet.contextPath"),
                    InetAddress.getLocalHost().getHostAddress(),
                    env.getProperty("server.port") + env.getProperty("server.servlet.contextPath") + "/swagger-ui.html",
                    env.getProperty("server.port") + env.getProperty("server.servlet.contextPath")
                            + "/swagger-ui.html");

        } catch (UnknownHostException e) {
            LOG.error("Falha ao executar aplicacao: {}", e);
            ctx.close();
        }
    }
}
