package br.gov.datasus.scodes.acl.api;

import br.gov.datasus.scodes.acl.model.dto.IdentificacaoUsuarioDTO;
import br.gov.datasus.scodes.acl.model.dto.UsuarioDTO;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RequestMapping(value = "/usuario")
public interface UsuarioApi {

    @GetMapping(path = "/identificacao", produces = MediaType.APPLICATION_JSON_VALUE)
    ResponseEntity<IdentificacaoUsuarioDTO> obterIdentificacaoUsuario(
        @RequestParam(name = "id", required = false) Long id,
        @RequestParam(name = "email", required = false) String email,
        @RequestParam(name = "cpf", required = false) String cpf);

    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    ResponseEntity<UsuarioDTO> obterUsuarioCompleto(
        @RequestParam(name = "id", required = false) Long id,
        @RequestParam(name = "email", required = false) String email,
        @RequestParam(name = "cpf", required = false) String cpf);

    @ApiOperation(value = "Obter lista de usuarios por sistema, perfil e esfera")
    @GetMapping(path = "/lista", produces = MediaType.APPLICATION_JSON_VALUE)
    ResponseEntity<List<IdentificacaoUsuarioDTO>> obterUsuariosPorSistemaPerfilEsfera(
        @RequestParam(name = "siglaSistema", required = true) String siglaSistema,
        @RequestParam(name = "siglaPerfil", required = true) String siglaPerfil,
        @RequestParam(name = "codigoEsferaAtuacao", required = true) String codigoEsferaAtuacao
    );

    @ApiOperation(value = "Obter lista de CPFs por sistema ou sistema e perfil ou sistema, perfil e esfera")
    @GetMapping(path = "/cpf", produces = MediaType.APPLICATION_JSON_VALUE)
    ResponseEntity<List<String>> obterCpfsPorSistemaPerfilEsfera(
        @RequestParam(name = "siglaSistema", required = true) String siglaSistema,
        @RequestParam(name = "siglaPerfil", required = false) String siglaPerfil,
        @RequestParam(name = "codigoEsferaAtuacao", required = false) String codigoEsferaAtuacao
    );

    @ApiOperation(value = "Autenticação do usuário logado.")
    @PostMapping(path = "/autenticacao", produces = MediaType.APPLICATION_JSON_VALUE)
    ResponseEntity<Boolean> autenticarUsuario(
        @RequestBody String hash
    );
}
