package br.gov.datasus.scodes.acl.model.enums;

import br.gov.datasus.api.core.model.annotations.Dominio;
import com.fasterxml.jackson.annotation.JsonValue;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;


@Dominio("Informar descrição")
@JsonSerialize
@AllArgsConstructor
public enum SituacaoAcessoEnum implements Serializable {
    REJEITADO("REJ", "Rejeitado"),
    PARTICIPA("SIM", "Participa no sistema"),
    NAO_PARTICIPA_BLOQUEADO("NAO", "Não Participa no sistema/bloqueado"),
    PENDENTE("PEN", "Pendente"),
    REJEITADO_CHEFIA_IMEDIATA("RCH", "Rejeitado pela chefia imediata"),
    ACEITO_CHEFIA_IMEDIATA("ACH", "Aceito pela chefia imediata"),
    PENDENTE_CHEFIA_IMEDIATA("PCH", "Pendente pela chefia imediata");

	@Getter
	@JsonValue
	private String codigo;

	@Getter
	private String descricao;

	public static Optional<SituacaoAcessoEnum> obterPorCodigo(final String codigo) {
		return SituacaoAcessoEnum.stream().filter(e -> e.getCodigo().equals(codigo)).findFirst();
	}

	public static Optional<SituacaoAcessoEnum> obterPorDescricao(final String descricao) {
		return SituacaoAcessoEnum.stream().filter(e -> e.getDescricao().equals(descricao)).findFirst();
	}

	public static Stream<SituacaoAcessoEnum> stream() {
		return Stream.of(SituacaoAcessoEnum.values());
	}

	public static List<SituacaoAcessoEnum> list() {
		return Arrays.asList(SituacaoAcessoEnum.values());
	}
}
