package br.gov.datasus.scodes.acl.service;

import br.gov.datasus.api.core.model.dto.acesso.*;
import br.gov.datasus.scodes.acl.model.dto.*;
import br.gov.datasus.scodes.acl.model.mapper.acesso.*;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

@Service
@AllArgsConstructor
public class AcessoService {

	private final UsuarioAcessoMapper mapperUsuario;
	private final PerfilAcessoMapper mapperPerfil;
	private final EsferaAtuacaoAcessoMapper mapperEsfera;
	private final PermissaoAcessoMapper mapperPermissao;
	private final FuncionalidadeAcessoMapper mapperFuncionalidade;
	private final SistemaAcessoMapper mapperSistema;

	private final UsuarioService usuarioService;
	private final PerfilService perfilService;
	private final EsferaService esferaService;
	private final PermissaoService permissaoService;
	private final FuncionalidadeService funcionalidadeService;
	private final SistemaService sistemaService;

	public UsuarioAcessoDTO obterUsuario(UserInfoDTO userInfo) {
		UsuarioDTO usuarioDto = usuarioService.obterUsuario(userInfo);
		return mapperUsuario.convert(usuarioDto);
	}

	public List<PerfilAcessoDTO> obterPerfil(UserInfoDTO userInfo, String siglaSistema) {
		UsuarioAcessoDTO usuario = obterUsuario(userInfo);
		List<PerfilDTO> perfilsSCPA = perfilService.obterPorUsuarioSistema(usuario.getId(), siglaSistema);
		List<PerfilAcessoDTO> perfis = mapperPerfil.convert(perfilsSCPA);
		for (PerfilAcessoDTO perfilAcessoDTO : perfis) {
			List<EsferaAtuacaoDTO> esferasSCPA = esferaService.obterPorUsuarioSistemaPerfil(usuario.getId(),
					siglaSistema, perfilAcessoDTO.getSigla());
			List<EsferaAtuacaoAcessoDTO> esferas = mapperEsfera.convert(esferasSCPA);
			if (!esferas.isEmpty()) {
				perfilAcessoDTO.getEsferas().addAll(esferas);
			}
		}
		return perfis;
	}

	public List<MenuAcessoDTO> obterMenu(String siglaSistema, String siglaPerfil) {
		PermissaoDTO permissaoSCPA = permissaoService.obterArvorePermissoes(siglaSistema, siglaPerfil);
		PermissaoAcessoDTO permissao = mapperPermissao.convert(permissaoSCPA);
		List<MenuAcessoDTO> menus = new ArrayList<>();
		montarMenu(permissao, menus);
		return menus;
	}

	private boolean montarMenu(PermissaoAcessoDTO permissao, List<MenuAcessoDTO> menus) {
		boolean addItem = false;

		List<PermissaoAcessoDTO> permList = permissao.getPermissoes();

		Collections.sort(permList, (PermissaoAcessoDTO p1, PermissaoAcessoDTO p2) -> {
			if (p1.getOrdem().longValue() == p2.getOrdem().longValue()) {
				return 0;
			}
			return p1.getOrdem() < p2.getOrdem() ? -1 : 1;
		});

		for (PermissaoAcessoDTO perm : permList) {
			MenuAcessoDTO menuItem = new MenuAcessoDTO();
			addItem = montarMenu(perm, menuItem.getItens());
			if (addItem) {
				menuItem.setDescricao(perm.getPermissao());
				if (perm.getFuncionalidade() != null) {
					menuItem.setPath(perm.getFuncionalidade().getPath());
				}
				menus.add(menuItem);
			}
		}

		if (permissao.getFuncionalidade() != null && permissao.getFuncionalidade().getExibeMenu()
				&& !permissao.getFuncionalidade().getBloqueado()) {
			addItem = true;
		}
		return addItem;
	}

	public List<FuncionalidadeAcessoDTO> obterFuncionaliade(String siglaSistema, String siglaPerfil) {
		List<FuncionalidadeDTO> funcionalidadeSCPA = funcionalidadeService
				.obterDesbloqueadosPorSistemaPerfil(siglaSistema, siglaPerfil);
		List<FuncionalidadeAcessoDTO> funcionalidades = mapperFuncionalidade.convert(funcionalidadeSCPA);
		return funcionalidades;
	}

	public List<SistemaAcessoDTO> obterSistemas(UserInfoDTO userInfo) {
		UsuarioDTO usuarioDto = usuarioService.obterUsuario(userInfo);
		return mapperSistema.convert(sistemaService.obterSistemasUsuario(usuarioDto.getId()));
	}

}
