package br.gov.datasus.scodes.acl.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.acl.model.dto.SistemaDTO;
import br.gov.datasus.scodes.acl.model.entity.Sistema;
import br.gov.datasus.scodes.acl.model.mapper.SistemaMapper;
import br.gov.datasus.scodes.acl.repository.SistemaRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class SistemaService {

    private final SistemaRepository repository;
    private final SistemaMapper mapper;

    public SistemaDTO obterPorCodigo(Long id) {
        Optional<Sistema> one = repository.findById(id);
        Sistema entity = one.orElseThrow(() -> new ResourceNotFoundException());
        return mapper.toDto(entity);
    }

    public List<SistemaDTO> obterSistemasUsuario(Long idUsuario) {
        Iterable<Sistema> itens = repository.obterSistemasUsuario(idUsuario);
        return mapper.toDto(itens);
    }

    public PageableResponseModel<SistemaDTO> obterTodos(PageableRequestModel<?> pageableRequestModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableRequestModel, "sistema");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<Sistema> pageableResponseModel = repository.findAll(Sistema.OBTER_PAGINADO, parameters,
            pagination, Sistema.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public SistemaDTO obterPorIdSigla(Optional<Long> id, Optional<String> sigla) {
        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("id", id.orElse(null));
        parameters.append("sigla", sigla.orElse(null));
        Sistema entity = repository.findOne(Sistema.OBTER_POR_ID_SIGLA, parameters, Sistema.class);
        if (entity == null) {
            throw new ResourceNotFoundException();
        }
        return mapper.toDto(entity);
    }

}
