package br.gov.datasus.scodes.acl.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.scodes.acl.model.dto.IdentificacaoUsuarioDTO;
import br.gov.datasus.scodes.acl.model.dto.UserInfoDTO;
import br.gov.datasus.scodes.acl.model.dto.UsuarioDTO;
import br.gov.datasus.scodes.acl.model.entity.IdentificacaoUsuario;
import br.gov.datasus.scodes.acl.model.entity.Usuario;
import br.gov.datasus.scodes.acl.model.mapper.*;
import br.gov.datasus.scodes.acl.repository.UsuarioRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class UsuarioService {

    private final UsuarioRepository repository;
    private final UsuarioMapper mapper;
    private final IdentificacaoUsuarioMapper identificacaoUsuarioMapper;
    private final UsuariosSistemaPerfilEsferaMapper usuariosSistemaPerfilEsferaMapper;
    private final CpfsSistemaPerfilEsferaMapper cpfsSistemaPerfilEsferaMapper;
    private final AutenticaUsuarioMapper autenticaUsuarioMapper;

    public UsuarioDTO obterUsuario(UserInfoDTO userInfo) {
        DynamicParameters parameters = DynamicParameters.get();
        if (userInfo.getKey() != null) {
            parameters.append("id", userInfo.getKey());
        } else {
            //parameters.append("email", userInfo.getEmail());
            parameters.append("cpf", userInfo.getCpf());
        }
        Usuario entity = repository.findOne(Usuario.OBTER_POR_ID_CPF_EMAIL, parameters, Usuario.class);
        return mapper.toDto(entity);
    }

    public UsuarioDTO obterUsuario(Long id, String email, String cpf) {
        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("id", id);
        parameters.append("email", email);
        parameters.append("cpf", cpf);
        Usuario entity = repository.findOne(Usuario.OBTER_POR_ID_CPF_EMAIL, parameters, Usuario.class);
        return mapper.toDto(entity);
    }

    public IdentificacaoUsuarioDTO obterIdentificacao(Long id, String email, String cpf) {
        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("id", id);
        parameters.append("email", email);
        parameters.append("cpf", cpf);
        IdentificacaoUsuario entity = repository.findOne(IdentificacaoUsuario.OBTER_IDENTIFICACAO, parameters,
            IdentificacaoUsuario.class);
        return identificacaoUsuarioMapper.toDto(entity);
    }

    public List<IdentificacaoUsuarioDTO> obterUsuariosPorSistemaPerfilEsfera(String siglaSistema, String siglaPerfil,
                                                                             String codigoEsferaAtuacao) {
        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("siglaSistema", siglaSistema);
        parameters.append("siglaPerfil", siglaPerfil);
        parameters.append("codigoEsferaAtuacao", codigoEsferaAtuacao);
        return repository.findAllNative("query-usuarios-sistema-perfil-esfera", parameters,
            usuariosSistemaPerfilEsferaMapper);
    }

    public List<String> obterCpfsPorSistemaPerfilEsfera(String siglaSistema, String siglaPerfil,
                                                        String codigoEsferaAtuacao) {
        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("siglaSistema", siglaSistema);
        parameters.append("siglaPerfil", siglaPerfil);
        parameters.append("codigoEsferaAtuacao", codigoEsferaAtuacao);
        return repository.findAllNative("query-cpfs-sistema-perfil-esfera", parameters,
            cpfsSistemaPerfilEsferaMapper).stream().map(IdentificacaoUsuarioDTO::getCpf).collect(Collectors.toList());
    }

    public Boolean autenticarUsuario(String cpf, String hash) {

        DynamicParameters parameters = DynamicParameters.get();
        parameters.append("cpf", cpf);
        parameters.append("hash", hash);
        return repository.findAllNative("query-autenticar-usuario", parameters,
            autenticaUsuarioMapper).size() > 0;
    }
}
