package br.gov.datasus.authorization;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.autoconfigure.jdbc.DataSourceProperties;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.orm.jpa.EntityManagerFactoryBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import javax.persistence.EntityManagerFactory;
import javax.sql.DataSource;

@Configuration
@EnableTransactionManagement
@EntityScan(basePackages = "br.gov.datasus.authorization.model.scodes")
@EnableJpaRepositories(
    entityManagerFactoryRef = "scodesEntityManagerFactory",
    transactionManagerRef = "scodesTransactionManager",
    basePackages = {"br.gov.datasus.authorization.repository.scodes"}
)
public class ScodesDbConfig {

    @Bean
    @ConfigurationProperties(prefix = "scodes.datasource")
    public DataSourceProperties scodesDataSourceProperties() {
        return new DataSourceProperties();
    }

    @Bean(name = "scodesDataSource")
    public DataSource dataSource() {
        return scodesDataSourceProperties().initializeDataSourceBuilder().build();
    }

    @Bean(name = "scodesEntityManagerFactory")
    public LocalContainerEntityManagerFactoryBean scodesEntityManagerFactory(EntityManagerFactoryBuilder builder,
                                                                            @Qualifier("scodesDataSource") DataSource dataSource) {

        return builder.dataSource(dataSource).packages("br.gov.datasus.authorization.model.scodes").persistenceUnit("scodesPU").build();
    }

    @Bean(name = "scodesTransactionManager")
    public PlatformTransactionManager scodesTransactionManager(@Qualifier("scodesEntityManagerFactory") EntityManagerFactory scodesEntityManagerFactory) {
        return new JpaTransactionManager(scodesEntityManagerFactory);
    }

}
