package br.gov.datasus.authorization.configuration.cache;

import br.gov.datasus.authorization.configuration.properties.ApplicationProperties;
import br.gov.datasus.authorization.enums.CacheEnum;
import com.hazelcast.client.HazelcastClient;
import com.hazelcast.client.config.ClientConfig;
import com.hazelcast.client.config.ClientConnectionStrategyConfig.ReconnectMode;
import com.hazelcast.config.Config;
import com.hazelcast.config.EvictionPolicy;
import com.hazelcast.config.MapConfig;
import com.hazelcast.config.MaxSizeConfig;
import com.hazelcast.core.Hazelcast;
import com.hazelcast.core.HazelcastInstance;
import lombok.RequiredArgsConstructor;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Component;

@Component
@Configuration
@RequiredArgsConstructor
public class HazelcastConfiguration {

    private static final String SERVICE_DNS = "service-dns-scodes";
    private static final String HAZELCAST_INSTANCE = "hazelcast-instance-scodes";

    private final ApplicationProperties.Hazelcast hazelcastProp;

    @Bean
    public HazelcastInstance hazelcastInstance() {
        if (hazelcastProp != null && hazelcastProp.serverUrl != null && !hazelcastProp.serverUrl.isEmpty()) {
            return HazelcastClient.newHazelcastClient(getHazelcastClientConfig());
        }
        return Hazelcast.newHazelcastInstance(getHazelCastLocalConfig());
    };

    private ClientConfig getHazelcastClientConfig() {
        ClientConfig clientConfig = new ClientConfig();
        clientConfig.getNetworkConfig().addAddress(hazelcastProp.serverUrl);
        clientConfig.getConnectionStrategyConfig().setReconnectMode(ReconnectMode.ON);
        clientConfig.getNetworkConfig().setConnectionAttemptLimit(20);
        clientConfig.getNetworkConfig().setConnectionAttemptPeriod(5000);
        return clientConfig;
    }

    private Config getHazelCastLocalConfig() {
        Config config = new Config().setInstanceName(HAZELCAST_INSTANCE)
                .addMapConfig(new MapConfig().setName("token")
                        .setMaxSizeConfig(new MaxSizeConfig(200, MaxSizeConfig.MaxSizePolicy.FREE_HEAP_SIZE))
                        .setEvictionPolicy(EvictionPolicy.LRU).setTimeToLiveSeconds(200));

        config.addMapConfig(new MapConfig().setName(CacheEnum.ACCESS_TOKENSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.AUTHENTICATION_TO_ACCESS_TOKENSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.USERNAME_TO_ACCESS_TOKENSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.CLIENT_ID_TO_ACCESS_TOKENSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.REFRESH_TOKENSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.ACCESS_TOKEN_TO_REFRESH_TOKENSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.AUTHENTICATION_STORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.REFRESH_TOKEN_AUTHENTICATIONSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.REFRESH_TOKEN_TO_ACCESS_TOKENSTORE.getCacheName()));
        config.addMapConfig(new MapConfig().setName(CacheEnum.AUTHORIZATION_CODESTORE.getCacheName()));
        config.getNetworkConfig().getJoin().getMulticastConfig().setEnabled(false);

        return config;
    }
}
