package br.gov.datasus.authorization.enums;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import com.fasterxml.jackson.annotation.JsonValue;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;

import lombok.AllArgsConstructor;
import lombok.Getter;

@JsonSerialize
@AllArgsConstructor
public enum AuthorizationScopeEnum implements Serializable {
	SCPA("SCPA", "Sistema de controle de permissão e acesso"), 
	GOV_BR("GOVBR", "Acesso único aos serviços do Governo"),
	CNS_DIGITAL("CNSDIGITAL", "Cartão nacional de saúde");

	@Getter
	@JsonValue
	private String codigo;

	@Getter
	private String descricao;

	public static Optional<AuthorizationScopeEnum> obterPorCodigo(final String codigo) {
		return AuthorizationScopeEnum.stream().filter(e -> e.getCodigo().equals(codigo)).findFirst();
	}

	public static Optional<AuthorizationScopeEnum> obterPorDescricao(final String descricao) {
		return AuthorizationScopeEnum.stream().filter(e -> e.getDescricao().equals(descricao)).findFirst();
	}

	public static Stream<AuthorizationScopeEnum> stream() {
		return Stream.of(AuthorizationScopeEnum.values());
	}

	public static List<AuthorizationScopeEnum> list() {
		return Arrays.asList(AuthorizationScopeEnum.values());
	}
}
