package br.gov.datasus.authorization.model.scodes;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import javax.persistence.*;
import java.util.Date;
import java.util.Set;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Inheritance(strategy = InheritanceType.JOINED)
@Table(name = "TB_USUARIO", schema = "DBSCODES")
@NamedQueries({
    @NamedQuery(name = Usuario.OBTER_TODOS_PAGINADO, query = "SELECT usuario FROM Usuario usuario WHERE 1=1"
        + "AND (UPPER(usuario.nomeLogin) LIKE UPPER(:nomeLogin) )")
    , @NamedQuery(name = Usuario.OBTER_DOMINIO, query = "SELECT new br.gov.datasus.api.core.model.dto.DominioItemModel(usuario.id, usuario.nomeLogin) FROM Usuario usuario")
})
public class Usuario extends BaseEntity<Long> {

    private static final long serialVersionUID = -1242812534163505788L;
    public static final String OBTER_TODOS_PAGINADO = "Usuario.obterTodosPaginado";
    public static final String OBTER_DOMINIO = "Usuario.obterDominio";

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_USUARIO")
    @SequenceGenerator(name = "SEQ_GENERATOR_USUARIO", sequenceName = "SQ_USUARIO_COSEQUSUARIO", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_USUARIO")
    private Long id;

    @Column(name = "DS_EMAIL", length = 50)
    private String descricaoEmail;

    @Column(name = "DS_FALTA_ACESSO", length = 255)
    private String descricaoFaltaAcesso;

    @Column(name = "DS_SENHA", length = 32)
    private String descricaoSenha;

    @Column(name = "DT_CADASTRO")
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataCadastro;

    @Column(name = "DT_ULTIMO_ACESSO")
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataUltimoAcesso;

    @Column(name = "NO_LOGIN", length = 12)
    private String nomeLogin;

    @Column(name = "NU_TENTATIVAS")
    private Long numeroTentativas;

    @Column(name = "ST_ATIVO")
    private Boolean situacaoAtivo;

    @Column(name = "ST_TRAVADO")
    private Boolean situacaoTravado;

    @ManyToMany
    @JoinTable(name = "RL_PERFIL_USUARIO", schema = "DBSCODES",
        joinColumns = { @JoinColumn(name = "CO_USUARIO") },
        inverseJoinColumns = { @JoinColumn(name = "CO_PERFIL") })
    private Set<Perfil> perfis;

}
