package br.gov.datasus.authorization.security.oauth2.filter;

import java.io.IOException;

import javax.servlet.RequestDispatcher;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationServiceException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;

import br.gov.datasus.authorization.configuration.properties.ApplicationProperties;
import br.gov.datasus.authorization.configuration.properties.MessageConstants;
import br.gov.datasus.authorization.configuration.properties.MessageSource;
import br.gov.datasus.authorization.service.GoogleCaptchaIntegrationService;
import br.gov.datasus.authorization.service.UsuarioScpaService;

public class LoginAuthenticationFilter extends UsernamePasswordAuthenticationFilter implements InitializingBean {

	private static final String URL_LOGIN_ERROR_1 = "/login?error=1";

	private static final String URL_LOGIN_ERROR_2 = "/login?error=2";

	private final String USERNAME = "username";

	private final String PASSWORD = "password";

	@Autowired
	private ApplicationProperties.Security security;

	@Autowired
	private GoogleCaptchaIntegrationService captchaIntegrationService;

	@Autowired
	private UsuarioScpaService usuarioScpaService;

	public Authentication attemptAuthentication(HttpServletRequest request, HttpServletResponse response)
			throws AuthenticationException {
		String username = request.getParameter(USERNAME);
		String password = request.getParameter(PASSWORD);
		if (security.captchaEnable && !captchaIntegrationService.validarCaptcha(request, response)) {
			errorRedirect(request, response, URL_LOGIN_ERROR_2);
		} else if (!usuarioScpaService.authenticate(username, password)) {
			errorRedirect(request, response, URL_LOGIN_ERROR_1);
		} else {
			return super.attemptAuthentication(request, response);
		}
		return null;
	}

	private void errorRedirect(HttpServletRequest request, HttpServletResponse response, String url) {
		try {
			RequestDispatcher dispatcher = request.getRequestDispatcher(url);
			dispatcher.forward(request, response);
		} catch (ServletException | IOException e) {
			throw new AuthenticationServiceException(MessageSource.get().message(MessageConstants.REDIRECT_ERROR, url));
		}
	}
}
