package br.gov.datasus.authorization.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.StreamSupport;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import br.gov.datasus.authorization.configuration.properties.ApplicationProperties;
import br.gov.datasus.authorization.enums.AuthorizationScopeEnum;
import br.gov.datasus.authorization.model.scpa.UrlPermitida;
import br.gov.datasus.authorization.repository.scpa.SistemaRepository;
import br.gov.datasus.authorization.repository.scpa.UrlPermitidaRepository;
import br.gov.datasus.authorization.security.oauth2.model.Client;

@Service
public class AuthorizationClientScpaService {

	private static final String CLIENT_SUFIX_NAME_INTEGRACAO = "_INTEGRACAO";

	private static final String FECHA_CHAVE = "}";

	private static final String ABRE_CHAVE = "{";

	@Autowired
	private SistemaRepository sistemaScpaRepository;

	@Autowired
	private UrlPermitidaRepository urlPermitidaRepository;

	@Autowired
	private ApplicationProperties.ClientIntegration clientIntegration;


	public List<Client> getClients() {
		Map<String, Set<String>> clientRoles = criarClientRoles();
		Iterable<UrlPermitida> urls = urlPermitidaRepository.findAll();
		List<Client> clients = new ArrayList<>();
		sistemaScpaRepository.findAll().forEach(sistema -> {
			Set<String> criarCallbackUrls = criarCallbackUrls(urls);
			// Add Client SCPA
			Client clientSistema = createClient(sistema.getSigla(), sistema.getToken());
			clientSistema.setUrls(criarCallbackUrls);
			clients.add(clientSistema);
			// Cliente para fluxo client_credentials
			Client clientIntegracao = createClient(sistema.getSigla() + CLIENT_SUFIX_NAME_INTEGRACAO,
					sistema.getToken());
			clientIntegracao.setUrls(criarCallbackUrls);
			if (clientRoles.containsKey(clientIntegracao.getClientId())) {
				clientIntegracao.setClientRoles(clientRoles.get(clientIntegracao.getClientId()));
			}
			clients.add(clientIntegracao);
		});
		return clients;
	}

	private Client createClient(String clientId, String clientSecret) {
		Client client = new Client();
		client.setClientId(clientId);
		client.setClientSecret(clientSecret);
		client.getScopes().add(AuthorizationScopeEnum.SCPA.getCodigo());
		client.getScopes().add(AuthorizationScopeEnum.GOV_BR.getCodigo());
		client.getScopes().add(AuthorizationScopeEnum.CNS_DIGITAL.getCodigo());
		return client;
	}

	private Set<String> criarCallbackUrls(Iterable<UrlPermitida> urls) {
		Set<String> permitidas = new HashSet<String>();
		StreamSupport.stream(urls.spliterator(), false).filter(url -> url.getIdSistema().equals(1L)).forEach(url -> {
			permitidas.add(url.getUrl());
		});
		return permitidas;
	}

	private Map<String, Set<String>> criarClientRoles() {
		Map<String, Set<String>> mapClientRoles = new HashMap<>();
		List<String> clients = clientIntegration.clients;
		for (String client : clients) {
			String key = client.substring(0, client.indexOf(ABRE_CHAVE)).trim();
			client = client.substring(client.indexOf(ABRE_CHAVE) + 1, client.indexOf(FECHA_CHAVE));
			String[] clientPart = client.split(",");
			Set<String> roles = new HashSet<>();
			for (int i = 0; i < clientPart.length; i++) {
				roles.add(clientPart[i].trim());
			}
			mapClientRoles.put(key, roles);
		}
		return mapClientRoles;
	}
}
