package br.gov.datasus.authorization.service;

import java.util.ArrayList;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Service;

import br.gov.datasus.authorization.configuration.properties.ApplicationProperties;
import br.gov.datasus.authorization.configuration.properties.MessageConstants;
import br.gov.datasus.authorization.configuration.properties.MessageSource;
import br.gov.datasus.authorization.enums.AuthorizationScopeEnum;
import br.gov.datasus.authorization.exception.AuthorizationException;
import br.gov.datasus.authorization.integracao.clients.UsuarioSusClient;
import br.gov.datasus.authorization.model.scpa.Usuario;
import br.gov.datasus.authorization.security.oauth2.model.User;
import br.gov.datasus.authorization.security.oauth2.model.UserAuthority;
import br.gov.datasus.cadsus.integracao.model.dto.CNESUsuarioDTO;
import br.gov.datasus.cadsus.integracao.model.dto.UsuarioSusDTO;

@Service
public class IntegracaoCnsDigitalServide {

    private static final String ROLE_CNS_DIGITAL = "ROLE_CNS_DIGITAL";

    @Autowired
    private ApplicationProperties.IntegracaoCadsus integracaoCadsus;

    @Autowired
    private UsuarioSusClient usuarioSusClient;

    @Autowired
    private UsuarioScpaService usuarioScpaService;

    public UserDetails loadUser(String username) throws AuthenticationException {
        User user = new User();
        ArrayList<UserAuthority> list = new ArrayList<UserAuthority>();
        list.add(new UserAuthority(ROLE_CNS_DIGITAL));
        user.setAuthorities(list);
        UsuarioSusDTO usuarioSus = obterDadosUsuarioSus(username);
        user.getUserInfo().setNome(usuarioSus.getNomeCompleto());
        user.getUserInfo().setCnes(usuarioSus.getCns());
        user.getUserInfo().setEmail(usuarioSus.getEmail());
        user.getUserInfo().setCpf(usuarioSus.getCpf());
        user.getUserInfo().setOrigem(AuthorizationScopeEnum.CNS_DIGITAL.getCodigo());
        user.setUsername(usuarioSus.getCpf());

        Usuario usuarioScpa = usuarioScpaService.findUserByCpf(user.getUsername());
        if (usuarioScpa != null) {
            usuarioScpaService.addRolesScpa(user, usuarioScpa);
        }

        return user;
    }

    private UsuarioSusDTO obterDadosUsuarioSus(String username) {
        CNESUsuarioDTO cnesUsuario = new CNESUsuarioDTO();
        cnesUsuario.setCnes("CNES");
        cnesUsuario.setUsuario(integracaoCadsus.cadsusUsername);
        cnesUsuario.setSenha(integracaoCadsus.cadsusPassword);
        ResponseEntity<UsuarioSusDTO> response = null;
        if (username.length() == 11) {
            response = usuarioSusClient.obterPorCPF(username, cnesUsuario);
        } else {
            response = usuarioSusClient.obterPorCNS(username, cnesUsuario);
        }
        if (response.getStatusCode().equals(HttpStatus.OK)) {
            return response.getBody();
        }
        throw new AuthorizationException(
                MessageSource.get().message(MessageConstants.HTTP_ERROR, String.valueOf(response.getStatusCode())));
    }
}
