package br.gov.datasus.scodes.controller;

import java.util.List;

import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.scodes.model.dto.AreaDTO;
import br.gov.datasus.scodes.service.AreaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;

@Api(tags = "Area")
@RequestMapping(value = "/area")
@RequiredArgsConstructor
@RestController
public class AreaController {
	
    private final AreaService service;
    
    @ApiOperation(value = "Obter área por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<AreaDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todas áreas")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<AreaDTO>> obterTodos() {
        List<AreaDTO> lista = service.obterTodos();
        return ResponseEntity.ok(lista);
    }
    
    @ApiOperation(value = "Obter todas áreas como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        List<DominioItemModel> lista = service.obterDominio();
        return ResponseEntity.ok(lista);
    }
}

