package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.AutorDemandaDTO;
import br.gov.datasus.scodes.model.dto.filter.DemandaFilterDTO;
import br.gov.datasus.scodes.service.AutorDemandaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(tags = "AutorDemanda")
@RequestMapping(value = "/autor-demanda")
@RequiredArgsConstructor
@RestController
public class AutorDemandaController {

    private final AutorDemandaService service;

    @ApiOperation(value = "Obter autor-demanda por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<AutorDemandaDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }


    @ApiOperation(value = "Pesquisa de autor-demanda paginada")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<AutorDemandaDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<DemandaFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.pesquisarPaginado(pageableModel));
    }

    @ApiOperation(value = "Incluir autor-demanda")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<AutorDemandaDTO> incluir(@Valid @RequestBody AutorDemandaDTO dto) {
        return new ResponseEntity<AutorDemandaDTO>(service.validateAndSave(dto), HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar autor-demanda")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<AutorDemandaDTO> alterar(@Valid @RequestBody AutorDemandaDTO dto, @PathVariable(name = "id") Long id) {
        dto.setId(id);
        return ResponseEntity.ok(service.validateAndSave(dto));
    }

    @ApiOperation(value = "Excluir autor-demanda por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluir(id);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }
}
