package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.CompraModalidadeDTO;
import br.gov.datasus.scodes.service.CompraModalidadeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "CompraModalidade")
@RequestMapping(value = "/compra-modalidade")
@RequiredArgsConstructor
@RestController
public class CompraModalidadeController {
	
    private final CompraModalidadeService service;
    
    @ApiOperation(value = "Obter modalidade de compra por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<CompraModalidadeDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todas modalidades de compra")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<CompraModalidadeDTO>> obterTodos() {
        return ResponseEntity.ok(service.obterTodos());
    }

    @ApiOperation(value = "Obter todos modalidades de compra")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<CompraModalidadeDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<Void> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }
    
    @ApiOperation(value = "Obter todas modalidades de compra como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        return ResponseEntity.ok(service.obterDominio());
    }
}
