package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.EntradaEstoqueDTO;
import br.gov.datasus.scodes.service.EntradaEstoqueService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(tags = "EntradaEstoque")
@RequestMapping(value = "/entrada-estoque")
@RequiredArgsConstructor
@RestController
public class EntradaEstoqueController {
	
    private final EntradaEstoqueService service;
    
    @ApiOperation(value = "Obter entrada de estoque por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<EntradaEstoqueDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todas entradas de estoque paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<EntradaEstoqueDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<Void> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Incluir entrada de estoque")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<EntradaEstoqueDTO> incluir(@Valid @RequestBody EntradaEstoqueDTO dto) {
        /* @todo buscar id do usuário logado */
        Long currentUserId = 1L;
        dto.setIdUsuario(currentUserId);
        /* ------------------------------------ */
        return new ResponseEntity<>(service.incluir(dto), HttpStatus.CREATED);
    }
}
