package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.FabricanteDTO;
import br.gov.datasus.scodes.service.FabricanteService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "Fabricante")
@RequestMapping(value = "/fabricante")
@RequiredArgsConstructor
@RestController
public class FabricanteController {
	
    private final FabricanteService service;
    
    @ApiOperation(value = "Obter fabricante por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<FabricanteDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos fabricantes")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<FabricanteDTO>> obterTodos() {
        List<FabricanteDTO> lista = service.obterTodos();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos fabricantes paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<FabricanteDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<Void> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }
    
    @ApiOperation(value = "Obter todos fabricantes como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        List<DominioItemModel> lista = service.obterDominio();
        return ResponseEntity.ok(lista);
    }
}
