package br.gov.datasus.scodes.controller;

import br.gov.datasus.scodes.model.dto.filter.GrupoEspecificacaoFilterDTO;
import br.gov.datasus.scodes.service.GrupoEspecificacaoService;
import br.gov.datasus.scodes.model.dto.GrupoEspecificacaoDTO;
import javax.validation.Valid;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import lombok.RequiredArgsConstructor;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Api(tags = "GrupoEspecificacao")
@RequestMapping(value = "/grupo-especificacao")
@RequiredArgsConstructor
@RestController
public class GrupoEspecificacaoController {

    private final GrupoEspecificacaoService service;

    @ApiOperation(value = "Obter grupo-especificacao por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<GrupoEspecificacaoDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos grupo-especificacao")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<GrupoEspecificacaoDTO>> obterTodos() {
        List<GrupoEspecificacaoDTO> lista = service.obterTodos();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos grupo-especificacaos paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<GrupoEspecificacaoDTO>> obterTodosPaginado(
            @Valid @RequestBody PageableRequestModel<GrupoEspecificacaoFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Obter todos grupo-especificacao como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio(
        @RequestParam(name = "categoria", required = false) Long idCategoria
    ) {
        List<DominioItemModel> lista = service.obterDominio(idCategoria);
        return ResponseEntity.ok(lista);
    }

}

