package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.ArquivoBase64DTO;
import br.gov.datasus.scodes.model.dto.InventarioDTO;
import br.gov.datasus.scodes.model.dto.filter.InventarioFilterDTO;
import br.gov.datasus.scodes.service.InventarioService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Set;

@Api(tags = "Inventario")
@RequestMapping(value = "/inventario")
@RequiredArgsConstructor
@RestController
public class InventarioController {

    private final InventarioService service;

    @ApiOperation(value = "Obter inventario por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<InventarioDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos inventarios paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<InventarioDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<InventarioFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Obter todos inventarios como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        return ResponseEntity.ok(service.obterDominio());
    }

    @ApiOperation(value = "Incluir inventario")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<InventarioDTO> incluir(@Valid @RequestBody InventarioDTO dto) {
        /* @todo buscar id do usuário logado */
        Long currentUserId = 1L;
        dto.setIdUsuario(currentUserId);
        /* ------------------------------------ */
        return new ResponseEntity<>(service.incluir(dto), HttpStatus.CREATED);
    }

    @ApiOperation(value = "Consolidar inventário")
    @PutMapping(value = "/{id}/consolidar", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> alterar(@PathVariable(name = "id") Long id) {
        service.consolidar(id);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @ApiOperation(value = "Gerar relatório de itens no inventário")
    @GetMapping(value = "/{id}/relatorio-item", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ArquivoBase64DTO> gerarRelatorioItem(
        @PathVariable(name = "id") Long id,
        @RequestParam(name = "idItens", required = false) Set<Long> idItens) {
        return ResponseEntity.ok(service.gerarRelatorioItem(id, idItens));
    }

    @ApiOperation(value = "Gerar relatório consolidado do inventário")
    @GetMapping(value = "/{id}/relatorio-consolidado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ArquivoBase64DTO> gerarRelatorioConsolidado(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.gerarRelatorioConsolidado(id));
    }
}
