package br.gov.datasus.scodes.controller;

import br.gov.datasus.scodes.model.dto.MarcaDTO;
import br.gov.datasus.scodes.service.MarcaService;
import br.gov.datasus.scodes.model.dto.filter.MarcaFilterDTO;
import javax.validation.Valid;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import lombok.RequiredArgsConstructor;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import java.util.List;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.PostMapping;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Api(tags = "Marca")
@RequestMapping(value = "/marca")
@RequiredArgsConstructor
@RestController
public class MarcaController {

    private final MarcaService service;
    
    @ApiOperation(value = "Obter marca por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<MarcaDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos marca")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<MarcaDTO>> obterTodos() {
        List<MarcaDTO> lista = service.obterTodos();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos marcas paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<MarcaDTO>> obterTodosPaginado(
            @Valid @RequestBody PageableRequestModel<MarcaFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Obter todos marca como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        List<DominioItemModel> lista = service.obterDominio();
        return ResponseEntity.ok(lista);
    }

}

