package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.PerfilDTO;
import br.gov.datasus.scodes.model.dto.filter.PerfilFilterDTO;
import br.gov.datasus.scodes.service.PerfilService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "Perfil")
@RequestMapping(value = "/perfil")
@RequiredArgsConstructor
@RestController
public class PerfilController {

    private final PerfilService service;

    @ApiOperation(value = "Obter perfil por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PerfilDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos perfil")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<PerfilDTO>> obterTodos() {
        List<PerfilDTO> lista = service.obterTodos();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos perfils paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<PerfilDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<PerfilFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Obter todos perfil como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        List<DominioItemModel> lista = service.obterDominio();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Incluir perfil")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PerfilDTO> incluir(@Valid @RequestBody PerfilDTO dto) {
        dto = service.incluir(dto);
        return new ResponseEntity<PerfilDTO>(dto, HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar perfil")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PerfilDTO> alterar(@Valid @RequestBody PerfilDTO dto, @PathVariable(name = "id") Long id) {
        dto = service.alterar(id, dto);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "Excluir perfil por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluir(id);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }

    @ApiOperation(value = "Obter perfil de demanda administrativa por codigo de area")
    @GetMapping(value = "/administrativo/area/{idArea}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterPerfilDemandaAdmPorArea(@PathVariable(name = "idArea") Long idArea) {
        List<DominioItemModel> lista = service.obterPerfilDemandaAdmPorArea(idArea);
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter perfil de demanda judicial por codigo de area")
    @GetMapping(value = "/judicial/area/{idArea}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterPerfilDemandaJudicialPorArea(@PathVariable(name = "idArea") Long idArea) {
        List<DominioItemModel> lista = service.obterPerfilDemandaJudicialPorArea(idArea);
        return ResponseEntity.ok(lista);
    }
}

