package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.ProdutoDTO;
import br.gov.datasus.scodes.model.dto.filter.ProdutoFilterDTO;
import br.gov.datasus.scodes.service.ProdutoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

@Api(tags = "Produto")
@RequestMapping(value = "/produto")
@RequiredArgsConstructor
@RestController
public class ProdutoController {

    private final ProdutoService service;

    @ApiOperation(value = "Obter produto por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ProdutoDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos produto")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<ProdutoDTO>> obterTodos() {
        return ResponseEntity.ok(service.obterTodos());
    }

    @ApiOperation(value = "Obter todos produtos paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<ProdutoDTO>> obterTodosPaginado(
            @Valid @RequestBody PageableRequestModel<ProdutoFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Obter todos produto como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        return ResponseEntity.ok(service.obterDominio());
    }

    @ApiOperation(value = "Obter autocomplete de produto por termo")
    @GetMapping(value = "/autocomplete", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> autocomplete(
        @RequestParam(name = "termo")
            String termoProduto,
        @RequestParam(name = "categoria", required = false)
            Long idCategoria
    ) {
        return ResponseEntity.ok(service.autocomplete(termoProduto, idCategoria));
    }

    @ApiOperation(value = "Incluir produto")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ProdutoDTO> incluir(@Valid @RequestBody ProdutoDTO dto) {
        return new ResponseEntity<>(service.incluir(dto), HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar produto")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ProdutoDTO> alterar(@Valid @RequestBody ProdutoDTO dto, @PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.alterar(id, dto));
    }

    @ApiOperation(value = "Excluir produto por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluir(id);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}

