package br.gov.datasus.scodes.controller;

import java.util.List;

import javax.validation.Valid;

import br.gov.datasus.scodes.model.dto.UfDTO;
import br.gov.datasus.scodes.model.dto.filter.ValidarAlteracaoNovaDataRetornoFilterDTO;
import br.gov.datasus.scodes.model.entity.Profissional;
import br.gov.datasus.scodes.model.entity.Uf;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.ProfissionalDTO;
import br.gov.datasus.scodes.model.dto.filter.ProfissionalFilterDTO;
import br.gov.datasus.scodes.service.ProfissionalService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;

@Api(tags = "Profissional")
@RequestMapping(value = "/profissional")
@RequiredArgsConstructor
@RestController
public class ProfissionalController {

    private final ProfissionalService service;

    @ApiOperation(value = "Obter profissional por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ProfissionalDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Pesquisa de profissionals paginada")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<ProfissionalDTO>> obterTodosPaginado(
            @Valid @RequestBody PageableRequestModel<ProfissionalFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.pesquisarPaginado(pageableModel));
    }

    @ApiOperation(value = "Incluir profissional")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ProfissionalDTO> incluir(@Valid @RequestBody ProfissionalDTO dto) {
        dto = service.incluir(dto);
        return new ResponseEntity<ProfissionalDTO>(dto, HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar profissional")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<ProfissionalDTO> alterar(@Valid @RequestBody ProfissionalDTO dto, @PathVariable(name = "id") Long id) {
        dto = service.alterar(id, dto);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "Excluir profissional por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluir(id);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }

    @ApiOperation(value = "Pesquisar Patrono (Profissional)")
    @PostMapping(value = "/pesquisar-patrono/tipoProfissaoId/{tipoProfissaoId}/uf/{uf}/oab/{oab}/nome/{nome}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<ProfissionalDTO>> pesquisarPatrono(
            @PathVariable(name = "tipoProfissaoId") Integer tipoProfissaoId,
            @PathVariable(name = "uf") String uf,
            @PathVariable(name = "oab") String oab,
            @PathVariable(name = "nome") String nome) {
        return ResponseEntity.ok(service.pesquisarPatrono(tipoProfissaoId, uf, oab, nome));
    }

    @ApiOperation(value = "Pesquisar Patrono (Profissional)")
    @PostMapping("/pesquisar-patrono")
    public ResponseEntity<List<ProfissionalDTO>> pesquisarPatrono(@Valid @RequestBody ProfissionalFilterDTO dtoFilter) {
        return ResponseEntity.ok(service.pesquisarPatrono(dtoFilter));
    }

    @ApiOperation(value = "Obter ufs profissinal")
    @GetMapping(value = "/ufs", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<UfDTO>> obterUfsProfissional() {
        return ResponseEntity.ok(service.obterUfsProfissional());
    }

}

