package br.gov.datasus.scodes.controller;

import br.gov.datasus.scodes.model.dto.TipoDocumentoDTO;
import br.gov.datasus.scodes.model.dto.filter.TipoDocumentoFilterDTO;
import br.gov.datasus.scodes.service.TipoDocumentoService;
import javax.validation.Valid;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import lombok.RequiredArgsConstructor;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import java.util.List;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.PostMapping;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Api(tags = "TipoDocumento")
@RequestMapping(value = "/tipo-documento")
@RequiredArgsConstructor
@RestController
public class TipoDocumentoController {

    private final TipoDocumentoService service;
    
    @ApiOperation(value = "Obter tipo-documento por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<TipoDocumentoDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos tipo-documento")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<TipoDocumentoDTO>> obterTodos() {
        List<TipoDocumentoDTO> lista = service.obterTodos();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos tipo-documentos paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<TipoDocumentoDTO>> obterTodosPaginado(
            @Valid @RequestBody PageableRequestModel<TipoDocumentoFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Obter todos tipo-documento como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        List<DominioItemModel> lista = service.obterDominio();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Incluir tipo-documento")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<TipoDocumentoDTO> incluir(@Valid @RequestBody TipoDocumentoDTO dto) {
        dto = service.incluir(dto);
        return new ResponseEntity<TipoDocumentoDTO>(dto, HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar tipo-documento")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<TipoDocumentoDTO> alterar(@Valid @RequestBody TipoDocumentoDTO dto, @PathVariable(name = "id") Long id) {
        dto = service.alterar(id, dto);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "Excluir tipo-documento por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluir(id);
        return new ResponseEntity<Void>(HttpStatus.OK);
    }
}

