package br.gov.datasus.scodes.controller;

import br.gov.datasus.scodes.model.dto.TipoProfissaoDTO;
import br.gov.datasus.scodes.model.dto.filter.TipoProfissaoFilterDTO;
import br.gov.datasus.scodes.model.entity.TipoProfissao;
import br.gov.datasus.scodes.model.enums.TipoDemandaEnum;
import br.gov.datasus.scodes.model.enums.TipoProfissaoEnum;
import br.gov.datasus.scodes.service.TipoProfissaoService;
import javax.validation.Valid;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import lombok.RequiredArgsConstructor;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.PostMapping;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

import static br.gov.datasus.scodes.model.enums.TipoProfissaoEnum.*;

@Api(tags = "TipoProfissao")
@RequestMapping(value = "/tipo-profissao")
@RequiredArgsConstructor
@RestController
public class TipoProfissaoController {

    private final TipoProfissaoService service;

    @ApiOperation(value = "Obter tipo-profissao por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<TipoProfissaoDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos tipo-profissao")
    @GetMapping(produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<TipoProfissaoDTO>> obterTodos() {
        List<TipoProfissaoDTO> lista = service.obterTodos();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos tipo-profissaos paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<TipoProfissaoDTO>> obterTodosPaginado(
            @Valid @RequestBody PageableRequestModel<TipoProfissaoFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Obter todos tipo-profissao como domínio id/descricao")
    @GetMapping(value = "/dominio", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominio() {
        List<DominioItemModel> lista = service.obterDominio();
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos tipo-profissao protocolo como domínio id/descricao")
    @GetMapping(value = "/dominio-protocolo", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<DominioItemModel>> obterDominioProtocolo() {
        final List<DominioItemModel> lista = service.obterDominio()
                                        .stream()
                                        .filter(tipoProfissao ->
                                                service.getTiposProfissionalProtocoloIds()
                                                    .contains(Long.valueOf(tipoProfissao.getCodigo()))
                                        ).collect(Collectors.toList());
        return ResponseEntity.ok(lista);
    }

    @ApiOperation(value = "Obter todos tipo-profissao protocolo")
    @GetMapping(value = "/protocolo", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<List<TipoProfissaoDTO>> obterTipoProfissaoProtocolo() {
        List<TipoProfissaoDTO> lista = service.obterTodos().stream()
            .filter(tipoProfissao -> service.getTiposProfissionalProtocoloIds().contains(tipoProfissao.getId())).collect(Collectors.toList());
        return ResponseEntity.ok(lista);
    }
}

