package br.gov.datasus.scodes.controller;

import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.UsuarioLocalCadastroDTO;
import br.gov.datasus.scodes.model.dto.UsuarioLocalDTO;
import br.gov.datasus.scodes.model.dto.UsuarioLocalSenhaDTO;
import br.gov.datasus.scodes.model.dto.filter.UsuarioLocalFilterDTO;
import br.gov.datasus.scodes.service.UsuarioLocalService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@Api(tags = "UsuarioLocal")
@RequestMapping(value = "/usuario-local")
@RequiredArgsConstructor
@RestController
public class UsuarioLocalController {

    private final UsuarioLocalService service;
    
    @ApiOperation(value = "Obter usuário local por código")
    @GetMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<UsuarioLocalDTO> obterPorId(@PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.obterPorId(id));
    }

    @ApiOperation(value = "Obter todos usuários locais paginado")
    @PostMapping(value = "/paginado", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<PageableResponseModel<UsuarioLocalDTO>> obterTodosPaginado(
        @Valid @RequestBody PageableRequestModel<UsuarioLocalFilterDTO> pageableModel) {
        return ResponseEntity.ok(service.obterTodosPaginado(pageableModel));
    }

    @ApiOperation(value = "Incluir usuário local")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<UsuarioLocalDTO> incluir(@Valid @RequestBody UsuarioLocalDTO dto) {
        return new ResponseEntity<>(service.incluir(dto), HttpStatus.CREATED);
    }

    @ApiOperation(value = "Atualizar usuário local")
    @PutMapping(value = "/{id}", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<UsuarioLocalDTO> alterar(@Valid @RequestBody UsuarioLocalDTO dto, @PathVariable(name = "id") Long id) {
        return ResponseEntity.ok(service.alterar(id, dto));
    }

    @ApiOperation(value = "Excluir usuário local por id")
    @DeleteMapping(value = "/{id}", produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> excluir(@PathVariable(name = "id") Long id) {
        service.excluir(id);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @ApiOperation(value = "Atualizar cadastro do usuário logado")
    @PostMapping(value = "/cadastro", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> atualizarCadastro(@Valid @RequestBody UsuarioLocalCadastroDTO dto) {
        /* @todo buscar id do usuário logado */
        Long currentUserId = 1L;
        /* -------------------------------------*/
        service.alterar(currentUserId, dto);
        return new ResponseEntity<>(HttpStatus.OK);
    }

    @ApiOperation(value = "Alterar senha do usuário logado")
    @PostMapping(value = "/senha", consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    public ResponseEntity<Void> alterarSenha(@Valid @RequestBody UsuarioLocalSenhaDTO dto) {
        /* @todo buscar id do usuário logado */
        Long currentUserId = 1L;
        /* -------------------------------------*/
        service.alterar(currentUserId, dto);
        return new ResponseEntity<>(HttpStatus.OK);
    }
}

