package br.gov.datasus.scodes.model.entity;

import java.util.Date;

import javax.persistence.*;

import br.gov.datasus.api.core.model.entity.BaseEntity;

import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_ACESSO", schema = "DBSCODES")
@NamedQueries({
    @NamedQuery(name = Acesso.OBTER_TODOS_PAGINADO, query = "SELECT a FROM  Acesso a"
        + " INNER JOIN FETCH a.usuarioLocal ul"
        + " WHERE 1=1"
        + " AND (UPPER(ul.nomeLogin) LIKE UPPER(:nomeLogin))"),
    
    @NamedQuery(name = Acesso.OBTER_TODOS_RELATORIO, query = "SELECT a FROM Acesso a"
        + " INNER JOIN FETCH a.usuarioLocal ul"
        + " LEFT JOIN FETCH ul.unidadeOrganizacional uo"
        + " LEFT JOIN FETCH ul.unidadesDispensadoras ud"
        + " WHERE 1=1"
        + " AND (UPPER(ul.nomeLogin) LIKE UPPER(:nomeLogin))"
        + " AND (UPPER(ul.nomeUsuario) LIKE UPPER(:nomeUsuario))"
        + " AND (a.dataEntrada >= :dataEntrada)"
        + " AND (a.dataSaida < :dataSaida)"
        + " ORDER BY a.id DESC")
})
public class Acesso extends BaseEntity<Long> {

	private static final long serialVersionUID = 1L;

	public static final String OBTER_TODOS_PAGINADO = "Acesso.obterTodosPaginado";
	public static final String OBTER_TODOS_RELATORIO = "Acesso.obterTodosRelatorio";

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_USUARIO")
	@SequenceGenerator(name = "SEQ_GENERATOR_ACESSO", sequenceName = "SQ_ACESSO_COSEQACESSO", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
	@Column(name = "CO_SEQ_ACESSO", nullable = false)
	private Long id;

	@ManyToOne
	@JoinColumn(name = "CO_USUARIO", nullable = false)
	private UsuarioLocal usuarioLocal;

	@Column(name = "CO_SESSAO", length = 50, nullable = false)
	private String codigoSessao;

	@Column(name = "NU_IP_CLIENTE", length = 50)
	private String ipCliente;

	@Column(name = "NU_IP_SERVIDOR", length = 50)
	private String ipServidor;

	@Column(name = "DT_ENTRADA", nullable = false)
	@Temporal(TemporalType.TIMESTAMP)
	private Date dataEntrada;

	@Column(name = "DT_SAIDA")
	@Temporal(TemporalType.TIMESTAMP)
	private Date dataSaida;

	@Column(name = "DS_FALHA", length = 100)
	private String descricaoFalha;
}
