package br.gov.datasus.scodes.model.entity;

import java.util.Date;

import javax.persistence.*;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_ARQUIVO", schema = "DBSCODES")
@NamedQuery(
    name = Arquivo.OBTER_ARQUIVOS_POR_DIRETORIO_ID,
    query = " SELECT arquivo FROM Arquivo arquivo"
            + " WHERE arquivo.diretorio.id = :diretorioId "
            + " AND arquivo.autorDemanda.id = :autorDemandaId "
)
public class Arquivo extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;

    public static final String OBTER_ARQUIVOS_POR_DIRETORIO_ID = "Arquivo.obterArquivosPorDiretorioId";

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_ARQUIVO")
    @SequenceGenerator(name = "SEQ_GENERATOR_ARQUIVO", sequenceName = "SQ_ARQUIVO_COSEQARQUIVO", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_ARQUIVO", nullable = false)
    private Long id;

    @ManyToOne
    @JoinColumn(name = "CO_AUTOR_DEMANDA", nullable = false)
    private AutorDemanda autorDemanda;

    @ManyToOne
    @JoinColumn(name = "CO_DIRETORIO", nullable = false)
    private Diretorio diretorio;

    @Column(name = "DT_INCLUSAO")
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataInclusao;

    @Column(name = "DT_PROTOCOLO")
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataProtocolo;

    @Column(name = "DS_DESCRICAO", length = 80)
    private String descricao;

    @Column(name = "DS_EXTENSAO", length = 80)
    private String extencao;

    @Column(name = "AR_ARQUIVO")
    private byte[] arquivo;
}
