package br.gov.datasus.scodes.model.entity;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Convert;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQuery;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import br.gov.datasus.scodes.model.converter.TipoSexoEnumConverter;
import br.gov.datasus.scodes.model.enums.TipoSexoEnum;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_AUTOR", schema = "DBSCODES")
@NamedQuery(
    name = Autor.PESQUISAR_POR_CPF,
    query = " SELECT autor FROM Autor autor WHERE autor.cpf like :cpf "
)
public class Autor extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;
    
    public static final String PESQUISAR_POR_CPF = "Autor.pesquisarPorCpf";

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_AUTOR")
    @SequenceGenerator(name = "SEQ_GENERATOR_AUTOR", sequenceName = "SQ_AUTOR_COSEQAUTOR", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_AUTOR", nullable = false)
    private Long id;
    
    @Column(name = "NO_NOME", nullable = false, length = 80)
    private String nome;

    @Column(name = "DT_NASCIMENTO", nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataNascimento;
    
    @Convert(converter = TipoSexoEnumConverter.class)
    @Column(name = "TP_SEXO", nullable = false)
    private TipoSexoEnum sexo;

    @Column(name = "NU_RG", length = 16)
    private String rg;

    @Column(name = "NU_CPF", length = 15)
    private String cpf;
    
    @Column(name = "NU_CNS", length = 20)
    private String cns;
    
    @Column(name = "DS_MAE", length = 80)
    private String nomeMae;
    
    @Column(name = "NU_TELEFONE", length = 20)
    private String telefone;

    @Column(name = "DS_OBS_TELEFONE", length = 100)
    private String obsTelefone;

    @Column(name = "NU_OUTRO_TELEFONE", length = 20)
    private String outroTelefone;
    
    @Column(name = "DS_OBS_OUTRO_TELEFONE", length = 100)
    private String obsOutroTelefone;

    @Column(name = "NU_CEP", length = 9)
    private String cep;

    @Column(name = "DS_BAIRRO", length = 60)
    private String bairro;

    @Column(name = "DS_LOGRADOURO", length = 100)
    private String logradouro;

    @Column(name = "NU_NUMERO", length = 5)
    private String numero;

    @Column(name = "DS_COMPLEMENTO", length = 30)
    private String complemento;

    @Column(name = "DS_EMAIL", length = 80)
    private String email;
    
    @ManyToOne
    @JoinColumn(name = "CO_CIDADE")
    private Cidade cidade;

}
