package br.gov.datasus.scodes.model.entity;

import java.util.Date;

import javax.persistence.*;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import br.gov.datasus.scodes.model.converter.TipoProcessoEnumConverter;
import br.gov.datasus.scodes.model.enums.TipoProcessoEnum;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_DEMANDA", schema = "DBSCODES")
public class Demanda extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_DEMANDA")
    @SequenceGenerator(name = "SEQ_GENERATOR_DEMANDA", sequenceName = "SQ_DEMANDA_COSEQDEMANDA", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_DEMANDA", nullable = false)
    private Long id;

    @ManyToOne(cascade = {CascadeType.MERGE})
    @JoinColumn(name = "CO_UNIDADE_ORGANIZACIONAL", nullable = false)
    private UnidadeOrganizacional unidadeOrganizacional;

    @ManyToOne
    @JoinColumn(name = "CO_RELATORIO_TECNICO")
    private RelatorioTecnico relatorioTecnico;

    @Column(name = "DT_ENTRADA", nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataEntrada;

    @Column(name = "NU_PROTOCOLO", length = 50, nullable = false)
    private String protocolo;

    @Column(name = "NU_PROCESSO", length = 50)
    private String processo;

    @Column(name = "NU_SENHA_PROCESSO", length = 100)
    private String senhaProcesso;

    @ManyToOne
    @JoinColumn(name = "CO_TIPO_DEMANDA")
    private TipoDemanda tipoDemanda;

    @Column(name = "TP_PROCESSO", nullable = false)
    @Convert(converter = TipoProcessoEnumConverter.class)
    private TipoProcessoEnum tipoProcesso;

    @ManyToOne
    @JoinColumn(name = "CO_CIDADE")
    private Cidade cidade;

    @Column(name = "DT_INCLUSAO", nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataInclusao;

}
