package br.gov.datasus.scodes.model.entity;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import javax.persistence.*;
import java.util.Date;
import java.util.List;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_ENTRADA_ESTOQUE", schema = "DBSCODES")
@NamedQuery(name = EntradaEstoque.OBTER_TODOS_PAGINADO, query = "SELECT ee FROM EntradaEstoque ee")
public class EntradaEstoque extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;

    public static final String OBTER_TODOS_PAGINADO = "EntradaEstoque.obterTodosPaginado";

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_ENTRADA_ESTOQUE")
    @SequenceGenerator(name = "SEQ_GENERATOR_ENTRADA_ESTOQUE", sequenceName = "SQ_ENTRADAESTOQUE_COSEQENTRADA", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_ENTRADA_ESTOQUE", nullable = false)
    private Long id;

    @Column(name = "DT_ENTRADA", nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @Setter(AccessLevel.NONE)
    private Date dataEntrada;

    @ManyToOne
    @JoinColumn(name = "CO_USUARIO", nullable = false)
    private Usuario usuario;

    @ManyToOne
    @JoinColumn(name = "CO_TIPO_MOVIMENTACAO", nullable = false)
    private TipoMovimentacao tipoMovimentacao;

    @ManyToOne(cascade = CascadeType.ALL)
    @JoinColumn(name = "CO_COMPRA_TROCA")
    private CompraTroca compraTroca;

    @ManyToOne(cascade = CascadeType.ALL)
    @JoinColumn(name = "CO_DOACAO")
    private Doacao doacao;

    @ManyToOne(cascade = CascadeType.ALL)
    @JoinColumn(name = "CO_TRANSFERENCIA")
    private Transferencia transferencia;

    @Column(name = "DS_MOTIVO", length = 200)
    private String motivo;

    @OneToMany(mappedBy = "entradaEstoque", cascade = CascadeType.ALL)
    private List<EntradaEstoqueItem> itens;

    @PrePersist
    public void prePersist() {
        this.dataEntrada = new Date();
    }
}
