package br.gov.datasus.scodes.model.entity;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import javax.persistence.*;
import java.math.BigDecimal;
import java.util.Date;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_ENTRADA_ESTOQUE_ITEM", schema = "DBSCODES")
public class EntradaEstoqueItem extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_ENTRADA_ESTOQUE_ITEM")
    @SequenceGenerator(name = "SEQ_GENERATOR_ENTRADA_ESTOQUE_ITEM", sequenceName = "SQ_ENTRADAESTOITEM_COSEQENTRAAESTOITEM", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_ENTRADA_ESTOQUE_ITEM", nullable = false)
    private Long id;

    @ManyToOne
    @JoinColumn(name = "CO_ENTRADA_ESTOQUE", nullable = false)
    private EntradaEstoque entradaEstoque;

    @ManyToOne
    @JoinColumn(name = "CO_ITEM", nullable = false)
    private Item item;

    @ManyToOne
    @JoinColumn(name = "CO_UNIDADE_DISPENSADORA", nullable = false)
    private UnidadeDispensadora unidadeDispensadora;

    @Column(name = "DT_ENTRADA_ITEM", nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @Setter(AccessLevel.NONE)
    private Date dataEntradaItem;

    @Column(name = "QT_ITEM", nullable = false)
    private Long quantidade;

    @Column(name = "ST_TERMOLABEL", nullable = false)
    private Boolean termolabel;

    @ManyToOne
    @JoinColumn(name = "CO_FABRICANTE", nullable = false)
    private Fabricante fabricante;

    @Column(name = "NU_VALOR_ITEM")
    private BigDecimal valor;

    @Column(name = "QT_ACERTO")
    private Long acertoEmbalagem;

    @PrePersist
    public void prePersist() {
        this.dataEntradaItem = new Date();
    }

    public Long getQuantidadeFinal() {
        return Math.addExact(this.quantidade, this.acertoEmbalagem != null ? this.acertoEmbalagem : 0L);
    }

    public TipoMovimentacao getTipoMovimentacao() {
        return this.entradaEstoque != null ? this.entradaEstoque.getTipoMovimentacao() : null;
    }
}
