package br.gov.datasus.scodes.model.entity;

import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_ESPECIFICACAO", schema = "DBSCODES")
@NamedQuery(
    name = Especificacao.OBTER_TODOS_PAGINADO,
    query = "SELECT espec FROM Especificacao espec" +
        " INNER JOIN FETCH espec.apresentacao grupoe" +
        " INNER JOIN FETCH espec.grupoEspecificacao grupoe" +
        " INNER JOIN FETCH espec.produto prod" +
        " INNER JOIN FETCH prod.categoria cat" +
        " INNER JOIN FETCH prod.tipoControlado tipocont" +
        " WHERE 1 = 1" +
        " AND (prod.id = :idProduto)"
)
@NamedQuery(name = Especificacao.OBTER_DOMINIO, query = "SELECT new br.gov.datasus.api.core.model.dto.DominioItemModel(espec.id, espec.nome) FROM Especificacao espec ORDER BY espec.nome ASC")
public class Especificacao extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;
    public static final String OBTER_TODOS_PAGINADO = "Especificacao.obterTodosPaginado";
    public static final String OBTER_DOMINIO = "Especificacao.obterDominio";

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_ESPECIFICACAO")
    @SequenceGenerator(name = "SEQ_GENERATOR_ESPECIFICACAO", sequenceName = "SQ_ESPECIFICACAO_COSEQESPECIF", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_ESPECIFICACAO", nullable = false)
    private Long id;

    @ManyToOne
    @JoinColumn(name = "CO_APRESENTACAO", nullable = false)
    private Apresentacao apresentacao;

    @ManyToOne
    @JoinColumn(name = "CO_GRUPO_ESPECIFICACAO", nullable = false)
    private GrupoEspecificacao grupoEspecificacao;

    @ManyToOne
    @JoinColumn(name = "CO_PRODUTO", nullable = false)
    private Produto produto;

    @Column(name = "NO_ESPECIFICACAO", length = 500, nullable = false)
    private String nome;

    @OneToMany(mappedBy = "especificacao")
    private List<Item> itens;

}
