package br.gov.datasus.scodes.model.entity;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Set;

import javax.persistence.*;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.*;
import org.hibernate.annotations.Formula;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_ITEM", schema = "DBSCODES")
@NamedQuery(
    name = Item.OBTER_TODOS_PAGINADO,
    query = "SELECT item FROM Item item" +
        " INNER JOIN FETCH item.especificacao espec" +
        " INNER JOIN FETCH item.tipoItem tipoitem" +
        " INNER JOIN FETCH item.programa prog" +
        " INNER JOIN FETCH item.tipoDispensacao tipodisp" +
        "  LEFT JOIN FETCH item.naturezaDespesa natdesp" +
        "  LEFT JOIN FETCH item.marca marca" +
        " INNER JOIN FETCH espec.apresentacao apr" +
        " INNER JOIN FETCH espec.grupoEspecificacao grupoesp" +
        " INNER JOIN FETCH espec.produto prod" +
        " INNER JOIN FETCH prod.categoria cat" +
        " INNER JOIN FETCH prod.tipoControlado tipocont" +
        " WHERE 1=1" +
        " AND (prod.id = :idProduto)" +
        " AND (prod.categoria.id = :idCategoria)" +
        " AND (espec.id = :idEspecificacao)" +
        " AND (UPPER(item.codigo) LIKE UPPER(:codigo))" +
        " AND (UPPER(FUNCTION('DBSCODES.FC_DESCRICAO_ITEM', item.id)) LIKE UPPER(:descricao))"
)
@NamedQuery(name = Item.OBTER_DOMINIO, query = "SELECT new br.gov.datasus.api.core.model.dto.DominioItemModel(item.id, item.codigo) FROM Item item")
public class Item extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;

    public static final String OBTER_TODOS_PAGINADO = "Item.obterTodosPaginado";
    public static final String OBTER_DOMINIO = "Item.obterDominio";

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_ITEM")
    @SequenceGenerator(name = "SEQ_GENERATOR_ITEM", sequenceName = "SQ_ITEM_COSEQITEM", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_ITEM", nullable = false)
    private Long id;

    @ManyToOne
    @JoinColumn(name = "CO_ESPECIFICACAO", nullable = false)
    private Especificacao especificacao;

    @Column(name = "CO_ITEM", length = 30)
    private String codigo;

    @ManyToOne
    @JoinColumn(name = "CO_MARCA")
    private Marca marca;

    @ManyToOne
    @JoinColumn(name = "CO_PROGRAMA", nullable = false)
    private Programa programa;

    @ManyToOne
    @JoinColumn(name = "CO_NATUREZA_DESPESA")
    private NaturezaDespesa naturezaDespesa;

    @ManyToOne
    @JoinColumn(name = "CO_TIPO_DISPENSACAO", nullable = false)
    private TipoDispensacao tipoDispensacao;

    @ManyToOne
    @JoinColumn(name = "CO_TIPO_ITEM", nullable = false)
    private TipoItem tipoItem;

    @Column(name = "CO_SIAFISICO", length = 20)
    private String codigoSiafisico;

    @Column(name = "DS_SIAFISICO", length = 400)
    private String descricaoSiafisico;

    @Column(name = "DS_OBSERVACAO", length = 500)
    private String observacao;

    @Column(name = "DT_IMPORTADO")
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataImportado;

    @Column(name = "DT_REVISADO_SIAFISICO")
    @Temporal(TemporalType.TIMESTAMP)
    private Date dataRevisadoSiafisico;

    @Column(name = "NU_CUSTO")
    private BigDecimal custo;

    @Column(name = "NU_MINIMO_ALERTA", nullable = false)
    private Long minimoAlerta;

    @Column(name = "NU_MINIMO_DISPENSACAO", nullable = false)
    private Long minimoDispensacao;

    @Column(name = "ST_IMPORTADO", nullable = false)
    private Boolean importado = false;

    @ManyToMany
    @JoinTable(
        name = "RL_TIPO_DEMANDA_ITEM",
        schema = "DBSCODES",
        joinColumns = @JoinColumn(name = "CO_ITEM"),
        inverseJoinColumns = @JoinColumn(name = "CO_TIPO_DEMANDA")
    )
    private Set<TipoDemanda> tipoDemandas;

    @ManyToMany
    @JoinTable(
        name = "RL_ITEM_SIMILARIDADE",
        schema = "DBSCODES",
        joinColumns = @JoinColumn(name = "CO_ITEM"),
        inverseJoinColumns = @JoinColumn(name = "CO_ITEM_SIMILAR")
    )
    private Set<Item> similares;

    @Formula(value = "DBSCODES.FC_DESCRICAO_ITEM(CO_SEQ_ITEM)")
    @Setter(value = AccessLevel.NONE)
    private String descricao;
}
