package br.gov.datasus.scodes.model.entity;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import javax.persistence.*;
import java.util.Date;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_MOVIMENTACAO", schema = "DBSCODES")
@NamedQuery(name = Movimentacao.OBTER_TODOS_PAGINADO, query = "SELECT mov FROM Movimentacao mov" +
    " INNER JOIN FETCH mov.item item" +
    " INNER JOIN FETCH mov.unidadeDispensadora uo" +
    " INNER JOIN FETCH mov.tipoMovimentacao tpmov" +
    " LEFT JOIN FETCH mov.entradaEstoque entrada" +
    " LEFT JOIN FETCH mov.saidaEstoque saida" +
    " WHERE 1=1" +
    " AND (mov.item.id = :idItem)" +
    " AND (mov.unidadeDispensadora.id = :idUnidadeDispensadora)")
public class Movimentacao extends BaseEntity<Long> {

	private static final long serialVersionUID = 1L;

	public static final String OBTER_TODOS_PAGINADO = "Movimentacao.obterTodosPaginado";

	@Id
	@GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_MOVIMENTACAO")
	@SequenceGenerator(name = "SEQ_GENERATOR_MOVIMENTACAO", sequenceName = "SQ_MOVIMENTACAO_COSEQMOVIMENTACAO", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
	@Column(name = "CO_SEQ_MOVIMENTACAO", nullable = false)
	private Long id;

	@Column(name = "DT_MOVIMENTACAO", nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @Setter(AccessLevel.NONE)
	private Date dataMovimentacao;

    @ManyToOne
    @JoinColumn(name = "CO_ITEM", nullable = false)
    private Item item;

    @ManyToOne
    @JoinColumn(name = "CO_UNIDADE_DISPENSADORA", nullable = false)
    private UnidadeDispensadora unidadeDispensadora;

    @ManyToOne
    @JoinColumn(name = "CO_TIPO_MOVIMENTACAO", nullable = false)
    private TipoMovimentacao tipoMovimentacao;

    @Column(name = "QT_MOVIMENTACAO", nullable = false)
    private Long quantidadeMovimentacao;

    @Column(name = "QT_MOVIMENTACAO_TOTAL", nullable = false)
    private Long quantidadeMovimentacaoTotal;

    @ManyToOne
    @JoinColumn(name = "CO_ENTRADA_ESTOQUE")
    private EntradaEstoque entradaEstoque;

    @ManyToOne
    @JoinColumn(name = "CO_SAIDA_ESTOQUE")
    private SaidaEstoque saidaEstoque;

    @PrePersist
    public void prePersist() {
        this.dataMovimentacao = new Date();
    }
}
