package br.gov.datasus.scodes.model.entity;

import br.gov.datasus.api.core.model.entity.BaseEntity;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;

import javax.persistence.*;
import java.util.Date;
import java.util.List;

@Getter
@Setter
@NoArgsConstructor
@Entity
@Table(name = "TB_SAIDA_ESTOQUE", schema = "DBSCODES")
@NamedQuery(name = SaidaEstoque.OBTER_TODOS_PAGINADO, query = "SELECT se FROM SaidaEstoque se")
public class SaidaEstoque extends BaseEntity<Long> {

    private static final long serialVersionUID = 1L;

    public static final String OBTER_TODOS_PAGINADO = "SaidaEstoque.obterTodosPaginado";

    @Id
    @GeneratedValue(strategy = GenerationType.SEQUENCE, generator = "SEQ_GENERATOR_SAIDA_ESTOQUE")
    @SequenceGenerator(name = "SEQ_GENERATOR_SAIDA_ESTOQUE", sequenceName = "SQ_SAIDAESTOQUE_COSEQSAIDAESTOQUE", initialValue = 1, allocationSize = 1, schema = "DBSCODES")
    @Column(name = "CO_SEQ_SAIDA_ESTOQUE", nullable = false)
    private Long id;

    @Column(name = "DT_SAIDA", nullable = false)
    @Temporal(TemporalType.TIMESTAMP)
    @Setter(AccessLevel.NONE)
    private Date dataSaida;

    @ManyToOne
    @JoinColumn(name = "CO_USUARIO", nullable = false)
    private Usuario usuario;

    @ManyToOne
    @JoinColumn(name = "CO_TIPO_MOVIMENTACAO", nullable = false)
    private TipoMovimentacao tipoMovimentacao;

    @ManyToOne(cascade = CascadeType.ALL)
    @JoinColumn(name = "CO_COMPRA_TROCA")
    private CompraTroca compraTroca;

    @ManyToOne(cascade = CascadeType.ALL)
    @JoinColumn(name = "CO_TRANSFERENCIA")
    private Transferencia transferencia;

    @Column(name = "DS_MOTIVO", length = 200)
    private String motivo;

    @OneToMany(mappedBy = "saidaEstoque", cascade = CascadeType.ALL)
    private List<SaidaEstoqueItem> itens;

    @PrePersist
    public void prePersist() {
        this.dataSaida = new Date();
    }
}
