package br.gov.datasus.scodes.model.enums;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import com.fasterxml.jackson.annotation.JsonValue;

import lombok.Getter;

public enum PatronoEnum {
    
    ADVOGADO (4, "Advogado"),
    DEFENSOR_PUBLICO (5, "Defensor Público"),
    PROMOTOR (6, "Promotor"),
    PROCURADOR (8, "Procurador"),
    JUDICIAL (51, "Judicial"),
    JEFAZ (88, "Jefaz"),
    JEC (29, "JEC");
    
    private PatronoEnum(Integer codigo, String descricao) {
        this.codigo = codigo;
        this.descricao = descricao;
    }

    @JsonValue
    @Getter
    private Integer codigo;

    @Getter
    private String descricao;
    
    @Override
    public String toString() {
        return this.descricao;
    }

    public static Optional<PatronoEnum> obterPorCodigo(final Integer codigo) {
        return PatronoEnum.stream().filter(e -> e.getCodigo().equals(codigo)).findFirst();
    }

    public static Optional<PatronoEnum> obterPorDescricao(final String descricao) {
        return PatronoEnum.stream().filter(e -> e.getDescricao().equals(descricao)).findFirst();
    }

    public static Stream<PatronoEnum> stream() {
        return Stream.of(PatronoEnum.values());
    }

    public static List<PatronoEnum> list() {
        return Arrays.asList(PatronoEnum.values());
    }
}
