package br.gov.datasus.scodes.model.enums;

import br.gov.datasus.api.core.model.annotations.Dominio;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

@Dominio("Tipo de Movimentação")
@AllArgsConstructor
public enum TipoMovimentacaoEnum implements Serializable {

    // Entrada
    COMPRA_ENTRADA(1L, "Compra", TipoMovimentacaoTipoEnum.ENTRADA),
    CARTA_TROCA_ENTRADA(2L, "Carta de Troca", TipoMovimentacaoTipoEnum.ENTRADA),
    OUTRA_ENTRADA(3L, "Outra", TipoMovimentacaoTipoEnum.ENTRADA),

    // Saida
    TRANSFERENCIA_SAIDA(4L, "Transferência", TipoMovimentacaoTipoEnum.SAIDA),
    PERDA_VENCIMENTO_SAIDA(5L, "Perda Vencimento", TipoMovimentacaoTipoEnum.SAIDA),
    QUEBRA_SAIDA(6L, "Quebra", TipoMovimentacaoTipoEnum.SAIDA),
    AJUSTES_SAIDA(7L, "Ajustes", TipoMovimentacaoTipoEnum.SAIDA),
    CARTA_TROCA_SAIDA(8L, "Carta de Troca", TipoMovimentacaoTipoEnum.SAIDA),
    OUTRA_SAIDA(9L, "Outra", TipoMovimentacaoTipoEnum.SAIDA),

    // Transferência
    TROCA_TRANSFERENCIA(10L, "Troca", TipoMovimentacaoTipoEnum.TRANSFERENCIA),
    PERDA_TRANSFERENCIA(11L, "Perda", TipoMovimentacaoTipoEnum.TRANSFERENCIA),
    SUSPENSAO_TRANSFERENCIA(12L, "Suspensão", TipoMovimentacaoTipoEnum.TRANSFERENCIA),

    // Dispensação
    DISPENSACAO(13L, "Dispensação", TipoMovimentacaoTipoEnum.DISPENSACAO),
    ESTORNO_DISPENSACAO(14L, "Estorno", TipoMovimentacaoTipoEnum.DISPENSACAO),
    RECIBO_INVALIDADO_DISPENSACAO(15L, "Recibo Invalidado", TipoMovimentacaoTipoEnum.DISPENSACAO);

    @JsonValue
    @Getter
    private final Long codigo;

    @Getter
    private final String descricao;

    @Getter
    private final TipoMovimentacaoTipoEnum tipo;

    @Override
    public String toString() {
        return this.codigo.toString();
    }

    public static Optional<TipoMovimentacaoEnum> obterPorCodigo(final Long codigo) {
        return TipoMovimentacaoEnum.stream().filter(e -> e.getCodigo().equals(codigo)).findFirst();
    }

    public static Optional<TipoMovimentacaoEnum> obterPorDescricao(final String descricao) {
        return TipoMovimentacaoEnum.stream().filter(e -> e.getDescricao().equals(descricao)).findFirst();
    }

    public static Stream<TipoMovimentacaoEnum> stream() {
        return Stream.of(TipoMovimentacaoEnum.values());
    }

    public static List<TipoMovimentacaoEnum> list() {
        return Arrays.asList(TipoMovimentacaoEnum.values());
    }
}
