package br.gov.datasus.scodes.model.enums;

import br.gov.datasus.api.core.model.annotations.Dominio;
import com.fasterxml.jackson.annotation.JsonValue;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

@Dominio("Tipos do Tipo de Movimentação")
@AllArgsConstructor
public enum TipoMovimentacaoTipoEnum implements Serializable {
    ENTRADA("E", "Entrada"),
    SAIDA("S", "Saída"),
    TRANSFERENCIA("T", "Transferência"),
    DISPENSACAO("D", "Dispensação");

    @JsonValue
    @Getter
    private final String codigo;

    @Getter
    private final String descricao;

    @Override
    public String toString() {
        return this.codigo;
    }

    public static Optional<TipoMovimentacaoTipoEnum> obterPorCodigo(final String codigo) {
        return TipoMovimentacaoTipoEnum.stream().filter(e -> e.getCodigo().equals(codigo)).findFirst();
    }

    public static Optional<TipoMovimentacaoTipoEnum> obterPorDescricao(final String descricao) {
        return TipoMovimentacaoTipoEnum.stream().filter(e -> e.getDescricao().equals(descricao)).findFirst();
    }

    public static Stream<TipoMovimentacaoTipoEnum> stream() {
        return Stream.of(TipoMovimentacaoTipoEnum.values());
    }

    public static List<TipoMovimentacaoTipoEnum> list() {
        return Arrays.asList(TipoMovimentacaoTipoEnum.values());
    }
}
