package br.gov.datasus.scodes.model.enums;

import br.gov.datasus.api.core.model.annotations.Dominio;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Getter;

import java.io.Serializable;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

@Dominio("Tipos de plano de saúde")
@JsonSerialize
@AllArgsConstructor
public enum TipoPlanoSaudeEnum implements Serializable {
    INDIVIDUAL("I", "Individual"),
    EMPRESARIAL("E", "Empresarial");

    @Getter
    @JsonValue
    private final String codigo;

    @Getter
    private final String descricao;

    @JsonCreator
    public static Optional<TipoPlanoSaudeEnum> obterPorCodigo(final String codigo) {
        return TipoPlanoSaudeEnum.stream().filter(e -> e.getCodigo().equals(codigo)).findFirst();
    }

    public static Optional<TipoPlanoSaudeEnum> obterPorDescricao(final String descricao) {
        return TipoPlanoSaudeEnum.stream().filter(e -> e.getDescricao().equals(descricao)).findFirst();
    }

    public static Stream<TipoPlanoSaudeEnum> stream() {
        return Stream.of(TipoPlanoSaudeEnum.values());
    }

    public static List<TipoPlanoSaudeEnum> list() {
        return Arrays.asList(TipoPlanoSaudeEnum.values());
    }
}
