package br.gov.datasus.scodes.model.mapper;

import br.gov.datasus.api.core.model.mapper.BaseMapper;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.*;
import br.gov.datasus.scodes.model.entity.*;
import org.mapstruct.*;

import java.util.List;

@Mapper(componentModel = "spring", unmappedTargetPolicy = ReportingPolicy.IGNORE, uses = {EntradaEstoqueItemMapper.class})
public interface EntradaEstoqueMapper extends BaseMapper<EntradaEstoque, EntradaEstoqueDTO> {

    @Mapping(source = "usuario.id", target = "idUsuario")
	EntradaEstoqueDTO toDto(EntradaEstoque entity);

    @InheritConfiguration
    @Named("toSimpleDto")
    @Mapping(target = "itens", ignore = true)
    EntradaEstoqueDTO toSimpleDto(EntradaEstoque entity);

    @IterableMapping(qualifiedByName="toSimpleDto")
	List<EntradaEstoqueDTO> toDto(List<EntradaEstoque> entities);

    @IterableMapping(qualifiedByName="toSimpleDto")
	List<EntradaEstoqueDTO> toDto(Iterable<EntradaEstoque> entities);

    @IterableMapping(qualifiedByName="toSimpleDto")
	PageableResponseModel<EntradaEstoqueDTO> toDtoPageableResponse(PageableResponseModel<EntradaEstoque> pageableResponseModel);

    @InheritInverseConfiguration(name = "toDto")
	EntradaEstoque toEntity(EntradaEstoqueDTO dto);

	List<EntradaEstoque> toEntity(List<EntradaEstoqueDTO> dtos);
	
	@InheritConfiguration
    @Mappings({
        @Mapping(target = "usuario", ignore = true),
        @Mapping(target = "dataEntrada", ignore = true),
    })
	void fromDto(EntradaEstoqueDTO dto, @MappingTarget EntradaEstoque entity);

    TipoMovimentacaoDTO tipoMovimentacaoToDto(TipoMovimentacao entity);

    TipoMovimentacao tipoMovimentacaoToEntity(TipoMovimentacaoDTO dto);

    CompraTrocaDTO compraTrocaToDto(CompraTroca entity);

    CompraTroca compraTrocaToEntity(CompraTrocaDTO dto);

    DoacaoDTO docaoToDto(Doacao entity);

    Doacao doacaoToEntity(DoacaoDTO dto);

    TransferenciaDTO transferenciaToDto(Transferencia entity);

    Transferencia transferenciaToEntity(TransferenciaDTO dto);

    @AfterMapping
    default void afterMapEntity(@MappingTarget EntradaEstoque entity) {
        List<EntradaEstoqueItem> itens = entity.getItens();
        if (itens != null) {
            itens.forEach(item -> item.setEntradaEstoque(entity));
        }
    }
}
