package br.gov.datasus.scodes.model.mapper;

import br.gov.datasus.api.core.model.mapper.BaseMapper;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.*;
import br.gov.datasus.scodes.model.entity.*;
import org.mapstruct.*;

import java.util.List;

@Mapper(componentModel = "spring", unmappedTargetPolicy = ReportingPolicy.IGNORE, uses = {SaidaEstoqueItemMapper.class})
public interface SaidaEstoqueMapper extends BaseMapper<SaidaEstoque, SaidaEstoqueDTO> {

    @Mapping(source = "usuario.id", target = "idUsuario")
    SaidaEstoqueDTO toDto(SaidaEstoque entity);

    @InheritConfiguration
    @Named("toSimpleDto")
    @Mapping(target = "itens", ignore = true)
    SaidaEstoqueDTO toSimpleDto(SaidaEstoque entity);

    @IterableMapping(qualifiedByName = "toSimpleDto")
    List<SaidaEstoqueDTO> toDto(List<SaidaEstoque> entities);

    @IterableMapping(qualifiedByName = "toSimpleDto")
    List<SaidaEstoqueDTO> toDto(Iterable<SaidaEstoque> entities);

    @IterableMapping(qualifiedByName = "toSimpleDto")
    PageableResponseModel<SaidaEstoqueDTO> toDtoPageableResponse(PageableResponseModel<SaidaEstoque> pageableResponseModel);

    @InheritInverseConfiguration(name = "toDto")
    SaidaEstoque toEntity(SaidaEstoqueDTO dto);

    List<SaidaEstoque> toEntity(List<SaidaEstoqueDTO> dtos);

    @InheritConfiguration
    @Mappings({
        @Mapping(target = "usuario", ignore = true),
        @Mapping(target = "dataSaida", ignore = true)
    })
    void fromDto(SaidaEstoqueDTO dto, @MappingTarget SaidaEstoque entity);

    TipoMovimentacaoDTO tipoMovimentacaoToDto(TipoMovimentacao entity);

    TipoMovimentacao tipoMovimentacaoToEntity(TipoMovimentacaoDTO dto);

    CompraTrocaDTO compraTrocaToDto(CompraTroca entity);

    CompraTroca compraTrocaToEntity(CompraTrocaDTO dto);

    TransferenciaDTO transferenciaToDto(Transferencia entity);

    Transferencia transferenciaToEntity(TransferenciaDTO dto);

    @AfterMapping
    default void afterMapEntity(@MappingTarget SaidaEstoque entity) {
        List<SaidaEstoqueItem> itens = entity.getItens();
        if (itens != null) {
            itens.forEach(item -> item.setSaidaEstoque(entity));
        }
    }
}
