package br.gov.datasus.scodes.repository;

import br.gov.datasus.api.core.dynamicquery.repository.DynamicNativeQueryRepository;
import br.gov.datasus.api.core.dynamicquery.repository.DynamicQueryRepository;
import br.gov.datasus.scodes.model.entity.Item;
import br.gov.datasus.scodes.model.entity.Movimentacao;
import br.gov.datasus.scodes.model.entity.TipoMovimentacao;
import br.gov.datasus.scodes.model.entity.UnidadeDispensadora;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;

import java.util.Date;
import java.util.Optional;

public interface MovimentacaoRepository extends CrudRepository<Movimentacao, Long>, DynamicQueryRepository, DynamicNativeQueryRepository {

    @Query(value = "SELECT quantidadeMovimentacaoTotal - quantidadeMovimentacao AS qtd " +
        "FROM Movimentacao WHERE id = (" +
        "   SELECT MIN(id) FROM Movimentacao " +
        "   WHERE item = :item " +
        "   AND unidadeDispensadora = :unidadeDispensadora " +
        "   AND dataMovimentacao > :dataMovimentacao" +
        ")")
    Optional<Long> findQuantidadeByDataMovimentacao(Item item, UnidadeDispensadora unidadeDispensadora,
                                                    Date dataMovimentacao);

    @Query(value = "SELECT SUM(quantidadeMovimentacao) AS qtd " +
        "FROM Movimentacao " +
        "WHERE item = :item " +
        "AND unidadeDispensadora = :unidadeDispensadora " +
        "AND dataMovimentacao > :dataMovimentacao " +
        "AND tipoMovimentacao = :tipoMovimentacao")
    Optional<Long> findQuantidadeTotalByTipoMovimentacao(Item item, UnidadeDispensadora unidadeDispensadora,
                                               Date dataMovimentacao, TipoMovimentacao tipoMovimentacao);
}
