package br.gov.datasus.scodes.repository;

import br.gov.datasus.scodes.model.entity.Produto;

import java.util.List;

import br.gov.datasus.api.core.model.dto.DominioItemModel;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import br.gov.datasus.api.core.dynamicquery.repository.DynamicNativeQueryRepository;
import br.gov.datasus.api.core.dynamicquery.repository.DynamicQueryRepository;
import org.springframework.data.repository.query.Param;

public interface ProdutoRepository extends CrudRepository<Produto, Long>, DynamicQueryRepository, DynamicNativeQueryRepository {

    List<DominioItemModel> obterDominio();

    @Query(
        " SELECT new br.gov.datasus.api.core.model.dto.DominioItemModel(produto.id, produto.nome)" +
        " FROM Produto produto" +
        " WHERE produto.categoria.id = :idCategoria" +
        " AND UPPER(produto.nome) LIKE '%' || UPPER(:termoProduto) || '%'"
    )
    List<DominioItemModel> autocomplete(
        @Param("termoProduto") String termoProduto,
        @Param("idCategoria") Long idCategoria
    );

    @Query(
        " SELECT new br.gov.datasus.api.core.model.dto.DominioItemModel(produto.id, produto.nome)" +
            " FROM Produto produto" +
            " WHERE UPPER(produto.nome) LIKE '%' || UPPER(:termoProduto) || '%'"
    )
    List<DominioItemModel> autocomplete(@Param("termoProduto") String termoProduto);

    @Query("SELECT count(e) FROM Produto p INNER JOIN p.especificacoes e WHERE p = :produto")
    Integer countVinculosEspecificacoes(Produto produto);
}
