package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.scodes.model.dto.AlteracaoDataRetornoProdutoDTO;
import br.gov.datasus.scodes.model.entity.AlteracaoDataRetornoProduto;
import br.gov.datasus.scodes.model.mapper.AlteracaoDataRetornoProdutoMapper;
import br.gov.datasus.scodes.repository.AlteracaoDataRetornoProdutoRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class AlteracaoDataRetornoProdutoService {

    private final AlteracaoDataRetornoProdutoRepository repository;
    private final AlteracaoDataRetornoProdutoMapper mapper;

    public AlteracaoDataRetornoProdutoDTO obterAlteracaoDataRetornoPorId(Long id) {
        List<AlteracaoDataRetornoProduto> list = repository.findByAlteracaoDataRetornoId(id);
        AlteracaoDataRetornoProduto entity = list.stream().findFirst().orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public AlteracaoDataRetornoProdutoDTO incluir(AlteracaoDataRetornoProdutoDTO dto) {
        AlteracaoDataRetornoProduto entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void excluirPorAlteracaoDataRetornoId(Long id) {
        repository.deleteByAlteracaoDataRetornoId(id);
    }

}
