package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.repository.ApresentacaoRepository;
import br.gov.datasus.scodes.model.entity.Apresentacao;
import br.gov.datasus.scodes.model.mapper.ApresentacaoMapper;
import br.gov.datasus.scodes.model.dto.filter.ApresentacaoFilterDTO;
import br.gov.datasus.scodes.model.dto.ApresentacaoDTO;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ApresentacaoService {

    private final ApresentacaoRepository repository;
    private final ApresentacaoMapper mapper;
    public ApresentacaoDTO obterPorId(Long id) {
        Optional<Apresentacao> one = repository.findById(id);
        Apresentacao entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<ApresentacaoDTO> obterTodos() {
        Iterable<Apresentacao> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<ApresentacaoDTO> obterTodosPaginado(PageableRequestModel<ApresentacaoFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "apresentacao");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<Apresentacao> pageableResponseModel = repository.findAll(Apresentacao.OBTER_TODOS_PAGINADO, parameters, pagination, Apresentacao.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

}
