package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.model.mapper.ClassificacaoCobrancaMapper;
import br.gov.datasus.scodes.model.entity.ClassificacaoCobranca;
import br.gov.datasus.scodes.repository.ClassificacaoCobrancaRepository;
import br.gov.datasus.scodes.model.dto.ClassificacaoCobrancaDTO;
import br.gov.datasus.scodes.model.dto.filter.ClassificacaoCobrancaFilterDTO;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ClassificacaoCobrancaService {

    private final ClassificacaoCobrancaRepository repository;
    private final ClassificacaoCobrancaMapper mapper;
    public ClassificacaoCobrancaDTO obterPorId(Long id) {
        Optional<ClassificacaoCobranca> one = repository.findById(id);
        ClassificacaoCobranca entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<ClassificacaoCobrancaDTO> obterTodos() {
        Iterable<ClassificacaoCobranca> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<ClassificacaoCobrancaDTO> obterTodosPaginado(PageableRequestModel<ClassificacaoCobrancaFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "classificacaoCobranca");
        final ClassificacaoCobrancaFilterDTO dto = pageableModel.getFilter().orElseThrow(ResourceNotFoundException::new);
        DynamicParameters parameters = DynamicParameters.get()
            .appendLike("descricao", dto.getDescricao());
        PageableResponseModel<ClassificacaoCobranca> pageableResponseModel = repository.findAll(ClassificacaoCobranca.OBTER_TODOS_PAGINADO, parameters, pagination, ClassificacaoCobranca.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)    
    public ClassificacaoCobrancaDTO incluir(ClassificacaoCobrancaDTO dto) {
        ClassificacaoCobranca entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)    
    public ClassificacaoCobrancaDTO alterar(Long id, ClassificacaoCobrancaDTO dto) {
        Optional<ClassificacaoCobranca> one = repository.findById(id);
        ClassificacaoCobranca entity = one.orElseThrow(() -> new ResourceNotFoundException());
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Optional<ClassificacaoCobranca> one = repository.findById(id);
        ClassificacaoCobranca entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }
}
