package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.message.IMessageProperty;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.model.dto.DemandaDTO;
import br.gov.datasus.scodes.model.dto.filter.DemandaFilterDTO;
import br.gov.datasus.scodes.model.entity.Demanda;
import br.gov.datasus.scodes.model.mapper.DemandaMapper;
import br.gov.datasus.scodes.repository.DemandaRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

import static java.util.Objects.nonNull;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class DemandaService {

    private final DemandaRepository repository;
    private final DemandaMapper mapper;

    public DemandaDTO obterPorId(Long id) {
        Demanda entity = obterEntityPorId(id);
       return mapper.toDto(entity);
    }

    public List<DemandaDTO> obterTodos() {
        Iterable<Demanda> itens = repository.findAll();
        return mapper.toDto(itens);
    }


    @Transactional(propagation = Propagation.REQUIRED)
    public void excluir(final Long id) {
        Demanda entity = obterEntityPorId(id);
        repository.delete(entity);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public DemandaDTO salvar(final DemandaDTO dto) {
        return nonNull(dto.getId()) ? alterar(dto) : incluir(dto);
    }

    private DemandaDTO incluir(final DemandaDTO dto) {
        Demanda entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    private DemandaDTO alterar(final DemandaDTO dto) {
        Demanda entity = obterEntityPorId(dto.getId());
        dto.setId(dto.getId());
        mapper.fromDto(dto, entity);
        return mapper.toDto(repository.save(entity));
    }

    private void parametersAppendFilter(DynamicParameters parameters, Object objFilter, String field) {
        Optional.ofNullable(objFilter).ifPresent(
            filter -> parameters.append(field, filter)
        );
    }

    private Demanda obterEntityPorId(Long id) {
        Optional<Demanda> one = repository.findById(id);
        return one.orElseThrow(ResourceNotFoundException::new);
    }

    public Demanda jaExisteDemandaComProtocoloProcessoEUnidadeOrganizacional(DemandaFilterDTO dto) {
        Optional<Demanda> demanda = repository.findByProtocoloAndProcessoAndUnidadeOrganizacionalId(dto.getProtocolo(), dto.getProcesso(), dto.getIdUnidadeOrganizacional());
        return demanda.map(d ->{
            if (dto.getProtocolo().equals(d.getProtocolo()) && dto.getProcesso().equals(d.getProcesso()) && dto.getIdUnidadeOrganizacional().equals(d.getUnidadeOrganizacional().getId())) {
                IMessageProperty bind = MessageProperty.PROTOCOLO_PROCESSO_UNIDADE_ORGANIZACIONAL_JA_EXISTE.bind(d.getProtocolo(), d.getProcesso(), d.getUnidadeOrganizacional().getDescricao());
                throw new BusinessException(bind);
            }
            return d;
        }).orElse(null);
    }
}
