package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.model.entity.Drs;
import br.gov.datasus.scodes.repository.DrsRepository;
import br.gov.datasus.scodes.model.dto.DrsDTO;
import br.gov.datasus.scodes.model.mapper.DrsMapper;
import br.gov.datasus.scodes.model.dto.filter.DrsFilterDTO;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class DrsService {

    private final DrsRepository repository;
    private final DrsMapper mapper;
    public DrsDTO obterPorId(Long id) {
        Optional<Drs> one = repository.findById(id);
        Drs entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<DrsDTO> obterTodos() {
        Iterable<Drs> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<DrsDTO> obterTodosPaginado(PageableRequestModel<DrsFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "drs");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<Drs> pageableResponseModel = repository.findAll(Drs.OBTER_TODOS_PAGINADO, parameters, pagination, Drs.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

}
