package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.EntradaEstoqueDTO;
import br.gov.datasus.scodes.model.entity.EntradaEstoque;
import br.gov.datasus.scodes.model.mapper.EntradaEstoqueMapper;
import br.gov.datasus.scodes.repository.EntradaEstoqueRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class EntradaEstoqueService {

    private final EntradaEstoqueRepository repository;
    private final EntradaEstoqueMapper mapper;
    private final MovimentacaoService movimentacaoService;

    public EntradaEstoqueDTO obterPorId(Long id) {
        EntradaEstoque entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public PageableResponseModel<EntradaEstoqueDTO> obterTodosPaginado(PageableRequestModel<Void> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "ee");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<EntradaEstoque> pageableResponseModel = repository.findAll(EntradaEstoque.OBTER_TODOS_PAGINADO,
            parameters, pagination, EntradaEstoque.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public EntradaEstoqueDTO incluir(EntradaEstoqueDTO dto) {
        EntradaEstoque entity = mapper.toEntity(dto);
        repository.save(entity);
        entity.getItens().forEach(movimentacaoService::incluirEntrada);
        return mapper.toDto(entity);
    }

}
