package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.FabricanteDTO;
import br.gov.datasus.scodes.model.entity.Fabricante;
import br.gov.datasus.scodes.model.mapper.FabricanteMapper;
import br.gov.datasus.scodes.repository.FabricanteRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class FabricanteService {

	private final FabricanteRepository repository;
	private final FabricanteMapper mapper;

	public FabricanteDTO obterPorId(Long id) {
		Fabricante entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		return mapper.toDto(entity);
	}

	public List<FabricanteDTO> obterTodos() {
		Iterable<Fabricante> itens = repository.findAll();
		return mapper.toDto(itens);
	}

    public PageableResponseModel<FabricanteDTO> obterTodosPaginado(PageableRequestModel<Void> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "fa");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<Fabricante> pageableResponseModel = repository.findAll(Fabricante.OBTER_TODOS_PAGINADO,
            parameters, pagination, Fabricante.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }
	
	public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }
}
