package br.gov.datasus.scodes.service;

import java.util.List;
import java.util.Optional;

import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.FuncionalidadeDTO;
import br.gov.datasus.scodes.model.dto.filter.FuncionalidadeFilterDTO;
import br.gov.datasus.scodes.model.entity.Funcionalidade;
import br.gov.datasus.scodes.model.mapper.FuncionalidadeMapper;
import br.gov.datasus.scodes.repository.FuncionalidadeRepository;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class FuncionalidadeService {

    private final FuncionalidadeRepository repository;
    private final FuncionalidadeMapper mapper;
    public FuncionalidadeDTO obterPorId(Long id) {
        Optional<Funcionalidade> one = repository.findById(id);
        Funcionalidade entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<FuncionalidadeDTO> obterTodos() {
        Iterable<Funcionalidade> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<FuncionalidadeDTO> obterTodosPaginado(PageableRequestModel<FuncionalidadeFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "f");
        DynamicParameters parameters = DynamicParameters.get();
        parameters.appendLike("nome", pageableModel.getFilter().get().getNome());
        PageableResponseModel<Funcionalidade> pageableResponseModel = repository.findAll(Funcionalidade.OBTER_TODOS_PAGINADO, parameters, pagination, Funcionalidade.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

}
