package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.model.dto.filter.GrupoEspecificacaoFilterDTO;
import br.gov.datasus.scodes.model.mapper.GrupoEspecificacaoMapper;
import br.gov.datasus.scodes.model.dto.GrupoEspecificacaoDTO;
import br.gov.datasus.scodes.model.entity.GrupoEspecificacao;
import br.gov.datasus.scodes.repository.GrupoEspecificacaoRepository;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class GrupoEspecificacaoService {

    private final GrupoEspecificacaoRepository repository;
    private final GrupoEspecificacaoMapper mapper;
    public GrupoEspecificacaoDTO obterPorId(Long id) {
        Optional<GrupoEspecificacao> one = repository.findById(id);
        GrupoEspecificacao entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<GrupoEspecificacaoDTO> obterTodos() {
        Iterable<GrupoEspecificacao> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<GrupoEspecificacaoDTO> obterTodosPaginado(PageableRequestModel<GrupoEspecificacaoFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "grupoEspecificacao");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<GrupoEspecificacao> pageableResponseModel = repository.findAll(GrupoEspecificacao.OBTER_TODOS_PAGINADO, parameters, pagination, GrupoEspecificacao.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio(Long idCategoria) {
        if(idCategoria == null) {
            return repository.obterDominio();
        }
        return repository.obterDominio(idCategoria);
    }

}
