package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.LocalJefazDTO;
import br.gov.datasus.scodes.model.dto.filter.LocalJefazFilterDTO;
import br.gov.datasus.scodes.model.entity.LocalJefaz;
import br.gov.datasus.scodes.model.mapper.LocalJefazMapper;
import br.gov.datasus.scodes.repository.LocalJefazRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class LocalJefazService {

    private final LocalJefazRepository repository;
    private final LocalJefazMapper mapper;
    public LocalJefazDTO obterPorId(Long id) {
        Optional<LocalJefaz> one = repository.findById(id);
        LocalJefaz entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<LocalJefazDTO> obterTodos() {
        Iterable<LocalJefaz> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<LocalJefazDTO> obterTodosPaginado(PageableRequestModel<LocalJefazFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "localJefaz");
        DynamicParameters parameters = DynamicParameters.get();
        parameters.appendLike("descricao", pageableModel.getFilter().get().getDescricao());
        PageableResponseModel<LocalJefaz> pageableResponseModel = repository.findAll(LocalJefaz.OBTER_TODOS_PAGINADO, parameters, pagination, LocalJefaz.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public LocalJefazDTO incluir(LocalJefazDTO dto) {
        LocalJefaz entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public LocalJefazDTO alterar(Long id, LocalJefazDTO dto) {
        Optional<LocalJefaz> one = repository.findById(id);
        LocalJefaz entity = one.orElseThrow(() -> new ResourceNotFoundException());
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Optional<LocalJefaz> one = repository.findById(id);
        LocalJefaz entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }
}
