package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.model.entity.*;
import br.gov.datasus.scodes.model.enums.TipoMovimentacaoEnum;
import br.gov.datasus.scodes.repository.MovimentacaoRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class MovimentacaoService {

	private final MovimentacaoRepository repository;
	private final EstoqueAuxiliarService estoqueAuxiliarService;

    @Transactional(propagation = Propagation.REQUIRED)
	public void incluirEntrada(EntradaEstoqueItem entradaItem) {
        Long quantidadeEstoque = this.estoqueAuxiliarService.adicionarEstoque(entradaItem);
        Movimentacao movimentacao = new Movimentacao();
        movimentacao.setTipoMovimentacao(entradaItem.getTipoMovimentacao());
        movimentacao.setItem(entradaItem.getItem());
        movimentacao.setUnidadeDispensadora(entradaItem.getUnidadeDispensadora());
        movimentacao.setEntradaEstoque(entradaItem.getEntradaEstoque());
        movimentacao.setQuantidadeMovimentacao(entradaItem.getQuantidadeFinal());
        movimentacao.setQuantidadeMovimentacaoTotal(quantidadeEstoque);
        repository.save(movimentacao);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public void incluirSaida(SaidaEstoqueItem saidaItem) {
        Long quantidadeEstoque = this.estoqueAuxiliarService.removerEstoque(saidaItem);
        Movimentacao movimentacao = new Movimentacao();
        movimentacao.setTipoMovimentacao(saidaItem.getTipoMovimentacao());
        movimentacao.setItem(saidaItem.getItem());
        movimentacao.setUnidadeDispensadora(saidaItem.getUnidadeDispensadora());
        movimentacao.setSaidaEstoque(saidaItem.getSaidaEstoque());
        movimentacao.setQuantidadeMovimentacao(Math.negateExact(saidaItem.getQuantidade()));
        movimentacao.setQuantidadeMovimentacaoTotal(quantidadeEstoque);
        repository.save(movimentacao);
    }

    public Long obterQuantidadeItemPorDataMovimentacao(Item item, UnidadeDispensadora unidadeDispensadora, Date dataMovimentacao) {
        return repository.findQuantidadeByDataMovimentacao(item, unidadeDispensadora, dataMovimentacao)
            .orElseGet(() -> estoqueAuxiliarService.obterQuantidade(item, unidadeDispensadora));
    }

    public Long obterQuantidadeItemPorTipoMovimentacao(Item item, UnidadeDispensadora unidadeDispensadora, Date dataMovimentacao, TipoMovimentacao tipoMovimentacao) {
        return repository.findQuantidadeTotalByTipoMovimentacao(item, unidadeDispensadora, dataMovimentacao, tipoMovimentacao)
            .orElse(0L);
    }
    public Long obterQuantidadeItemPorTipoMovimentacao(Item item, UnidadeDispensadora unidadeDispensadora, Date dataMovimentacao, TipoMovimentacaoEnum tipoMovimentacaoEnum) {
        return this.obterQuantidadeItemPorTipoMovimentacao(item, unidadeDispensadora, dataMovimentacao, TipoMovimentacao.of(tipoMovimentacaoEnum));
    }
}
