package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.NaturezaDespesaDTO;
import br.gov.datasus.scodes.model.dto.filter.NaturezaDespesaFilterDTO;
import br.gov.datasus.scodes.model.entity.NaturezaDespesa;
import br.gov.datasus.scodes.model.mapper.NaturezaDespesaMapper;
import br.gov.datasus.scodes.repository.NaturezaDespesaRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class NaturezaDespesaService {

    private final NaturezaDespesaRepository repository;
    private final NaturezaDespesaMapper mapper;
    public NaturezaDespesaDTO obterPorId(Long id) {
        Optional<NaturezaDespesa> one = repository.findById(id);
        NaturezaDespesa entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<NaturezaDespesaDTO> obterTodos() {
        Iterable<NaturezaDespesa> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<NaturezaDespesaDTO> obterTodosPaginado(PageableRequestModel<NaturezaDespesaFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "naturezaDespesa");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getNumero()).ifPresent(
                numero -> parameters.append("numero", numero)
            );
            Optional.ofNullable(filter.getDescricao()).ifPresent(
                descricao -> parameters.appendLike("descricao", descricao)
            );
        });
        PageableResponseModel<NaturezaDespesa> pageableResponseModel = repository.findAll(NaturezaDespesa.OBTER_TODOS_PAGINADO, parameters, pagination, NaturezaDespesa.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public NaturezaDespesaDTO incluir(NaturezaDespesaDTO dto) {
        NaturezaDespesa entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public NaturezaDespesaDTO alterar(Long id, NaturezaDespesaDTO dto) {
        Optional<NaturezaDespesa> one = repository.findById(id);
        NaturezaDespesa entity = one.orElseThrow(() -> new ResourceNotFoundException());
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Optional<NaturezaDespesa> one = repository.findById(id);
        NaturezaDespesa entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }
}
