package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.PerfilDTO;
import br.gov.datasus.scodes.model.dto.filter.PerfilFilterDTO;
import br.gov.datasus.scodes.model.entity.Perfil;
import br.gov.datasus.scodes.model.mapper.PerfilMapper;
import br.gov.datasus.scodes.repository.PerfilRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class PerfilService {

    private final PerfilRepository repository;
    private final PerfilMapper mapper;

    public PerfilDTO obterPorId(Long id) {
        Perfil entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<PerfilDTO> obterTodos() {
        Iterable<Perfil> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<PerfilDTO> obterTodosPaginado(PageableRequestModel<PerfilFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "p");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getNome())
                .ifPresent(nome -> parameters.appendLike("nome", nome));
        });
        PageableResponseModel<Perfil> pageableResponseModel = repository.findAll(Perfil.OBTER_TODOS_PAGINADO, parameters, pagination, Perfil.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public PerfilDTO incluir(PerfilDTO dto) {
        Perfil entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public PerfilDTO alterar(Long id, PerfilDTO dto) {
        Perfil entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Perfil entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        repository.delete(entity);
    }

    public List<DominioItemModel> obterPerfilDemandaAdmPorArea(Long idArea) {
        return repository.obterDominioDemandaAdmPorArea(idArea);
    }

    public List<DominioItemModel> obterPerfilDemandaJudicialPorArea(Long idArea) {
        return repository.obterDominioDemandaJudicialPorArea(idArea);
    }
}
