package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.PlanoSaudeDTO;
import br.gov.datasus.scodes.model.dto.filter.PlanoSaudeFilterDTO;
import br.gov.datasus.scodes.model.entity.PlanoSaude;
import br.gov.datasus.scodes.model.mapper.PlanoSaudeMapper;
import br.gov.datasus.scodes.repository.PlanoSaudeRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class PlanoSaudeService {

    private final PlanoSaudeRepository repository;
    private final PlanoSaudeMapper mapper;
    public PlanoSaudeDTO obterPorId(Long id) {
        Optional<PlanoSaude> one = repository.findById(id);
        PlanoSaude entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<PlanoSaudeDTO> obterTodos() {
        Iterable<PlanoSaude> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<PlanoSaudeDTO> obterTodosPaginado(PageableRequestModel<PlanoSaudeFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "planoSaude");
        DynamicParameters parameters = DynamicParameters.get();
        parameters.appendLike("descricao", pageableModel.getFilter().get().getDescricao());
        parameters.append("tipo", pageableModel.getFilter().get().getTipo());
        PageableResponseModel<PlanoSaude> pageableResponseModel = repository.findAll(PlanoSaude.OBTER_TODOS_PAGINADO, parameters, pagination, PlanoSaude.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public PlanoSaudeDTO incluir(PlanoSaudeDTO dto) {
        PlanoSaude entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public PlanoSaudeDTO alterar(Long id, PlanoSaudeDTO dto) {
        Optional<PlanoSaude> one = repository.findById(id);
        PlanoSaude entity = one.orElseThrow(() -> new ResourceNotFoundException());
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Optional<PlanoSaude> one = repository.findById(id);
        PlanoSaude entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }
}
