package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.repository.ProgramaRepository;
import br.gov.datasus.scodes.model.dto.filter.ProgramaFilterDTO;
import br.gov.datasus.scodes.model.entity.Programa;
import br.gov.datasus.scodes.model.dto.ProgramaDTO;
import br.gov.datasus.scodes.model.mapper.ProgramaMapper;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class ProgramaService {

    private final ProgramaRepository repository;
    private final ProgramaMapper mapper;
    public ProgramaDTO obterPorId(Long id) {
        Optional<Programa> one = repository.findById(id);
        Programa entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<ProgramaDTO> obterTodos() {
        Iterable<Programa> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<ProgramaDTO> obterTodosPaginado(PageableRequestModel<ProgramaFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "programa");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<Programa> pageableResponseModel = repository.findAll(Programa.OBTER_TODOS_PAGINADO, parameters, pagination, Programa.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

}
