package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.SaidaEstoqueDTO;
import br.gov.datasus.scodes.model.entity.SaidaEstoque;
import br.gov.datasus.scodes.model.mapper.SaidaEstoqueMapper;
import br.gov.datasus.scodes.repository.SaidaEstoqueRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class SaidaEstoqueService {

    private final SaidaEstoqueRepository repository;
    private final SaidaEstoqueMapper mapper;
    private final MovimentacaoService movimentacaoService;

    public SaidaEstoqueDTO obterPorId(Long id) {
        SaidaEstoque entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public PageableResponseModel<SaidaEstoqueDTO> obterTodosPaginado(PageableRequestModel<Void> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "se");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<SaidaEstoque> pageableResponseModel = repository.findAll(SaidaEstoque.OBTER_TODOS_PAGINADO,
            parameters, pagination, SaidaEstoque.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    @Transactional(propagation = Propagation.REQUIRED)
    public SaidaEstoqueDTO incluir(SaidaEstoqueDTO dto) {
        SaidaEstoque entity = mapper.toEntity(dto);
        repository.save(entity);
        entity.getItens().forEach(movimentacaoService::incluirSaida);
        return mapper.toDto(entity);
    }

}
