package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.repository.TipoControladoRepository;
import br.gov.datasus.scodes.model.dto.filter.TipoControladoFilterDTO;
import br.gov.datasus.scodes.model.dto.TipoControladoDTO;
import br.gov.datasus.scodes.model.entity.TipoControlado;
import br.gov.datasus.scodes.model.mapper.TipoControladoMapper;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class TipoControladoService {

    private final TipoControladoRepository repository;
    private final TipoControladoMapper mapper;
    public TipoControladoDTO obterPorId(Long id) {
        Optional<TipoControlado> one = repository.findById(id);
        TipoControlado entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<TipoControladoDTO> obterTodos() {
        Iterable<TipoControlado> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<TipoControladoDTO> obterTodosPaginado(PageableRequestModel<TipoControladoFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "tipoControlado");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<TipoControlado> pageableResponseModel = repository.findAll(TipoControlado.OBTER_TODOS_PAGINADO, parameters, pagination, TipoControlado.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

}
