package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.repository.TipoItemRepository;
import br.gov.datasus.scodes.model.entity.TipoItem;
import br.gov.datasus.scodes.model.mapper.TipoItemMapper;
import br.gov.datasus.scodes.model.dto.filter.TipoItemFilterDTO;
import br.gov.datasus.scodes.model.dto.TipoItemDTO;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class TipoItemService {

    private final TipoItemRepository repository;
    private final TipoItemMapper mapper;
    public TipoItemDTO obterPorId(Long id) {
        Optional<TipoItem> one = repository.findById(id);
        TipoItem entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<TipoItemDTO> obterTodos() {
        Iterable<TipoItem> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<TipoItemDTO> obterTodosPaginado(PageableRequestModel<TipoItemFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "tipoItem");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<TipoItem> pageableResponseModel = repository.findAll(TipoItem.OBTER_TODOS_PAGINADO, parameters, pagination, TipoItem.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

}
