package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.TipoMovimentacaoDTO;
import br.gov.datasus.scodes.model.entity.TipoMovimentacao;
import br.gov.datasus.scodes.model.enums.TipoMovimentacaoTipoEnum;
import br.gov.datasus.scodes.model.mapper.TipoMovimentacaoMapper;
import br.gov.datasus.scodes.repository.TipoMovimentacaoRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class TipoMovimentacaoService {

	private final TipoMovimentacaoRepository repository;
	private final TipoMovimentacaoMapper mapper;

	public TipoMovimentacaoDTO obterPorId(Long id) {
		TipoMovimentacao entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
		return mapper.toDto(entity);
	}

	public List<TipoMovimentacaoDTO> obterTodos() {
		Iterable<TipoMovimentacao> itens = repository.findAll();
		return mapper.toDto(itens);
	}

    public PageableResponseModel<TipoMovimentacaoDTO> obterTodosPaginado(PageableRequestModel<Void> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "tm");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<TipoMovimentacao> pageableResponseModel = repository.findAll(TipoMovimentacao.OBTER_TODOS_PAGINADO,
            parameters, pagination, TipoMovimentacao.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }
	
	public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    public List<DominioItemModel> obterDominio(TipoMovimentacaoTipoEnum tipo) {
        return repository.obterDominioPorTipo(tipo);
    }
}
