package br.gov.datasus.scodes.service;

import br.gov.datasus.scodes.model.dto.UnidadeDispensadoraDTO;
import br.gov.datasus.scodes.repository.UnidadeDispensadoraRepository;
import br.gov.datasus.scodes.model.mapper.UnidadeDispensadoraMapper;
import br.gov.datasus.scodes.model.dto.filter.UnidadeDispensadoraFilterDTO;
import br.gov.datasus.scodes.model.entity.UnidadeDispensadora;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.annotation.Propagation;
import lombok.RequiredArgsConstructor;
import java.util.List;
import java.util.Optional;
import org.springframework.data.domain.Pageable;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.api.core.model.dto.DominioItemModel;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class UnidadeDispensadoraService {

    private final UnidadeDispensadoraRepository repository;
    private final UnidadeDispensadoraMapper mapper;
    public UnidadeDispensadoraDTO obterPorId(Long id) {
        Optional<UnidadeDispensadora> one = repository.findById(id);
        UnidadeDispensadora entity = one.orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<UnidadeDispensadoraDTO> obterTodos() {
        Iterable<UnidadeDispensadora> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<UnidadeDispensadoraDTO> obterTodosPaginado(PageableRequestModel<UnidadeDispensadoraFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "unidadeDispensadora");
        DynamicParameters parameters = DynamicParameters.get();
        PageableResponseModel<UnidadeDispensadora> pageableResponseModel = repository.findAll(UnidadeDispensadora.OBTER_TODOS_PAGINADO, parameters, pagination, UnidadeDispensadora.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)    
    public UnidadeDispensadoraDTO incluir(UnidadeDispensadoraDTO dto) {
        UnidadeDispensadora entity = mapper.toEntity(dto);
        return mapper.toDto(repository.save(entity));
    }

    @Transactional(propagation= Propagation.REQUIRED)    
    public UnidadeDispensadoraDTO alterar(Long id, UnidadeDispensadoraDTO dto) {
        Optional<UnidadeDispensadora> one = repository.findById(id);
        UnidadeDispensadora entity = one.orElseThrow(() -> new ResourceNotFoundException());
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        Optional<UnidadeDispensadora> one = repository.findById(id);
        UnidadeDispensadora entity = one.orElseThrow(() -> new ResourceNotFoundException());
        repository.delete(entity);
    }
}
