package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.model.dto.DominioItemModel;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.UnidadeTransferenciaDTO;
import br.gov.datasus.scodes.model.dto.filter.UnidadeTransferenciaFilterDTO;
import br.gov.datasus.scodes.model.entity.UnidadeTransferencia;
import br.gov.datasus.scodes.model.mapper.UnidadeTransferenciaMapper;
import br.gov.datasus.scodes.repository.UnidadeTransferenciaRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation= Propagation.SUPPORTS)
public class UnidadeTransferenciaService {

    private final UnidadeTransferenciaRepository repository;
    private final UnidadeTransferenciaMapper mapper;

    public UnidadeTransferenciaDTO obterPorId(Long id) {
        UnidadeTransferencia entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public List<UnidadeTransferenciaDTO> obterTodos() {
        Iterable<UnidadeTransferencia> itens = repository.findAll();
        return mapper.toDto(itens);
    }

    public PageableResponseModel<UnidadeTransferenciaDTO> obterTodosPaginado(PageableRequestModel<UnidadeTransferenciaFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "ut");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> {
            Optional.ofNullable(filter.getDescricao()).ifPresent(descricao -> parameters.appendLike("descricao", descricao));
        });
        PageableResponseModel<UnidadeTransferencia> pageableResponseModel = repository.findAll(UnidadeTransferencia.OBTER_TODOS_PAGINADO, parameters, pagination, UnidadeTransferencia.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

    public List<DominioItemModel> obterDominio() {
        return repository.obterDominio();
    }

    @Transactional(propagation= Propagation.REQUIRED)    
    public UnidadeTransferenciaDTO incluir(UnidadeTransferenciaDTO dto) {
        UnidadeTransferencia entity = mapper.toEntity(dto);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)    
    public UnidadeTransferenciaDTO alterar(Long id, UnidadeTransferenciaDTO dto) {
        UnidadeTransferencia entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        dto.setId(id);
        mapper.fromDto(dto, entity);
        repository.save(entity);
        return mapper.toDto(entity);
    }

    @Transactional(propagation= Propagation.REQUIRED)
    public void excluir(Long id) {
        UnidadeTransferencia entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        repository.delete(entity);
    }
}
