package br.gov.datasus.scodes.service;

import br.gov.datasus.api.core.dynamicquery.parameters.DynamicParameters;
import br.gov.datasus.api.core.exception.ResourceNotFoundException;
import br.gov.datasus.api.core.helper.PaginationHelper;
import br.gov.datasus.api.core.pagination.PageableRequestModel;
import br.gov.datasus.api.core.pagination.PageableResponseModel;
import br.gov.datasus.scodes.model.dto.UsuarioDTO;
import br.gov.datasus.scodes.model.dto.filter.UsuarioFilterDTO;
import br.gov.datasus.scodes.model.entity.Usuario;
import br.gov.datasus.scodes.model.mapper.UsuarioMapper;
import br.gov.datasus.scodes.repository.UsuarioRepository;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Optional;

@Service
@RequiredArgsConstructor
@Transactional(propagation = Propagation.SUPPORTS)
public class UsuarioService {

    private final UsuarioRepository repository;
    private final UsuarioMapper mapper;

    public UsuarioDTO obterPorId(Long id) {
        Usuario entity = repository.findById(id).orElseThrow(ResourceNotFoundException::new);
        return mapper.toDto(entity);
    }

    public PageableResponseModel<UsuarioDTO> obterTodosPaginado(PageableRequestModel<UsuarioFilterDTO> pageableModel) {
        Pageable pagination = PaginationHelper.toPageable(pageableModel, "u");
        DynamicParameters parameters = DynamicParameters.get();
        pageableModel.getFilter().ifPresent(filter -> Optional.ofNullable(filter.getNomeLogin())
            .ifPresent(nomeLogin -> parameters.appendLike("nomeLogin", nomeLogin)));
        PageableResponseModel<Usuario> pageableResponseModel = repository.findAll(Usuario.OBTER_TODOS_PAGINADO, parameters, pagination, Usuario.class);
        return mapper.toDtoPageableResponse(pageableResponseModel);
    }

}
