package br.gov.datasus.scodes.util;

import br.gov.datasus.api.core.exception.BusinessException;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import java.time.DayOfWeek;
import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;

import static java.util.Objects.isNull;

@Component
public class DataUtil {

    public LocalDate convertToLocalDate(Date date) {
        if(isNull(date)) {
            throw new BusinessException("A data para conversão não pode ser nula");
        }
        return Instant.ofEpochMilli(date.getTime()).atZone(ZoneId.systemDefault()).toLocalDate();
    }

    public Date convertToDate(LocalDate localDate) {
        if(isNull(localDate)) {
            throw new BusinessException("A data para conversão não pode ser nula");
        }
        return Date.from(localDate.atStartOfDay(ZoneId.systemDefault()).toInstant());
    }

    public  boolean isFimDeSemana(LocalDate localDate) {
        if(isNull(localDate)) {
            throw new BusinessException("A data para conversão não pode ser nula");
        }
        DayOfWeek dayOfWeek = localDate.getDayOfWeek();
        return dayOfWeek == DayOfWeek.SATURDAY || dayOfWeek == DayOfWeek.SUNDAY;
    }
}
