package br.gov.datasus.scodes.validation;

import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.scodes.configuration.properties.MessageProperty;
import br.gov.datasus.scodes.util.DataUtil;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.util.Date;

import static br.gov.datasus.scodes.configuration.properties.MessageProperty.*;

@Service
@RequiredArgsConstructor
public class AlteracaoDataRetornoServiceValidation {

    private final DataUtil dataUtil;
    private final FeriadoServiceValidation feriadoServiceValidation;

    public void validate(Date novaDataRetorno, Date dataRetornoAnterior) {
        this.dataRetornoMenorDataAtualValidate(novaDataRetorno);
        this.dataRetornoEFimDeSemanaValidate(novaDataRetorno);
        this.feriadoServiceValidation.validate(novaDataRetorno);
        this.dataRetornoMenorQue15DiasDataRetornoAnteriorValidate(novaDataRetorno, dataRetornoAnterior);
        this.dataRetornoMaiorQue15DiasDataRetornoAnteriorValidate(novaDataRetorno, dataRetornoAnterior);
    }

    private void dataRetornoMenorDataAtualValidate(Date dataRetorno) {
        LocalDate ldDataRetorno = dataUtil.convertToLocalDate(dataRetorno);
        if (ldDataRetorno.isBefore(LocalDate.now())) {
            throw new BusinessException(NOVA_DATA_RETORNO_NAO_PODE_SER_MENOR_DATA_ATUAL);
        }
    }


    private  void dataRetornoEFimDeSemanaValidate(Date dataRetorno) {
        LocalDate ldDataRetorno = dataUtil.convertToLocalDate(dataRetorno);
        if (dataUtil.isFimDeSemana(ldDataRetorno)) {
            throw new BusinessException(NOVA_DATA_RETORNO_NAO_PODE_SER_FINAL_SEMANA);
        }
    }

    private void dataRetornoMenorQue15DiasDataRetornoAnteriorValidate(Date novaDataRetorno, Date dataRetornoAnterior) {
        LocalDate lNovaDataRetorno = dataUtil.convertToLocalDate(novaDataRetorno);
        LocalDate lDataRetornoAnterior = dataUtil.convertToLocalDate(dataRetornoAnterior);
        LocalDate lDataRetornoAnterior15days = lDataRetornoAnterior.minusDays(15);

        if (lNovaDataRetorno.isBefore(lDataRetornoAnterior15days)) {
            throw new BusinessException(NOVA_DATA_RETORNO_NAO_PODE_SER_MENOR_15_DIAS_DATA_ATUAL_RETORNO);
        }
    }

    private void dataRetornoMaiorQue15DiasDataRetornoAnteriorValidate(Date novaDataRetorno, Date dataRetornoAnterior) {
        LocalDate lNovaDataRetorno = dataUtil.convertToLocalDate(novaDataRetorno);
        LocalDate lDataRetornoAnterior = dataUtil.convertToLocalDate(dataRetornoAnterior);
        LocalDate lDataRetornoAnterior15days = lDataRetornoAnterior.plusDays(15);
        if (lNovaDataRetorno.isAfter(lDataRetornoAnterior15days)) {
            throw new BusinessException(NOVA_DATA_RETORNO_NAO_PODE_SER_MAIOR_15_DIAS_DATA_ATUAL_RETORNO);
        }

    }
}
