package br.gov.datasus.scodes.test.validation;

import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.scodes.ApplicationBoot;
import br.gov.datasus.scodes.model.dto.*;
import br.gov.datasus.scodes.validation.AlteracaoDataRetornoServiceValidation;
import br.gov.datasus.scodes.util.DataUtil;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.junit4.SpringRunner;

import java.time.LocalDate;
import java.util.Date;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = { ApplicationBoot.class, AlteracaoDataRetornoServiceValidation.class})
public class AlteracaoDataRetornoServiceValidationTest {

    @Autowired
    private AlteracaoDataRetornoServiceValidation serviceValidation;

    private DataUtil dataUtil;

    private AlteracaoDataRetornoDTO alteracaoDataRetornoDTO;

    @Before
    public void setUp() {
        dataUtil = new DataUtil();
        alteracaoDataRetornoDTO = criarAlteracaoDataRetornoDTO();
    }

    @Test(expected = BusinessException.class)
    public void deveLancarBusinessExceptionSeADataRetornoEMenorQueADataAtual() {
        LocalDate ldHoje = LocalDate.now();
        LocalDate ldOntem = ldHoje.minusDays(1);
        Date dOntem = dataUtil.convertToDate(ldOntem);
        this.serviceValidation.validate(dOntem, null);
    }

    @Test(expected = BusinessException.class)
    public void deveLancarBusinessExceptionSeADataRetornoForFimDeSemana() {
        LocalDate ldDomingo = LocalDate.of(2017, 12, 24);
        this.serviceValidation.validate(dataUtil.convertToDate(ldDomingo), null);
    }

    @Test(expected = BusinessException.class)
    public void deveLancarBusinessExceptionSeANovaDataRetornoEMenorQue15DiasQueADataRetornoAnterior() {
        LocalDate ldNovaDataRetorno = LocalDate.of(2020,02,7);
        LocalDate ldDataRetornoAnterior = LocalDate.of(2020,02,28);

        Date dNovaDataRetorno = dataUtil.convertToDate(ldNovaDataRetorno);
        Date dDataRetornoAnterior = dataUtil.convertToDate(ldDataRetornoAnterior);

        this.serviceValidation.validate(dNovaDataRetorno, dDataRetornoAnterior);
    }

    @Test(expected = BusinessException.class)
    public void deveLancarBusinessExceptionSeANovaDataRetornoEMaiorQue15DiasQueADataRetornoAnterior() {
        LocalDate ldNovaDataRetorno = LocalDate.of(2020,03,31);
        LocalDate ldDataRetornoAnterior = LocalDate.of(2020,02,28);

        Date dNovaDataRetorno = dataUtil.convertToDate(ldNovaDataRetorno);
        Date dDataRetornoAnterior = dataUtil.convertToDate(ldDataRetornoAnterior);

        this.serviceValidation.validate(dNovaDataRetorno, dDataRetornoAnterior);
    }

    @After
    public void tearDown() {
        alteracaoDataRetornoDTO = new AlteracaoDataRetornoDTO();
    }

    private AlteracaoDataRetornoDTO criarAlteracaoDataRetornoDTO() {
        return AlteracaoDataRetornoDTO.builder()
            .dataCadastro(new Date())
            .novaDataRetorno(new Date())
            .titulo("Xpto")
            .status(new StatusAlteracaoDataRetornoDTO()).build();
    }

}
