package br.gov.datasus.scodes.test.validation;


import br.gov.datasus.api.core.exception.BusinessException;
import br.gov.datasus.scodes.model.dto.AutorDemandaDTO;
import br.gov.datasus.scodes.model.dto.DemandaDTO;
import br.gov.datasus.scodes.model.dto.TipoDemandaDTO;
import br.gov.datasus.scodes.model.enums.TipoDemandaEnum;
import br.gov.datasus.scodes.repository.DemandaRepository;
import br.gov.datasus.scodes.repository.ProtocoloJefazRepository;
import br.gov.datasus.scodes.service.DemandaService;
import br.gov.datasus.scodes.service.ProtocoloJefazService;
import br.gov.datasus.scodes.test.builder.AutorDemandaBuilder;
import br.gov.datasus.scodes.validation.DemandaServiceValidation;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.junit4.SpringRunner;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Objects;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

@RunWith(SpringRunner.class)
@SpringBootTest(classes = {DemandaServiceValidation.class})
public class DemandaServiceValidationTest {

    @Autowired
    private DemandaServiceValidation validation;

    private AutorDemandaDTO autorDemandaDTO;

    @Before
    public void setUp() {
        autorDemandaDTO = AutorDemandaBuilder.buildAutorDemanda();
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeADemandaEhObrigatorio() {
        autorDemandaDTO.setDemanda(new DemandaDTO());
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeADataEntradaEhObrigatorio() {
        autorDemandaDTO.getDemanda().setDataEntrada(null);
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeOTipoDemandaEhObrigatorio() {
        autorDemandaDTO.getDemanda().setTipoDemanda(null);
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeOTipoDemandaIdEhObrigatorio() {
        autorDemandaDTO.getDemanda().getTipoDemanda().setId(null);
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeAUnidadeOrganizacionalEhObrigatorio() {
        autorDemandaDTO.getDemanda().setUnidadeOrganizacional(null);
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeAUnidadeOrganizacionalIdEhObrigatorio() {
        autorDemandaDTO.getDemanda().getUnidadeOrganizacional().setId(null);
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeOTipoProcessoEhObrigatorio() {
        autorDemandaDTO.getDemanda().setTipoProcesso(null);
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeOsAutoresEhObrigatorio() {
        autorDemandaDTO.setAutores(null);
        validation.validate(autorDemandaDTO);
    }

    @Test(expected = BusinessException.class)
    public void deveValidarSeOsAutoresECidadeEhObrigatorio() {
        autorDemandaDTO.getAutores().forEach(autor -> autor.getCidade().setId(null));
        validation.validate(autorDemandaDTO);
    }

    @Test
    public void deveSetarAutorDemandaDataEntradaSeDataVierNull() {
        autorDemandaDTO.setDataEntrada(null);
        validation.validate(autorDemandaDTO);
        boolean actual = Objects.nonNull(autorDemandaDTO.getDataEntrada()) ? true : false;
        assertEquals(true, actual);
    }

    @Test
    public void deveSetarDemandaDataEntrada() {
        validation.validate(autorDemandaDTO);
        boolean actual = Objects.nonNull(autorDemandaDTO.getDemanda().getDataEntrada()) ? true : false;
        assertEquals(true , actual);
    }

    @Test
    public void deveSetarDemandaDataInclusao() {
        validation.validate(autorDemandaDTO);
        boolean actual = Objects.nonNull(autorDemandaDTO.getDemanda().getDataInclusao()) ? true : false;
        assertEquals(true , actual);
    }

    /*
    @Test
    public void deveSetarProtocoloSeOTipoDemandaJefaz() {

        TipoDemandaDTO tipoDemandaDTO = new TipoDemandaDTO();
        tipoDemandaDTO.setId(3L);
        autorDemandaDTO.getDemanda().setTipoDemanda(tipoDemandaDTO);
        autorDemandaDTO.getDemanda().setProtocolo(null);
        validation.validate(autorDemandaDTO);

        assertNotNull(autorDemandaDTO.getDemanda().getProtocolo());
    }
    */

    @After
    public void tearDown() {
        autorDemandaDTO = new AutorDemandaDTO();
    }
}
