import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

import { NotFoundComponent, guardsDefault } from '@datasus/commons';

import { GrupoResolver } from './resolvers/grupo.resolver';
import { GrupoListComponent } from './pages/list/grupo.list.component';
import { GrupoFormComponent } from './pages/form/grupo.form.component';
import { PerfisResolver } from '../perfil/resolvers/perfis.resolver';

const routes: Routes = [
  {
    path: '',
    component: NotFoundComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'error',
    }
  },
  {
    path: 'listar',
    component: GrupoListComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'primary',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: 'editar/:id',
    component: GrupoFormComponent,
    resolve: {
      item: GrupoResolver,
      perfis: PerfisResolver,
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'dusky',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: 'inserir',
    component: GrupoFormComponent,
    resolve: {
      perfis: PerfisResolver,
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'success',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: '**',
    redirectTo: '',
    pathMatch: 'full'
  },
];

@NgModule({
  imports: [
    RouterModule.forChild(routes),
  ],
  exports: [
    RouterModule,
  ],
})
export class GrupoRoutingModule { }
