import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService, DominioItemModel, PageableRequestModel, PageableResponseModel } from '@datasus/core';

import { GrupoModel } from '../models/grupo.model';
import { GrupoFilterModel } from '../models/grupo-filter.model';

@Injectable()
export class GrupoService extends BaseService {

  obterPorId(id: number): Observable<GrupoModel> {
    return this.baseGet<GrupoModel>(`/grupo/${id}`);
  }

  obterTodos(): Observable<GrupoModel[]> {
    return this.baseGet<GrupoModel[]>('/grupo');
  }

  obterTodosPaginado(body: PageableRequestModel<GrupoFilterModel>): Observable<PageableResponseModel<GrupoModel>> {
    return this.basePost<PageableResponseModel<GrupoModel>>('/grupo/paginado', body);
  }

  obterDominio(): Observable<DominioItemModel[]> {
    return this.baseGet<DominioItemModel[]>('/grupo/dominio');
  }

  incluir(body: GrupoModel): Observable<GrupoModel> {
    return this.basePost<GrupoModel>('/grupo', body);
  }

  alterar(body: GrupoModel, id: number): Observable<GrupoModel> {
    return this.basePut<GrupoModel>(`/grupo/${id}`, body);
  }

  excluir(id: number): Observable<void> {
    return this.baseDelete<void>(`/grupo/${id}`);
  }

}
