import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { FlexLayoutModule } from '@angular/flex-layout';
import { NgxMaskModule } from 'ngx-mask';

import { MaterialModule } from '../../../material/material.module';
import { DatasusModule } from '../../../datasus/datasus.module';
import { SharedModule } from '../../shared/shared.module';
import { PerfilRoutingModule } from './perfil-routing.module';
import { PerfilService } from './services/perfil.service';
import { FuncionalidadeService } from './services/funcionalidade.service';
import { PerfilResolver } from './resolvers/perfil.resolver';
import { PerfilListComponent } from './pages/list/perfil.list.component';
import { PerfilFormComponent } from './pages/form/perfil.form.component';
import { TreeFuncionalidadeComponent } from './pages/form/tree-funcionalidade/tree-funcionalidade.component';
import { PerfisResolver } from './resolvers/perfis.resolver';

@NgModule({
  imports: [
    // Angular
    CommonModule,
    FormsModule,
    ReactiveFormsModule,
    // Angular Flex Layout
    FlexLayoutModule,
    // Angular Material
    MaterialModule,
    // JSDADDY Ngx Mask
    NgxMaskModule.forChild({ dropSpecialCharacters: true }),
    // Datasus
    DatasusModule,
    // App
    SharedModule,
    // Perfil
    PerfilRoutingModule,
  ],
  declarations: [
    /*declaration component*/
    PerfilListComponent,
    PerfilFormComponent,
    TreeFuncionalidadeComponent,
  ],
  providers: [
    /*declaration provider*/
    PerfilResolver,
    PerfisResolver,
    PerfilService,
    FuncionalidadeService,
  ],
})
export class PerfilModule { }
