import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

import { NotFoundComponent, guardsDefault } from '@datasus/commons';

import { DemandasListComponent } from './pages/demandas-list/demandas-list.component';
import { ProtocoloFormComponent } from './pages/protocolo-form/protocolo-form.component';
import { DemandaResolver } from './resolvers/demanda.resolver';
import { MenuHearderComponent } from './components/menu-hearder/menu-hearder.component';
import { ProcessualFormComponent } from './pages/processual-form/processual-form.component';
import { TecnicoFormComponent } from './pages/tecnico-form/tecnico-form.component';
import { AtendimentoFormComponent } from './pages/atendimento-form/atendimento-form.component';
import { HistoricoFormComponent } from './pages/historico-form/historico-form.component';
import { DispensacaoFormComponent } from './pages/dispensacao-form/dispensacao-form.component';
import { ProntuarioFormComponent } from './pages/prontuario-form/prontuario-form.component';

const routes: Routes = [
  {
    path: '',
    component: NotFoundComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'error'
    }
  },
  {
    path: 'listar',
    component: DemandasListComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'primary',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: 'menu',
    component: MenuHearderComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'success',
    },
    canActivate: [
      ...guardsDefault,
    ],

    children: [
      {
        path: 'protocolo',
        component: ProtocoloFormComponent,
        canActivate: [
          ...guardsDefault,
        ],
      },
      {
        path: 'protocolo/:id',
        component: ProtocoloFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },
      {
        path: 'processual',
        component: ProcessualFormComponent,
        canActivate: [
          ...guardsDefault,
        ],
      },
      {
        path: 'processual/:id',
        component: ProcessualFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },

      {
        path: 'tecnico/:id',
        component: TecnicoFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },

      {
        path: 'atendimento/:id',
        component: AtendimentoFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },

      {
        path: 'dispensacao/:id',
        component: DispensacaoFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },

      {
        path: 'historico/:id',
        component: HistoricoFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },

      {
        path: 'prontuario/:id',
        component: ProntuarioFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },
      {
        path: 'prontuario',
        component: ProntuarioFormComponent,
        resolve: {
          item: DemandaResolver,
        },
        canActivate: [
          ...guardsDefault,
        ],
      },


    ],
  },

  {
    path: '**',
    redirectTo: '',
    pathMatch: 'full'
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule]
})
export class DemandasRoutingModule { }
