import { ProtocoloConstants } from './../../models/protoloco.constants';
import { ListaProtocoloUtil } from './../../models/lista-protocolo.util';
import { AutorModel } from './../../../shared/models/autor.model';
import { DemandaModel } from './../../models/demanda.model';
import { Component, OnInit, ViewChild } from '@angular/core';
import { MatAutocompleteSelectedEvent, MatTreeNestedDataSource, MatDialog } from '@angular/material';
import { Router, ActivatedRoute } from '@angular/router';
import { MatTabGroup } from '@angular/material/tabs';
import { FormBuilder, FormGroup, Validators, AbstractControl } from '@angular/forms';
import { Observable } from 'rxjs';
import { switchMap, finalize, debounceTime, tap, map, take, filter } from 'rxjs/operators';
import { NestedTreeControl } from '@angular/cdk/tree';
import { DatePipe } from '@angular/common';

import { FormValidators } from '@datasus/forms';
import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel, download, stringToBlob } from '@datasus/core';

import { UfModel } from 'src/app/shared/models/uf.model';
import { DemandasService } from '../../services/demandas.service';
import { AutorDemandaModel } from '../../models/autor-demanda.model';
import { UnidadeOrganizacionalService } from 'src/app/shared/services/unidade-organizacional.service';
import { TipoDemandaService } from 'src/app/shared/services/tipo-demanda.service';
import { UfService } from 'src/app/shared/services/uf.service';
import { CidadeService } from 'src/app/shared/services/cidade.service';
import { CidadeModel } from 'src/app/shared/models/cidade.model';
import { ArvoreDemandaModel } from '../../models/arvore-demanda.model';
import { ArquivoArvoreService } from '../../services/arquivo-arvore.service';
import { ArquivoDemandaModel } from '../../models/arquivo-demanda.model';
import { DialogAdicionarArquivoComponent } from '../../components/dialog-adicionar-arquivo/dialog-adicionar-arquivo.component';
import { AutorComponent } from '../../components/autor/autor.component';
import { ProfissionalModel } from 'src/app/shared/models/profissional.model';


@Component({
  selector: 'scodes-protocolo-form.component',
  templateUrl: './protocolo-form.component.html',
  styleUrls: ['./protocolo-form.component.scss']
})

export class ProtocoloFormComponent implements OnInit {

  @ViewChild(AutorComponent) autorComponent: AutorComponent;

  // Form Group
  formGroup: FormGroup;

  // Viewport Observable
  onMobile$: Observable<boolean>;
  private autorDemdandaModel: AutorDemandaModel;

  private id: number;
  listaUfs: UfModel[];
  listaCidades: CidadeModel[] = [];
  isSalvo: boolean;
  listaUnidadeOrganizacional: DominioItemModel[];
  showField: boolean;
  placeholder: string;
  exibirMunicipio: boolean;
  isJefaz: boolean;
  isComissao: boolean;
  isJudicial: boolean;
  isAcaoContra: boolean;
  isOab: boolean;
  listaTipoDemanda: DominioItemModel[];
  oabLoading: boolean;
  judicialRequiredCondition: any;
  jefazRequiredCondition: any;
  comissaoRequiredCondition: any;
  acaoContraRequiredCondition: any;
  dataCadastro: Date;
  loadingCpf: boolean;
  demandaId: number;
  autorId: number;
  isEstadoMunicipio: boolean;
  listaProfissionais: ProfissionalModel[];

  dataSource = new MatTreeNestedDataSource<ArvoreDemandaModel>();
  treeControl = new NestedTreeControl<ArvoreDemandaModel>((node: ArvoreDemandaModel) => node.arquivos as any);
  actionButtonCtrl: Map<ArvoreDemandaModel, boolean> = new Map();

  listaDocumentos = ListaProtocoloUtil.listaDocumentos;
  listaTipoProcesso = ListaProtocoloUtil.listaTipoProcesso;
  listaTipoAcaoContra = ListaProtocoloUtil.listaTipoAcaoContra;
  listaSexos = ListaProtocoloUtil.listaSexos;
  listaPatrono = ListaProtocoloUtil.listaPatrono;

  constructor(
    private route: ActivatedRoute,
    private fb: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private demandaService: DemandasService,
    private unidadeOrganizacionalService: UnidadeOrganizacionalService,
    private tipoDemandaService: TipoDemandaService,
    private cidadeService: CidadeService,
    private arquivoArvoreService: ArquivoArvoreService,
    private matDialog: MatDialog,
    private ufService: UfService,
  ) { }

  ngOnInit(): void {
    this.autorDemdandaModel = new AutorDemandaModel();
    this.onMobile$ = this.layoutService.onMobile();
    this.layoutService.setTitle('Cadastro de Demanada - Protocolo');
    this.setDataCadastro();
    this.judicialRequiredCondition = () => this.isJudicial;
    this.jefazRequiredCondition = () => this.isJefaz;
    this.comissaoRequiredCondition = () => this.isComissao;
    this.acaoContraRequiredCondition = () => this.isJefaz && this.isAcaoContra;
    this.construirFormulario();
    this.obterAll();
    this.validacaoAll();
    this.setItemAutorDemandaModel();
    this.disableAllControls();

  }

  public isDemandaJudicialEProcessoColetivo(): Boolean {
    return +this.getTipoDemanda().value === ProtocoloConstants.ID_TIPO_DEMANDA_JUDICIAL &&
      this.getTipoProcesso().value === ProtocoloConstants.ID_TIPO_PROCESSO_COLETIVO;
  }

  public isDemandaComissaoFarmacologica(): Boolean {
    return +this.getTipoDemanda().value === ProtocoloConstants.ID_TIPO_DEMANDA_COMISSAO_FARMACOLOGICA &&
    this.getTipoProcesso().value === ProtocoloConstants.ID_TIPO_PROCESSO_INDIVIDUAL;
  }

  private setDataCadastro(): void {
    this.dataCadastro = new Date();
  }

  private isEditar = (): boolean => this.autorDemdandaModel.id ? true : false;

  private construirFormulario(): void {
    const item: AutorDemandaModel = this.getDadosAutorDemandaModel();

    this.formGroup = this.fb.group({
      demanda: this.getDemandaFormGroup(item.demanda),
      autor: this.getAutorFormGroup(item.autores)
    });
  }

  private getDemandaFormGroup(demanda: DemandaModel): FormGroup {
    return this.fb.group({
      dataEntrada: [demanda ? demanda.dataEntrada : null, Validators.required],
      horaEntrada: [demanda ? this.getHoraEMinutos(demanda.dataEntrada) : null, Validators.required],
      idTipoDemanda: [demanda ? demanda.tipoDemanda.id.toString() : null, Validators.required],
      idTipoAcaoContra: [null, FormValidators.requiredIf(this.jefazRequiredCondition)],
      idTipoProcesso: [demanda ? demanda.tipoProcesso : null, Validators.required],
      idUnidadeOrganizacional: [demanda ? demanda.unidadeOrganizacional.id.toString() : null, Validators.required],
      processo: [demanda ? demanda.processo : null, FormValidators.requiredIf(this.judicialRequiredCondition)],
      protocolo: [demanda ? demanda.protocolo : null, FormValidators.requiredIf(() => this.isComissao || this.isJudicial)],
      senhaProcesso: [demanda ? demanda.senhaProcesso : null],
      estadoDemanda: [null, FormValidators.requiredIf(this.acaoContraRequiredCondition)],
      cidadeDemanda: [null, FormValidators.requiredIf(this.acaoContraRequiredCondition)],
      idPatrono: [null, FormValidators.requiredIf(this.jefazRequiredCondition)],
      estadoPatrono: [null, FormValidators.requiredIf(() => this.isJefaz && this.isEstadoMunicipio)],
      oab: [null, FormValidators.requiredIf(() => this.isJefaz && this.isOab)],
      profissional: [null, FormValidators.requiredIf(() => this.isJefaz && this.isEstadoMunicipio)],
    });
  }

  private getAutorFormGroup(autores: AutorModel[]): FormGroup {
    return this.fb.group({
      nome: [autores ? autores[0].nome : null, Validators.compose([Validators.required, Validators.minLength(10)])],
      dataNascimento: [autores ? autores[0].dataNascimento : null, Validators.required],
      cpf: [autores ? autores[0].cpf : null, Validators.compose([Validators.required, FormValidators.cpf])],
      cns: [autores ? autores[0].cns : null],
      rg: [autores ? autores[0].rg : null],
      cep: [autores ? autores[0].cep : null],
      nomeMae: [autores ? autores[0].nomeMae : null],
      sexo: [autores ? autores[0].sexo : null],
      logradouro: [autores ? autores[0].logradouro : null],
      bairro: [autores ? autores[0].bairro : null],
      estado: [autores ? autores[0].cidade.uf : null],
      cidade: [autores ? autores[0].cidade : null, Validators.required],
      numero: [autores ? autores[0].numero : null],
      complemento: [autores ? autores[0].complemento : null],
      email: [autores ? autores[0].email : null, Validators.compose([Validators.email, Validators.minLength(5)])],
      telefone: [autores ? autores[0].telefone : null],
      obsTelefone: [autores ? autores[0].obsTelefone : null],
      outroTelefone: [autores ? autores[0].outroTelefone : null],
      obsOutroTelefone: [autores ? autores[0].obsOutroTelefone : null],
    });
  }

  getAutorId(id: number): void {
    this.autorId = id;
  }

  private getHoraEMinutos(dataEntrada: Date | string) {
    const dataHora = new Date(dataEntrada.toString());
    const hora = dataHora.getHours() < 10 ? '0' + dataHora.getHours() : dataHora.getHours();
    const minutos = dataHora.getMinutes() < 10 ? '0' + dataHora.getMinutes() : dataHora.getMinutes();
    return `${hora}:${minutos}`;
  }

  editItem(item: AutorDemandaModel): void {
    this.layoutService.setTitle(`Alteração da Demanda - Protocolo`);
    this.id = item.id;
    this.autorId = item.autores[0].id;
    this.demandaId = item.demanda.id;
    this.autorDemdandaModel = item;
    console.log('item -->', item);
    this.construirFormulario();
    this.obterArquivos();
  }

  compareUf(option: UfModel, selectValue: UfModel): boolean {
    return !!selectValue && selectValue.id === option.id;
  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.get(controlName).hasError(errorName);
  }


  displayFn(cidadeDominio?: CidadeModel): string | undefined {
    return cidadeDominio ? cidadeDominio.descricao : null;
  }

  displayProfissioanal(profissional: ProfissionalModel): string | undefined {
    return profissional ? profissional.nome : null;
  }

  obterCidades(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.cidadeService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('cidade').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }

  obterCidadesDemanda(event: MatAutocompleteSelectedEvent) {
    if (event.option.value && event.option.value.descricao) {
      this.cidadeService
        .obterPorId(event.option.value.id)
        .subscribe(
          (response) => this.formGroup.get('cidadeDemanda').setValue(response),
          (error: ErrorModel) => this.dialogService.showNotification(error.message));
    }
  }

  prevTab(tabs: MatTabGroup) {
    tabs.selectedIndex--;
  }

  adicionarArquivo(node: ArvoreDemandaModel): void {
    const dialogAdicionarArquivo = this.matDialog.open(DialogAdicionarArquivoComponent, {
      disableClose: true,
      minWidth: '50%',
      data: { diretorio: node, autorDemandaId: this.id }
    });

    dialogAdicionarArquivo.afterClosed().subscribe((resultado: boolean) => {
      if (resultado) {
        this.obterArquivos();
      }
    });
  }

  removerArquivoPorId(id: number): void {
    this.arquivoArvoreService.excluirPorId(id)
      .subscribe(
        () => this.obterArquivos(),
        (error: ErrorModel) => this.dialogService.showAlert(error.message));
  }

  removerArquivo(arquivo: ArquivoDemandaModel): void {
    console.log('arquivo --> ', arquivo);
    const date = (new DatePipe('pt-br')).transform(arquivo.dataProtocolo, 'dd/MM/yyyy');

    const buttons: Map<string, () => void> = new Map();
    buttons.set('SIM', () => this.removerArquivoPorId(arquivo.id));
    buttons.set('NÃO', () => console.log());

    this.dialogService.showConfirmation({
      title: 'Confirmação',
      question: `Tem certeza que deseja excluir o arquivo?
                  ${arquivo.descricao}${arquivo.extencao} - ${date}`,
      anwsers: buttons,
    });
  }

  obterNode(node: ArvoreDemandaModel) {
    node.isLoading = true;

    this.arquivoArvoreService
      .obterArquivoPorDiretoriosAutor(node.id, this.id)
      .pipe(finalize(() => node.isLoading = false))
      .subscribe(
        (response) => {
          console.log('obterArquivoPorDiretoriosAutor -->', response);
          node.arquivos = response;
        },
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  downloadArquivo(arquivo: ArquivoDemandaModel): void {
    download(stringToBlob(atob(arquivo.arquivo), 'application/octet-stream'), arquivo.descricao + arquivo.extencao);
  }

  getLevel = (node: ArvoreDemandaModel) => !!node.quantidadeArquivos && node.quantidadeArquivos > 0;

  isExpandable = (node: ArvoreDemandaModel) => node.expandable;

  hasChild = (_: number, node: ArvoreDemandaModel) => !!node.quantidadeArquivos && node.quantidadeArquivos > 0;

  oabFinder(event: Event): void {
    event.preventDefault();

  }

  submitAutorDemanda(event: Event) {
    console.log('form --->', this.formGroup);

    event.preventDefault();
    this.autorComponent.onSubmit();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos obrigatórios');
    }
    this.isSalvo = true;
    this.request().pipe(
      finalize(() => this.isSalvo = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/demanda/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  private obterArquivos(): void {
    this.arquivoArvoreService
      .obterDiretoriosPorAutor(this.id)
      .subscribe(
        (response) => this.dataSource.data = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private obterDataEHora(): string {
    const data = new Date(this.formGroup.value.demanda.dataEntrada);
    const hora = this.formGroup.value.demanda.horaEntrada.split(':')[0];
    const minutos = this.formGroup.value.demanda.horaEntrada.split(':')[1];
    data.setHours(Number(hora));
    data.setMinutes(Number(minutos));
    return data.toISOString();
  }

  private montarAutorDemanda(): AutorDemandaModel {
    const autorDemanda: AutorDemandaModel = {
      atendido: false,
      autores: [
        {
          id: this.autorId,
          cns: this.formGroup.value.autor.cns,
          cpf: this.formGroup.value.autor.cpf,
          rg: this.formGroup.value.autor.rg,
          dataNascimento: this.formGroup.value.autor.dataNascimento,
          sexo: this.formGroup.value.autor.sexo,
          nome: this.formGroup.value.autor.nome,
          nomeMae: this.formGroup.value.autor.nomeMae,
          cep: this.formGroup.value.autor.cep,
          bairro: this.formGroup.value.autor.bairro,
          cidade: {
            id: this.formGroup.value.autor.cidade ? this.formGroup.value.autor.cidade.id : null
          },
          logradouro: this.formGroup.value.autor.logradouro,
          numero: this.formGroup.value.autor.numero,
          complemento: this.formGroup.value.autor.complemento,
          telefone: this.formGroup.value.autor.telefone,
          obsTelefone: this.formGroup.value.autor.telefone,
          outroTelefone: this.formGroup.value.autor.outroTelefone,
          obsOutroTelefone: this.formGroup.value.autor.obsOutroTelefone,
          email: this.formGroup.value.autor.email,
        }
      ],
      dataEntrada: this.obterDataEHora(),
      demanda: {
        id: this.demandaId,
        dataEntrada: this.obterDataEHora(),
        unidadeOrganizacional: { id: this.formGroup.get('demanda.idUnidadeOrganizacional').value },
        tipoDemanda: {
          id: this.formGroup.get('demanda.idTipoDemanda').value
        },
        dataInclusao: this.obterDataEHora(),
        tipoProcesso: this.formGroup.get('demanda.idTipoProcesso').value,
        processo: this.formGroup.value.demanda.processo,
        protocolo: this.formGroup.value.demanda.protocolo,
        senhaProcesso: this.formGroup.value.demanda.senhaProcesso,
      },
      // processual: {
      //   tipoAcaoContra: { id: this.formGroup.get('idTipoAcaoContra').value },
      // }


    } as AutorDemandaModel;

    return autorDemanda;
  }

  private getDadosAutorDemandaModel(): AutorDemandaModel {
    return this.autorDemdandaModel ? this.autorDemdandaModel : new AutorDemandaModel();
  }

  private request(): Observable<AutorDemandaModel> {
    const postData: AutorDemandaModel = this.montarAutorDemanda();
    console.log('postData --->', postData);

    if (this.id) {
      return this.demandaService.alterar(<AutorDemandaModel>{
        id: this.id,
        ...postData,
      }, this.id);
    }
    return this.demandaService.incluir(postData);
  }

  private obterUfs(): void {
    this.ufService.obterTodos().subscribe(
      (response) => this.listaUfs = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  private obterUnidadesOrganizacionais(): void {
    this.unidadeOrganizacionalService.obterDominio().subscribe(
      (response) => this.listaUnidadeOrganizacional = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );
  }

  private obterTipoDemanda(): void {
    this.tipoDemandaService.obterDominio()
      .pipe(
        map((tipoDemanda: DominioItemModel[]) => {
          const arr: DominioItemModel[] = [];
          tipoDemanda.map((i) => {
            if (i.codigo !== '1') {
              arr.push(i);
            }
          });
          return arr;
        })
      )
      .subscribe(
        (response) => this.listaTipoDemanda = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private obterCidadesPorUf(): void {
    this.formGroup.get('demanda.estadoDemanda').valueChanges
      .pipe(
        tap(() => this.formGroup.get('demanda.cidadeDemanda').setValue(null)),
        switchMap((uf: UfModel) => {
          if (uf) {
            return this.cidadeService.obterCidadesPorUf(uf.id);
          }
          return [];
        }),
      ).subscribe(
        (resp: any) => this.listaCidades = resp);
  }

  private obterCidadesPorNome(): void {
    this.formGroup.get('demanda.cidadeDemanda').valueChanges
      .pipe(debounceTime(350))
      .subscribe(
        (cidade) => {
          if (!!this.formGroup.get('demanda.estadoDemanda').value && typeof cidade === 'string') {
            return this.cidadeService.obterCidadesPorNome(cidade, this.formGroup.get('demanda.estadoDemanda').value.id)
              .subscribe(
                (response) => this.listaCidades = response,
                (error: ErrorModel) => this.dialogService.showNotification(error.message));
          }
        });
  }

  private validacaoTipoAcaocontra(): void {
    this.formGroup.get('demanda.idTipoAcaoContra').valueChanges
      .subscribe((valor) => {
        if (valor === 'M' || valor === 'EM' || valor === 'UEM' || valor === 'UM') {
          this.exibirMunicipio = true;
          this.isAcaoContra = true;
        } else {
          this.exibirMunicipio = false;
          this.isAcaoContra = false;
        }
      });
  }

  private validacaoTipoDemanda(): void {
    this.formGroup.get('demanda.idTipoDemanda').valueChanges
      .subscribe((valor) => {
        this.isJudicial = false;
        this.isJefaz = false;
        this.isComissao = false;
        this.formGroup.get('demanda.idTipoProcesso').setValue(valor === '3' || valor === '4' ? 'I' : null);
        this.formGroup.get('demanda.idTipoProcesso').enable();
        this.formGroup.get('demanda.protocolo').enable();
        this.formGroup.get('demanda.protocolo').setValue(null);
        this.formGroup.get('demanda.idTipoAcaoContra').setValue(null);
        switch (valor) {
          case '2':
            this.isJudicial = true;
            break;
          case '3':
            this.isJefaz = true;
            this.formGroup.get('demanda.protocolo').disable();
            break;
          case '4':
            this.isComissao = true;
            this.formGroup.get('demanda.idTipoProcesso').disable();
            break;
        }
      });
  }

  private validacaoPatrono(): void {
    this.formGroup.get('demanda.idPatrono').valueChanges
      .subscribe((valor) => {
        if (valor === '4' || valor === '5' || valor === '8') {
          this.isOab = true;
          this.isEstadoMunicipio = true;
        } else if (valor === '88' || valor === '29') {
          this.isEstadoMunicipio = false;
          this.isOab = false;
        } else if (valor === '6') {
          this.isEstadoMunicipio = true;
          this.isOab = false;
        } else {
          this.isEstadoMunicipio = false;
          this.isOab = false;
        }
      });
  }

  private disableControlsDemandaJudicialEProcessoSeletivo(): void {
    if (this.isEditar() && this.isDemandaJudicialEProcessoColetivo()) {
      const controls = ['demanda.dataEntrada',
                        'demanda.horaEntrada',
                        'demanda.idTipoDemanda',
                        'demanda.idTipoProcesso',
                        'demanda.idUnidadeOrganizacional',
                        'demanda.protocolo'];
      this.disableControls(controls);
    }
  }

  private disableControlsDemandaComissaoFamacologica(): void {
    if (this.isEditar() && this.isDemandaComissaoFarmacologica()) {
      const controls = ['demanda.idTipoDemanda', 'demanda.idTipoProcesso'];
      this.disableControls(controls);
    }
  }

  private setItemAutorDemandaModel(): void {
    this.route.data.pipe(
      take(1),
      map((data: { item: AutorDemandaModel }) => data.item),
      filter((item: AutorDemandaModel) => !!item),
      tap((item: AutorDemandaModel) => this.editItem(item)),
    ).subscribe();
  }

  private getTipoDemanda(): AbstractControl {
    return this.formGroup.get('demanda.idTipoDemanda');
  }

  private getTipoProcesso(): AbstractControl {
    return this.formGroup.get('demanda.idTipoProcesso');
  }

  private disableControls(controls: string[]): void {
    controls.forEach(control => {
      this.formGroup.get(control).disable();
    });
  }

  private obterAll(): void {
    this.obterUfs();
    this.obterUnidadesOrganizacionais();
    this.obterTipoDemanda();
    this.obterCidadesPorUf();
    this.obterCidadesPorNome();
  }

  private validacaoAll(): void {
    this.validacaoTipoAcaocontra();
    this.validacaoTipoDemanda();
    this.validacaoPatrono();
  }

  private disableAllControls(): void {
    this.disableControlsDemandaJudicialEProcessoSeletivo();
    this.disableControlsDemandaComissaoFamacologica();
  }
}
