import { Injectable } from '@angular/core';
import { BaseService, PageableRequestModel, PageableResponseModel } from '@datasus/core';
import { Observable } from 'rxjs';
import { DemandaFilterModel } from '../models/demanda-filter.model';
import { AutorDemandaModel } from '../models/autor-demanda.model';
import { AutorModel } from 'src/app/shared/models/autor.model';

@Injectable({
  providedIn: 'root'
})
export class DemandasService extends BaseService {

  obterTodosPaginado(body: PageableRequestModel<DemandaFilterModel>): Observable<PageableResponseModel<AutorDemandaModel>> {
    return this.basePost<PageableResponseModel<AutorDemandaModel>>('/demanda/paginado', body);
  }

  incluir(body: AutorDemandaModel): Observable<AutorDemandaModel> {
    return this.basePost<AutorDemandaModel>('/demanda', body);
  }

  alterar(body: AutorDemandaModel, id: number): Observable<AutorDemandaModel> {
    return this.basePut<AutorDemandaModel>(`/demanda/${id}`, body);
  }

  excluir(id: number): Observable<void> {
    return this.baseDelete<void>(`/demanda/${id}`);
  }

  obterPorId(id: number): Observable<AutorDemandaModel> {
    return this.baseGet<AutorDemandaModel>(`/demanda/${id}`);
  }

  getCep(cep: number): Observable<number> {
    return this.baseGet<number>(`/demanda/cep/${cep}`);
  }

  obterAutorPorCpf(cpf: number): Observable<AutorModel[]> {
    const data = {};
    return this.basePost<AutorModel[]>(`/demanda/pesquisar-autor-cpf/${cpf}`, data);
  }

}
