import { ItemService } from './../../../../shared/services/item.service';
import { ValidarAlteracaoNovaDataRetornoFilterModel } from './../../models/validar-alteracao-nova-data-retorno-filter.model';
import { PesquisaDRAutorDemandaReciboFilterBuilder } from './../../models/pesquisa-dr-autor-demanda-recibo-filter-builder.model';
import { PesquisaDRAutorDemandaReciboFilterModel } from './../../models/pesquisa-dr-autor-demanda-recibo-filter.model';
import { ArquivoBase64Model } from './../../../../shared/models/arquivo-base64.model';
import { RelatorioDataRetornoAutorDemandaFilterModel } from './../../models/relatorio-data-retorno-autor-demanda-filter.model.';
import {
  ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoModel 
} from './../../models/resultado-pesquisa-dr-autor-demanda-recibo-ordem-produto.model';
import { SalvarAlteracaoDataRetornoModel } from './../../models/salvar-alteracao-data-retorno.model';
import { ResultadoPesquisaAutorDemandaModel } from './../../models/resultado-pesquisa-autor-demanda.model';
import { AlteracaoDataRetornoService } from './../../services/alteracao-data-retorno.service';
import { AlteracaoDataRetornoModel } from './../../models/alteracao-data-retorno.model';
import { TipoDemandaService } from 'src/app/shared/services/tipo-demanda.service';
import { UnidadeDispensadoraService } from 'src/app/shared/services/unidade-dispensadora.service';
import { UnidadeOrganizacionalService } from 'src/app/shared/services/unidade-organizacional.service';
import { Component, OnInit, ViewChild, OnDestroy, ChangeDetectorRef, AfterViewInit, HostListener, AfterContentChecked } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { FormGroup, FormBuilder, Validators, AbstractControl } from '@angular/forms';
import { take, map, filter, tap, finalize, debounceTime } from 'rxjs/operators';
import { trigger, transition, style, animate, query } from '@angular/animations';
import { Observable, Subscription, EMPTY } from 'rxjs';
import { MatPaginator, MatTableDataSource, MatAutocompleteSelectedEvent } from '@angular/material';
import { CdkScrollable, ScrollDispatcher } from '@angular/cdk/overlay';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel, requestItemsParams, download, base64DecodeToBlob, stringToBlob } from '@datasus/core';
import { SelectionModel } from '@angular/cdk/collections';
import { 
  PesquisaDRAutorDemandaReciboOrdemProdutoFilterModel 
} from '../../models/pesquisa-dr-autor-demanda-recibo-ordem-produto-filter.model';

@Component({
  selector: 'scodes-alteracao-data-retorno.form',
  templateUrl: './alteracao-data-retorno-form.component.html',
  styleUrls: ['./alteracao-data-retorno-form.component.scss'],
  animations: [
    trigger('card', [
      transition(':enter', [
        style({ transform: 'translateX(100%)', opacity: 0 }),
        animate('.4s .8s cubic-bezier(0.52, 0.04, 0.32, 1.26)'),
      ]),
    ]),
    trigger('tabs', [
      transition(':enter', [
        query('mat-tab-header', [
          style({ transform: 'translateY(-100%)', opacity: 0 }),
          animate('.4s 2s ease-in-out'),
        ]),
      ]),
    ]),
  ],
})
export class AlteracaoDataRetornoFormComponent implements OnInit, OnDestroy, AfterViewInit, AfterContentChecked {

  public filtro: PesquisaDRAutorDemandaReciboFilterModel;
  public filtroRelatorioCsv: RelatorioDataRetornoAutorDemandaFilterModel;
  public items: ResultadoPesquisaAutorDemandaModel[] = [];
  public itemsEditar: ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoModel[] = [];
  
  public infinityItems: ResultadoPesquisaAutorDemandaModel[] = [];
  public infinityItemsEditar: ResultadoPesquisaDRAutorDemandaReciboOrdemProdutoModel[] = [];
  public displayedColumns: string[];
  public displayedColumnsEditar: string[];

  public altaracaoDtRetorno: AlteracaoDataRetornoModel;
  public formGroup: FormGroup;
  public saving: boolean;

  // listas
  public listaUnidadeOrganizacional: DominioItemModel[];
  public listaUnidadeDispensadora: DominioItemModel[];
  public listaTipoDemanda: DominioItemModel[];
  public listaItems: DominioItemModel[];

  // Viewport Observable
  public onMobile$: Observable<boolean>;
  public onMobileEditar$: Observable<boolean>;
  

  // View Children
  @ViewChild('paginator') paginator: MatPaginator;

   // Data Table Controls (begin)
   public loading = false;
   public infinityItemsMessage = '';
   public infinityItemsMessageEditar = '';
   public total = 0;
   public pageSize = 0;
   // Data Table Controls (end)

   // View Controls (begin)
   public viewChars = false;
   // View Controls (end)

  public showTitulo = false;
  public isSelecionarAutor = false;

  dataSource: any;
  selection = new SelectionModel<ResultadoPesquisaAutorDemandaModel>(true, []);
  autoresDemanda: ResultadoPesquisaAutorDemandaModel[] = [];

  private scrollDispatcherSubscription: Subscription;

  public isDisabledCheckBoxRow = true;

  public showEdit = false;
  public showInserir = false;
  public jaExecutado = false;
  public processingCsv = false;
  public processingPdf = false;
  public showNovaDataRetorno = true;
  public itemDescricao: string;
  public showItemDescricao = false;
  
  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private alteracaoDataRetornoService: AlteracaoDataRetornoService,
    private itemService: ItemService,
    private scrollDispatcher: ScrollDispatcher,
    private unidadeOrganizacionalService: UnidadeOrganizacionalService,
    private unidadeDispensadoraService: UnidadeDispensadoraService,
    private tipoDemandaService: TipoDemandaService,
    private changeDetectorRef: ChangeDetectorRef,
    
  ) { }

  ngOnInit(): void {
    this.altaracaoDtRetorno = new AlteracaoDataRetornoModel();
    this.contruirFormulario();

    this.obterUnidadeOrganizacional();
    this.obterUnidadeDispensadora();
    this.obterTipoDemanda();

    this.setOnMobile();
    this.setOnMibileEditar();
    this.setDisplayedColumns();
    this.setDisplayedColumnsEditar();

    this.setItemAlteracaoDtRetornoModel();
    this.autoCompletePesquisarAutorDemanda();
    this.autoCompleteProduto();
  }

  ngAfterViewInit(): void {
    this.setTitle();
    this.setItemsEditar();
    this.setDisableNovaDataRetornoETitulo();
    this.validacaoSelecionarAutorDemanda();
    this.changeDetectorRef.detectChanges();
  }

  ngAfterContentChecked(): void {
    const showEdit = this.isEditar();
    if (showEdit !== this.showEdit) {
      this.showEdit = showEdit;
      this.changeDetectorRef.detectChanges();
    }

  }

  // TODO: melhoria - implementar status executando.
  @HostListener('window:beforeunload', ['$event'])
  beforeunloadHandler($event) {
    $event.preventDefault();    
  }

  ngOnDestroy(): void {
    this.scrollRegister(false);
    this.infinityItems = [];
    this.infinityItemsEditar = [];
  }

  obterItem(event: MatAutocompleteSelectedEvent) {
    const id = event.option.value.codigo;
    if (event.option.value && id) {
      this.itemService.obterPorId(id).subscribe(resp => {
        const { descricao } = resp;
        this.itemDescricao = descricao;
        this.showItemDescricao = true;
      });
    }
  }

  public displayFn(produtoDominio?: DominioItemModel): string | undefined {
    return produtoDominio ? produtoDominio.descricao : undefined;
  }

  public gerarRelatorioCsv(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.showNotificationFormInvalid();
    }
  
    const filtroRelatorioCsv = this.setFiltroRelatorioCsv(
                              this.altaracaoDtRetorno.id, 
                              this.formGroup.get('novaDataRetorno').value); 
    this.processingCsv = true;
    this.alteracaoDataRetornoService.relatorioCsv(filtroRelatorioCsv).pipe(
      finalize(() => this.processingCsv = false)
    ).subscribe(
      (arquivo: ArquivoBase64Model) => download(base64DecodeToBlob(arquivo.base64, arquivo.mimeType), arquivo.filename),
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  public gerarRelatorioPdf(event: Event): void {
    event.preventDefault();

    if (this.formGroup.invalid) {
      return this.showNotificationFormInvalid();
    }

    this.processingPdf = true;
    this.alteracaoDataRetornoService.relatorioPdf(this.altaracaoDtRetorno.id).pipe(
      finalize(() => this.processingPdf = false)
    ).subscribe(
      (arquivo: ArquivoBase64Model) => { 
        download(stringToBlob(atob(arquivo.base64), arquivo.mimeType), arquivo.filename);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message),
    );
  }

  public onChangeCheckeBoxAllHeader(event) {

    if (event.checked) {
      this.dataSource.data.forEach(row => {
        this.autoresDemanda.push(row);
      });
    } else {
      this.autoresDemanda = [];
    }

    return event ? this.masterToggle() : null;
  }

  public onChangeCheckeBoxRow(event, row) {
    if (event.checked) {
      this.autoresDemanda.push(row);
    } else {
      this.autoresDemanda = this.removeAutorDemanda(this.autoresDemanda, row);
    }
    return event ? this.selection.toggle(row) : null;
  }

  editItem(item: AlteracaoDataRetornoModel): void {
    this.layoutService.setTitle(`Alteração do ${item.titulo}`);
    this.altaracaoDtRetorno = item;
    this.contruirFormulario();
  }

  public compareFn(item1: DominioItemModel, itme2: DominioItemModel): boolean {
    return item1 && itme2 ? item1.codigo === itme2.codigo : item1 === itme2;
  }

  validateAndSave(event: Event): void {
    event.preventDefault();
    if (this.formGroup.invalid) {
      return this.showNotificationFormInvalid();
    }
    this.saving = true;
    this.save().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      (resp) => {
        const msg = this.altaracaoDtRetorno.id ? 'alterado' : 'salvo';
        this.dialogService.showNotification(`Registro ${msg} com sucesso`);
        // TODO: seta para  lista ou para o editar ? ! ?
        // this.router.navigate([`/rotina/tabela/produto/${resp.id}/especificacao/inserir`]);
          this.router.navigate([`/rotina/alteracao-data-retorno/listar`]);
      },
      (error: ErrorModel) => {
        this.dialogService.showAlert(error.message);
      },
    );
  }

  hasError = (controlName: string, errorName: string) => {
    return this.formGroup.controls[controlName].hasError(errorName);
  }

  public isAllSelected(): boolean {
    const numSelected = this.selection.selected.length;
    const numRows = this.dataSource.data.length;
    return numSelected === numRows;

  }

  public getIndeterminate() {
    return this.selection.hasValue() && !this.isAllSelected();
  }

  public checkedAll() {
    return this.selection.hasValue() && this.isAllSelected();
  }

  public checkedRow(row) {
    return this.selection.isSelected(row);
  }

  public masterToggle() {
    this.isAllSelected() ?
        this.selection.clear() :
        this.dataSource.data.forEach(row => this.selection.select(row));
  }

  public checkboxLabel(row?: ResultadoPesquisaAutorDemandaModel): string {
    if (!row) {
      return `${this.isAllSelected() ? 'select' : 'deselect'} all`;
    }
    return `${this.selection.isSelected(row) ? 'deselect' : 'select'} row ${row.codigoReciboOrdemProduto + 1}`;
  }

  public salvarDisabledInvalid(): boolean {
    let valid = true;

    if (!this.isEditar() && this.getDataRetorno().valid 
        && this.getNovaDataRetorno().valid 
        && this.getTitulo().valid 
        && this.autoresDemanda.length > 0) {
      valid = false;
    }

      if (this.isEditar()
        && this.getNovaDataRetorno().valid
        && this.getTitulo().valid) {
      valid = false;
    }
    return valid;
  }

  public get botaoSalvarEExecutar(): string {
    return this.isEditar() ? 'EXECUTAR' : 'SALVAR';
  }

  public isPaginaDetalhar(): boolean {
    const url = window.location.pathname;
    return url.includes('detalhar') ? true : false;
  }

  private getDataRetorno(): AbstractControl {
    return this.formGroup.get('dataRetorno');
  }

  private getNovaDataRetorno(): AbstractControl {
    return this.formGroup.get('novaDataRetorno');
  }

  private getTitulo(): AbstractControl {
    return this.formGroup.get('titulo');
  }

  private contruirFormulario(): void {
    const item: AlteracaoDataRetornoModel = this.getDadosAlteracaoDtRetorno();

    this.formGroup = this.formBuilder.group({
      dataRetorno: [Validators.required],
      protocolo: [],
      processo: [],
      autor: [],
      produto: [],
      unidadeOrganizacional: [],
      unidadesDispensadoras: [],
      tipoDemanda:  [],
      status: [item.status ? item.status.descricao : null ],
      novaDataRetorno: [item.novaDataRetorno],
      titulo: [item.titulo]
    });

    if (item.status) {
      this.formGroup.get('status').disable();
    }

  }

  private getDadosAlteracaoDtRetorno(): AlteracaoDataRetornoModel {
    return this.altaracaoDtRetorno ? this.altaracaoDtRetorno : new AlteracaoDataRetornoModel();
  }

  private isEditar = (): boolean => this.altaracaoDtRetorno.id ? true : false;
  
  // Item Actions (end)
  private scrollRegister(register: boolean): void {

    if (!register) {
      return this.scrollDispatcherSubscription
        ? this.scrollDispatcherSubscription.unsubscribe()
        : void 0;
    }

    this.scrollDispatcherSubscription = this.scrollDispatcher.scrolled().subscribe(
      (data: CdkScrollable) => {
        if (!data) {
          return;
        }
        const bottom = data.measureScrollOffset('bottom');
        const top = data.measureScrollOffset('top');
        if ((bottom / top) <= 0.5) {
          if (!this.loading) {
            this.paginator.nextPage();
          }
        }
      }
    );
  }

  private save(): Observable<SalvarAlteracaoDataRetornoModel> {
    const objSave = this.getSalvarAlteracaoDataRetornoModel();
    return this.incluirOuAlerar(objSave, this.altaracaoDtRetorno.id);
  }

  private incluirOuAlerar(obj: SalvarAlteracaoDataRetornoModel, id: number): Observable<SalvarAlteracaoDataRetornoModel> {
    if (id) {
      return this.alteracaoDataRetornoService.alterar({ ...obj }, id);
    }
    return this.alteracaoDataRetornoService.incluir(obj);
  }

  private getSalvarAlteracaoDataRetornoModel(): SalvarAlteracaoDataRetornoModel {
    return new SalvarAlteracaoDataRetornoModel(
      this.getAlteracaoDataRetornoModel(),
      this.getPesquisaDRAutorDemandaReciboOrdemProdutoFilterModel());
  }

  private getAlteracaoDataRetornoModel(): AlteracaoDataRetornoModel {
    return new AlteracaoDataRetornoModel(
      this.formGroup.value.novaDataRetorno, this.formGroup.value.dataRetorno, this.formGroup.value.titulo);
  }

  private getPesquisaDRAutorDemandaReciboOrdemProdutoFilterModel(): PesquisaDRAutorDemandaReciboOrdemProdutoFilterModel {
    return new PesquisaDRAutorDemandaReciboOrdemProdutoFilterModel(
      this.formGroup.value.dataRetorno, this.gerarCodigosAutoresFilter());
  }

  get produto() {
    return this.formGroup.value.produto;
  }

  private gerarCodigosAutoresFilter() {
    return this.autoresDemanda ?
      this.autoresDemanda.map((vl: ResultadoPesquisaAutorDemandaModel) => ( vl.codigoAutorDemanda )) : null;
  }

  private obterUnidadeOrganizacional(): void {
    this.unidadeOrganizacionalService
      .obterDominio()
      .subscribe((value) => this.listaUnidadeOrganizacional = value);
  }

  private obterUnidadeDispensadora(): void {
    this.unidadeDispensadoraService
      .obterDominio()
      .subscribe((value) => this.listaUnidadeDispensadora = value);
  }

  private obterTipoDemanda(): void {
    this.tipoDemandaService.obterDominio().subscribe(
      (response: DominioItemModel[]) => this.listaTipoDemanda = response,
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );
  }
  private setOnMobile(): void {
    this.onMobile$ = this.layoutService.onMobile().pipe(
      tap((onMobile: boolean) => this.scrollRegister(onMobile))
    );
  }

  private setOnMibileEditar(): void {
    this.onMobileEditar$ = this.layoutService.onMobile().pipe(
      tap((onMobileEditar: boolean) => this.scrollRegister(onMobileEditar))
    );
  }

  private setItemAlteracaoDtRetornoModel(): void {
    this.route.data.pipe(
      take(1),
      map((data: {item: AlteracaoDataRetornoModel}) => data.item),
      filter((item: AlteracaoDataRetornoModel) => !!item),
      tap((item: AlteracaoDataRetornoModel) => this.editItem(item)),
    ).subscribe();
  }

  private autoCompletePesquisarAutorDemanda(): void {
    this.formGroup.valueChanges
      .pipe(debounceTime(350))
      .subscribe((filter: any) => {
        const filtro = this.setFilter(filter);

        if (filtro.dataRetorno !== null &&
          filtro.dataRetorno.getDate !== undefined && 
          !this.isEditar() && this.getNovaDataRetorno().value === null) {
          this.pesquisaAutorDemandaPaginado(filtro);
        }

        if (filtro.dataRetorno === null) {
          this.items = [];
        }

    });
  }

  private autoCompleteProduto(): void {
    this.formGroup.get('produto').valueChanges
      .pipe(debounceTime(350))
      .subscribe(
        (codigo) => {
          if (codigo.length === 0) {
            this.showItemDescricao = false;
          }

          if (typeof codigo === 'string') {
            return this.itemService
              .obterAutocompletePorCodigo(codigo)
              .subscribe(
                (response) => this.listaItems = response,
                (error: ErrorModel) => this.dialogService.showNotification(error.message));
          }
        });
  }

  private gerarUnidadeDispensadoraFitler(unidadesDispensadoras: DominioItemModel[]) {
    return unidadesDispensadoras ? unidadesDispensadoras
                                    .map((vl: DominioItemModel) => ( +vl.codigo )) : null;
  }

  private pesquisaAutorDemandaPaginado(filter: PesquisaDRAutorDemandaReciboFilterModel) {
    this.alteracaoDataRetornoService.pesquisaDRAutorDemandaReciboPaginado({
      filter,
      ...requestItemsParams({
        to: this,
      }),
    }).pipe(
      finalize(() => {
        this.loading = false;
        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [ ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      })
    ).subscribe(
      (page) => {

        this.total = page.total;
        this.items = page.items;

        this.dataSource = new MatTableDataSource<ResultadoPesquisaAutorDemandaModel>(this.items);
   
        if (this.items) {
          this.setValidatorsNovaDataRetornoETitulo();
        }

        if (!this.items) {
          return this.dialogService.showNotification('Nenhum registro encontrado.');
        }

        if (this.scrollDispatcherSubscription) {
          this.infinityItems = [...this.infinityItems, ...this.items];
          this.infinityItemsMessage = `${this.infinityItems.length} de ${this.total}`;
          this.changeDetectorRef.detectChanges();
        }
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );
  }

  private setDisplayedColumns(): void {
    this.displayedColumns = [
      'select',
      'autor',
      'dataCadastro',
      'protocolo',
      'processo',
      'unidadeDispensadora',
      'tipoDemanda',
      'dataRetorno'
    ];
  }

  private setDisplayedColumnsEditar(): void {
    this.displayedColumnsEditar = [
      'autor',
      'dataCadastro',
      'protocolo',
      'processo',
      'unidadeDispensadora',
      'tipoDemanda',
      'dataRetorno'
    ];
  }

  private validacaoSelecionarAutorDemanda(): void {
    this.formGroup.valueChanges.subscribe(val => {
 
      const novaDataRetorno = val.novaDataRetorno ? val.novaDataRetorno : null;
      const dataRetornoAnterior = this.getDataRetorno().value ? this.getDataRetorno().value : null; 
      const titulo = val.titulo ? val.titulo : null;
      const dto = new ValidarAlteracaoNovaDataRetornoFilterModel(novaDataRetorno, dataRetornoAnterior, titulo);

      if (this.isEditar()) {
        this.validarNovaDataRetornoEditar(this.altaracaoDtRetorno.id, dto);
      } else {
        this.validarNovaDataRetorno(dto);
      }


      if (novaDataRetorno !== null && titulo !== null) {
        this.isDisabledCheckBoxRow = false;
        this.isSelecionarAutor = true;
      }
      
      if (this.dataSource !== undefined && (novaDataRetorno === null || titulo === null)) {
        this.dataSource.data.forEach(row => {
            this.selection.clear();
            this.autoresDemanda = [];
        });

        this.isDisabledCheckBoxRow = true;
        this.isSelecionarAutor = false;
      }
    });
  }

  private removeAutorDemanda(autores, element) {
    return autores.filter(autor => autor !== element);
  }

  private setValidatorsNovaDataRetornoETitulo(): void {
    this.formGroup.get('novaDataRetorno').setValidators([Validators.required]);
    this.formGroup.get('titulo').setValidators([Validators.required]);
  }

  private setItemsEditar(): void {
   
    if (this.isEditar()) {
      this.showEdit = true;
      this.setValidatorsNovaDataRetornoETitulo();
      this.alteracaoDataRetornoService.pesquisaDRAutorDemandaADRPorCodigo(this.altaracaoDtRetorno.id)
          .subscribe(resp => {
            this.itemsEditar = resp;
            if (this.scrollDispatcherSubscription) {
              this.infinityItemsEditar = [...this.infinityItemsEditar, ...this.itemsEditar];
              this.infinityItemsMessageEditar = `${this.infinityItemsEditar.length}`;
              this.changeDetectorRef.detectChanges();
            }
      });
    } else {
      this.showInserir = true;
    }

  }

  private setDisableNovaDataRetornoETitulo(): void {
    if (this.altaracaoDtRetorno.status !== undefined && this.altaracaoDtRetorno.status.id === 3) {
      this.jaExecutado = true;
      this.formGroup.get('novaDataRetorno').disable();
      this.formGroup.get('titulo').disable();
      this.showNovaDataRetorno = false;
   }
  }

  private setFilter(filter: any): PesquisaDRAutorDemandaReciboFilterModel {
    return new PesquisaDRAutorDemandaReciboFilterBuilder()
      .setDataRetorno(filter.dataRetorno)
      .setProtocolo(filter.protocolo)
      .setProcesso(filter.processo)
      .setAutor(filter.autor)
      .setIdsUnidadesDispensadoras(this.gerarUnidadeDispensadoraFitler(filter.unidadesDispensadoras))
      .setCodigoTipoDemanda(filter.tipoDemanda)
      .build();
  }


  private setFiltroRelatorioCsv(codigoAlteracaoDataRetorno: number, dataRetornoAtual: Date): RelatorioDataRetornoAutorDemandaFilterModel {
    return new RelatorioDataRetornoAutorDemandaFilterModel(
      codigoAlteracaoDataRetorno,
      dataRetornoAtual);
  }

  private showNotificationFormInvalid() {
    return this.dialogService.showNotification('Preencha os campos corretamente');
  }

  private validarNovaDataRetorno(dto: ValidarAlteracaoNovaDataRetornoFilterModel): void {

    if (dto !== null && dto.novaDataRetorno !== null && dto.titulo === null) {
      this.alteracaoDataRetornoService.validarNovaDataRetorno(dto)
        .subscribe(() => {},
        (error: ErrorModel) => this.dialogService.showAlert(error.message));
    }
  }

  private validarNovaDataRetornoEditar(id: number, dto: ValidarAlteracaoNovaDataRetornoFilterModel): void {
    if (id !== null && dto !== null && dto.novaDataRetorno !== null) {
      this.alteracaoDataRetornoService.validarNovaDataRetornoEditar(id, dto)
      .subscribe(() => {},
      (error: ErrorModel) => this.dialogService.showAlert(error.message));
    }
  }

  private setTitle(): void {
    if (this.isEditar()) {
      this.layoutService.setTitle('Executar Data Retorno');
    } if (this.isPaginaDetalhar()) {
      this.layoutService.setTitle('Detalhar Data Retorno');
    } else if (!this.isEditar()) {
      this.layoutService.setTitle('Cadastrar Data Retorno');
    }
  }


}
