import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

import { guardsDefault } from '@datasus/commons';

import { GestaoEstoqueComponent } from './pages/gestao-estoque/gestao-estoque.component';
import { EntradaEstoqueComponent } from './pages/entrada-estoque/entrada-estoque.component';
import { SaidaEstoqueComponent } from './pages/saida-estoque/saida-estoque.component';
import { UnidadeDispensadoraResolver } from './resolvers/unidade-dispensadora.resolver';

const routes: Routes = [
  {
    path: '',
    component: GestaoEstoqueComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'primary',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: 'entrada/:ud',
    component: EntradaEstoqueComponent,
    resolve: {
      unidadeDispensadora: UnidadeDispensadoraResolver,
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'success',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: 'saida/:ud',
    component: SaidaEstoqueComponent,
    resolve: {
      unidadeDispensadora: UnidadeDispensadoraResolver,
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'dusky',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: '**',
    redirectTo: '',
    pathMatch: 'full'
  },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule]
})
export class GestaoEstoqueRoutingModule { }
