import { Component, OnInit } from '@angular/core';
import { FormGroup, FormBuilder } from '@angular/forms';
import { ActivatedRoute, Router } from '@angular/router';
import { finalize } from 'rxjs/operators';
import { Observable } from 'rxjs';

import { LayoutService, DialogService } from '@datasus/layout';
import { ErrorModel, DominioItemModel } from '@datasus/core';

import { InativarDemandaService } from '../../services/inativar-demanda.service';
import { InativarDemandaModel } from '../../models/inativar-demanda.model';
import { UnidadeOrganizacionalModel } from 'src/app/shared/models/unidade-organizacional.model';
import { UnidadeOrganizacionalService } from 'src/app/shared/services/unidade-organizacional.service';
import { TipoDemandaService } from 'src/app/shared/services/tipo-demanda.service';
import { UnidadeDispensadoraService } from 'src/app/shared/services/unidade-dispensadora.service';


@Component({
  selector: 'scodes-inativar-demanda-form',
  templateUrl: './inativar-demanda-form.component.html',
  styleUrls: ['./inativar-demanda-form.component.scss']
})
export class InativarDemandaFormComponent implements OnInit {


  formGroup: FormGroup;
  saving: boolean;
  listaUnidadeOrganizacional: DominioItemModel[];
  listaTipoDemanda: DominioItemModel[];
  listaUnidadeDispensadora: DominioItemModel[];
  items: any;


  displayedColumns: string[] = [
    'autor',
    'dataCadastro',
    'protocolo',
    'processo',
    'unidadeDispensadora',
    'tipoDemanda'
  ];

  private id: number;

  constructor(
    private route: ActivatedRoute,
    private formBuilder: FormBuilder,
    private router: Router,
    private layoutService: LayoutService,
    private dialogService: DialogService,
    private unidadeOrganizacionalService: UnidadeOrganizacionalService,
    private inativarDemandaService: InativarDemandaService,
    private tipoDemandaService: TipoDemandaService,
    private unidadeDispensadoraService: UnidadeDispensadoraService,
  ) { }

  ngOnInit() {
    this.layoutService.setTitle('Novo cadastro');
    this.id = null;
    this.obterTipoDemanda();
    this.obterUnidadeOrganizacional();
    this.obterUnidadeDispensadora();
    this.construirFormulario();
  }

  save(event: Event): void {
    event.preventDefault();

    if (!this.formGroup.valid) {
      return this.dialogService.showNotification('Preencha os campos corretamente');
    }

    this.saving = true;
    this.request().pipe(
      finalize(() => this.saving = false)
    ).subscribe(
      () => {
        this.dialogService.showNotification('Registro salvo com sucesso');
        this.router.navigate(['/rotina/inativar-demanda/listar']);
      },
      (error: ErrorModel) => this.dialogService.showAlert(error.message)
    );
  }

  private obterTipoDemanda(): void {
    this.tipoDemandaService
      .obterDominio()
      .subscribe(
        (response) => this.listaTipoDemanda = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private obterUnidadeOrganizacional(): void {
    this.unidadeOrganizacionalService
      .obterDominio()
      .subscribe(
        (response) => this.listaUnidadeOrganizacional = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private obterUnidadeDispensadora(): void {
    this.unidadeDispensadoraService
      .obterDominio()
      .subscribe(
        (response) => this.listaUnidadeDispensadora = response,
        (error: ErrorModel) => this.dialogService.showAlert(error.message)
      );
  }

  private construirFormulario(): void {
    this.formGroup = this.formBuilder.group({
      rotina: [1],
      parametro: [null],
      tipoDemanda: [null],
      unidadeOrganizacional: [null],
      unidadeDispensadora: [null],
    });
  }

  private request(): Observable<InativarDemandaModel> {
    if (this.id) {
      return this.inativarDemandaService.alterar(this.id, {
        id: this.id,
        ...this.formGroup.value,
      });
    }
    return this.inativarDemandaService.incluir(this.formGroup.value);
  }
}

