import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService, PageableRequestModel, PageableResponseModel } from '@datasus/core';

import { ArquivoBase64Model } from 'src/app/shared/models/arquivo-base64.model';

import { InventarioFilterModel } from '../models/inventario-filter.model';
import { InventarioModel } from '../models/inventario.model';

@Injectable()
export class InventarioService extends BaseService {

  obterPorId(id: number): Observable<InventarioModel> {
    return this.baseGet<InventarioModel>(`/inventario/${id}`);
  }

  obterTodosPaginado(body: PageableRequestModel<InventarioFilterModel>): Observable<PageableResponseModel<InventarioModel>> {
    return this.basePost<PageableResponseModel<InventarioModel>>('/inventario/paginado', body);
  }

  incluir(body: InventarioModel): Observable<InventarioModel> {
    return this.basePost<InventarioModel>('/inventario', body);
  }

  consolidar(id: number): Observable<void> {
    return this.basePut<void>(`/inventario/${id}/consolidar`, null);
  }

  relatorioItem(id: number, idItens?: number[]): Observable<ArquivoBase64Model> {
    const params = idItens != null ? {'idItens':  idItens.map(v => String(v))} : null;
    return this.baseGetWithParams<ArquivoBase64Model>(`/inventario/${id}/relatorio-item`, params);
  }

  relatorioConsolidado(id: number): Observable<ArquivoBase64Model> {
    return this.baseGet<ArquivoBase64Model>(`/inventario/${id}/relatorio-consolidado`);
  }
}
