import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService, DominioItemModel, PageableRequestModel, PageableResponseModel } from '@datasus/core';

import { LocalJefazModel } from '../models/local-jefaz.model';
import { LocalJefazFilterModel } from '../models/local-jefaz-filter.model';

@Injectable()
export class LocalJefazService extends BaseService {

  obterPorId(id: number): Observable<LocalJefazModel> {
    return this.baseGet<LocalJefazModel>(`/local-jefaz/${id}`);
  }

  obterTodos(): Observable<LocalJefazModel[]> {
    return this.baseGet<LocalJefazModel[]>('/local-jefaz');
  }

  obterTodosPaginado(body: PageableRequestModel<LocalJefazFilterModel>): Observable<PageableResponseModel<LocalJefazModel>> {
    return this.basePost<PageableResponseModel<LocalJefazModel>>('/local-jefaz/paginado', body);
  }

  obterDominio(): Observable<DominioItemModel[]> {
    return this.baseGet<DominioItemModel[]>('/local-jefaz/dominio');
  }

  incluir(body: LocalJefazModel): Observable<LocalJefazModel> {
    return this.basePost<LocalJefazModel>('/local-jefaz', body);
  }

  alterar(body: LocalJefazModel, id: number): Observable<LocalJefazModel> {
    return this.basePut<LocalJefazModel>(`/local-jefaz/${id}`, body);
  }

  excluir(id: number): Observable<void> {
    return this.baseDelete<void>(`/local-jefaz/${id}`);
  }

}
