import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService, DominioItemModel, PageableRequestModel, PageableResponseModel } from '@datasus/core';

import { PlanoSaudeModel } from '../models/plano-saude.model';
import { PlanoSaudeFilterModel } from '../models/plano-saude-filter.model';

@Injectable()
export class PlanoSaudeService extends BaseService {

  obterPorId(id: number): Observable<PlanoSaudeModel> {
    return this.baseGet<PlanoSaudeModel>(`/plano-saude/${id}`);
  }

  obterTodos(): Observable<PlanoSaudeModel[]> {
    return this.baseGet<PlanoSaudeModel[]>('/plano-saude');
  }

  obterTodosPaginado(body: PageableRequestModel<PlanoSaudeFilterModel>): Observable<PageableResponseModel<PlanoSaudeModel>> {
    return this.basePost<PageableResponseModel<PlanoSaudeModel>>('/plano-saude/paginado', body);
  }

  obterDominio(): Observable<DominioItemModel[]> {
    return this.baseGet<DominioItemModel[]>('/plano-saude/dominio');
  }

  incluir(body: PlanoSaudeModel): Observable<PlanoSaudeModel> {
    return this.basePost<PlanoSaudeModel>('/plano-saude', body);
  }

  alterar(body: PlanoSaudeModel, id: number): Observable<PlanoSaudeModel> {
    return this.basePut<PlanoSaudeModel>(`/plano-saude/${id}`, body);
  }

  excluir(id: number): Observable<void> {
    return this.baseDelete<void>(`/plano-saude/${id}`);
  }

}
