import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';

import { NotFoundComponent, guardsDefault } from '@datasus/commons';

import { ListComponent } from './pages/list/list.component';
import { EspecificacaoFormComponent } from './pages/especificacao-form/especificacao-form.component';
import { ProdutoFormComponent } from './pages/produto-form/produto-form.component';
import { ItemFormComponent } from './pages/item-form/item-form.component';
import { ProdutoResolver } from './resolvers/produto.resolver';
import { EspecificacaoResolver } from './resolvers/especificacao.resolver';
import { ItemResolver } from './resolvers/item.resolver';


const routes: Routes = [
  { path: '', component: NotFoundComponent, data: { layoutAnimation: true, layoutColor: 'error', } },
  {
    path: 'listar',
    component: ListComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'primary',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: 'inserir',
    component: ProdutoFormComponent,
    data: {
      layoutAnimation: true,
      layoutColor: 'success',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: ':idProduto/editar',
    component: ProdutoFormComponent,
    resolve: {
      produto: ProdutoResolver,
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'dusky',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },

  {
    path: ':idProduto/especificacao/inserir',
    component: EspecificacaoFormComponent,
    resolve: {
      produto: ProdutoResolver
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'dusky',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: ':idProduto/especificacao/:idEspecificacao/editar',
    component: EspecificacaoFormComponent,
    resolve: {
      produto: ProdutoResolver,
      especificacao: EspecificacaoResolver
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'dusky',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  {
    path: ':idProduto/especificacao/:idEspecificacao/item/inserir',
    component: ItemFormComponent,
    resolve: {
      produto: ProdutoResolver,
      especificacao: EspecificacaoResolver
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'dusky',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },

  {
    path: ':idProduto/especificacao/:idEspecificacao/item/:idItem/editar',
    component: ItemFormComponent,
    resolve: {
      produto: ProdutoResolver,
      especificacao: EspecificacaoResolver,
      item: ItemResolver,
    },
    data: {
      layoutAnimation: true,
      layoutColor: 'dusky',
    },
    canActivate: [
      ...guardsDefault,
    ],
  },
  /*path component*/
  { path: '**', redirectTo: '', pathMatch: 'full' },
];

@NgModule({
  imports: [RouterModule.forChild(routes)],
  exports: [RouterModule]
})
export class ProdutoRoutingModule { }
