import { NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { FlexLayoutModule } from '@angular/flex-layout';
import { NgxCurrencyModule } from 'ngx-currency';
import { NgxMaskModule } from 'ngx-mask';

import { ProdutoRoutingModule } from './produto-routing.module';
import { MaterialModule } from 'src/material/material.module';
import { DatasusModule } from 'src/datasus/datasus.module';
import { SharedModule } from 'src/app/shared/shared.module';
import { ItemResolver } from './resolvers/item.resolver';
import { ListComponent } from './pages/list/list.component';
import { EspecificacaoService } from './services/especificacao.service';
import { EspecificacaoResolver } from './resolvers/especificacao.resolver';
import { EspecificacaoFormComponent } from './pages/especificacao-form/especificacao-form.component';
import { ItemFormComponent } from './pages/item-form/item-form.component';
import { ProdutoFormComponent } from './pages/produto-form/produto-form.component';
import { CategoriaService } from './services/categoria.service';
import { ProdutoResolver } from './resolvers/produto.resolver';
import { TipoControladoService } from './services/tipo-controlado.service';
import { ProdutoService } from './services/produto.service';
import { GrupoEspecificacaoService } from './services/grupo-especificacao.service';
import { ApresentacaoService } from './services/apresentacao.service';
import { TipoItemService } from './services/tipo-item.service';
import { MarcaService } from './services/marca.service';
import { NaturezaDespesaService } from '../natureza-despesa/services/natureza-despesa.service';
import { AreaService } from 'src/app/shared/services/area.service';
import { TipoDispensacaoService } from './services/tipo-dispensacao.service';
import { ProgramaService } from './services/programa.service';

@NgModule({
  imports: [
    // Angular
    CommonModule,
    FormsModule,
    ReactiveFormsModule,
    // Angular Flex Layout
    FlexLayoutModule,
    // Angular Material
    MaterialModule,
    // JSDADDY Ngx Mask
    NgxMaskModule.forChild({ dropSpecialCharacters: true }),
    // Ngx Currency
    NgxCurrencyModule,
    // Datasus
    DatasusModule,
    // App
    SharedModule,
    // Item
    ProdutoRoutingModule,
  ],
  declarations: [
    /*declaration component*/
    ListComponent,
    EspecificacaoFormComponent,
    ProdutoFormComponent,
    ItemFormComponent,
  ],
  providers: [
    /*declaration provider*/
    ItemResolver,
    EspecificacaoService,
    EspecificacaoResolver,
    CategoriaService,
    ProdutoService,
    ProdutoResolver,
    TipoControladoService,
    GrupoEspecificacaoService,
    ApresentacaoService,
    TipoItemService,
    MarcaService,
    NaturezaDespesaService,
    AreaService,
    TipoDispensacaoService,
    ProgramaService,

  ],
})
export class ProdutoModule { }
