import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService, DominioItemModel, PageableRequestModel, PageableResponseModel } from '@datasus/core';

import { TipoDocumentoModel } from '../models/tipo-documento.model';
import { TipoDocumentoFilterModel } from '../models/tipo-documento-filter.model';

@Injectable()
export class TipoDocumentoService extends BaseService {

  obterPorId(id: number): Observable<TipoDocumentoModel> {
    return this.baseGet<TipoDocumentoModel>(`/tipo-documento/${id}`);
  }

  obterTodos(): Observable<TipoDocumentoModel[]> {
    return this.baseGet<TipoDocumentoModel[]>('/tipo-documento');
  }

  obterTodosPaginado(body: PageableRequestModel<TipoDocumentoFilterModel>): Observable<PageableResponseModel<TipoDocumentoModel>> {
    return this.basePost<PageableResponseModel<TipoDocumentoModel>>('/tipo-documento/paginado', body);
  }

  obterDominio(): Observable<DominioItemModel[]> {
    return this.baseGet<DominioItemModel[]>('/tipo-documento/dominio');
  }

  incluir(body: TipoDocumentoModel): Observable<TipoDocumentoModel> {
    return this.basePost<TipoDocumentoModel>('/tipo-documento', body);
  }

  alterar(body: TipoDocumentoModel, id: number): Observable<TipoDocumentoModel> {
    return this.basePut<TipoDocumentoModel>(`/tipo-documento/${id}`, body);
  }

  excluir(id: number): Observable<void> {
    return this.baseDelete<void>(`/tipo-documento/${id}`);
  }

}
