import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService, PageableRequestModel, PageableResponseModel } from '@datasus/core';

import { ModeloRelatorioTecnicoModel } from '../models/modelo-relatorio-tecnico.model';
import { TipoDemandaModel } from '../models/tipo-demanda.model';

@Injectable()
export class ModeloRelatorioTecnicoService extends BaseService {

  obterPorId(id: number): Observable<ModeloRelatorioTecnicoModel> {
    return this.baseGet<ModeloRelatorioTecnicoModel>(`/modelo-relatorio-tecnico/${id}`);
  }

  obterTodosPaginado(body: PageableRequestModel<ModeloRelatorioTecnicoModel>): Observable<PageableResponseModel<ModeloRelatorioTecnicoModel>> {
    return this.basePost<PageableResponseModel<ModeloRelatorioTecnicoModel>>('/modelo-relatorio-tecnico/paginado', body);
  }

  listarTiposRelatorioTecnico(body: PageableRequestModel<TipoDemandaModel>): Observable<PageableResponseModel<TipoDemandaModel>> {
    return this.basePost<PageableResponseModel<TipoDemandaModel>>('/modelo-relatorio-tecnico/tipos_demandas_relatorio_tecnico', body);
  }

  incluir(body: ModeloRelatorioTecnicoModel): Observable<ModeloRelatorioTecnicoModel> {
    return this.basePost<ModeloRelatorioTecnicoModel>('/modelo-relatorio-tecnico', body);
  }

  alterar(body: ModeloRelatorioTecnicoModel, id: number): Observable<ModeloRelatorioTecnicoModel> {
    return this.basePut<ModeloRelatorioTecnicoModel>(`/modelo-relatorio-tecnico/${id}`, body);
  }

  excluir(id: number): Observable<void> {
    return this.baseDelete<void>(`/modelo-relatorio-tecnico/${id}`);
  }

}
