import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';

import { BaseService, PageableRequestModel, PageableResponseModel } from '@datasus/core';

import { VaraModel } from '../models/vara.model';
import { CidadeModel } from '../models/cidade.model';

@Injectable()
export class VaraService extends BaseService {

  obterPorId(id: number): Observable<VaraModel> {
    return this.baseGet<VaraModel>(`/vara/${id}`);
  }

  obterTodosPaginado(body: PageableRequestModel<VaraModel>): Observable<PageableResponseModel<VaraModel>> {
    return this.basePost<PageableResponseModel<VaraModel>>('/vara/paginado', body);
  }

  pesquisarMunicipioVara(body: PageableRequestModel<CidadeModel>,  uf: string,
    nome: string): Observable<PageableResponseModel<CidadeModel>> {
    return this.basePost<PageableResponseModel<CidadeModel>>(`/vara/pesquisar-municipio-vara/uf${uf}/municipio/${nome}`, body);
  }

  incluir(body: VaraModel): Observable<VaraModel> {
    return this.basePost<VaraModel>('/vara', body);
  }

  alterar(body: VaraModel, id: number): Observable<VaraModel> {
    return this.basePut<VaraModel>(`/vara/${id}`, body);
  }

  excluir(id: number): Observable<void> {
    return this.baseDelete<void>(`/vara/${id}`);
  }

}
